<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:om="http://www.openmath.org/OpenMath"
  xmlns:dc="http://purl.org/DC"
  xmlns:OD="http://www.mathweb.org/omdoc"
  version="1.0">

  <xsl:include href="od2ht_dc.xsl"/>
  <xsl:include href="od2ht_om_oms.xsl"/>
  <xsl:include href="od2ht_om.xsl"/>

  <xsl:output method="html"/>

  <xsl:variable name="format">MathML</xsl:variable>

  <xsl:template match="/">
    <html>
      <head>
        <title><xsl:value-of select="OD:omdoc/metadata/dc:Title"/></title>
      </head>
      <body>
        <xsl:apply-templates/>
      </body>
    </html>
  </xsl:template>

<!-- ============================================================ -->
<!-- ==================   Symbol definitions   ================== --> 
<!-- ============================================================ -->

  <xsl:template match="OD:symbol[xml:lang('en')]">
    symbol_en
  </xsl:template>
  
  <xsl:template match="OD:symbol[xml:lang('es')]">
    symbol_es
  </xsl:template>
  
  <xsl:template match="OD:symbol">
    <xsl:variable name="id" select="@id"/>
    <xsl:choose>
      <xsl:when test="xml:lang('en')">
        <b>Symbol definition:</b><xsl:value-of select="OD:commonname"/><br />
      </xsl:when>
      <xsl:when test="xml:lang('es')">
        <b>Definición de símbolo:</b><xsl:value-of select="OD:commonname"/><br />
      </xsl:when>
      <xsl:otherwise>
        <b>sym:</b><xsl:value-of select="OD:commonname"/><br />
      </xsl:otherwise>
    </xsl:choose>
    <xsl:apply-templates select="//OD:presentation[@for=$id]" mode="show"/>
  </xsl:template>
  
  <xsl:template match="OD:presentation" mode="show">
    <xsl:choose>
      <xsl:when test="lang('en')">
        <b>Presentation:</b>
        <xsl:value-of select="OD:use[@format=$format]" disable-output-escaping="yes"/>
        <br />
      </xsl:when>
      <xsl:when test="lang('es')">
        <b>Se representa con:</b>
        <xsl:value-of select="OD:use[@format=$format]" disable-output-escaping="yes"/>
        <br />
      </xsl:when>
      <xsl:otherwise>
        <b>pres:</b>
        <xsl:value-of select="OD:use[@format=$format]" disable-output-escaping="yes"/>
        <br />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="OD:presentation">
  </xsl:template>
  
<!-- ============================================================ -->
<!-- ==================     Paragraph types    ================== --> 
<!-- ============================================================ -->

  <xsl:template match="OD:example">
    <xsl:apply-templates select="metadata">
      <xsl:with-param name="layout">extensive</xsl:with-param>
    </xsl:apply-templates>
    <xsl:element name="a">
      <xsl:attribute name="name"><xsl:value-of select="@id"/></xsl:attribute>
      Example:
    </xsl:element>
    <xsl:apply-templates select="*[not(self::OD:metadata)]"/>
  </xsl:template>

  <xsl:template match="OD:assertion">
    <xsl:apply-templates select="metadata">
      <xsl:with-param name="layout">extensive</xsl:with-param>
    </xsl:apply-templates>
    <xsl:element name="a">
      <xsl:attribute name="name"><xsl:value-of select="@id"/></xsl:attribute>
      Example:
    </xsl:element>
    <xsl:apply-templates select="*[not(self::OD:metadata)]"/>
  </xsl:template>

  <xsl:template match="OD:omtext">
    <p class="omtext">
      <xsl:apply-templates select="metadata">
        <xsl:with-param name="layout">extensive</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="*[not(self::OD:metadata)]"/>
    </p>
  </xsl:template>

  <xsl:template match="OD:omtext[@type]">
    <p class="omtext">
      <xsl:apply-templates select="metadata"/>
      <h3><xsl:value-of select="@type"/></h3>
      <xsl:apply-templates select="*[not(self::OD:metadata)]"/>
    </p>
  </xsl:template>

  <xsl:template match="OD:omtext[@id and not(@type)]">
    <dl>
      <dt>
        <xsl:element name="a">
          <xsl:attribute name="name"><xsl:value-of select="@id"/></xsl:attribute>
          [<xsl:value-of select="@id"/>]<xsl:apply-templates select="metadata"/>
        </xsl:element>
      </dt>
      <xsl:apply-templates select="*[not(self::OD:metadata)]"/>
    </dl>
  </xsl:template>
  
  <xsl:template match="OD:CMP">
    <p class="cmp1"><xsl:apply-templates/></p>
  </xsl:template>
  
  <xsl:template match="OD:CMP[../@id]">
    <xsl:choose>
      <xsl:when test="../OD:omtext[@type]">
        <p class="cmp2"><xsl:apply-templates/></p>
      </xsl:when>
      <xsl:otherwise>
        <dd class="cmp3"><xsl:apply-templates/></dd>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="OD:FMP">
    <p class="fmp1">Formalmente:<xsl:apply-templates/></p>
  </xsl:template>
  
<!-- ============================================================ -->
<!-- ==================       References       ================== --> 
<!-- ============================================================ -->

  <xsl:template match="OD:ref[@xref]">
    <xsl:element name="a">
      <xsl:attribute name="href">#<xsl:value-of select="@xref"/></xsl:attribute>
      <xsl:choose>
        <xsl:when test="node()">
          <xsl:apply-templates/>
        </xsl:when>
        <xsl:otherwise>
          [<xsl:value-of select="@xref"/>]
        </xsl:otherwise>
      </xsl:choose>
    </xsl:element>
  </xsl:template>
  
</xsl:stylesheet>
