<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:om="http://www.openmath.org/OpenMath"
  version="1.0">

  <!-- xsl:include href="o2ht_om_oms.xsl"/ -->

  <xsl:output method="html"/>

<!-- ============================================================ -->
<!-- ==================        OpenMath        ================== --> 
<!-- ============================================================ -->

  <xsl:template match="om:OMOBJ">
    <math>
      <xsl:apply-templates/>
    </math>
  </xsl:template>
  
  <xsl:template name="infix_notation">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:choose>
      <xsl:when test="$parent_precedence &gt; $precedence">
        <xsl:text disable-output-escaping="yes">&lt;mfenced separators=""&gt;</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text disable-output-escaping="yes">&lt;mrow&gt;</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:for-each select="*">
      <xsl:choose>
        <xsl:when test="position()=1"></xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates select=".">
            <xsl:with-param name="parent_precedence" select="$precedence"/>
          </xsl:apply-templates>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:if test="not(position()=1 or (position()=last()))">
        <xsl:apply-templates select="../*[1]"/>
      </xsl:if>
    </xsl:for-each>
    <xsl:choose>
      <xsl:when test="$parent_precedence &gt; $precedence">
        <xsl:text disable-output-escaping="yes">&lt;/mfenced&gt;</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text disable-output-escaping="yes">&lt;/mrow&gt;</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="superindex_notation">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:if test="$parent_precedence &gt; $precedence">
      <xsl:text disable-output-escaping="yes">&lt;mfenced separators=""&gt;</xsl:text>
    </xsl:if>
    <msup>
      <xsl:for-each select="*">
        <xsl:if test="position()!=1">
          <xsl:apply-templates select=".">
            <xsl:with-param name="parent_precedence" select="$precedence"/>
          </xsl:apply-templates>
        </xsl:if>
      </xsl:for-each>
    </msup>
    <xsl:if test="$parent_precedence &gt; $precedence">
      <xsl:text disable-output-escaping="yes">&lt;/mfenced&gt;</xsl:text>
    </xsl:if>
  </xsl:template>

  <xsl:template name="root_notation">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:if test="$parent_precedence &gt; $precedence">
      <xsl:text disable-output-escaping="yes">&lt;mfenced separators=""&gt;</xsl:text>
    </xsl:if>
    <mroot>
      <xsl:for-each select="*">
        <xsl:if test="position()!=1">
          <xsl:apply-templates select=".">
            <xsl:with-param name="parent_precedence" select="$precedence"/>
          </xsl:apply-templates>
        </xsl:if>
      </xsl:for-each>
    </mroot>
    <xsl:if test="$parent_precedence &gt; $precedence">
      <xsl:text disable-output-escaping="yes">&lt;/mfenced&gt;</xsl:text>
    </xsl:if>
  </xsl:template>

  <xsl:template name="function_notation">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <mrow>
      <xsl:apply-templates select="*[1]"/>
      <xsl:choose>
        <xsl:when test="count(*) &gt; 2 or om:OMA or $parent_precedence &gt; $precedence">
          <xsl:text disable-output-escaping="yes">&lt;mfenced separators=","&gt;</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text disable-output-escaping="yes">&lt;mrow&gt;</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:for-each select="*">
        <xsl:if test="position()!=1">
          <xsl:apply-templates select=".">
            <xsl:with-param name="parent_precedence" select="$precedence"/>
          </xsl:apply-templates>
        </xsl:if>
      </xsl:for-each>
      <xsl:choose>
        <xsl:when test="count(*) &gt; 2 or om:OMA or $parent_precedence &gt; $precedence">
          <xsl:text disable-output-escaping="yes">&lt;/mfenced&gt;</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text disable-output-escaping="yes">&lt;/mrow&gt;</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </mrow>
  </xsl:template>
  
  <xsl:template name="prefix_notation">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:if test="$parent_precedence &gt; $precedence">
      <xsl:text disable-output-escaping="yes">&lt;mfenced separators=""&gt;</xsl:text>
    </xsl:if>
    <mrow>
      <xsl:apply-templates select="*[1]"/>
      <xsl:apply-templates select="*[2]"/>
    </mrow>
    <xsl:if test="$parent_precedence &gt; $precedence">
      <xsl:text disable-output-escaping="yes">&lt;/mfenced&gt;</xsl:text>
    </xsl:if>
  </xsl:template>
  
  <xsl:template name="suffix_notation">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:if test="$parent_precedence &gt; $precedence">
      <xsl:text disable-output-escaping="yes">&lt;mfenced separators=""&gt;</xsl:text>
    </xsl:if>
    <mrow>
      <xsl:apply-templates select="*[2]"/>
      <xsl:apply-templates select="*[1]"/>
    </mrow>
    <xsl:if test="$parent_precedence &gt; $precedence">
      <xsl:text disable-output-escaping="yes">&lt;/mfenced&gt;</xsl:text>
    </xsl:if>
  </xsl:template>
  
  <xsl:template name="sumatory_notation">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:if test="$parent_precedence &gt; $precedence">
      <xsl:text disable-output-escaping="yes">&lt;mfenced separators=""&gt;</xsl:text>
    </xsl:if>
    <mrow>
      <munderover>
        <xsl:apply-templates select="*[1]"/>
        <xsl:apply-templates select="*[2]"/>
        <xsl:apply-templates select="*[3]"/>
      </munderover>
      <xsl:apply-templates select="*[4]"/>
    </mrow>
    <xsl:if test="$parent_precedence &gt; $precedence">
      <xsl:text disable-output-escaping="yes">&lt;/mfenced&gt;</xsl:text>
    </xsl:if>
  </xsl:template>
  
  <xsl:template name="fenced_notation">
    <xsl:param name="open">(</xsl:param>
    <xsl:param name="close">)</xsl:param>
    <xsl:param name="separators">,</xsl:param>
    <mrow>
      <xsl:element name="mfenced">
        <xsl:attribute name="open"><xsl:value-of select="$open"/></xsl:attribute>
        <xsl:attribute name="close"><xsl:value-of select="$close"/></xsl:attribute>
        <xsl:attribute name="separators"><xsl:value-of select="$separators"/></xsl:attribute>
        <xsl:for-each select="*">
          <xsl:choose>
            <xsl:when test="position()=1"></xsl:when>
            <xsl:otherwise>
              <xsl:apply-templates select="."/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:for-each>
      </xsl:element>
    </mrow>
  </xsl:template>
  
  <xsl:template name="fraction_notation">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:if test="$parent_precedence &gt; $precedence">
      <xsl:text disable-output-escaping="yes">&lt;mfenced separators=""&gt;</xsl:text>
    </xsl:if>
    <mfrac>
      <mrow>
        <xsl:for-each select="*">
          <xsl:choose>
            <xsl:when test="position()=1"></xsl:when>
            <xsl:otherwise>
              <xsl:apply-templates select="."/>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:if test="not(position()=1 or (position()=last()))">
            <xsl:text disable-output-escaping="yes">&lt;/mrow&gt;&lt;mrow&gt;</xsl:text>
          </xsl:if>
        </xsl:for-each>
      </mrow>
    </mfrac>
    <xsl:if test="$parent_precedence &gt; $precedence">
      <xsl:text disable-output-escaping="yes">&lt;/mfenced&gt;</xsl:text>
    </xsl:if>
  </xsl:template>
  
  <xsl:template name="rational_notation">
    <xsl:param name="precedence">0</xsl:param>
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:if test="$parent_precedence &gt; $precedence">
      <xsl:text disable-output-escaping="yes">&lt;mfenced separators=""&gt;</xsl:text>
    </xsl:if>
    <mn>
      <mfrac>
        <xsl:apply-templates select="*[2]"/>
        <xsl:apply-templates select="*[3]"/>
      </mfrac>
    </mn>
    <xsl:if test="$parent_precedence &gt; $precedence">
      <xsl:text disable-output-escaping="yes">&lt;/mfenced&gt;</xsl:text>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="om:OMA">
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:call-template name="function_notation">
      <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
    </xsl:call-template>
  </xsl:template>
  
  <xsl:template match="om:OMA[*[1]/@cd='arith1']">
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:variable name="name" select="*[1]/@name"/>
    <xsl:choose>
      <xsl:when test="$name='abs'">
        <xsl:call-template name="fenced_notation">
          <xsl:with-param name="open">|</xsl:with-param>
          <xsl:with-param name="close">|</xsl:with-param>
          <xsl:with-param name="separators">,</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='divide'">
        <xsl:call-template name="fraction_notation">
          <xsl:with-param name="precedence">60</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='minus'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">50</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='plus'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">50</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='power'">
        <xsl:call-template name="superindex_notation">
          <xsl:with-param name="precedence">70</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='product'">
        <xsl:call-template name="sumatory_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='root'">
        <xsl:call-template name="root_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='sum'">
        <xsl:call-template name="sumatory_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='times'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">60</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='unary_minus'">
        <xsl:call-template name="prefix_notation">
          <xsl:with-param name="precedence">80</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="function_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="om:OMA[*[1]/@cd='interval1']">
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:variable name="name" select="*[1]/@name"/>
    <xsl:choose>
      <xsl:when test="$name='interval_oo'">
        <xsl:call-template name="fenced_notation">
          <xsl:with-param name="open">(</xsl:with-param>
          <xsl:with-param name="close">)</xsl:with-param>
          <xsl:with-param name="separators">,</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='interval_oc'">
        <xsl:call-template name="fenced_notation">
          <xsl:with-param name="open">(</xsl:with-param>
          <xsl:with-param name="close">]</xsl:with-param>
          <xsl:with-param name="separators">,</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='interval_co'">
        <xsl:call-template name="fenced_notation">
          <xsl:with-param name="open">[</xsl:with-param>
          <xsl:with-param name="close">)</xsl:with-param>
          <xsl:with-param name="separators">,</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='interval_cc'">
        <xsl:call-template name="fenced_notation">
          <xsl:with-param name="open">[</xsl:with-param>
          <xsl:with-param name="close">]</xsl:with-param>
          <xsl:with-param name="separators">,</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="function_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="om:OMA[*[1]/@cd='integer1']">
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:variable name="name" select="*[1]/@name"/>
    <xsl:choose>
      <xsl:when test="$name='factorial'">
        <xsl:call-template name="suffix_notation">
          <xsl:with-param name="precedence">90</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="function_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="om:OMA[*[1]/@cd='logic1']">
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:variable name="name" select="*[1]/@name"/>
    <xsl:choose>
      <xsl:when test="$name='and'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">11</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='equals'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">5</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='implies'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">5</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='or'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">10</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='not'">
        <xsl:call-template name="prefix_notation">
          <xsl:with-param name="precedence">12</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="function_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="om:OMA[*[1]/@cd='nums1']">
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:variable name="name" select="*[1]/@name"/>
    <xsl:choose>
      <xsl:when test="$name='rational'">
        <xsl:call-template name="rational_notation">
          <xsl:with-param name="precedence">65</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="function_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="om:OMA[*[1]/@cd='relation1']">
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:variable name="name" select="*[1]/@name"/>
    <xsl:choose>
      <xsl:when test="$name='eq'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">20</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='geq'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">20</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='leq'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">20</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='gt'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">20</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='lt'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">20</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="function_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="om:OMA[*[1]/@cd='set1']">
    <xsl:param name="parent_precedence">0</xsl:param>
    <xsl:variable name="name" select="*[1]/@name"/>
    <xsl:choose>
      <xsl:when test="$name='cartesian_product'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">60</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='in'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">44</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='intersect'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">46</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='map'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">50</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='notin'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">44</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='notsubset'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">45</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='prsubset'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">45</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='set'">
        <xsl:call-template name="fenced_notation">
          <xsl:with-param name="open">{</xsl:with-param>
          <xsl:with-param name="close">}</xsl:with-param>
          <xsl:with-param name="separators">,</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='setdiff'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">50</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='subset'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">45</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='suchthat'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">10</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$name='union'">
        <xsl:call-template name="infix_notation">
          <xsl:with-param name="precedence">45</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="function_notation">
          <xsl:with-param name="precedence">0</xsl:with-param>
          <xsl:with-param name="parent_precedence" select="$parent_precedence"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="om:OMBIND">
    <mrow>
      <xsl:apply-templates select="*[1]"/>
      <xsl:apply-templates select="*[2]"/>
      <mo>.</mo><mfenced><xsl:apply-templates select="*[3]"/></mfenced>
    </mrow>
  </xsl:template>
  
  <xsl:template match="om:OMBVAR">
    <mfenced open="[" close="]" separators="">
      <xsl:apply-templates/>
    </mfenced>
  </xsl:template>
  
  <xsl:template match="om:OMI">
    <mn><xsl:value-of select="."/></mn>
  </xsl:template>

  <xsl:template match="om:OMF">
    <xsl:choose>
      <xsl:when test="@dec">
        <mn><xsl:value-of select="@dec"/></mn>
      </xsl:when>
      <xsl:when test="@hex">
        <mn>#x<xsl:value-of select="@hex"/></mn>
      </xsl:when>
      <xsl:otherwise>
        <mn>NaN</mn>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="om:OMV">
    <mi><xsl:value-of select="@name"/></mi>
  </xsl:template>

  <xsl:template match="om:OMSTR">
    <mtext><xsl:apply-templates/></mtext>
  </xsl:template>

  <xsl:template match="om:OME">
    <merror><xsl:apply-templates/></merror>
  </xsl:template>

</xsl:stylesheet>
