// Clase:    OpenMath_object
// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:38 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#ifndef _H_OpenMath_object
#define _H_OpenMath_object

#include <stdarg.h>
#include "../../XML_infoset/XML_string/XML_string.h"
#include <string.h>
#include <list>
using std::list;
#include <iostream>
#include <algorithm> // STL
#include <vector>
using std::vector;
#include "../../XML_infoset/XML_elem_info_item/XML_elem_info_item.h"
typedef class OpenMath_object OpenMath_object_t, *OpenMath_object_p_t;

class OpenMath_object
{
 public:
  enum OpenMath_object_type
  {//     Basic objects
    //           Implementation:  Type:
    none,       //  -------------bugtrap
    integer,    //  vector<char> Bignum (array of digits of unlimited lenght).
    fp_number,  //  double float IEEE 754-1985 standard.
    char_string,//  XML_string   Unicode wide char array.
    byte_array, //  vector<char> Byte array.
    symbol,     //  vector<char> OpenMath symbol (ASCII char string).
    variable,   //  vector<char> OpenMath variable (ASCII char string).
    //    Constructs
    application,
    attribution,
    binding,
    error
  };
  typedef enum OpenMath_object_type OpenMath_object_type_t;
  static const unsigned int SAFE_BUFFER_SIZE_FOR_DOUBLE_FLOAT_TO_STRING = 256;
 private:
  OpenMath_object_type_t type;
  vector<unsigned char>* basic_object;
  double double_float_number;
  XML_string_p_t xml_string;
  list<OpenMath_object_p_t>* construct;
 public:
  OpenMath_object();
  ~OpenMath_object();
  OpenMath_object_p_t Binding(int num_args, ...);
  OpenMath_object_p_t Binding(const list<OpenMath_object_p_t>* object_list);
  OpenMath_object_p_t Byte_array(char* byte_array, int count);
  vector<unsigned char>* Byte_array();
  static bool Output(OpenMath_object_p_t object, ostream &output);
 private:
  static void Output_construct_objects(ostream &output, list<OpenMath_object_p_t>::iterator &construct_iterator, int count, list<OpenMath_object_p_t>::iterator &construct_end);
  static bool Output_subobject(OpenMath_object_p_t object, ostream &output);
 public:
  OpenMath_object_p_t Attribution(int num_args, ...);
  OpenMath_object_p_t Attribution(const list<OpenMath_object_p_t>* object_list);
  OpenMath_object_type_t Type(OpenMath_object_type_t type);
  OpenMath_object_type_t Type();
  OpenMath_object_p_t Char_string(vector<XML_UCS4_char_t> &xml_string);
  OpenMath_object_p_t Char_string(vector<char> &xml_string);
  OpenMath_object_p_t Char_string(string &xml_string);
  OpenMath_object_p_t Char_string(const char* xml_string);
  XML_string_p_t Char_string();
  OpenMath_object_p_t Application(int num_args, ...);
  OpenMath_object_p_t Application(const list<OpenMath_object_p_t>* object_list);
  OpenMath_object_p_t Symbol(vector<char> &symbol);
  OpenMath_object_p_t Symbol(string &symbol);
  OpenMath_object_p_t Symbol(const char* symbol);
  OpenMath_object_p_t Symbol(const char* symbol, int count);
  vector<unsigned char>* Symbol();
  OpenMath_object_p_t Variable(vector<char> &variable);
  OpenMath_object_p_t Variable(string &variable);
  OpenMath_object_p_t Variable(const char* variable);
  OpenMath_object_p_t Variable(const char* variable, int count);
  vector<unsigned char>* Variable();
  XML_elem_info_item_p_t To_XML_elem_info_item();
 private:
  XML_elem_info_item_p_t To_XML_elem_info_item_subobject();
 public:
  const list<OpenMath_object_p_t>* Construct();
  OpenMath_object_p_t Integer(int integer);
  OpenMath_object_p_t Integer(vector<char> &integer);
  OpenMath_object_p_t Integer(string &integer);
  OpenMath_object_p_t Integer(const char* integer);
  OpenMath_object_p_t Integer(const char* integer, int count);
  vector<unsigned char>* Integer();
  OpenMath_object_p_t Fp_number(double fp_number);
  OpenMath_object_p_t Fp_number(string &fp_number);
  OpenMath_object_p_t Fp_number(vector<char> &fp_number);
  OpenMath_object_p_t Fp_number(const char* fp_number);
  OpenMath_object_p_t Fp_number(const char* fp_number, unsigned int count);
  double Fp_number();
  OpenMath_object_p_t Error(int num_args, ...);
  OpenMath_object_p_t Error(const list<OpenMath_object_p_t>* object_list);
  static bool Delete(OpenMath_object_p_t object);
  static bool Delete_recursive(OpenMath_object_p_t object);
};

extern ostream &operator<< (ostream &, OpenMath_object_p_t);

#endif
