/* QMath mathematical notation system--QMath */

%{
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
void yyerror (const char *s);
int no_main(void);
#include "OpenMath_object.H"
#include "Bison_typedefs.h"
/* The following is to receive an XML_elem_info_item_p_t cast to (void*) */
#define YYPARSE_PARAM root_element
typedef struct 
{
  int indentation;
  string tag;
  string tag_id;
  XML_elem_info_item_p_t parent;
} Indentation_tag_t, *Indentation_tag_p_t;
#include <map>
using std::map;
map<XML_elem_info_item_p_t, Indentation_tag_p_t> indentation_tags;
unsigned int error_count = 0;
XML_string_p_t root_element_name_p = 0;
XML_string_p_t mime_format_p = 0;
XML_string_p_t metadata_mime_format_element_name_p = 0;
XML_string_p_t id_attribute_name_p = 0;
XML_string_p_t paragraph_default_element_name_p = 0;
XML_string_p_t paragraph_text_container_element_name_p = 0;
XML_string_p_t paragraph_formula_container_element_name_p = 0;
XML_string_p_t metadata_element_name_p = 0;
XML_string_p_t metadata_title_element_name_p = 0;
XML_string_p_t metadata_author_element_name_p = 0;
XML_string_p_t metadata_date_element_name_p = 0;
XML_string_p_t reference_element_name_p = 0;
XML_string_p_t system_identifier_p = 0;
XML_string_p_t public_identifier_p = 0;
XML_string_p_t root_element_identifier_p = 0;
#define DC_NAMESPACE_URI "http://purl.org/dc/elements/1.1/"
/* Helper functions: */
void manage_metadata_xmlns_in(XML_elem_info_item_p_t metadata);
void manage_metadata_lang_in(XML_elem_info_item_p_t metadata, XML_elem_info_item_p_t destination);
void metadata_from_parts(XML_elem_info_item_p_t metadata, XML_string_p_t token, XML_string_p_t argument);
void insert_block_by_indentation_level(vector<XML_elem_info_item_p_t>* block_p, XML_elem_info_item_p_t xml_document_root);
XML_elem_info_item_p_t build_element_from_description(XML_string_p_t description);
void set_values_from_directives(XML_elem_info_item_p_t root_element);
%}

/* Make this function reentrant.
   The lexer likely isn't, so this is useless for now. */
%pure_parser

/* Don't worry about shift/reduce conflicts. */
%expect 121

/* Possible types for tokens. */
%union {
  void* markup;/* For markup tokens, whose value is not used. */
  XML_UCS4_char_t character;
  XML_string_p_t text;
  OpenMath_object_p_t OM_object;
  list<OpenMath_object_p_t>* OM_object_list;
  XML_elem_info_item_p_t XML_element;
  vector<XML_elem_info_item_p_t>* XML_element_list;
  vector<XML_info_item_p_t>*      XML_item_list;
  vector<Presentation_entry_t>* presentations_list;
}

%{
#define _H_QMATH_PARSER_TAB_CC
#include "Tokenizer/Tokenizer.h"
Tokenizer lexer;
int yylex (YYSTYPE *yylval_p) { int token_type; token_type = lexer.Get_next_token(yylval_p); return token_type; };
%}

/* BISON Declarations */
/* [AGP] 20001106:
   The previous versions used shorter names, such as ID instead of T_SYMBOL.
   This caused name collisions with the XML infoset classes, which was reported
   by the compiler as weird "Parsing error before 260" messages (260 was the
   value Bison assigned to the ID token).
   The C declaration section ('%{...%}') is inserted in the generated .tab.c
   file after the #define directives for the following tokens, so any included
   files must avoid these symbols.
   I took the idea of using the "T_" prefix from GiNaC's source code.
*/
%token <OM_object> T_INT_NUM
%token <OM_object> T_FP_NUM
%token <OM_object> T_STR
%token <OM_object> T_SYMBOL
%token <OM_object> T_VARIABLE
%token <OM_object> T_APPLICATION
%token <OM_object> T_ATTRIBUTION
%token <OM_object> T_BINDING
%token <OM_object> T_OP_IMPL
%token <OM_object> T_OP_OR
%token <OM_object> T_OP_AND
%token <OM_object> T_OP_EQ
/* At first I used "OP_SUM" instead of "OP_PLUS", but for some
   reason Bison didn't like it.*/
%token <OM_object> T_OP_PLUS
%token <OM_object> T_OP_PROD
%token <OM_object> T_OP_EXP
%token <OM_object> T_OP_FACT
%token <OM_object> T_OP_BIND

%token <markup> T_EMBEDDED_MARKER

%token <markup> T_SPACE
%token <markup> T_TAB
%token <markup> T_END_OF_LINE
%token <markup> T_SEP_COMMA
%token <markup> T_SEP_SCOLON
%token <markup> T_SEP_COLON
%token <character> T_FENCE_OPEN
%token <character> T_FENCE_CLOSE

%token <text> T_UCS4_CHAR_RUN
%token <character> T_UCS4_CHAR

%token <markup> T_MATH_BEGIN
%token <markup> T_MATH_END
%token <markup> T_DOUBLE_QUOTE
%token <markup> T_SINGLE_QUOTE
%token <markup> T_BACK_QUOTE
%token <markup> T_HASH
%token <markup> T_NEW_PARAGRAPH
%token <markup> T_DOCUMENT_END

%token <text> T_IDENTIFIER
%token <markup> T_METADATA_BEGIN
%token <text> T_METADATA_TITLE
%token <text> T_METADATA_AUTHOR
%token <text> T_METADATA_DATE
%token <text> T_METADATA_LANG
%token <text> T_METADATA_REFERENCE
%token <text> T_METADATA
%token <text> T_PARAGRAPH_TYPE
%token <text> T_CONTAINER_TYPE
%token <text> T_PARAGRAPH_TYPE_LEVEL_0
%token <text> T_CONTAINER_TYPE_LEVEL_0
%token <text> T_PARAGRAPH_TYPE_LEVEL_1
%token <text> T_CONTAINER_TYPE_LEVEL_1
%token <text> T_PARAGRAPH_TYPE_LEVEL_2
%token <text> T_CONTAINER_TYPE_LEVEL_2
%token <text> T_PARAGRAPH_TYPE_LEVEL_3
%token <text> T_CONTAINER_TYPE_LEVEL_3
%token <text> T_PARAGRAPH_TYPE_LEVEL_4
%token <text> T_CONTAINER_TYPE_LEVEL_4
%token <text> T_PARAGRAPH_TYPE_LEVEL_5
%token <text> T_CONTAINER_TYPE_LEVEL_5
%token <markup> T_PARAGRAPH_AS_CONTAINER_MARKER
%token <markup> T_PARAGRAPH_SECTION_MARKER
%token <text>   T_PARAGRAPH_SECTION_MARKER_FMP
%token <text> T_REFERENCE_ID
%token <text> T_REFERENCE
%token <text> T_REFERENCE_OPEN
%token <text> T_REFERENCE_CLOSE
%token <text>   T_TAG_OPEN
%token <markup> T_TAG_CLOSE

%token <markup> T_QMATH_DIRECTIVE
%token <markup> T_DIRECTIVE_DEFINITION
%token <markup> T_DIRECTIVE_CONTEXT
%token <markup> T_DIRECTIVE_ROOT_ELEMENT_NAME
%token <markup> T_DIRECTIVE_MIME_FORMAT
%token <markup> T_DIRECTIVE_METADATA_MIME_FORMAT_ELEMENT_NAME
%token <markup> T_DIRECTIVE_ID_ATTRIBUTE_NAME
%token <markup> T_DIRECTIVE_PARAGRAPH_DEFAULT_ELEMENT_NAME
%token <markup> T_DIRECTIVE_PARAGRAPH_TEXT_CONTAINER_ELEMENT_NAME
%token <markup> T_DIRECTIVE_PARAGRAPH_FORMULA_CONTAINER_ELEMENT_NAME
%token <markup> T_DIRECTIVE_METADATA_ELEMENT_NAME
%token <markup> T_DIRECTIVE_METADATA_TITLE_ELEMENT_NAME
%token <markup> T_DIRECTIVE_METADATA_AUTHOR_ELEMENT_NAME
%token <markup> T_DIRECTIVE_METADATA_DATE_ELEMENT_NAME
%token <markup> T_DIRECTIVE_REFERENCE_ELEMENT_NAME
%token <markup> T_DIRECTIVE_SYSTEM_IDENTIFIER
%token <markup> T_DIRECTIVE_PUBLIC_IDENTIFIER

%token <markup> T_START_DEBUGGING
%token <text>   T_DEBUGGING_REPORT
%token <markup> T_STOP_DEBUGGING
/* T_STRING is *not* the same as T_STR, which is an OpenMath object. */
%token <text>   T_STRING
%token <text>   T_SYMBOL_DEFINITION_TYPE
%token <text>   T_PARAMETER_LABEL

/* Precedence and associativity are determined here: */
/* From least precedence ... */
%left T_SEP_COLON
%left T_SEP_SCOLON
%left T_SEP_COMMA
%left T_OP_BIND
%right T_OP_IMPL
%left T_OP_OR
%left T_OP_AND
%right T_OP_EQ
%left T_OP_PLUS
%left T_OP_PROD
%right T_OP_EXP
%right T_OP_FACT
/* ... to most precedence */

/* Non-terminals: */
%type <XML_element_list>   block
%type <XML_element>        metadata
%type <XML_element_list>   symbol_definition
%type <presentations_list> symbol_definition_presentations_list
%type <text>               definition_identifier
%type <markup>             qmath_directive
%type <XML_element_list>   paragraph
%type <XML_element>        paragraph_header
%type <XML_element>        paragraph_body
%type <XML_element>        paragraph_body_CMP
%type <XML_element>        paragraph_body_FMP
%type <XML_element>        tag_element
%type <XML_item_list>      tag_element_content
%type <character> single_character
%type <markup>    blanks
%type <text>      literal
%type <character> literal_character
%type <OM_object>      prefix_unary
%type <OM_object>      infix_binary
%type <OM_object>      postfix_unary
%type <OM_object>      exp
%type <OM_object_list> exp_list
%type <OM_object_list> fenced_exp_list
%type <OM_object>      symbol_variable_number

/* Grammar follows. */
%%
input: /* empty string */  {
  // Defaults:
  root_element_name_p
    = (new XML_string())->UTF8("document");
  id_attribute_name_p
    = (new XML_string())->UTF8("xml:id");
  metadata_element_name_p
    = (new XML_string())->UTF8("metadata");
  metadata_title_element_name_p
    = (new XML_string())->UTF8("title");
  metadata_author_element_name_p
    = (new XML_string())->UTF8("author");
  metadata_date_element_name_p
    = (new XML_string())->UTF8("date");
  reference_element_name_p
    = (new XML_string())->UTF8("reference");
  mime_format_p = 0;//"application/omdoc+xml";
  metadata_mime_format_element_name_p
    = (new XML_string())->UTF8("format");// Only used if mime_format_p != 0.
  system_identifier_p = 0;
  public_identifier_p = 0;
  paragraph_default_element_name_p
    = (new XML_string())->UTF8("paragraph");
  paragraph_text_container_element_name_p
    = (new XML_string())->UTF8("text");
  paragraph_formula_container_element_name_p
    = (new XML_string())->UTF8("formula");
  ;}
| input T_EMBEDDED_MARKER T_MATH_BEGIN exp T_MATH_END {
  cout << $4;
  OpenMath_object::Delete_recursive($4);
  ;}
| input T_EMBEDDED_MARKER T_MATH_BEGIN error T_MATH_END {
  cerr << lexer.Current_file_name() << ":"
       << lexer.Current_line_number() << ":"
       << " Parsing error in math expression"
       << endl << flush;
  OpenMath_object_p_t tmp_om_object;
  (tmp_om_object = new OpenMath_object)
    ->Error(2,
            (new OpenMath_object)
            ->Char_string("[broken expression]")
            ,
            (new OpenMath_object)
            ->Char_string("[parse error in expression]")
            )
    ->To_XML_elem_info_item()
    ->Attributes((new XML_attr_info_item)
                 ->Prefix((char*)NULL)
                 ->Local_name("xmlns")
                 ->Normalized_value("http://www.openmath.org/"
                                    "OpenMath")
                 );
  cout << tmp_om_object;
  OpenMath_object::Delete_recursive(tmp_om_object);
  ;}
| input T_EMBEDDED_MARKER T_MATH_BEGIN exp error T_MATH_END {
  cerr << lexer.Current_file_name() << ":"
       << lexer.Current_line_number() << ":"
       << " Parsing error in math expression after:\n" << $4
       << endl << flush;
  OpenMath_object_p_t tmp_om_object;
  (tmp_om_object = new OpenMath_object)
    ->Error(2,
            $4,
            (new OpenMath_object)
            ->Char_string("[parse error in expression]")
            )
    ->To_XML_elem_info_item()
    ->Attributes((new XML_attr_info_item)
                 ->Prefix((char*)NULL)
                 ->Local_name("xmlns")
                 ->Normalized_value("http://www.openmath.org/"
                                    "OpenMath")
                 );
  cout << tmp_om_object;
  OpenMath_object::Delete_recursive(tmp_om_object);
  ;}
| input T_EMBEDDED_MARKER T_MATH_BEGIN exp error T_DOCUMENT_END {
  cerr << lexer.Current_file_name() << ":"
       << lexer.Current_line_number() << ":"
       << " Unterminated math expression:\n" << $4
       << endl << flush;
  OpenMath_object_p_t tmp_om_object;
  (tmp_om_object = new OpenMath_object)
    ->Error(2,
            $4,
            (new OpenMath_object)
            ->Char_string("[unterminated expression]")
            )
    ->To_XML_elem_info_item()
    ->Attributes((new XML_attr_info_item)
                 ->Prefix((char*)NULL)
                 ->Local_name("xmlns")
                 ->Normalized_value("http://www.openmath.org/"
                                    "OpenMath")
                 );
  OpenMath_object::Delete_recursive(tmp_om_object);
  YYABORT; /* Return fail */ }
| input T_EMBEDDED_MARKER T_UCS4_CHAR {
  vector<char> buffer;
  Unicode_translator::UTF8($3, &buffer);
  if (buffer.size() > 0)// Else, it's an invalid UCS4 char.
    {
      for (vector<char>::const_iterator
             buffer_iterator = buffer.begin();
           buffer_iterator != buffer.end();
           ++buffer_iterator)
        {
          cout << *buffer_iterator;
        }
    }
  ;}
| input T_EMBEDDED_MARKER T_UCS4_CHAR_RUN {
  vector<char> buffer;
  vector<Unicode_data::UCS4_char_t>::const_iterator current;
  vector<Unicode_data::UCS4_char_t>::const_iterator end;
  current = $3->UCS4_string().begin();
  end     = $3->UCS4_string().end();
  while (current < end) Unicode_translator::UTF8(*(current++), &buffer);
  for (vector<char>::const_iterator
         buffer_iterator = buffer.begin();
       buffer_iterator != buffer.end();
       ++buffer_iterator)
    {
      cout << *buffer_iterator;
    }
  if ($3) { delete $3; $3=NULL; }
  ;}
| input T_EMBEDDED_MARKER T_END_OF_LINE {
  cout << endl;
  ;}
| input T_END_OF_LINE   { ;/* NOP */ }
| input T_NEW_PARAGRAPH { ;/* NOP */ }
| input T_DOCUMENT_END {
  set_values_from_directives(static_cast<XML_elem_info_item_p_t>
                             (root_element));
  if (error_count || lexer.Error_count())
    {
      YYABORT; /* Return fail. */
    }
  else
    {
      YYACCEPT; /* Return OK. */
    }
  ;}
| input T_START_DEBUGGING T_NEW_PARAGRAPH { yydebug = 1; }
| input T_DEBUGGING_REPORT T_NEW_PARAGRAPH {
  cerr << $2 << endl << flush;
  if ($2) { delete $2; $2=NULL; }
  ;}
| input T_STOP_DEBUGGING T_NEW_PARAGRAPH { yydebug = 0; }
| input T_REFERENCE_ID T_NEW_PARAGRAPH {
  root_element_identifier_p = $2;
  ;}
| input block T_NEW_PARAGRAPH {
  insert_block_by_indentation_level($2, XML_elem_info_item_p_t(root_element));
  ;}
| input symbol_definition {// T_NEW_PARAGRAPH
  insert_block_by_indentation_level($2, XML_elem_info_item_p_t(root_element));
  ;}
| input qmath_directive {
  ;// A QMath directive doesn't produce any XML element by itself.
  ;}
| input block T_DOCUMENT_END {
  insert_block_by_indentation_level($2, XML_elem_info_item_p_t(root_element));
  set_values_from_directives(static_cast<XML_elem_info_item_p_t>
                             (root_element));
  YYACCEPT; /* Return OK. */ }
;

block: metadata {
  $$ = new vector<XML_elem_info_item_p_t>;
  if ($1)
    {
      manage_metadata_xmlns_in($1);
      $$->push_back($1);
      $1 = (XML_elem_info_item_p_t)NULL;
    }
  ;}
| T_PARAGRAPH_AS_CONTAINER_MARKER paragraph {
  $$ = $2;
  $2 = (vector<XML_elem_info_item_p_t>*)NULL;
  vector<XML_elem_info_item_p_t> temporary_container;
  XML_elem_info_item_p_t cmp_element;
  for (vector<XML_elem_info_item_p_t>::iterator
         paragraph_item_iterator = $$->begin();
       paragraph_item_iterator != $$->end();
       ++ paragraph_item_iterator)
    {
      cmp_element = (*paragraph_item_iterator)
        ->Child(paragraph_text_container_element_name_p);
      if (cmp_element)
        {
          for (list<XML_info_item_p_t>::const_iterator
                 cmp_child_i = cmp_element->Children().begin();
               cmp_child_i  != cmp_element->Children().end();
               ++cmp_child_i)
            {
              if (XML_info_item::elem_info_item == (*cmp_child_i)->Type())
                {
                  temporary_container
                    .push_back(XML_elem_info_item_p_t(*cmp_child_i)
                               ->Parent(XML_elem_info_item_p_t(0))
                               );
                }
            }
          (*paragraph_item_iterator)->Child_erase(cmp_element);
          (*paragraph_item_iterator)->Children(temporary_container);
        }
    }
  ;}
| paragraph {
  $$ = $1;
  $1 = (vector<XML_elem_info_item_p_t>*)NULL;
  ;}
| block symbol_definition {
  if ($2)
    {
      for (vector<XML_elem_info_item_p_t>::iterator
             element_iterator = $2->begin();
           element_iterator != $2->end();
           ++element_iterator)
        {
          $$->push_back(*element_iterator);
          *element_iterator = (XML_elem_info_item_p_t)NULL;
        }
      $2->erase($2->begin(), $2->end());
      delete $2;
      $2 = (vector<XML_elem_info_item_p_t>*)NULL;
    }
  ;}
| block T_END_OF_LINE {;}
;

metadata: T_METADATA_BEGIN {
  $$ = build_element_from_description(metadata_element_name_p);
  ;}
| metadata T_METADATA_BEGIN { $$ = $1; }
| metadata T_METADATA_TITLE {
  $$ = $1
    ->Children(build_element_from_description(metadata_title_element_name_p)
               ->Children($2)
               );
  if ($2) { delete $2; $2=NULL; }
  ;}
/*  metadata: T_METADATA_TITLE { */
/*    $$ = ((new XML_elem_info_item) */
/*          ->Local_name(metadata_element_name_p) */
/*          ->Children((new XML_elem_info_item) */
/*                     ->Local_name("Title") */
/*                     ->Children($1) */
/*                     ) */
/*          ); */
/*    if ($1) { delete $1; $1=NULL; } */
/*    ;} */
| metadata T_METADATA_AUTHOR {
  $$ = $1
    ->Children(build_element_from_description(metadata_author_element_name_p)
               ->Children($2)
               );
  if ($2) { delete $2; $2=(XML_string_p_t)NULL; }
  ;}
| metadata T_METADATA_DATE {
  $$ = $1
    ->Children(build_element_from_description(metadata_date_element_name_p)
               ->Children($2)
               );
  if ($2) { delete $2; $2=(XML_string_p_t)NULL; }
  ;}
| metadata T_METADATA_LANG {
  $$ = $1->Attributes((new XML_attr_info_item)
                      ->Prefix("xml")
                      ->Local_name("lang")
                      ->Normalized_value($2)
                      );
  if ($2) { delete $2; $2=(XML_string_p_t)NULL; }
  ;}
| metadata T_METADATA T_METADATA_BEGIN T_METADATA_REFERENCE {
  metadata_from_parts($1, $2, $4);
  
  $$ = $1;
  
  if ($2) { delete $2; $2=(XML_string_p_t)NULL; }
  if ($4) { delete $4; $4=(XML_string_p_t)NULL; }
  ;}
| metadata T_METADATA T_METADATA_BEGIN T_METADATA_TITLE {
  metadata_from_parts($1, $2, $4);
  
  $$ = $1;
  
  if ($2) { delete $2; $2=(XML_string_p_t)NULL; }
  if ($4) { delete $4; $4=(XML_string_p_t)NULL; }
  ;}
| metadata T_METADATA T_METADATA_BEGIN T_METADATA_DATE {
  metadata_from_parts($1, $2, $4);
  
  $$ = $1;
  
  if ($2) { delete $2; $2=(XML_string_p_t)NULL; }
  if ($4) { delete $4; $4=(XML_string_p_t)NULL; }
  ;}
| metadata T_METADATA T_METADATA_BEGIN T_METADATA_AUTHOR {
  metadata_from_parts($1, $2, $4);
  
  $$ = $1;
  
  if ($2) { delete $2; $2=(XML_string_p_t)NULL; }
  if ($4) { delete $4; $4=(XML_string_p_t)NULL; }
  ;}
| metadata T_METADATA T_METADATA_BEGIN T_METADATA_LANG {
  metadata_from_parts($1, $2, $4);
  
  $$ = $1;
  
  if ($2) { delete $2; $2=(XML_string_p_t)NULL; }
  if ($4) { delete $4; $4=(XML_string_p_t)NULL; }
  ;}
| metadata blanks { $$ = $1; }
;

symbol_definition_presentations_list: T_PARAMETER_LABEL T_STRING {
  $$ = new vector<Presentation_entry_t>;
  Presentation_entry_t presentation_entry;
  presentation_entry.format = $1;
  presentation_entry.presentation = $2;
  $$->push_back(presentation_entry);
  ;}
| symbol_definition_presentations_list T_PARAMETER_LABEL T_STRING {
  Presentation_entry_t presentation_entry;
  presentation_entry.format = $2;
  presentation_entry.presentation = $3;
  $1->push_back(presentation_entry);
  $$ = $1;
  ;}
;

definition_identifier
: T_IDENTIFIER
| T_FENCE_OPEN T_FENCE_CLOSE {
  vector<XML_UCS4_char_t> combined_fence_symbol;
  combined_fence_symbol.push_back($1);
  combined_fence_symbol.push_back($2);
  $$ = (new XML_string())->UCS4(combined_fence_symbol);
}
;

symbol_definition
: T_DIRECTIVE_DEFINITION definition_identifier T_SYMBOL_DEFINITION_TYPE T_STRING T_NEW_PARAGRAPH {
  $$ = (vector<XML_elem_info_item_p_t>*)NULL;// This doesn't go into the OMDoc.
  XML_string_t context_name;
  context_name = "local";// TODO: This will be the id of the enclosing theory.
  lexer.Append_translation_entry(context_name, *$2, *$3, *$4);
  if  ($2) { delete  $2;  $2=(XML_string_p_t)NULL;      }// Token
  if  ($3) { delete  $3;  $3=(XML_string_p_t)NULL;      }// Type
  if  ($4) { delete  $4;  $4=(XML_string_p_t)NULL;      }// Translation
  ;}
| T_DIRECTIVE_DEFINITION T_REFERENCE_ID definition_identifier T_SYMBOL_DEFINITION_TYPE T_STRING T_STRING symbol_definition_presentations_list {
  $$ = new vector<XML_elem_info_item_p_t>;
  XML_elem_info_item_p_t symbol_element;
  XML_elem_info_item_p_t presentation_element;
  XML_string_t symbol_id;
  XML_string_t symbol_namespace;
  symbol_id = *($2);
  XML_attr_info_item_p_t attribute;
  attribute = NULL;
  symbol_element = (new XML_elem_info_item)
    ->Local_name("symbol")
    ->Attributes((new XML_attr_info_item)
                 ->Local_name(id_attribute_name_p)
                 ->Normalized_value(symbol_id)
                 )
    ->Children((new XML_elem_info_item)
               ->Local_name("commonname")
               ->Children($6)
               )
    ;
  XML_string_t presentation_id;
  presentation_id = symbol_id;
  presentation_id += "_pres";
  presentation_element = (new XML_elem_info_item)
    ->Local_name("presentation")
    ->Attributes((new XML_attr_info_item)
                 ->Local_name(id_attribute_name_p)
                 ->Normalized_value(presentation_id)
                 )
    ->Attributes((new XML_attr_info_item)
                 ->Local_name("for")
                 ->Normalized_value(symbol_id)
                 )
    ;
  for (vector<Presentation_entry_t>::iterator
         presentation_iterator = $7->begin();
       presentation_iterator != $7->end();
       ++presentation_iterator)
    {
      presentation_element->Children((new XML_elem_info_item)
                     ->Local_name("use")
                     ->Attributes((new XML_attr_info_item)
                                  ->Local_name("format")
                                  ->Normalized_value(presentation_iterator->
                                                     format)
                                  )
                     ->Children(presentation_iterator->presentation)
                     );
      delete presentation_iterator->format;
      delete presentation_iterator->presentation;
      presentation_iterator->format       = (XML_string_p_t)NULL;
      presentation_iterator->presentation = (XML_string_p_t)NULL;
    }
  $$->push_back(symbol_element);
  $$->push_back(presentation_element);
  // Search for its parent:
  if (lexer.Tab_stops()->size() > 1)
    {
      Tab_stop_p_t tab_stop;
      tab_stop =  lexer.Tab_stops(lexer.Tab_stops()->size() - 2);
      if (!indentation_tags[symbol_element])
        indentation_tags[symbol_element] = new Indentation_tag_t;
      indentation_tags[symbol_element]->indentation = tab_stop->indentation;
      indentation_tags[symbol_element]->parent      = tab_stop->element;
    }
  else
    {
      indentation_tags[symbol_element]->indentation = 0;
      indentation_tags[symbol_element]->parent = (XML_elem_info_item_p_t)NULL;
    }
  // Search for its parent:
  if (lexer.Tab_stops()->size() > 1)
    {
      Tab_stop_p_t tab_stop;
      tab_stop =  lexer.Tab_stops(lexer.Tab_stops()->size() - 2);
      if (!indentation_tags[presentation_element])
        indentation_tags[presentation_element] = new Indentation_tag_t;
      indentation_tags[presentation_element]->indentation = tab_stop->indentation;
      indentation_tags[presentation_element]->parent      = tab_stop->element;
    }
  else
    {
      indentation_tags[presentation_element]->indentation = 0;
      indentation_tags[presentation_element]->parent = (XML_elem_info_item_p_t)NULL;
    }

  // Add the symbol to the lexer table.
  XML_string_t context_name;
  context_name = "local";// TODO: This will be the id of the enclosing theory.
  int token_type;
  token_type = lexer.Append_translation_entry(context_name, *$3, *$4, *$5);
  switch (token_type)
    {
    case T_APPLICATION:
      presentation_element
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("parent")
                     ->Normalized_value("OMA")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("fixity")
                     ->Normalized_value("prefix")
                     )
        ;
      break;
    case T_ATTRIBUTION:
      presentation_element
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("parent")
                     ->Normalized_value("OMATTR")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("fixity")
                     ->Normalized_value("prefix")
                     )
        ;
      break;
    case T_BINDING:
      presentation_element
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("parent")
                     ->Normalized_value("OMBIND")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("fixity")
                     ->Normalized_value("prefix")
                     )
        ;
      break;
    case T_OP_IMPL:
      presentation_element
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("parent")
                     ->Normalized_value("OMA")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("fixity")
                     ->Normalized_value("infix")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("precedence")
                     ->Normalized_value("700")
                     )
        ;
      break;
    case T_OP_OR:
      presentation_element
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("parent")
                     ->Normalized_value("OMA")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("fixity")
                     ->Normalized_value("infix")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("precedence")
                     ->Normalized_value("500")
                     )
        ;
      break;
    case T_OP_AND:
      presentation_element
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("parent")
                     ->Normalized_value("OMA")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("fixity")
                     ->Normalized_value("infix")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("precedence")
                     ->Normalized_value("500")
                     )
        ;
      break;
    case T_OP_EQ:
      presentation_element
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("parent")
                     ->Normalized_value("OMA")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("fixity")
                     ->Normalized_value("infix")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("precedence")
                     ->Normalized_value("700")
                     )
        ;
      break;
    case T_OP_PLUS:
      presentation_element
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("parent")
                     ->Normalized_value("OMA")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("fixity")
                     ->Normalized_value("assoc")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("precedence")
                     ->Normalized_value("500")
                     )
        ;
      break;
    case T_OP_PROD:
      presentation_element
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("parent")
                     ->Normalized_value("OMA")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("fixity")
                     ->Normalized_value("assoc")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("precedence")
                     ->Normalized_value("400")
                     )
        ;
      break;
    case T_OP_EXP:
      presentation_element
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("parent")
                     ->Normalized_value("OMA")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("fixity")
                     ->Normalized_value("infix")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("precedence")
                     ->Normalized_value("200")
                     )
        ;
      break;
    case T_OP_FACT:
      presentation_element
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("parent")
                     ->Normalized_value("OMA")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("fixity")
                     ->Normalized_value("postfix")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("precedence")
                     ->Normalized_value("200")
                     )
        ;
      break;
    case T_OP_BIND:
      presentation_element
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("parent")
                     ->Normalized_value("OMBIND")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("fixity")
                     ->Normalized_value("prefix")
                     )
        ->Attributes((new XML_attr_info_item)
                     ->Local_name("precedence")
                     ->Normalized_value("700")
                     )
        ;
      break;
    }
  
  if ($3) { delete $3; $3=(XML_string_p_t)NULL;                }// Token
  if ($4) { delete $4; $4=(XML_string_p_t)NULL;                }// Type
  if ($5) { delete $5; $5=(XML_string_p_t)NULL;                }// Translation
  if ($6) { delete $6; $6=(XML_string_p_t)NULL;                }// Common name
  if ($7) { delete $7; $7=(vector<Presentation_entry_t>*)NULL; }// Presentations
  ;}
;

qmath_directive
: T_DIRECTIVE_CONTEXT T_STRING {
  XML_string_t file_name;
  file_name = *$2;
  file_name += ".qmath";
  if (!lexer.Input_stream(file_name, Tokenizer::LEXER_STATE_QMATH))
    {
      cerr << lexer.Current_file_name() << ":"
           << lexer.Current_line_number() << ":"
           << " warning: can't open file: \""
           << file_name << "\"" << endl << flush;
    }
  if ($2) { delete $2; $2=(XML_string_p_t)NULL; }
  ;}
| T_DIRECTIVE_ROOT_ELEMENT_NAME T_STRING {
  if ($2)
    {
      if (! root_element_name_p) root_element_name_p = new XML_string();
      *(root_element_name_p) = *($2);
      delete $2;
      $2=(XML_string_p_t)NULL;
    }
  else
    {
      if (root_element_name_p)
        {
          delete root_element_name_p;
          root_element_name_p = 0;
        }
    }
  ;}
| T_DIRECTIVE_MIME_FORMAT T_STRING {
  if ($2)
    {
      if (! mime_format_p) mime_format_p = new XML_string();
      *(mime_format_p) = *($2);
      delete $2;
      $2=(XML_string_p_t)NULL;
    }
  else
    {
      if (mime_format_p)
        {
          delete mime_format_p;
          mime_format_p = 0;
        }
    }
  ;}
| T_DIRECTIVE_METADATA_MIME_FORMAT_ELEMENT_NAME T_STRING {
  if ($2)
    {
      if (! metadata_mime_format_element_name_p) metadata_mime_format_element_name_p = new XML_string();
      *(metadata_mime_format_element_name_p) = *($2);
      delete $2;
      $2=(XML_string_p_t)NULL;
    }
  else
    {
      if (metadata_mime_format_element_name_p)
        {
          delete metadata_mime_format_element_name_p;
          metadata_mime_format_element_name_p = 0;
        }
    }
  ;}
| T_DIRECTIVE_ID_ATTRIBUTE_NAME T_STRING {
  if ($2)
    {
      if (! id_attribute_name_p) id_attribute_name_p = new XML_string();
      *(id_attribute_name_p) = *($2);
      delete $2;
      $2=(XML_string_p_t)NULL;
    }
  else
    {
      if (id_attribute_name_p)
        {
          delete id_attribute_name_p;
          id_attribute_name_p = 0;
        }
    }
  ;}
| T_DIRECTIVE_PARAGRAPH_DEFAULT_ELEMENT_NAME T_STRING {
  if ($2)
    {
      if (! paragraph_default_element_name_p) paragraph_default_element_name_p = new XML_string();
      *(paragraph_default_element_name_p) = *($2);
      delete $2;
      $2=(XML_string_p_t)NULL;
    }
  else
    {
      if (paragraph_default_element_name_p)
        {
          delete paragraph_default_element_name_p;
          paragraph_default_element_name_p = 0;
        }
    }
  ;}
| T_DIRECTIVE_PARAGRAPH_TEXT_CONTAINER_ELEMENT_NAME T_STRING {
  if ($2)
    {
      if (! paragraph_text_container_element_name_p) paragraph_text_container_element_name_p = new XML_string();
      *(paragraph_text_container_element_name_p) = *($2);
      delete $2;
      $2=(XML_string_p_t)NULL;
    }
  else
    {
      if (paragraph_text_container_element_name_p)
        {
          delete paragraph_text_container_element_name_p;
          paragraph_text_container_element_name_p = 0;
        }
    }
  ;}
| T_DIRECTIVE_PARAGRAPH_FORMULA_CONTAINER_ELEMENT_NAME T_STRING {
  if ($2)
    {
      if (! paragraph_formula_container_element_name_p) paragraph_formula_container_element_name_p = new XML_string();
      *(paragraph_formula_container_element_name_p) = *($2);
      delete $2;
      $2=(XML_string_p_t)NULL;
    }
  else
    {
      if (paragraph_formula_container_element_name_p)
        {
          delete paragraph_formula_container_element_name_p;
          paragraph_formula_container_element_name_p = 0;
        }
    }
  ;}
| T_DIRECTIVE_METADATA_ELEMENT_NAME T_STRING {
  if ($2)
    {
      if (! metadata_element_name_p) metadata_element_name_p = new XML_string();
      *(metadata_element_name_p) = *($2);
      delete $2;
      $2=(XML_string_p_t)NULL;
    }
  else
    {
      if (metadata_element_name_p)
        {
          delete metadata_element_name_p;
          metadata_element_name_p = 0;
        }
    }
  ;}
| T_DIRECTIVE_METADATA_TITLE_ELEMENT_NAME T_STRING {
  if ($2)
    {
      if (! metadata_title_element_name_p) metadata_title_element_name_p = new XML_string();
      *(metadata_title_element_name_p) = *($2);
      delete $2;
      $2=(XML_string_p_t)NULL;
    }
  else
    {
      if (metadata_title_element_name_p)
        {
          delete metadata_title_element_name_p;
          metadata_title_element_name_p = 0;
        }
    }
  ;}
| T_DIRECTIVE_METADATA_AUTHOR_ELEMENT_NAME T_STRING {
  if ($2)
    {
      if (! metadata_author_element_name_p) metadata_author_element_name_p = new XML_string();
      *(metadata_author_element_name_p) = *($2);
      delete $2;
      $2=(XML_string_p_t)NULL;
    }
  else
    {
      if (metadata_author_element_name_p)
        {
          delete metadata_author_element_name_p;
          metadata_author_element_name_p = 0;
        }
    }
  ;}
| T_DIRECTIVE_METADATA_DATE_ELEMENT_NAME T_STRING {
  if ($2)
    {
      if (! metadata_date_element_name_p) metadata_date_element_name_p = new XML_string();
      *(metadata_date_element_name_p) = *($2);
      delete $2;
      $2=(XML_string_p_t)NULL;
    }
  else
    {
      if (metadata_date_element_name_p)
        {
          delete metadata_date_element_name_p;
          metadata_date_element_name_p = 0;
        }
    }
  ;}
| T_DIRECTIVE_REFERENCE_ELEMENT_NAME T_STRING {
  if ($2)
    {
      if (! reference_element_name_p) reference_element_name_p = new XML_string();
      *(reference_element_name_p) = *($2);
      delete $2;
      $2=(XML_string_p_t)NULL;
    }
  else
    {
      if (reference_element_name_p)
        {
          delete reference_element_name_p;
          reference_element_name_p = 0;
        }
    }
  ;}
| T_DIRECTIVE_SYSTEM_IDENTIFIER T_STRING {
  if ($2)
    {
      if (! system_identifier_p) system_identifier_p = new XML_string();
      *(system_identifier_p) = *($2);
      delete $2;
      $2=(XML_string_p_t)NULL;
    }
  else
    {
      if (system_identifier_p)
        {
          delete system_identifier_p;
          system_identifier_p = 0;
        }
    }
  ;}
| T_DIRECTIVE_PUBLIC_IDENTIFIER T_STRING {
  if ($2)
    {
      if (! public_identifier_p) public_identifier_p = new XML_string();
      *(public_identifier_p) = *($2);
      delete $2;
      $2=(XML_string_p_t)NULL;
    }
  else
    {
      if (public_identifier_p)
        {
          delete public_identifier_p;
          public_identifier_p = 0;
        }
    }
  ;}
;

paragraph
: paragraph_header {
  $$ = new vector<XML_elem_info_item_p_t>;
  $$->push_back($1);
  ;}
| metadata paragraph_body {
  $$ = new vector<XML_elem_info_item_p_t>;
  XML_elem_info_item_p_t omdoc_paragraph;
  omdoc_paragraph = new XML_elem_info_item;
  manage_metadata_xmlns_in($1);
  manage_metadata_lang_in($1, $2);
  if ($1 && $1->Attribute("for"))
    {
      XML_attr_info_item_p_t attribute;
      attribute
        = $1->Attribute("for");
      omdoc_paragraph
        ->Attributes((new XML_attr_info_item)
                     ->Prefix(attribute->Prefix())
                     ->Local_name(attribute->Local_name())
                     ->Normalized_value(attribute->Normalized_value())
                     );
      $1->Attribute_erase("for");
    }
  if ($1 && $1->Attribute("name"))
    {
      XML_attr_info_item_p_t attribute;
      attribute
        = $1->Attribute("name");
      omdoc_paragraph
        ->Attributes((new XML_attr_info_item)
                     ->Prefix(attribute->Prefix())
                     ->Local_name(attribute->Local_name())
                     ->Normalized_value(attribute->Normalized_value())
                     );
      $1->Attribute_erase("name");
    }
  if ($1 && $1->Attribute("xref"))
    {
      XML_attr_info_item_p_t attribute;
      attribute
        = $1->Attribute("xref");
      omdoc_paragraph
        ->Attributes((new XML_attr_info_item)
                     ->Prefix(attribute->Prefix())
                     ->Local_name(attribute->Local_name())
                     ->Normalized_value(attribute->Normalized_value())
                     );
      $1->Attribute_erase("xref");
    }
  if ($1 && $1->Children().empty())
    {
      if ($1->Parent()
          && $1->Parent()->Type()
            == XML_info_item::elem_info_item)
        {
          static_cast<XML_elem_info_item_p_t>($1->Parent())
            ->Child_erase($1);
        }
      delete $1;
      $1 = 0;
    }
  $$->push_back(omdoc_paragraph
                ->Local_name(paragraph_default_element_name_p)
                ->Children($1)
                ->Children($2)
                );
  int indentation;
  indentation = (int)lexer.Indentation_base();
  while (lexer.Tab_stops()->size() > 0)
    {
      if (indentation == lexer.Tab_stops()->back().indentation)
        break;
      else if (indentation > lexer.Tab_stops()->back().indentation)
        {
          Tab_stop_t tab_stop;
          lexer.Tab_stops()->push_back(tab_stop);
          break;
        }
      else
        lexer.Tab_stops()->pop_back();
    }
  lexer.Tab_stops()->back().indentation = indentation;
  lexer.Tab_stops()->back().element = omdoc_paragraph;
  // Search for its parent:
  if (lexer.Tab_stops()->size() > 1)
    {
      Tab_stop_p_t tab_stop;
      tab_stop =  lexer.Tab_stops(lexer.Tab_stops()->size() - 2);
      if (!indentation_tags[omdoc_paragraph])
        indentation_tags[omdoc_paragraph] = new Indentation_tag_t;
      indentation_tags[omdoc_paragraph]->indentation = tab_stop->indentation;
      indentation_tags[omdoc_paragraph]->parent      = tab_stop->element;
    }
  else
    {
      indentation_tags[omdoc_paragraph]->indentation = 0;
      indentation_tags[omdoc_paragraph]->parent = (XML_elem_info_item_p_t)NULL;
    }
  ;}
| paragraph_body {
  $$ = new vector<XML_elem_info_item_p_t>;
  XML_elem_info_item_p_t omdoc_paragraph;
  omdoc_paragraph = new XML_elem_info_item;
  $$->push_back(omdoc_paragraph
                ->Local_name(paragraph_default_element_name_p)
                ->Children($1)
                );
  int indentation;
  indentation = (int)lexer.Indentation_base();
  while (lexer.Tab_stops()->size() > 0)
    {
      if (indentation == lexer.Tab_stops()->back().indentation)
        break;
      else if (indentation > lexer.Tab_stops()->back().indentation)
        {
          Tab_stop_t tab_stop;
          lexer.Tab_stops()->push_back(tab_stop);
          break;
        }
      else
        lexer.Tab_stops()->pop_back();
    }
  lexer.Tab_stops()->back().indentation = indentation;
  lexer.Tab_stops()->back().element = omdoc_paragraph;
  // Search for its parent:
  if (lexer.Tab_stops()->size() > 1)
    {
      Tab_stop_p_t tab_stop;
      tab_stop =  lexer.Tab_stops(lexer.Tab_stops()->size() - 2);
      if (!indentation_tags[omdoc_paragraph])
        indentation_tags[omdoc_paragraph] = new Indentation_tag_t;
      indentation_tags[omdoc_paragraph]->indentation = tab_stop->indentation;
      indentation_tags[omdoc_paragraph]->parent      = tab_stop->element;
    }
  else
    {
      indentation_tags[omdoc_paragraph]->indentation = 0;
      indentation_tags[omdoc_paragraph]->parent = (XML_elem_info_item_p_t)NULL;
    }
  ;}
| paragraph_header paragraph_body {
  $$ = new vector<XML_elem_info_item_p_t>;
  XML_elem_info_item_p_t omdoc_paragraph;
  omdoc_paragraph = $1;
  $$->push_back(omdoc_paragraph
                ->Children($2)
                );
  ;}
| paragraph_header metadata {
  $$ = new vector<XML_elem_info_item_p_t>;
  XML_elem_info_item_p_t omdoc_paragraph;
  omdoc_paragraph = $1;
  manage_metadata_xmlns_in($2);
  manage_metadata_lang_in($2, omdoc_paragraph);
  if ($2 && $2->Attribute("for"))
    {
      XML_attr_info_item_p_t attribute;
      attribute
        = $2->Attribute("for");
      omdoc_paragraph
        ->Attributes((new XML_attr_info_item)
                     ->Prefix(attribute->Prefix())
                     ->Local_name(attribute->Local_name())
                     ->Normalized_value(attribute->Normalized_value())
                     );
      $2->Attribute_erase("for");
    }
  if ($2 && $2->Attribute("name"))
    {
      XML_attr_info_item_p_t attribute;
      attribute
        = $2->Attribute("name");
      omdoc_paragraph
        ->Attributes((new XML_attr_info_item)
                     ->Prefix(attribute->Prefix())
                     ->Local_name(attribute->Local_name())
                     ->Normalized_value(attribute->Normalized_value())
                     );
      $2->Attribute_erase("name");
    }
  if ($2 && $2->Attribute("xref"))
    {
      XML_attr_info_item_p_t attribute;
      attribute
        = $2->Attribute("xref");
      omdoc_paragraph
        ->Attributes((new XML_attr_info_item)
                     ->Prefix(attribute->Prefix())
                     ->Local_name(attribute->Local_name())
                     ->Normalized_value(attribute->Normalized_value())
                     );
      $2->Attribute_erase("xref");
    }
  if ($2 && $2->Children().empty())
    {
      if ($2->Parent()
          && $2->Parent()->Type()
            == XML_info_item::elem_info_item)
        {
          static_cast<XML_elem_info_item_p_t>($2->Parent())
            ->Child_erase($2);
        }
      delete $2;
      $2 = 0;
    }
  $$->push_back(omdoc_paragraph
                ->Children($2)
                );
  ;}
| paragraph_header metadata paragraph_body {
  $$ = new vector<XML_elem_info_item_p_t>;
  XML_elem_info_item_p_t omdoc_paragraph;
  omdoc_paragraph = $1;
  manage_metadata_xmlns_in($2);
  manage_metadata_lang_in($2, $3);
  if ($2 && $2->Attribute("for"))
    {
      XML_attr_info_item_p_t attribute;
      attribute
        = $2->Attribute("for");
      omdoc_paragraph
        ->Attributes((new XML_attr_info_item)
                     ->Prefix(attribute->Prefix())
                     ->Local_name(attribute->Local_name())
                     ->Normalized_value(attribute->Normalized_value())
                     );
      $2->Attribute_erase("for");
    }
  if ($2 && $2->Attribute("name"))
    {
      XML_attr_info_item_p_t attribute;
      attribute
        = $2->Attribute("name");
      omdoc_paragraph
        ->Attributes((new XML_attr_info_item)
                     ->Prefix(attribute->Prefix())
                     ->Local_name(attribute->Local_name())
                     ->Normalized_value(attribute->Normalized_value())
                     );
      $2->Attribute_erase("name");
    }
  if ($2 && $2->Attribute("xref"))
    {
      XML_attr_info_item_p_t attribute;
      attribute
        = $2->Attribute("xref");
      omdoc_paragraph
        ->Attributes((new XML_attr_info_item)
                     ->Prefix(attribute->Prefix())
                     ->Local_name(attribute->Local_name())
                     ->Normalized_value(attribute->Normalized_value())
                     );
      $2->Attribute_erase("xref");
    }
  if ($2 && $2->Children().empty())
    {
      if ($2->Parent()
          && $2->Parent()->Type()
            == XML_info_item::elem_info_item)
        {
          static_cast<XML_elem_info_item_p_t>($2->Parent())
            ->Child_erase($2);
        }
      delete $2;
      $2 = 0;
    }
  $$->push_back(omdoc_paragraph
                ->Children($2)
                ->Children($3)
                );
  ;}
| paragraph metadata paragraph_body_CMP {
  manage_metadata_xmlns_in($2);
  manage_metadata_lang_in($2, $3);
  XML_elem_info_item_p_t previous_metadata_item;
  previous_metadata_item = $1->back()->Child(metadata_element_name_p);
  if (previous_metadata_item)
    if (previous_metadata_item->Type() != XML_info_item::elem_info_item)
      previous_metadata_item = (XML_elem_info_item_p_t)NULL;
  if (previous_metadata_item)
    {
      unsigned int transferred_children_count;
      transferred_children_count = 0;
      for (list<XML_info_item_p_t>::const_iterator
             item_iterator = $2->Children().begin();
           item_iterator != $2->Children().end();
           ++item_iterator)
        {
          if ((*item_iterator)->Type() == XML_info_item::elem_info_item)
            {
              previous_metadata_item
                ->Children((XML_elem_info_item_p_t)*item_iterator);
              ++transferred_children_count;
            }
        }
      if ($2->Children().size() == transferred_children_count)
        delete $2;
    }
  else
    {
      $1->back()->Children($2);
    }
  $1->back()->Children($3);
  ;}
| paragraph paragraph_body_FMP {
  if ($1->size() > 0)
    {
      if ($1->back()->Child(paragraph_formula_container_element_name_p))
        {
          yyerror("You can't put more than one FMP in a paragraph.");
        }
      else
        {
          $1->back()->Children($2);
        }
    }
  else yyerror("QMath internal error: void paragraph when adding FMP.");
  ;}
| paragraph tag_element {
  if ($1->size() > 0)
    {
      $1->back()->Children($2);
    }
  else yyerror("QMath internal error: void paragraph when adding tag element.");
  ;}
;

paragraph_header
: T_PARAGRAPH_TYPE {
  $$ = build_element_from_description($1);
  // Store with its indentation, for the next generation:
  if (lexer.Tab_stops()->back().indentation > -1)
    lexer.Tab_stops()->back().element = $$;
  else
    {
      yyerror("Internal error in QMath_parser.y: last tab stop is the root one.");
      YYABORT; /* Return fail */
    }
  // Search for its parent:
  if (lexer.Tab_stops()->size() > 1)
    {
      Tab_stop_p_t tab_stop;
      tab_stop =  lexer.Tab_stops(lexer.Tab_stops()->size() - 2);
      if (!indentation_tags[$$])
        indentation_tags[$$] = new Indentation_tag_t;
      indentation_tags[$$]->indentation = tab_stop->indentation;
      indentation_tags[$$]->parent      = tab_stop->element;
    }
  else
    {
      indentation_tags[$$]->indentation = 0;
      indentation_tags[$$]->parent = (XML_elem_info_item_p_t)NULL;
    }
  delete $1; $1=NULL; }
| T_REFERENCE_ID {
  $$ = ((new XML_elem_info_item)
        ->Local_name(paragraph_default_element_name_p)
        ->Attributes((new XML_attr_info_item)
                     ->Local_name(id_attribute_name_p)
                     ->Normalized_value($1)
                     )
        );
  // Store with its indentation, for the next generation:
  if (lexer.Tab_stops()->back().indentation > -1)
    lexer.Tab_stops()->back().element = $$;
  else
    {
      yyerror("Internal error in QMath_parser.y: last tab stop is the root one.");
      YYABORT; /* Return fail */
    }
  // Search for its parent:
  if (lexer.Tab_stops()->size() > 1)
    {
      Tab_stop_p_t tab_stop;
      tab_stop =  lexer.Tab_stops(lexer.Tab_stops()->size() - 2);
      if (!indentation_tags[$$])
        indentation_tags[$$] = new Indentation_tag_t;
      indentation_tags[$$]->indentation = tab_stop->indentation;
      indentation_tags[$$]->parent      = tab_stop->element;
    }
  else
    {
      indentation_tags[$$]->indentation = 0;
      indentation_tags[$$]->parent = (XML_elem_info_item_p_t)NULL;
    }
  delete $1; $1=NULL; }
| T_PARAGRAPH_TYPE T_REFERENCE_ID {
  $$ = build_element_from_description($1);
  // Store with its indentation, for the next generation:
  if (lexer.Tab_stops()->back().indentation > -1)
    lexer.Tab_stops()->back().element = $$;
  else
    {
      yyerror("Internal error in QMath_parser.y: last tab stop is the root one.");
      YYABORT; /* Return fail */
    }
  // Search for its parent:
  if (lexer.Tab_stops()->size() > 1)
    {
      Tab_stop_p_t tab_stop;
      tab_stop =  lexer.Tab_stops(lexer.Tab_stops()->size() - 2);
      if (!indentation_tags[$$])
        indentation_tags[$$] = new Indentation_tag_t;
      indentation_tags[$$]->indentation = tab_stop->indentation;
      indentation_tags[$$]->parent      = tab_stop->element;
    }
  else
    {
      indentation_tags[$$]->indentation = 0;
      indentation_tags[$$]->parent = (XML_elem_info_item_p_t)NULL;
    }
  $$->Attributes((new XML_attr_info_item)
                 ->Local_name(id_attribute_name_p)
                 ->Normalized_value($2)
                 );
  delete $1; $1=NULL;
  delete $2; $2=NULL; }
| paragraph_header blanks {
  $$=$1;// Ignore blanks (SPC, EOL, etc...).
  ;}
;

literal: T_STRING {
  $$ = new XML_string;
  *$$ += "\"";
  *$$ += *$1;
  *$$ += "\"";
  delete $1;
  $1 = (XML_string_p_t)NULL;
  ;}
| T_IDENTIFIER    { $$=$1; }
| T_UCS4_CHAR_RUN { $$=$1; }
;

literal_character: single_character { $$=$1; }
;

tag_element_content
: paragraph_body_CMP {
  $$=new vector<XML_info_item_p_t>;
  for (list<XML_info_item_p_t>::const_iterator
         cmp_child_i = $1->Children().begin();
       cmp_child_i  != $1->Children().end();
       ++cmp_child_i)
    {
      if (XML_info_item::elem_info_item == (*cmp_child_i)->Type())
        {
          $$->push_back(XML_elem_info_item_p_t(*cmp_child_i)
                        ->Parent(XML_elem_info_item_p_t(0))
                        );
        }
      else if (XML_info_item::character_info_item == (*cmp_child_i)->Type())
        {
          $$->push_back(XML_character_info_item_p_t(*cmp_child_i)
                        ->Parent(XML_elem_info_item_p_t(0))
                        );
        }
    }
  delete $1;
  $1=NULL;
  //$$->push_back($1);
  ;}
| tag_element {
  $$=new vector<XML_info_item_p_t>;
  $$->push_back($1);
  ;}
| tag_element_content tag_element {
  $$=$1;
  $$->push_back($2);
  ;}
| tag_element_content blanks {
  $$=$1;
  ;}
| tag_element_content paragraph_body_CMP {
  if ($1) $$=$1;
  else    $$=new vector<XML_info_item_p_t>;
  for (list<XML_info_item_p_t>::const_iterator
         cmp_child_i = $2->Children().begin();
       cmp_child_i  != $2->Children().end();
       ++cmp_child_i)
    {
      if (XML_info_item::elem_info_item == (*cmp_child_i)->Type())
        {
          $$->push_back(XML_elem_info_item_p_t(*cmp_child_i)
                        ->Parent(XML_elem_info_item_p_t(0))
                        );
        }
      else if (XML_info_item::character_info_item == (*cmp_child_i)->Type())
        {
          $$->push_back(XML_character_info_item_p_t(*cmp_child_i)
                        ->Parent(XML_elem_info_item_p_t(0))
                        );
        }
    }
  delete $2;
  $2=NULL;
 ;}
;

tag_element
: T_TAG_OPEN T_TAG_CLOSE {
  $$ = build_element_from_description($1);
  if ($1) { delete $1; $1=NULL; }
  ;}
| T_TAG_OPEN tag_element_content T_TAG_CLOSE {
  $$ = build_element_from_description($1);
  if ($2) { $$->Children(*$2); delete $2; $2=NULL; }
  if ($1) { delete $1; $1=NULL; }
  ;}
| T_TAG_OPEN blanks tag_element_content T_TAG_CLOSE {
  $$ = build_element_from_description($1);
  if ($3) { $$->Children(*$3); delete $3; $3=NULL; }
  if ($1) { delete $1; $1=NULL; }
  ;}
;

paragraph_body
: paragraph_body_FMP { $$ = $1; }
| paragraph_body_CMP { $$ = $1; }
;

paragraph_body_FMP: T_PARAGRAPH_SECTION_MARKER_FMP T_MATH_BEGIN exp T_MATH_END {
  $$ = ((new XML_elem_info_item)
        ->Local_name(paragraph_formula_container_element_name_p)
        ->Children($3->To_XML_elem_info_item()
                   ->Attributes((new XML_attr_info_item)
                                ->Prefix((char*)NULL)
                                ->Local_name("xmlns")
                                ->Normalized_value("http://www.openmath.org/"
                                                   "OpenMath")
                                )
                   )
        );
  OpenMath_object::Delete_recursive($3);
  ;}
| paragraph_body_FMP blanks {
  $$ = $1;// Ignore blanks after formula.
  ;}
;

paragraph_body_CMP
: literal_character {
  XML_elem_info_item_p_t omdoc_paragraph;
  /* [AGP] 20001108: This was done as "$$ = ((omdoc_paragraph = new ...)->...)"
     but it caused segmentation faults on some input sequences. At some point,
     what should have been the value of omdoc_paragraph in a function return
     value, became 0x0000000D. I suppose this is a bug in gcc 2.95.2, since it
     disappeared completely when I moved the object creation out of the
     assignment expression.
  */
  omdoc_paragraph = new XML_elem_info_item;
  $$ = (omdoc_paragraph
        ->Local_name(paragraph_text_container_element_name_p)
        ->Children((new XML_character_info_item)
                   ->Parent(omdoc_paragraph)
                   ->Character_code($1)
                   )
        );
  ;}
| literal {
  XML_elem_info_item_p_t omdoc_paragraph;
  /* [AGP] 20001108: This was done as "$$ = ((omdoc_paragraph = new ...)->...)"
     but it caused segmentation faults on some input sequences. At some point,
     what should have been the value of omdoc_paragraph in a function return
     value, became 0x0000000D. I suppose this is a bug in gcc 2.95.2, since it
     disappeared completely when I moved the object creation out of the
     assignment expression.
  */
  omdoc_paragraph = new XML_elem_info_item;
  $$ = (omdoc_paragraph
        ->Local_name(paragraph_text_container_element_name_p)
        ->Children($1)
        );
  if ($1) { delete $1; $1=NULL; }
  ;}
| T_MATH_BEGIN exp T_MATH_END {
  $$ = ((new XML_elem_info_item)
        ->Local_name(paragraph_text_container_element_name_p)
        ->Children($2->To_XML_elem_info_item()
                   ->Attributes((new XML_attr_info_item)
                                ->Prefix((char*)NULL)
                                ->Local_name("xmlns")
                                ->Normalized_value("http://www.openmath.org/"
                                                   "OpenMath")
                                )
                   )
        );
  OpenMath_object::Delete_recursive($2);
  ;}
| paragraph_body_CMP blanks {
  $$ = $1->Children(" ");
  ;}
| paragraph_body_CMP literal_character {
  $$ = $1->Children((new XML_character_info_item)
                    ->Parent($1)// character_info_items need a parent before
                    ->Character_code($2)// getting the character code.
                    );
  ;}
| paragraph_body_CMP literal {
  $$ = $1->Children($2);
  if ($2) { delete $2; $2=NULL; }
  ;}
| paragraph_body_CMP T_REFERENCE {
  vector<XML_string_t> parts;
  $2->Split(XML_UCS4_char_t(':'), &parts);
  if (parts.size() == 1)
    {
      $$ = $1->Children((new XML_elem_info_item)
                        ->Local_name(reference_element_name_p)
                        ->Attributes((new XML_attr_info_item)
                                     ->Local_name("xref")
                                     ->Normalized_value($2)
                                     )
                        ->Attributes((new XML_attr_info_item)
                                     ->Local_name("type")
                                     ->Normalized_value("cite")
                                     )
                        );
    }
  else //if (parts.size() == 2)
    {
      $$ = $1->Children((new XML_elem_info_item)
                        ->Local_name(reference_element_name_p)
                        ->Attributes((new XML_attr_info_item)
                                     ->Local_name("theory")
                                     ->Normalized_value(parts[0])
                                     )
                        ->Attributes((new XML_attr_info_item)
                                     ->Local_name("name")
                                     ->Normalized_value(parts[1])
                                     )
                        ->Attributes((new XML_attr_info_item)
                                     ->Local_name("type")
                                     ->Normalized_value("cite")
                                     )
                        );
    }
  // TODO: else error. (or warning?)
  if ($2) { delete $2; $2=NULL; }
  ;}
| paragraph_body_CMP T_REFERENCE_OPEN {
  vector<XML_string_t> parts;
  $2->Split(XML_UCS4_char_t(':'), &parts);
  if (parts.size() == 1)
    {
      $1->Children($$ = (new XML_elem_info_item)
                   ->Local_name(reference_element_name_p)
                   ->Attributes((new XML_attr_info_item)
                                ->Local_name("xref")
                                ->Normalized_value($2)
                                )
                   ->Attributes((new XML_attr_info_item)
                                ->Local_name("type")
                                ->Normalized_value("cite")
                                )
                   );
    }
  else //if (parts.size() == 2)
    {
      $1->Children($$ = (new XML_elem_info_item)
                   ->Local_name(reference_element_name_p)
                   ->Attributes((new XML_attr_info_item)
                                ->Local_name("theory")
                                ->Normalized_value(parts[0])
                                )
                   ->Attributes((new XML_attr_info_item)
                                ->Local_name("name")
                                ->Normalized_value(parts[1])
                                )
                   ->Attributes((new XML_attr_info_item)
                                ->Local_name("type")
                                ->Normalized_value("cite")
                                )
                   );
    }
  // TODO: else error. (or warning?)
  if ($2) { delete $2; $2=NULL; }
  ;}
| paragraph_body_CMP T_REFERENCE_CLOSE {
  if (*reference_element_name_p == *($1->Local_name()) && $1->Parent())
    $$ = (XML_elem_info_item_p_t)$1->Parent();
  else
    $$ = (XML_elem_info_item_p_t)$1->Children($2);
  ;}
| paragraph_body_CMP T_MATH_BEGIN exp T_MATH_END {
  $$ = $1->Children(($3->To_XML_elem_info_item())
                    ->Attributes((new XML_attr_info_item)
                                 ->Prefix((char*)NULL)
                                 ->Local_name("xmlns")
                                 ->Normalized_value("http://www.openmath.org/"
                                                    "OpenMath")
                                 )
                    );
  OpenMath_object::Delete_recursive($3);
  ;}
| paragraph_body_CMP tag_element {
  $$ = $1;
  if ($2) { $$->Children($2); }
  ;}
| paragraph_body_CMP T_START_DEBUGGING { yydebug = 1; $$=$1; }
| paragraph_body_CMP T_DEBUGGING_REPORT {
  cerr << $2;
  $$=$1;
  if ($2) { delete $2; $2=NULL; }
  ;}
| paragraph_body_CMP T_STOP_DEBUGGING  { yydebug = 0; $$=$1; }
| paragraph_body_CMP T_MATH_BEGIN error T_MATH_END {
  cerr << lexer.Current_file_name() << ":"
       << lexer.Current_line_number() << ":"
       << " Parsing error in math expression after\n" << $1
       << endl << flush;
  OpenMath_object_p_t tmp_om_object;
  $$ = $1->Children((tmp_om_object = new OpenMath_object)
                    ->Error(2,
                            (new OpenMath_object)
                            ->Char_string("[broken expression]")
                            ,
                            (new OpenMath_object)
                            ->Char_string("[parse error in expression]")
                            )
                    ->To_XML_elem_info_item()
                    ->Attributes((new XML_attr_info_item)
                                 ->Prefix((char*)NULL)
                                 ->Local_name("xmlns")
                                 ->Normalized_value("http://www.openmath.org/"
                                                    "OpenMath")
                                 )
                    );
  OpenMath_object::Delete_recursive(tmp_om_object);
  ;}
| paragraph_body_CMP T_MATH_BEGIN exp error T_MATH_END {
  cerr << lexer.Current_file_name() << ":"
       << lexer.Current_line_number() << ":"
       << " Parsing error in math expression after:\n" << $3
       << endl << flush;
  OpenMath_object_p_t tmp_om_object;
  $$ = $1->Children((tmp_om_object = new OpenMath_object)
                    ->Error(2,
                            $3,
                            (new OpenMath_object)
                            ->Char_string("[parse error in expression]")
                            )
                    ->To_XML_elem_info_item()
                    ->Attributes((new XML_attr_info_item)
                                 ->Prefix((char*)NULL)
                                 ->Local_name("xmlns")
                                 ->Normalized_value("http://www.openmath.org/"
                                                    "OpenMath")
                                 )
                    );
  OpenMath_object::Delete_recursive(tmp_om_object);
  ;}
| paragraph_body_CMP T_MATH_BEGIN exp error T_DOCUMENT_END {
  cerr << lexer.Current_file_name() << ":"
       << lexer.Current_line_number() << ":"
       << " Unterminated math expression:\n" << $3
       << endl << flush;
  OpenMath_object_p_t tmp_om_object;
  $$ = $1->Children((tmp_om_object = new OpenMath_object)
                    ->Error(2,
                            $3,
                            (new OpenMath_object)
                            ->Char_string("[unterminated expression]")
                            )
                    ->To_XML_elem_info_item()
                    ->Attributes((new XML_attr_info_item)
                                 ->Prefix((char*)NULL)
                                 ->Local_name("xmlns")
                                 ->Normalized_value("http://www.openmath.org/"
                                                    "OpenMath")
                                 )
                    )
    ;
  OpenMath_object::Delete_recursive(tmp_om_object);
  // Insert partial paragraph.
  XML_elem_info_item_p_t(root_element)
    ->Children("\n  ");// TODO: Implement proper output indentation.
  XML_elem_info_item_p_t(root_element)
    ->Children((new XML_elem_info_item)
               ->Local_name(paragraph_default_element_name_p)
               ->Children($1)
               )
    ;
  YYABORT; /* Return fail */ }
| paragraph_body_CMP error {
  cerr << lexer.Current_file_name() << ":"
       << lexer.Current_line_number() << ":"
       << " Parsing error in paragraph after:\n" << $1
       << endl << flush;
  $$=$1; }
;

blanks: T_END_OF_LINE        { $$=NULL; }
|       T_SPACE              { $$=NULL; }
|       T_TAB                { $$=NULL; }
|       blanks T_END_OF_LINE { $$=NULL; }
|       blanks T_SPACE       { $$=NULL; }
|       blanks T_TAB         { $$=NULL; }
;

single_character: T_UCS4_CHAR            { $$ = $1; }
|                 T_SEP_COMMA            { $$ = XML_UCS4_char_t(','); }
;

prefix_unary
: T_OP_PLUS exp     { $$ = (new OpenMath_object)->Application(2, $1, $2); }
| T_OP_EXP exp      { $$ = (new OpenMath_object)->Application(2, $1, $2); }
| T_OP_FACT exp     { $$ = (new OpenMath_object)->Application(2, $1, $2); }
;

infix_binary
: exp T_OP_IMPL exp { $$ = (new OpenMath_object)->Application(3, $2, $1, $3); }
| exp T_OP_EQ exp   { $$ = (new OpenMath_object)->Application(3, $2, $1, $3); }
| exp T_OP_OR exp   { $$ = (new OpenMath_object)->Application(3, $2, $1, $3); }
| exp T_OP_AND exp  { $$ = (new OpenMath_object)->Application(3, $2, $1, $3); }
| exp T_OP_PLUS exp { $$ = (new OpenMath_object)->Application(3, $2, $1, $3); }
| exp T_OP_PROD exp { $$ = (new OpenMath_object)->Application(3, $2, $1, $3); }
| exp T_OP_EXP exp  { $$ = (new OpenMath_object)->Application(3, $2, $1, $3); }
| exp T_OP_BIND exp { $$ = (new OpenMath_object)->Binding(3, $2, $1, $3); }
;

postfix_unary
: exp T_OP_FACT     { $$ = (new OpenMath_object)->Application(2, $2, $1); }
;

fenced_exp_list
: T_FENCE_OPEN exp_list T_FENCE_CLOSE   {
  vector<XML_UCS4_char_t> combined_fence_symbol;
  combined_fence_symbol.push_back($1);
  combined_fence_symbol.push_back($3);
  string translated_input;
  int token_type;
  if (lexer.Translate_token(combined_fence_symbol,
                            &translated_input, &token_type))
    {
      $2->push_front((new OpenMath_object)
                     ->Symbol(translated_input));
    }
  else
    {
      $2->push_front((new OpenMath_object)
                     ->Char_string(combined_fence_symbol));
    }
  $$ = $2; }
;

exp
: symbol_variable_number
| T_APPLICATION
| T_OP_IMPL
| T_OP_OR
| T_OP_AND
| T_OP_EQ
| T_OP_PLUS
| T_OP_PROD
| T_OP_EXP
| T_OP_FACT
| T_OP_BIND
| T_BINDING
| T_STR
| T_APPLICATION fenced_exp_list {
  OpenMath_object::Delete_recursive($2->front());
  $2->pop_front();// Ignore. For multiplication, use f*(a,b).
  $$ = (new OpenMath_object)->Application(1, $1)->Application($2);
  ;}
| T_BINDING fenced_exp_list {
  OpenMath_object::Delete_recursive($2->front());
  $2->pop_front();// Ignore. For multiplication, use f*(a,b).
  $$ = (new OpenMath_object)->Binding(1, $1)->Binding($2);
  ;}
| fenced_exp_list {
  OpenMath_object_p_t enclosing_application;
  enclosing_application = $1->front();
  $1->pop_front();
  if (OpenMath_object::symbol == enclosing_application->Type())
    {
      $$ = (new OpenMath_object)
        ->Application(1, enclosing_application)
        ->Application($1);
    }
  else
    {
      $$ = $1->front();
      $1->pop_front();
      if (!$1->empty())
        {
          XML_string_t error_message;
          error_message = "Undefined fence pair: ";
          error_message += *(enclosing_application->Char_string());
          string error_message_utf8;
          error_message.UTF8(&error_message_utf8);
          yyerror(error_message_utf8.c_str());
          for (list<OpenMath_object_p_t>::iterator i = $1->begin();
               i != $1->end();
               ++i)
            {
              OpenMath_object::Delete_recursive(*i);
              *i = 0;
            }
        }
    }
  ;}
| exp fenced_exp_list {
  $2->pop_front();

  $$ = (new OpenMath_object)
    ->Application(3,(new OpenMath_object)->Symbol("arith1:times"), $1, $2->front());
  ;}
| exp symbol_variable_number {
  $$ = (new OpenMath_object)
    ->Application(3,(new OpenMath_object)->Symbol("arith1:times"), $1, $2);
  ;}
| prefix_unary
| infix_binary
| postfix_unary
;

symbol_variable_number
: T_SYMBOL
| T_VARIABLE
| T_INT_NUM
| T_FP_NUM
;

exp_list: exp {
  $$ = new list<OpenMath_object_p_t>;
  $$->push_back($1); }
| exp_list T_SEP_COMMA exp { $1->push_back($3); $$ = $1; }
;

%%

#include <ctype.h>
#include <stdio.h>

void yyerror (const char *s)  /* Called by yyparse on error */
{
  cerr << lexer.Current_file_name() << ":"
       << lexer.Current_line_number() << ":"
       << " error: "
       << "at column " << lexer.Current_column_number() << ": "
       << s
       << endl << flush;
  ++error_count;
}

/* Helper functions: */
void manage_metadata_xmlns_in(XML_elem_info_item_p_t metadata_element)
{
  return;// We don't need this any more.
  if (!metadata_element) return;
  
  for (list<XML_info_item_p_t>::const_iterator
         item_iterator = metadata_element->Children().begin();
       item_iterator  != metadata_element->Children().end();
       ++item_iterator)
    {
      switch ((*item_iterator)->Type())
        {
        case XML_info_item::elem_info_item:
          XML_elem_info_item_p_t item_p;
          item_p = (XML_elem_info_item_p_t)*item_iterator;
          if (item_p->Prefix())
            {
              if (*(item_p->Prefix()) == "dc")
                {
                  item_p
                    ->Prefix((char*)NULL)
                    ->Attributes((new XML_attr_info_item)
                                 ->Prefix((char*)NULL)
                                 ->Local_name("xmlns")
                                 ->Normalized_value(DC_NAMESPACE_URI)
                                 )
                    ;
                }
            }
          break;
        default:
          break;
        }
    }
}

void manage_metadata_lang_in(XML_elem_info_item_p_t metadata, XML_elem_info_item_p_t destination)
{
  if (metadata->Attribute("lang"))
    {
      XML_attr_info_item_p_t attribute;
      attribute
        = metadata->Attribute("lang");
      destination
        ->Attributes((new XML_attr_info_item)
                     ->Prefix(attribute->Prefix())
                     ->Local_name(attribute->Local_name())
                     ->Normalized_value(attribute->Normalized_value())
                     );
      for (list<XML_info_item_p_t>::const_iterator
             item_iterator = metadata->Children().begin();
           item_iterator != metadata->Children().end();
           ++item_iterator)
        {
          if ((*item_iterator)->Type() == XML_info_item::elem_info_item)
            {
              ((XML_elem_info_item_p_t)*item_iterator)
                ->Attributes((new XML_attr_info_item)
                             ->Prefix(attribute->Prefix())
                             ->Local_name(attribute->Local_name())
                             ->Normalized_value(attribute->Normalized_value())
                             );
            }
        }
      metadata->Attribute_erase("lang");
    }
}

void metadata_from_parts(XML_elem_info_item_p_t metadata, XML_string_p_t token, XML_string_p_t argument)
{
  bool unify_common_paths;
  XML_elem_info_item_p_t current_item;
  unify_common_paths = true;
  current_item = metadata;// Will be copied to 'parent' when starting the loop.
  XML_elem_info_item_p_t parent;
  parent = metadata;
  vector<XML_string_t> path_parts;
  token->Split(XML_UCS4_char_t('/'), &path_parts);
  vector<XML_string_t> attributes;
  vector<XML_string_t> assignment;
  vector<XML_string_t> name_parts;
  bool argument_used_for_attribute;
  argument_used_for_attribute = false;
  for (vector<XML_string_t>::iterator
         path_iterator = path_parts.begin();
       path_iterator != path_parts.end();
       ++path_iterator)
    {
      path_iterator->Split(XML_UCS4_char_t('@'), &attributes);
      attributes[0].Split(XML_UCS4_char_t(':'), &name_parts);
      if (! (attributes[0] == ""))
        {
          parent = current_item;
          current_item = (XML_elem_info_item_p_t)NULL;
          if (unify_common_paths && attributes.size() == 1)
            current_item = parent->Child(attributes[0]);
          if (!current_item)
            {
              parent->Children(current_item = (new XML_elem_info_item));
          switch (name_parts.size())
            {
            case 1:
              current_item
                ->Local_name(name_parts[0])
                ;
              break;
            case 2:
              current_item
                ->Prefix    (name_parts[0])
                ->Local_name(name_parts[1])
                ;
              break;
            default:
                  {
                    XML_string_t error_message;
                    error_message = "Bad metadata definition: '";
                    error_message += *token;
                    error_message += "', in '";
                    error_message += attributes[0];
                    error_message += "'";
                    string error_message_utf8;
                    error_message.UTF8(&error_message_utf8);
                    yyerror(error_message_utf8.c_str());
                  }
              break;
            }
            }
        }
      if (attributes.size() > 1)
        {
          for (vector<XML_string_t>::iterator
                 attributes_iterator = attributes.begin() + 1;
               attributes_iterator != attributes.end();
               ++attributes_iterator)
            {
              XML_attr_info_item_p_t attribute;
              current_item->Attributes(attribute = (new XML_attr_info_item));
              attributes_iterator->Split(XML_UCS4_char_t('='), &assignment);
              assignment[0].Split(XML_UCS4_char_t(':'), &name_parts);
              switch (name_parts.size())
                {
                case 1:
                  attribute
                    ->Local_name(name_parts[0])
                    ;
                  break;
                case 2:
                  attribute
                    ->Prefix    (name_parts[0])
                    ->Local_name(name_parts[1])
                    ;
                  break;
                default:
                      {
                        XML_string_t error_message;
                        error_message = "Bad metadata definition: '";
                        error_message += *token;
                        error_message += "', in '";
                        error_message += attributes[0];
                        error_message += "'";
                        string error_message_utf8;
                        error_message.UTF8(&error_message_utf8);
                        yyerror(error_message_utf8.c_str());
                      }
                  break;
                }
              switch (assignment.size())
                {
                case 1:
                  attribute
                    ->Normalized_value(*argument)
                    ;
                  argument_used_for_attribute = true;
                  break;
                case 2:
                  attribute
                    ->Normalized_value(assignment[1])
                    ;
                  break;
                default:
              {
                XML_string_t error_message;
                error_message = "Bad metadata definition: '";
                error_message += *token;
                error_message += "', in '";
                error_message += *attributes_iterator;
                error_message += "'";
                string error_message_utf8;
                error_message.UTF8(&error_message_utf8);
                yyerror(error_message_utf8.c_str());
              }
                  break;
                }
            }
        }
    }
  if (!argument_used_for_attribute)
    current_item->Children(argument);
}

void insert_block_by_indentation_level(vector<XML_elem_info_item_p_t>* block_p, XML_elem_info_item_p_t xml_document_root)
{
  if (!block_p) return;
  
      XML_elem_info_item_p_t parent;
      for (vector<XML_elem_info_item_p_t>::iterator
             elem_iterator = block_p->begin();
           elem_iterator != block_p->end();
           ++elem_iterator)
        {
          map<XML_elem_info_item_p_t, Indentation_tag_p_t>::iterator
            element_tag_pair;
          element_tag_pair = indentation_tags.find(*elem_iterator);
          if (indentation_tags.end() == element_tag_pair)
            {
              parent = xml_document_root;
            }
          else
            {
              if (element_tag_pair->second->parent)
                parent = element_tag_pair->second->parent;
              else
                parent = xml_document_root;
            }
          if (*((*elem_iterator)->Local_name()) == *metadata_element_name_p)
            {
              manage_metadata_lang_in(((XML_elem_info_item_p_t)*elem_iterator), parent);
              if ((*elem_iterator)->Attribute("for")
                  && !parent->Attribute("for"))
                {
                  XML_attr_info_item_p_t attribute;
                  attribute
                    = (*elem_iterator)->Attribute("for");
                  parent
                    ->Attributes((new XML_attr_info_item)
                                 ->Prefix(attribute->Prefix())
                                 ->Local_name(attribute->Local_name())
                                 ->Normalized_value(attribute->Normalized_value())
                                 );
                  (*elem_iterator)->Attribute_erase("for");
                }
              if ((*elem_iterator)->Attribute("catalogue")
                  && !parent->Attribute("catalogue"))
                {
                  XML_attr_info_item_p_t attribute;
                  attribute
                    = (*elem_iterator)->Attribute("catalogue");
                  parent
                    ->Attributes((new XML_attr_info_item)
                                 ->Prefix(attribute->Prefix())
                                 ->Local_name(attribute->Local_name())
                                 ->Normalized_value(attribute->Normalized_value())
                                 );
                  (*elem_iterator)->Attribute_erase("catalogue");
                }
              if ((*elem_iterator)->Attribute("xref")
                  && !parent->Attribute("xref"))
                {
                  XML_attr_info_item_p_t attribute;
                  attribute
                    = (*elem_iterator)->Attribute("xref");
                  parent
                    ->Attributes((new XML_attr_info_item)
                                 ->Prefix(attribute->Prefix())
                                 ->Local_name(attribute->Local_name())
                                 ->Normalized_value(attribute->Normalized_value())
                                 );
                  (*elem_iterator)->Attribute_erase("xref");
                }
              XML_elem_info_item_p_t previous_metadata_item;
              previous_metadata_item = parent->Child(metadata_element_name_p);
              if (previous_metadata_item)
                if (previous_metadata_item->Type() != XML_info_item::elem_info_item)
                  previous_metadata_item = (XML_elem_info_item_p_t)NULL;
              if (previous_metadata_item)
                {
                  unsigned int transferred_children_count;
                  transferred_children_count = 0;
                  for (list<XML_info_item_p_t>::const_iterator
                         item_iterator = (*elem_iterator)->Children().begin();
                       item_iterator != (*elem_iterator)->Children().end();
                       ++item_iterator)
                    {
                      if ((*item_iterator)->Type() == XML_info_item::elem_info_item)
                        {
                          previous_metadata_item
                            ->Children((XML_elem_info_item_p_t)*item_iterator);
                          ++transferred_children_count;
                        }
                    }
                  if ((*elem_iterator)->Children().size()
                      == transferred_children_count)
                    delete (*elem_iterator);
                }
              else parent
                ->Children(*elem_iterator)
                ;
            }
          else parent
            ->Children(*elem_iterator)
            ;
          *elem_iterator = (XML_elem_info_item_p_t)NULL;
        }
      delete block_p; block_p = (vector<XML_elem_info_item_p_t>*)NULL;
}


XML_elem_info_item_p_t build_element_from_description(XML_string_p_t description)
{
  XML_elem_info_item_p_t element;
  element = new XML_elem_info_item;
  vector<XML_string_t> parts;
  description->Split(XML_UCS4_char_t('@'), &parts);
  vector<XML_string_t> name_parts;
  parts[0].Split(XML_UCS4_char_t(':'), &name_parts);
  if (1 == name_parts.size())
    element->Local_name(parts[0]);
  else
    element->Prefix    (name_parts[0])
           ->Local_name(name_parts[1]);
  if (parts.size() > 1)
    {
      for (vector<XML_string_t>::iterator
             parts_iterator = parts.begin() + 1;
           parts_iterator != parts.end();
           ++parts_iterator)
        {
          vector<XML_string_t> attribute_assignement;
          parts_iterator->Split(XML_UCS4_char_t('='), &attribute_assignement);
          XML_attr_info_item_p_t attribute;
          attribute = new XML_attr_info_item;
          attribute_assignement[0].Split(XML_UCS4_char_t(':'), &name_parts);
          if (1 == name_parts.size())
            attribute->Local_name(attribute_assignement[0]);
          else
            attribute->Prefix    (name_parts[0])
                     ->Local_name(name_parts[1]);
          if (attribute_assignement.size() > 1)
            attribute->Normalized_value(attribute_assignement[1]);
          
          element->Attributes(attribute);
        }
    }
  // TODO: else error.
  
  return element;
}

void set_values_from_directives(XML_elem_info_item_p_t root_element)
{
  if (id_attribute_name_p && root_element_identifier_p)
    {
      root_element
        ->Attributes((new XML_attr_info_item)
                     ->Local_name(id_attribute_name_p)
                     ->Normalized_value(root_element_identifier_p)
                     );
    }
  if (root_element_name_p)
    {
      vector<XML_string_t> parts;
      root_element_name_p->Split(XML_UCS4_char_t(' '), &parts);
      if (0 == parts.size())
        {
          cerr << "The root element name property can not be parsed:\n"
               << root_element_name_p << endl;
        }
      else
        {
          root_element->Local_name(parts[0]);
          vector<Unicode_data::UCS4_char_t> attribute_value;
          vector<XML_string_t> attribute_parts;
          for (unsigned int i = 1; i < parts.size(); ++i)
            {
              parts[i].Split(XML_UCS4_char_t('='), &attribute_parts);
              if (attribute_parts.size() != 2)
                {
                  cerr << "Attribute in the root element name property does not have an assignment:\n"
                       << parts[i] << endl;
                }
              else
                {
                  attribute_value = attribute_parts[1].UCS4_string();
                  switch (attribute_value[0])
                    {
                    case XML_UCS4_char_t('"'):
                    case XML_UCS4_char_t('\''):
                      attribute_value.erase(attribute_value.begin());
                      attribute_value.pop_back();
                      break;
                    default:
                      cerr << "The attribute values in the root element property should be quoted:"
                           << attribute_parts[1] << endl;
                      break;
                    }
                  root_element->Attributes((new XML_attr_info_item)
                                           ->Local_name(attribute_parts[0])
                                           ->Normalized_value(attribute_value)
                                           );
                }
            }
        }
    }
  if ((public_identifier_p || system_identifier_p)
      && root_element->Parent()
      && root_element->Parent()->Type() == XML_info_item::doc_info_item)
    {
      XML_doc_info_item_p_t document;
      document = static_cast<XML_doc_info_item_p_t>(root_element->Parent());
      document
        ->Standalone(XML_doc_info_item::no)
        ->Children((new XML_doc_type_decl_info_item)
                   ->Public_identifier(public_identifier_p)
                   ->System_identifier(system_identifier_p)
                   )
        ;
    }
  if (mime_format_p)
    {
      XML_elem_info_item_p_t metadata_element;
      metadata_element = root_element->Child(metadata_element_name_p);
      if (!metadata_element)
        root_element
          ->Children(metadata_element =
                     build_element_from_description(metadata_element_name_p)
                     )
          ;
      metadata_element
        ->Children(build_element_from_description
                   (metadata_mime_format_element_name_p)
                   ->Children(mime_format_p)
                   )
        ;
    }
}
