// Clase:    Tokenizer
// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:39 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002, 2003, 2007 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#include "Tokenizer.h"

#define TOKEN_ID_NONE 0
#define TOKEN_ID_START_DEBUGGING 1
#define TOKEN_ID_STOP_DEBUGGING 2
#define TOKEN_ID_DEBUGGING_REPORT 3
#define TOKEN_ID_QMATH_DOC_BEGIN 4
#define TOKEN_ID_XML_DOC_BEGIN 5
#define TOKEN_ID_XML_QMATH_PROCESSING_INSTRUCTION 6
#define TOKEN_ID_XML_QMATH_PROCESSING_INSTRUCTION_EOL 7
#define TOKEN_ID_DEF_TYPE_SPECIFICATION 8
#define TOKEN_ID_DEF_PARAMETER_LABELLED 9
#define TOKEN_ID_LITERAL_DOLLAR 10
#define TOKEN_ID_MATH_BEGIN 11
#define TOKEN_ID_MATH_END 12
#define TOKEN_ID_INT_NUMBER 13
#define TOKEN_ID_FP_NUMBER 14
#define TOKEN_ID_CHAR_STRING 15
#define TOKEN_ID_SEPARATOR_SPACE 16
#define TOKEN_ID_SEPARATOR_COMMA 17
#define TOKEN_ID_SEPARATOR_COLON 18
#define TOKEN_ID_SEPARATOR_SEMICOLON 19
#define TOKEN_ID_DOUBLE_QUOTE 20
#define TOKEN_ID_SINGLE_QUOTE 21
#define TOKEN_ID_BACK_QUOTE 22
#define TOKEN_ID_EOL 23
#define TOKEN_ID_FENCE_OPEN 24
#define TOKEN_ID_FENCE_CLOSE 25
#define TOKEN_ID_HASH 26
#define TOKEN_ID_SPACE 27
#define TOKEN_ID_TAB 28
#define TOKEN_ID_PARAGRAPH_SEPARATOR 29
#define TOKEN_ID_FENCE_DEFINITION 30
#define TOKEN_ID_IDENTIFIER 31
#define TOKEN_ID_LANG 32
#define TOKEN_ID_DATE 33
#define TOKEN_ID_TITLE 34
#define TOKEN_ID_METADATA_LABEL 35
#define TOKEN_ID_AUTHOR 36
#define TOKEN_ID_METADATA_REFERENCE 37
#define TOKEN_ID_PARAGRAPH_TYPE 38
#define TOKEN_ID_BLOCK_BEGIN 39
#define TOKEN_ID_BLOCK_END 40
#define TOKEN_ID_PARAGRAPH_SECTION_MARKER 41
#define TOKEN_ID_REFERENCE_ID 42
#define TOKEN_ID_REFERENCE 43
#define TOKEN_ID_REFERENCE_OPEN 44
#define TOKEN_ID_REFERENCE_CLOSE 45
#define TOKEN_ID_TAG_OPEN 46
#define TOKEN_ID_TAG_CLOSE 47
#define TOKEN_ID_PLAIN_CHARACTERS 48
#define TOKEN_ID_SINGLE_CHARACTER 49
#define TOKEN_ID_COMMENT 50
#define TOKEN_ID_XML_ENTITY_AMP 51
#define TOKEN_ID_XML_ENTITY_LT 52
#define TOKEN_ID_XML_ENTITY_GT 53
#define TOKEN_ID_XML_ENTITY_APOS 54
#define TOKEN_ID_XML_ENTITY_QUOT 55
/////////////
// Tokenizer - Class constructor.
// no arguments.
//public:
Tokenizer::Tokenizer()
{
  initialized = false;
  index_of_current_character = 0;
  Input_stream(cin, LEXER_STATE_INITIAL);
  char* env_qmath_home;
  env_qmath_home = getenv("QMATH_HOME");
  if (env_qmath_home)
  {
    base_directory = env_qmath_home;
  }
  else
  {
    base_directory = DEFAULT_QMATH_HOME;
  }
  Tab_stop_t tab_stop;
  tab_stop.indentation = -1;
  tab_stop.element = (XML_elem_info_item_p_t)NULL;
  tab_stops.push_back(tab_stop);
  indentation_base = 0;
  at_beginning_of_line = true;
  at_end_of_line = false;
  lexer_state = LEXER_STATE_INITIAL;
  debug = false;
  single_characters_pending = 0;
  single_character_identifiers_pending = 0;
  previous_tag_open = "";
  lang = "";// There is no default language.
  initialized = Load_translation_table();
  error_count = 0;
  embedded_mode = false;
  
  Token_entry_t token_entry;
  token_entry.id = TOKEN_ID_START_DEBUGGING;
  token_entry.regexp.Compile("@<");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  token_entry.id = TOKEN_ID_STOP_DEBUGGING;
  token_entry.regexp.Compile("@>");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  token_entry.id = TOKEN_ID_DEBUGGING_REPORT;
  token_entry.regexp.Compile("@\\([^)\n]*\\)");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  token_entry.id = TOKEN_ID_QMATH_DOC_BEGIN;
  token_entry.regexp.Compile("^\\U0000FEFF?QMATH [0-9]+(\\.[0-9]+)+$");
  tokens[LEXER_STATE_INITIAL].push_back(token_entry);
  token_entry.id = TOKEN_ID_XML_DOC_BEGIN;
  token_entry.regexp.Compile("^\\U0000FEFF?<\\?xml ");
  tokens[LEXER_STATE_INITIAL].push_back(token_entry);
  token_entry.id = TOKEN_ID_XML_QMATH_PROCESSING_INSTRUCTION;
  token_entry.regexp.Compile("<\\?QMath[ \t]*");
  tokens[LEXER_STATE_XML].push_back(token_entry);
  token_entry.id = TOKEN_ID_XML_QMATH_PROCESSING_INSTRUCTION_EOL;
  token_entry.regexp.Compile("[ \t]*\\?>\n?");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_HEADER].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_SIMPLE].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_COMPLETE].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DIRECTIVE].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  token_entry.id = TOKEN_ID_DEF_TYPE_SPECIFICATION;
  token_entry.regexp.Compile("QMATH_DIRECTIVE|PARAGRAPH_TYPE|PARAGRAPH_TYPE_LEVEL_[0-9]+|CONTAINER_TYPE|CONTAINER_TYPE_LEVEL_[0-9]+|PARAGRAPH_SECTION_MARKER|METADATA|APPLICATION|ATTRIBUTION|BINDING|SYMBOL|VARIABLE|OP_IMPL|OP_OR|OP_AND|OP_EQ|OP_PLUS|OP_PROD|OP_EXP|OP_FACT|OP_BIND");
  tokens[LEXER_STATE_QMATH_DEF_SIMPLE].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_COMPLETE].push_back(token_entry);
  token_entry.id = TOKEN_ID_DEF_PARAMETER_LABELLED;
  token_entry.regexp.Compile("[a-zA-Z]+:");
  tokens[LEXER_STATE_QMATH_DEF_COMPLETE].push_back(token_entry);
  token_entry.id = TOKEN_ID_LITERAL_DOLLAR;
  token_entry.regexp.Compile("\\$\\$");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_SIMPLE].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_COMPLETE].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  tokens[LEXER_STATE_XML].push_back(token_entry);
  token_entry.id = TOKEN_ID_MATH_BEGIN;
  token_entry.regexp.Compile("\\$");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  tokens[LEXER_STATE_XML].push_back(token_entry);
  token_entry.id = TOKEN_ID_MATH_END;
  token_entry.regexp.Compile("\\$");
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  token_entry.id = TOKEN_ID_INT_NUMBER;
  token_entry.regexp.Compile("[[:Nd:]]+");
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  token_entry.id = TOKEN_ID_FP_NUMBER;
  token_entry.regexp.Compile("[[:Nd:]]+[.'][[:Nd:]]+");
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  token_entry.id = TOKEN_ID_CHAR_STRING;
  token_entry.regexp.Compile("\"[^\\\\\"]*(\\\\\"?[^\\\\\"]*)*\"");
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_SIMPLE].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_COMPLETE].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DIRECTIVE].push_back(token_entry);
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  token_entry.id = TOKEN_ID_SEPARATOR_SPACE;
  token_entry.regexp.Compile("[ \t]+");
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_HEADER].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_SIMPLE].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_COMPLETE].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DIRECTIVE].push_back(token_entry);
  token_entry.id = TOKEN_ID_SEPARATOR_COMMA;
  token_entry.regexp.Compile(",");
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_SIMPLE].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_COMPLETE].push_back(token_entry);
  token_entry.id = TOKEN_ID_SEPARATOR_COLON;
  token_entry.regexp.Compile(":");
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  token_entry.id = TOKEN_ID_SEPARATOR_SEMICOLON;
  token_entry.regexp.Compile(";");
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  token_entry.id = TOKEN_ID_DOUBLE_QUOTE;
  token_entry.regexp.Compile("\\\"");
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  token_entry.id = TOKEN_ID_SINGLE_QUOTE;
  token_entry.regexp.Compile("'");
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  token_entry.id = TOKEN_ID_BACK_QUOTE;
  token_entry.regexp.Compile("`");
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  token_entry.id = TOKEN_ID_EOL;
  token_entry.regexp.Compile("[ \t]*\n");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_HEADER].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_SIMPLE].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_COMPLETE].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DIRECTIVE].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  tokens[LEXER_STATE_XML].push_back(token_entry);
  token_entry.id = TOKEN_ID_FENCE_OPEN;
  token_entry.regexp.Compile("[[:Ps:]]");
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  token_entry.id = TOKEN_ID_FENCE_CLOSE;
  token_entry.regexp.Compile("[[:Pe:]]");
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  token_entry.id = TOKEN_ID_HASH;
  token_entry.regexp.Compile("#");
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  token_entry.id = TOKEN_ID_SPACE;
  token_entry.regexp.Compile(" ");
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  token_entry.id = TOKEN_ID_TAB;
  token_entry.regexp.Compile("\t");
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  token_entry.id = TOKEN_ID_PARAGRAPH_SEPARATOR;
  token_entry.regexp.Compile("[ \t]*\n[ \t]*\n");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_SIMPLE].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_COMPLETE].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DIRECTIVE].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  token_entry.id = TOKEN_ID_FENCE_DEFINITION;
  token_entry.regexp.Compile("[[:Ps:]][[:Pe:]]");
  tokens[LEXER_STATE_QMATH_DEF_SIMPLE].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_COMPLETE].push_back(token_entry);
  token_entry.id = TOKEN_ID_IDENTIFIER;
  token_entry.regexp.Compile("(([[:Lu:][:Ll:][:Lt:][:Lm:][:Lo:]][[:Lu:][:Ll:][:Lt:][:Lm:][:Lo:][:Pc:][:Nd:][:No:]]*)|([[:Pd:][:Po:][:Sm:][:Sk:][:So:][:No:]][[:Pc:][:Pd:][:Po:][:Sm:][:Sk:][:So:][:No:]]*))|([[:Sc:]])");
  tokens[LEXER_STATE_MATH].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_HEADER].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_SIMPLE].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_COMPLETE].push_back(token_entry);
  token_entry.id = TOKEN_ID_LANG;
  token_entry.regexp.Compile("^:(aa|ab|af|am|ar|as|ay|az|ba|be|bg|bh|bi|bn|bo|br|ca|co|cs|cy|da|de|dz|el|en|eo|es|et|eu|fa|fi|fj|fo|fr|fy|ga|gd|gl|gn|gu|gv|ha|he|hi|hr|hu|hy|ia|id|ie|ik|is|it|iu|ja|jw|ka|kk|kl|km|kn|ko|ks|ku|kw|ky|la|lb|ln|lo|lt|lv|mg|mi|mk|ml|mn|mo|mr|ms|mt|my|na|ne|nl|no|oc|om|or|pa|pl|ps|pt|qu|rm|rn|ro|ru|rw|sa|sd|se|sg|sh|si|sk|sl|sm|sn|so|sq|sr|ss|st|su|sv|sw|ta|te|tg|th|ti|tk|tl|tn|to|tr|ts|tt|tw|ug|uk|ur|uz|vi|vo|wo|xh|yi|yo|za|zh|zu)$");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  token_entry.id = TOKEN_ID_DATE;
  token_entry.regexp.Compile("^(:[[:Nd:]]{4,}-[[:Nd:]]{1,2}-[[:Nd:]]{1,2}( [[:Nd:]]{1,2}:[[:Nd:]]{1,2}(:[[:Nd:]]{1,2})?([+-][[:Nd:]]{2}:[[:Nd:]]{2})?)?)$");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  token_entry.id = TOKEN_ID_TITLE;
  token_entry.regexp.Compile("^:\\\"[^\"]*\\\"$");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  token_entry.id = TOKEN_ID_METADATA_LABEL;
  token_entry.regexp.Compile("^:((([[:Lu:][:Ll:][:Lt:][:Lm:][:Lo:]][[:Lu:][:Ll:][:Lt:][:Lm:][:Lo:][:Pc:][:Nd:][:No:]]*)|([[:Pd:][:Po:][:Sm:][:Sk:][:So:][:No:]][[:Pc:][:Pd:][:Po:][:Sm:][:Sk:][:So:][:No:]]*))):");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  token_entry.id = TOKEN_ID_AUTHOR;
  token_entry.regexp.Compile("^:[^[:Zl:][:Nd:]][^:[:Mn:][:Zl:][:Zp:][:Cc:][:Cf:][:Cs:][:Co:][:Cn:]]+$");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  token_entry.id = TOKEN_ID_METADATA_REFERENCE;
  token_entry.regexp.Compile("^:\\[[[:Lu:][:Ll:][:Lt:][:Lm:][:Lo:]][[:Lu:][:Ll:][:Lt:][:Lm:][:Lo:][:Pc:][:Nd:]:]+\\]$");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  token_entry.id = TOKEN_ID_PARAGRAPH_TYPE;
  token_entry.regexp.Compile("^([ \t]*[[:Lu:][:Ll:][:Lt:][:Lm:][:Lo:][:Pc:]]+:)");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_SIMPLE].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DIRECTIVE].push_back(token_entry);
  token_entry.id = TOKEN_ID_BLOCK_BEGIN;
  token_entry.regexp.Compile("^\\{$");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  token_entry.id = TOKEN_ID_BLOCK_END;
  token_entry.regexp.Compile("^\\}$");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  token_entry.id = TOKEN_ID_PARAGRAPH_SECTION_MARKER;
  token_entry.regexp.Compile("^([ \t]*[[:Lu:][:Ll:][:Lt:][:Lm:][:Lo:][:Pc:]]+:)");
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  token_entry.id = TOKEN_ID_REFERENCE_ID;
  token_entry.regexp.Compile("\\[<-[[:Lu:][:Ll:][:Lt:][:Lm:][:Lo:]][[:Lu:][:Ll:][:Lt:][:Lm:][:Lo:][:Pc:][:Nd:]]+\\]");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_HEADER].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  token_entry.id = TOKEN_ID_REFERENCE;
  token_entry.regexp.Compile("\\[[[:Lu:][:Ll:][:Lt:][:Lm:][:Lo:]][[:Lu:][:Ll:][:Lt:][:Lm:][:Lo:][:Pc:][:Nd:]:]+\\]");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  token_entry.id = TOKEN_ID_REFERENCE_OPEN;
  token_entry.regexp.Compile("\\[[[:Lu:][:Ll:][:Lt:][:Lm:][:Lo:]][[:Lu:][:Ll:][:Lt:][:Lm:][:Lo:][:Pc:][:Nd:]:]+ ");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  token_entry.id = TOKEN_ID_REFERENCE_CLOSE;
  token_entry.regexp.Compile(" \\]");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  token_entry.id = TOKEN_ID_TAG_OPEN;
  token_entry.regexp.Compile("\\[[[:Lu:][:Ll:][:Lt:][:Lm:][:Lo:]][[:Lu:][:Ll:][:Lt:][:Lm:][:Lo:][:Pc:][:Nd:][:No:]]*>|<>");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  token_entry.id = TOKEN_ID_TAG_CLOSE;
  token_entry.regexp.Compile("<\\]");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  token_entry.id = TOKEN_ID_PLAIN_CHARACTERS;
  token_entry.regexp.Compile("[[:Lu:][:Ll:][:Lt:][:Nd:][:Nl:][:No:]]+");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
  tokens[LEXER_STATE_XML].push_back(token_entry);
  token_entry.id = TOKEN_ID_SINGLE_CHARACTER;
  token_entry.regexp.Compile("[^\n]");
  tokens[LEXER_STATE_INITIAL].push_back(token_entry);
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_XML].push_back(token_entry);
  token_entry.id = TOKEN_ID_COMMENT;
  token_entry.regexp.Compile("\\|\\|[^\n]*$");
  tokens[LEXER_STATE_QMATH].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_SIMPLE].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DEF_COMPLETE].push_back(token_entry);
  tokens[LEXER_STATE_QMATH_DIRECTIVE].push_back(token_entry);
  tokens[LEXER_STATE_PARAGRAPH].push_back(token_entry);
}

/////////////
// ~Tokenizer - Class destructor.
// no arguments.
//public:
Tokenizer::~Tokenizer()
{

}

/////////////
// Current_column_number - Get current column number.
// no arguments.
// returns column number, counting from 0, or 0 if error.
//public:
int Tokenizer::Current_column_number()
{
  if (input_streams_stack.size() > 0)
    return input_streams_stack.back().current_input_column_number;
  else
    return 0;
}

using std::vector;

/////////////
// Translate_token - Translate token to OMDoc form.
// token: UCS4 string containing token.
// p_result: pointer to UCS4 string to receive result.
// returns token value (int), as defined in "*.tab.h".
//public:
bool Tokenizer::Translate_token(const vector<XML_UCS4_char_t> input, string *p_translated_input, int *p_token_type)
{
  for (Translation_table_t::reverse_iterator
         entry_iterator = translation_table.rbegin();
       entry_iterator != translation_table.rend();
       ++entry_iterator)
    {
      if (entry_iterator->ucs4_string == input)
        {
          p_translated_input->erase(p_translated_input->begin(),
                                    p_translated_input->end()
                                    );
          p_translated_input->insert(p_translated_input->begin(),
                                     entry_iterator->openmath_symbol.begin(),
                                     entry_iterator->openmath_symbol.end()
                                     );
          *p_token_type = entry_iterator->token_type;
          
          return true;
        }
    }
  
  return false;
}


/////////////
// Language - Tell whether or not the lang varible has been set.
// no arguments.
// returns true if lang had been set, false otherwise.
//public:
bool Tokenizer::Language()
{
  if ("" == lang)
    return false;
  
  return true;
}

/////////////
// Language - Set language for input.
// language: language as ISO 639-1 two letter identifier.
// returns true if identifier is valid, false otherwise.
//public:
bool Tokenizer::Language(XML_string_t &language)
{
  if (!(&language))
    return false;
  if ("" == language)
    return false;
  lang = language;
  
  return true;
}

/////////////
// Language - Put language for input into a string.
// p_language: pointer to string to receive ISO 639-1 two letter identifier.
// returns true on success, false on fail.
//public:
bool Tokenizer::Language(XML_string_p_t p_language)
{
  if (!p_language)
    return false;
  if ("" == lang)
    return false;
  *p_language = lang;
  
  return true;
}

/////////////
// Language - Put language for input into a string.
// p_language: pointer to string to receive ISO 639-1 two letter identifier.
// returns true on success, false on fail.
//public:
bool Tokenizer::Language(string* p_language)
{
  if (!p_language)
    return false;
  if ("" == lang)
    return false;
  lang.UTF8(p_language);
  
  return true;
}



/////////////
// Increment_current_line_number - Increment current line number.
// no arguments.
// returns line number, counting from 1, or 0 if error.
//public:
int Tokenizer::Increment_current_line_number()
{
  if (input_streams_stack.size() > 0)
    {
      input_streams_stack.back().current_input_column_number = 0;
      ++input_streams_stack.back().current_input_line_number;
      return input_streams_stack.back().current_input_line_number;
    }
  else
    return 0;
}

/////////////
// Increment_current_line_number - Increment current line number.
// count: number of lines to add.
// returns line number, counting from 1, or 0 if error.
//public:
int Tokenizer::Increment_current_line_number(unsigned int count)
{
  if (input_streams_stack.size() > 0)
    {
      input_streams_stack.back().current_input_column_number = 0;
      input_streams_stack.back().current_input_line_number += count;
      return input_streams_stack.back().current_input_line_number;
    }
  else
    return 0;
}


/////////////
// Current_line_number - Get current line number.
// no arguments.
// returns line number, counting from 1, or 0 if error.
//public:
int Tokenizer::Current_line_number()
{
  if (input_streams_stack.size() > 0)
    return input_streams_stack.back().current_input_line_number;
  else
    return 0;
}


/////////////
// Load_translation_table - Load minimal input token translation table.
// no arguments.
// returns true on success, false on fail.
//public:
bool Tokenizer::Load_translation_table()
{
  XML_string_t context_name;
  XML_string_t token;
  XML_string_t token_type;
  XML_string_t translation;
  token = "____";
  token_type = "QMATH_DIRECTIVE";
  translation = "qmath_symbol_definition";
  Append_translation_entry(context_name, token, token_type, translation);
  
  return true;
}

/////////////
// State - Tell lexer state.
// no arguments.
// returns current lexer state.
//public:
Tokenizer::Lexer_state_t Tokenizer::State()
{
  return lexer_state;
}

/////////////
// State - Set lexer state.
// state: new state.
// returns true if state is valid, false otherwise.
//public:
bool Tokenizer::State(Lexer_state_t state)
{
  if (debug)
    cerr << "Lexer state: " << lexer_state << "->" << state << endl << flush;
  lexer_state = state;// State is always valid, for now.
  for (vector<Token_entry_t>::iterator token_entry_iterator
         = tokens[lexer_state].begin();
       token_entry_iterator < tokens[lexer_state].end();
       ++token_entry_iterator)
    {
      token_entry_iterator->regexp.Reset();
      token_entry_iterator->match_length = 0;
    }
  
  return true;
}

/////////////
// State_push - Set lexer state, and push previous on the stack.
// state: new state.
// returns true if state is valid, false otherwise.
//public:
bool Tokenizer::State_push(Lexer_state_t state)
{
  lexer_state_stack.push_back(lexer_state);
  State(state);
  
  return true;
}

/////////////
// State_pop - Set lexer state from the stack.
// no arguments.
// returns true if state is valid, false otherwise.
//public:
bool Tokenizer::State_pop()
{
  if (lexer_state_stack.size() < 1)
    {
      cerr << "Tokenizer::State_pop(): no more states after state "
           << State() << endl << flush;
      return false;
    }
  
  State(lexer_state_stack.back());
  lexer_state_stack.pop_back();
  
  return true;
}


using std::ios;
using std::cout;
using std::cin;
using std::endl;
using std::flush;


/////////////
// Input_stream - Current input stream. Reset lexer state to QMATH on EOF.
// no arguments.
// returns current input stream pointer. If it's not valid (NULL) or is at
//         eof state, pops it from the stack and continues until finding a
//         valid one.
//public:
istream* Tokenizer::Input_stream()
{
  while (input_streams_stack.size() > 0)
    {
      if (input_streams_stack.back().p_istream)
        {
          if (!input_streams_stack.back().p_istream->eof())
            break;
          if (debug)
            {
              cerr << "\n    --- Exiting level "
                   << input_streams_stack.size()
                   << " stream." << endl << flush;
            }
        }
      if (input_streams_stack.back().p_ifstream)
        {
          if (!input_streams_stack.back().p_ifstream->eof())
            break;
          else
            input_streams_stack.back().p_ifstream->close();
          if (debug)
            {
              cerr << "\n    --- Exiting level "
                   << input_streams_stack.size()
                   << " stream, file name \""
                   << input_streams_stack.back().file_name
                   << "\"" << endl << flush;
            }
        }
      input_streams_stack.pop_back();
    }
  if (0 == input_streams_stack.size())
    return NULL;
  else
    {
      if (input_streams_stack.back().p_ifstream)
        return input_streams_stack.back().p_ifstream;
      else if (input_streams_stack.back().p_istream)
        return input_streams_stack.back().p_istream;
      else
        return NULL;
    }
}

/////////////
// Input_stream - Switch to a new input stream, and append it to the stack.
// p_input_stream: pointer to new input stream.
// returns true if appended, false otherwise.
//public:
bool Tokenizer::Input_stream(istream &input_stream)
{
  return Input_stream(input_stream, State());// Continue in same lexer state.
}

/////////////
// Input_stream - Switch to a new input stream, and append it to the stack.
// p_input_stream: pointer to new input stream.
// new_lexer_state: lexer state to switch to for this stream.
// returns true if appended, false otherwise.
//public:
bool Tokenizer::Input_stream(istream &input_stream, Lexer_state_t new_lexer_state)
{
  if (!(&input_stream))
    return false;
  Stream_entry_t stream_entry;
  stream_entry.p_istream = &input_stream;
  stream_entry.p_ifstream = NULL;
  stream_entry.current_input_line_number = 1;
  stream_entry.current_input_column_number = 0;
  stream_entry.file_name = "";
  input_streams_stack.push_back(stream_entry);
  State(new_lexer_state);
  if (debug)
    {
      cerr << "\n    --- Entering level "
           << input_streams_stack.size()
           << " stream." << endl << flush;
    }
  
  return true;
}

/////////////
// Input_stream - Switch to a new input stream, created by opening a file.
// file_name: name of file to open.
// returns true if file opened and stream appended, false otherwise.
//public:
bool Tokenizer::Input_stream(const char* file_name)
{
  return Input_stream(file_name, State());// Continue in same lexer state.
}

/////////////
// Input_stream - Switch to a new input stream, created by opening a file.
// file_name: name of file to open.
// new_lexer_state: lexer state to switch to for this stream.
// returns true if file opened and stream appended, false otherwise.
//public:
bool Tokenizer::Input_stream(const char* file_name, Lexer_state_t new_lexer_state)
{
  if (!file_name)
    return false;
  string language;
  Language(&language);
  string full_file_name;
  full_file_name = base_directory;
  if (!full_file_name.empty())
    {
      if (full_file_name[full_file_name.size() - 1] != '/')
        full_file_name += "/";
    }
  full_file_name += CONTEXTS_PATH_FROM_QMATH_HOME;
  full_file_name += language;
  full_file_name += "/";
  if (strlen(file_name) == 0)
    full_file_name += "QMath.qmath";
  else
    {
      if (strncmp("./", file_name, 2) == 0)
        {
          full_file_name = file_name;
        }
      else
        {
          full_file_name += file_name;
        }
    }
  for (vector<Stream_entry_t>::iterator
         stream_entry_iterator = input_streams_stack.begin();
       stream_entry_iterator != input_streams_stack.end();
       ++stream_entry_iterator)
    if (stream_entry_iterator->file_name == full_file_name)
      {
        string error_message;
        error_message = "Context loop found [";
        for (vector<Stream_entry_t>::iterator
               loop_iterator = input_streams_stack.begin();
             loop_iterator != input_streams_stack.end();
             ++loop_iterator)
          {
            if (loop_iterator->p_istream == &cin)
              error_message += "stdin";
            else
              error_message += loop_iterator->file_name;
            error_message += ", ";
          }
        error_message += full_file_name;
        error_message += "]";
        Report_error(error_message);
        return false;
      }
  ifstream* p_new_file;
  p_new_file = new ifstream;
  p_new_file->open(full_file_name.c_str(), ios::in);
  if (!p_new_file->is_open())
    {
      Report_error("Can not open file:");
      Report_error(full_file_name);
      return false;
    }
  
  Stream_entry_t stream_entry;
  stream_entry.p_istream = NULL;
  stream_entry.p_ifstream = p_new_file;
  stream_entry.current_input_line_number = 1;
  stream_entry.current_input_column_number = 0;
  stream_entry.file_name = full_file_name;
  input_streams_stack.push_back(stream_entry);
  State(new_lexer_state);
  if (debug)
    {
      cerr << "\n    --- Entering level "
           << input_streams_stack.size()
           << " stream, file name \""
           << input_streams_stack.back().file_name
           << "\"" << endl << flush;
    }
  
  return true;
}

/////////////
// Input_stream - Switch to a new input stream, created by opening a file.
// file_name: name of file to open.
// returns true if file opened and stream appended, false otherwise.
//public:
bool Tokenizer::Input_stream(const string& file_name)
{
  return Input_stream(file_name.c_str());
}

/////////////
// Input_stream - Switch to a new input stream, created by opening a file.
// file_name: name of file to open.
// new_lexer_state: lexer state to switch to for this stream.
// returns true if file opened and stream appended, false otherwise.
//public:
bool Tokenizer::Input_stream(const string& file_name, Lexer_state_t new_lexer_state)
{
  return Input_stream(file_name.c_str(), new_lexer_state);
}

/////////////
// Input_stream - Switch to a new input stream, created by opening a file.
// file_name: name of file to open.
// returns true if file opened and stream appended, false otherwise.
//public:
bool Tokenizer::Input_stream(XML_string_t &file_name)
{
  string file_name_utf8;
  file_name.UTF8(&file_name_utf8);
  
  return Input_stream(file_name_utf8);
}

/////////////
// Input_stream - Switch to a new input stream, created by opening a file.
// file_name: name of file to open.
// new_lexer_state: lexer state to switch to for this stream.
// returns true if file opened and stream appended, false otherwise.
//public:
bool Tokenizer::Input_stream(XML_string_t &file_name, Lexer_state_t new_lexer_state)
{
  string file_name_utf8;
  file_name.UTF8(&file_name_utf8);
  
  return Input_stream(file_name_utf8, new_lexer_state);
}


/////////////
// Indentation_base - Tell value of base indentation.
// no arguments.
// returns current indentation base.
//public:
unsigned int Tokenizer::Indentation_base()
{
  return indentation_base;
}


/////////////
// Current_file_name - Get current file name.
// no arguments.
// returns pointer to file name.
//public:
const char *Tokenizer::Current_file_name()
{
  if (input_streams_stack.size() > 0)
    return input_streams_stack.back().file_name.c_str();
  else
    return "";
}


using std::vector;

/////////////
// Get_next_token - Get next token from input.
// yylval_p: Pointer to yylval union to pass data to the parser.
// returns token value (int), as defined in "*.tab.h".
//public:
int Tokenizer::Get_next_token(YYSTYPE *yylval_p)
{
  if (!yylval_p)
    {
      Report_error("Tokenizer::Get_next_token(): yylval_p is null.");
      return T_DOCUMENT_END;
    }
  if (!initialized)
    {
      Report_error("Error: Tokenizer::Get_next_token(): class not initialized."
                   " No contexts available.");
      return T_DOCUMENT_END;
    }
  if (!Input_stream() && 0 == input_buffer.size())
    {
      if (debug)
        Report_debug("Get_next_token(): input streams exhausted.");
      Token_buffer_entry_t token_buffer_entry;
      token_buffer_entry.type = T_DOCUMENT_END;
      token_buffer.push_back(token_buffer_entry);
    }
  
  if (token_buffer.size() > 0)
    {
      Token_buffer_entry_t token;
      token = *(token_buffer.begin());
      token_buffer.erase(token_buffer.begin());
      *(yylval_p) = token.value;
      return token.type;
    }
  
  bool document_finished;
  document_finished = false;
  while (!document_finished)
    {
      vector<XML_UCS4_char_t> matched_input;
      matched_input.erase(matched_input.begin(), matched_input.end());
      index_of_current_character = single_characters_pending + single_character_identifiers_pending;
      
      XML_UCS4_char_t character;
      int token_type;
      token_type = T_DOCUMENT_END;
      string translated_input;
      
      if (single_characters_pending > 0)
        {
          yylval_p->character = input_buffer.front().character;
          input_buffer.erase(input_buffer.begin());
          --single_characters_pending;
          if (index_of_current_character > 0)
            --index_of_current_character;
          return T_UCS4_CHAR;
        }
      else if (single_character_identifiers_pending > 0)
        {
          matched_input.push_back(input_buffer.front().character);
          input_buffer.erase(input_buffer.begin());
          --single_character_identifiers_pending;
          if (index_of_current_character > 0)
            --index_of_current_character;
          if (Translate_token(matched_input,
                              &translated_input, &token_type))
            {
              yylval_p->OM_object = (new OpenMath_object)
                ->Symbol(translated_input);
              return token_type;
            }
          else
            {
              translated_input.erase(translated_input.begin(),
                                     translated_input.end());
              if ((input_buffer.front().character & 0xFFFFFF80) == 0)
                {
                  // TODO: Is this really useful?
                  //       Currently, only characters below 128 are accepted.
                  //       This rejects other useful symbols such as greek letters.
                  translated_input += (char(matched_input.front()));
                  yylval_p->OM_object = (new OpenMath_object)
                    ->Variable(translated_input);
                  return T_VARIABLE;
                }
              else
                {
                  Unicode_translator::UTF8(matched_input.front(),
                                           &translated_input);
                  yylval_p->OM_object = (new OpenMath_object)
                    ->Error(2,
                            (new OpenMath_object)
                            ->Char_string(translated_input)
                            ,
                            (new OpenMath_object)
                            ->Char_string("[Unrecognized symbol]")
                            );
                  return T_SYMBOL;// This error replaces a symbol.
                }
            }
        }
      
      for (vector<Token_entry_t>::iterator token_entry_iterator
             = tokens[lexer_state].begin();
           token_entry_iterator < tokens[lexer_state].end();
           ++token_entry_iterator)
        {
          token_entry_iterator->regexp.Reset();
          token_entry_iterator->match_length = 0;
        }
      
      bool all_regexps_failed;
      all_regexps_failed = false;
      bool completing_xml_entity;
      completing_xml_entity = false;
      
      Unicode_regexp::Special_condition_t character_special_condition;
      character_special_condition = Unicode_regexp::SPECIAL_CONDITION_NONE;
      while (!all_regexps_failed)
        {
          if (index_of_current_character == input_buffer.size())
            {
              do
                {
                  character = 0xFFFFFFFF;// Invalid UCS code point.
                  while (!Input_UCS4_from_UTF8_stream(*Input_stream(),
                                                      &character))
                    {
                      // Why did we fail?
                      if (0x0000FFFD == character)
                        {
                          // Fail was because of an encoding error.
                          Report_error("UTF-8 encoding error");
                          yylval_p->text = (new XML_string())
                            ->UTF8("[UTF8 encoding error after '");
                          *(yylval_p->text) += matched_input;
                          *(yylval_p->text) += "']";
                          return T_DEBUGGING_REPORT;
                        }
                      else if (!Input_stream())
                        {
                          // Fail was because Input_stream() is NULL, which
                          // should happen only at EOF or equivalent.
                          if (0 == input_buffer.size())
                            {
                              if (debug)
                                Report_error("Get_next_token(): EOF");
                              if (LEXER_STATE_INITIAL == lexer_state)
                                Report_error("Begin of document mark (QMATH x.x) not found");
                              return T_DOCUMENT_END;
                            }
                          // else continue until input buffer is empty.
                          if (debug)
                            cerr << "Emptying input buffer ("
                                 << input_buffer.size() << ")\n" << flush;
                          all_regexps_failed = true;// Let it drain.
                          break;
                        }
                      // else EOF. Keep reading until we exhaust all streams.
                    }
                  // Got a UCS4 character.
                  // Update position tracking:
                  Increment_current_column_number();
                  switch (character)
                    {
                    case XML_UCS4_char_t('\n'):
                      Increment_current_line_number();
                      break;
                    case XML_UCS4_char_t('&'):
                      if (embedded_mode && LEXER_STATE_XML != State())
                        {
                          completing_xml_entity = true;
                        }
                      break;
                    case XML_UCS4_char_t(';'):
                      if (completing_xml_entity)
                        {
                          string entity;
                          entity = "";
                          for (unsigned int i = index_of_current_character + 1;
                               i < input_buffer.size();
                               ++i)
                            {
                              entity += static_cast<char>(input_buffer[i]
                                                          .character);
                            }
                          if ("amp" == entity)
                            character = XML_UCS4_char_t('&');
                          else if ("lt" == entity)
                            character = XML_UCS4_char_t('<');
                          else if ("gt" == entity)
                            character = XML_UCS4_char_t('>');
                          else if ("apos" == entity)
                            character = XML_UCS4_char_t('\'');
                          else if ("quot" == entity)
                            character = XML_UCS4_char_t('"');
                          else
                            completing_xml_entity = false;
                          if (completing_xml_entity)
                            {
                              input_buffer.erase(input_buffer.begin()
                                                 + index_of_current_character,
                                                 input_buffer.end());
                              completing_xml_entity = false;
                            }
                        }
                      break;
                    default:
                      break;
                    }
                  
                  if (all_regexps_failed)
                    break;// We need to get out of while (!all_regexps_failed) ...
                  
                  if (at_beginning_of_line)
                    {
                      character_special_condition
                        = Unicode_regexp::
                        SPECIAL_CONDITION_AT_BEGINNING_OF_LINE;
                      at_beginning_of_line = false;
                    }
                  else
                    {
                      character_special_condition
                        = Unicode_regexp::SPECIAL_CONDITION_NONE;
                    }
                  
                  switch (character)
                    {
                    case XML_UCS4_char_t('\n'):
                      character_special_condition
                      = Unicode_regexp::
                      Special_condition_t(character_special_condition |
                                          Unicode_regexp::
                                          SPECIAL_CONDITION_AT_END_OF_LINE
                                          );
                      at_beginning_of_line = true;// For next one.
                      break;
                    }
                  Input_buffer_entry_t entry;
                  entry.character = character;
                  entry.special_condition = character_special_condition;
                  input_buffer.push_back(entry);
                }
              while (completing_xml_entity);
              //////////////////////////////////////////////////
              
              index_of_current_character = input_buffer.size();
            }
          else if (index_of_current_character > input_buffer.size())
            {
              // This is an error.
              index_of_current_character = 0;
              character_special_condition = Unicode_regexp::SPECIAL_CONDITION_NONE;
              yylval_p->text = (new XML_string())
                ->UTF8("[QMath internal error]");
              Report_error("Tokenizer::Get_next_token(): "
                           "index_of_current_character > input_buffer.size()");
              return T_DEBUGGING_REPORT;
            }
          else
            {
              character
                = input_buffer[index_of_current_character].character;
              character_special_condition
                = input_buffer[index_of_current_character].special_condition;
              ++index_of_current_character;
            }
          
          if (all_regexps_failed)
            break;// Out of while (!all_regexps_failed) ...
          
          // Signal special conditions.
          if (character_special_condition)
            {
              for (vector<Token_entry_t>::iterator token_entry_iterator
                     = tokens[lexer_state].begin();
                   token_entry_iterator < tokens[lexer_state].end();
                   ++token_entry_iterator)
                {
                  token_entry_iterator
                    ->regexp.Signal_condition(character_special_condition);
                }
            }
          
          // Match character.
          unsigned int matched_regexps_count;
          matched_regexps_count = 0;
          for (vector<Token_entry_t>::iterator token_entry_iterator
                 = tokens[lexer_state].begin();
               token_entry_iterator < tokens[lexer_state].end();
               ++token_entry_iterator)
            {
              if (token_entry_iterator->regexp.Match(character))
                {
                  ++(token_entry_iterator->match_length);
                  ++matched_regexps_count;
                }
            }
          if (matched_regexps_count > 0) matched_input.push_back(character);
          else                           all_regexps_failed = true;
        }// while (!all_regexps_failed) ...
      
      // Find longest completed match.
      unsigned int longest_match_token_id;
      unsigned int longest_match_size;
      longest_match_token_id = TOKEN_ID_NONE;
      longest_match_size = 0;
      for (vector<Token_entry_t>::iterator token_entry_iterator
             = tokens[lexer_state].begin();
           token_entry_iterator < tokens[lexer_state].end();
           ++token_entry_iterator)
        {
          unsigned int match_size;
          match_size = token_entry_iterator->match_length;
          if (match_size > longest_match_size)
            {
              if (token_entry_iterator->regexp.Match_is_complete())
                {
                  longest_match_size = match_size;
                  longest_match_token_id = token_entry_iterator->id;
                }
            }
        }
      
      matched_input.erase(matched_input.begin() + longest_match_size,
                          matched_input.end());
      
      if (debug)
        {
          XML_string buffer;
          buffer.UCS4(matched_input);
          switch (longest_match_token_id)
            {
            case TOKEN_ID_NONE:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_NONE '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_START_DEBUGGING:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_START_DEBUGGING '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_STOP_DEBUGGING:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_STOP_DEBUGGING '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_DEBUGGING_REPORT:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_DEBUGGING_REPORT '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_QMATH_DOC_BEGIN:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_QMATH_DOC_BEGIN '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_XML_DOC_BEGIN:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_XML_DOC_BEGIN '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_XML_QMATH_PROCESSING_INSTRUCTION:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_XML_QMATH_PROCESSING_INSTRUCTION '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_XML_QMATH_PROCESSING_INSTRUCTION_EOL:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_XML_QMATH_PROCESSING_INSTRUCTION_EOL '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_DEF_TYPE_SPECIFICATION:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_DEF_TYPE_SPECIFICATION '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_DEF_PARAMETER_LABELLED:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_DEF_PARAMETER_LABELLED '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_LITERAL_DOLLAR:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_LITERAL_DOLLAR '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_MATH_BEGIN:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_MATH_BEGIN '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_MATH_END:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_MATH_END '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_INT_NUMBER:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_INT_NUMBER '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_FP_NUMBER:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_FP_NUMBER '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_CHAR_STRING:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_CHAR_STRING '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_SEPARATOR_SPACE:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_SEPARATOR_SPACE '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_SEPARATOR_COMMA:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_SEPARATOR_COMMA '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_SEPARATOR_COLON:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_SEPARATOR_COLON '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_SEPARATOR_SEMICOLON:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_SEPARATOR_SEMICOLON '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_DOUBLE_QUOTE:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_DOUBLE_QUOTE '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_SINGLE_QUOTE:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_SINGLE_QUOTE '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_BACK_QUOTE:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_BACK_QUOTE '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_EOL:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_EOL '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_FENCE_OPEN:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_FENCE_OPEN '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_FENCE_CLOSE:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_FENCE_CLOSE '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_HASH:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_HASH '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_SPACE:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_SPACE '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_TAB:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_TAB '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_PARAGRAPH_SEPARATOR:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_PARAGRAPH_SEPARATOR '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_FENCE_DEFINITION:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_FENCE_DEFINITION '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_IDENTIFIER:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_IDENTIFIER '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_LANG:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_LANG '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_DATE:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_DATE '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_TITLE:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_TITLE '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_METADATA_LABEL:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_METADATA_LABEL '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_AUTHOR:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_AUTHOR '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_METADATA_REFERENCE:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_METADATA_REFERENCE '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_PARAGRAPH_TYPE:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_PARAGRAPH_TYPE '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_BLOCK_BEGIN:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_BLOCK_BEGIN '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_BLOCK_END:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_BLOCK_END '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_PARAGRAPH_SECTION_MARKER:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_PARAGRAPH_SECTION_MARKER '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_REFERENCE_ID:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_REFERENCE_ID '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_REFERENCE:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_REFERENCE '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_REFERENCE_OPEN:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_REFERENCE_OPEN '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_REFERENCE_CLOSE:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_REFERENCE_CLOSE '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_TAG_OPEN:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_TAG_OPEN '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_TAG_CLOSE:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_TAG_CLOSE '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_PLAIN_CHARACTERS:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_PLAIN_CHARACTERS '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_SINGLE_CHARACTER:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_SINGLE_CHARACTER '" << buffer << "'"
                   << endl << flush;
              break;
            case TOKEN_ID_COMMENT:
              cerr << "\n   --- [" << index_of_current_character
                   << "] TOKEN_ID_COMMENT '" << buffer << "'"
                   << endl << flush;
              break;
            default:
              cerr << "\n  --- UNKNOWN TOKEN:" << longest_match_token_id
                   << endl << flush;
            }
        }
      
      // Handle special tokens.
      switch (longest_match_token_id)
        {
        case TOKEN_ID_LITERAL_DOLLAR:
          // Remove the second '$' in '$$', but leave longest_match_size
          // the same, so that the whole token is removed from the input
          // buffer.
          matched_input.pop_back();
          // Change the interpretation depending on lexer state:
          switch (lexer_state)
            {
            case LEXER_STATE_QMATH:
            case LEXER_STATE_PARAGRAPH:
              longest_match_token_id = TOKEN_ID_PLAIN_CHARACTERS;
              break;
            case LEXER_STATE_QMATH_DEF_SIMPLE:
            case LEXER_STATE_QMATH_DEF_COMPLETE:
            case LEXER_STATE_MATH:
              longest_match_token_id = TOKEN_ID_IDENTIFIER;
              break;
            default:
              Report_error("Lexer state not handled in Get_next_token().");
              break;
            }
          break;
        case TOKEN_ID_BLOCK_BEGIN:
          // Increase the nesting level.
          ++indentation_base;
          longest_match_token_id = TOKEN_ID_PARAGRAPH_SEPARATOR;
          break;
        case TOKEN_ID_BLOCK_END:
          // Diminish the nesting level.
          if (indentation_base > 0) --indentation_base;
          else Report_error("Unexpected paragraph block ending.");
          longest_match_token_id = TOKEN_ID_PARAGRAPH_SEPARATOR;
          break;
        case TOKEN_ID_FENCE_DEFINITION:
          longest_match_token_id = TOKEN_ID_IDENTIFIER;
          break;
        case TOKEN_ID_TAG_OPEN:
          if (XML_UCS4_char_t('<') == matched_input.front())
            {
              Token_buffer_entry_t token_buffer_entry;
              token_buffer_entry.type = T_TAG_OPEN;
              token_buffer_entry.value.text = (new XML_string)->UTF8(previous_tag_open);
              token_buffer.push_back(token_buffer_entry);
              longest_match_token_id = TOKEN_ID_TAG_CLOSE;
            }
          break;
        case TOKEN_ID_TAG_CLOSE:
          previous_tag_open = "";
          break;
        case TOKEN_ID_XML_ENTITY_AMP:
          matched_input.clear();
          matched_input.push_back('&');
          break;
        case TOKEN_ID_XML_ENTITY_LT:
          matched_input.clear();
          matched_input.push_back('<');
          break;
        case TOKEN_ID_XML_ENTITY_GT:
          matched_input.clear();
          matched_input.push_back('>');
          break;
        case TOKEN_ID_XML_ENTITY_APOS:
          matched_input.clear();
          matched_input.push_back('\'');
          break;
        case TOKEN_ID_XML_ENTITY_QUOT:
          matched_input.clear();
          matched_input.push_back('"');
          break;
        }
      
      if (debug) cerr << "Lexer state is " << lexer_state << "." << endl;
      
      // Translate token, if needed.
      switch (lexer_state)
        {
        case LEXER_STATE_MATH:
          switch (longest_match_token_id)
            {
            case TOKEN_ID_IDENTIFIER:
              if (!Translate_token(matched_input,
                                   &translated_input, &token_type))
                {
                  matched_input.erase(matched_input.begin(),
                                      matched_input.end());
                  single_character_identifiers_pending = longest_match_size;
                  single_characters_pending = 0;
                  matched_input.push_back(input_buffer.front().character);
                  input_buffer.erase(input_buffer.begin());
                  --single_character_identifiers_pending;
                  if (index_of_current_character > 0)
                    --index_of_current_character;
                  if (Translate_token(matched_input,
                                      &translated_input, &token_type))
                    {
                      yylval_p->OM_object = (new OpenMath_object)
                        ->Symbol(translated_input);
                      return token_type;
                    }
                  else
                    {
                      translated_input.erase(translated_input.begin(),
                                             translated_input.end());
                      if ((input_buffer.front().character & 0xFFFFFF80) == 0)
                        {
                          // TODO: Is this really useful?
                          //       Currently, only characters below 128 are accepted.
                          //       This rejects other useful symbols such as greek letters.
                          translated_input += (char(matched_input.front()));
                          yylval_p->OM_object = (new OpenMath_object)
                            ->Variable(translated_input);
                          return T_VARIABLE;
                        }
                      else
                        {
                          Unicode_translator::UTF8(matched_input.front(),
                                                   &translated_input);
                          yylval_p->OM_object = (new OpenMath_object)
                            ->Error(2,
                                    (new OpenMath_object)
                                    ->Char_string(translated_input)
                                    ,
                                    (new OpenMath_object)
                                    ->Char_string("[Unrecognized symbol]")
                                    );
                          return T_SYMBOL;// This error replaces a symbol.
                        }
                    }
                }
              break;
            case TOKEN_ID_INT_NUMBER:
              for (vector<XML_UCS4_char_t>::iterator
                     character_iterator = matched_input.begin();
                   character_iterator < matched_input.end();
                   ++character_iterator)
                {
                  char ascii_char;
                  if ((*character_iterator & 0xFFFFFF80) == 0)
                    ascii_char = char(*character_iterator & 0x0000007F);
                  else
                    {
                      Unicode_data::character_entry_t character_entry;
                      if (unicode.Character_entry(*character_iterator,
                                                  &character_entry))
                        ascii_char = '0'
                          + character_entry.decimal_digit_value;
                      else
                        ascii_char = '?';// TODO: This is an error.
                    }
                  translated_input += (ascii_char);
                }
              break;
            case TOKEN_ID_FP_NUMBER:
              for (vector<XML_UCS4_char_t>::iterator
                     character_iterator = matched_input.begin();
                   character_iterator < matched_input.end();
                   ++character_iterator)
                {
                  char ascii_char;
                  ascii_char = 0x00;
                  if ((*character_iterator & 0xFFFFFF80) == 0)
                    ascii_char = char(*character_iterator & 0x0000007F);
                  else
                    {
                      Unicode_data::character_entry_t character_entry;
                      if (unicode.Character_entry(*character_iterator,
                                                  &character_entry))
                        ascii_char = '0'
                          + character_entry.decimal_digit_value;
                      else
                        ascii_char = '?';// TODO: This is an error.
                    }
                  switch (*character_iterator)
                    {
                    case 0x00000027:// APOSTROPHE
                    case 0x0000002C:// COMMA
                    case 0x0000066B:// ARABIC DECIMAL SEPARATOR
                    case 0x00002396:// DECIMAL SEPARATOR KEY SYMBOL(ISO 9995-7)
                      ascii_char = '.';
                      break;
                    }
                  if (ascii_char)
                    translated_input += (ascii_char);
                }
              break;
            case TOKEN_ID_CHAR_STRING:
              for (vector<XML_UCS4_char_t>::iterator
                     character_iterator = matched_input.begin();
                   character_iterator < matched_input.end();
                   ++character_iterator)
                {
                  //TODO: This must be an Unicode string, not an ASCII one.
                  char ascii_char;
                  ascii_char = 0x00;
                  if ((*character_iterator & 0xFFFFFF80) == 0)
                    ascii_char = char(*character_iterator & 0x0000007F);
                  if (ascii_char)
                    translated_input += (ascii_char);
                }
              break;
              // No default.
            }// switch (longest_match_token_id) ...
          break;
        case LEXER_STATE_QMATH_DEF_SIMPLE:
        case LEXER_STATE_QMATH_DIRECTIVE:
        case LEXER_STATE_QMATH:
        case LEXER_STATE_PARAGRAPH:
          switch (longest_match_token_id)
            {
              int counter;
              bool open_block;
            case TOKEN_ID_PARAGRAPH_TYPE:
              if (matched_input.size() < 2)
                {
                  XML_string_t match;
                  match.UCS4(matched_input);
                  cerr << "QMath: Get_next_token(): Wrong match for "
                       << "paragraph type: '" << match << "'"
                       << endl << flush;
                  longest_match_token_id = TOKEN_ID_NONE;
                  longest_match_size = 0;
                  break;
                }
              open_block = (XML_UCS4_char_t('{') == matched_input.back());
              matched_input.pop_back();// Get rid of the ending ':' or '{'.
              counter = 0;
              while (   XML_UCS4_char_t(' ')  == matched_input.front()
                     || XML_UCS4_char_t('\t') == matched_input.front())
                {
                  matched_input.erase(matched_input.begin());
                  ++counter;
                  // The result of this gets raised by indentation_base later.
                }
              if (!Translate_token(matched_input,
                                   &translated_input, &token_type))
                {
                  matched_input.erase(matched_input.begin(),
                                      matched_input.end());
                  single_characters_pending = longest_match_size;
                  single_character_identifiers_pending = 0;
                  yylval_p->character = input_buffer.front().character;
                  input_buffer.erase(input_buffer.begin());
                  --single_characters_pending;
                  if (index_of_current_character > 0)
                    --index_of_current_character;
                  return T_UCS4_CHAR;
                  break;
                }
              switch (token_type)
                {
                case T_PARAGRAPH_TYPE:
                case T_CONTAINER_TYPE:
                  counter += indentation_base;
                  break;
                case T_PARAGRAPH_TYPE_LEVEL_0:
                case T_CONTAINER_TYPE_LEVEL_0:
                  counter += 0;
                  indentation_base = 0 + 1;
                  break;
                case T_PARAGRAPH_TYPE_LEVEL_1:
                case T_CONTAINER_TYPE_LEVEL_1:
                  counter += 1;
                  indentation_base = 1 + 1;
                  break;
                case T_PARAGRAPH_TYPE_LEVEL_2:
                case T_CONTAINER_TYPE_LEVEL_2:
                  counter += 2;
                  indentation_base = 2 + 1;
                  break;
                case T_PARAGRAPH_TYPE_LEVEL_3:
                case T_CONTAINER_TYPE_LEVEL_3:
                  counter += 3;
                  indentation_base = 3 + 1;
                  break;
                case T_PARAGRAPH_TYPE_LEVEL_4:
                case T_CONTAINER_TYPE_LEVEL_4:
                  counter += 4;
                  indentation_base = 4 + 1;
                  break;
                case T_PARAGRAPH_TYPE_LEVEL_5:
                case T_CONTAINER_TYPE_LEVEL_5:
                  counter += 5;
                  indentation_base = 5 + 1;
                  break;
                case T_QMATH_DIRECTIVE:
                  counter = -1;
                  break;
                default:
                  cerr << "QMath: Get_next_token(): unknown token type:"
                       << token_type << endl << flush;
                  break;
                }
              if (counter >= 0)
                {
                  while (tab_stops.size() > 0)
                    {
                      if (counter == tab_stops.back().indentation)
                        break;
                      else if (counter > tab_stops.back().indentation)
                        {
                          Tab_stop_t tab_stop;
                          tab_stops.push_back(tab_stop);
                          break;
                        }
                      else
                        tab_stops.pop_back();
                    }
                  tab_stops.back().indentation = counter;
                  tab_stops.back().element = (XML_elem_info_item_p_t)NULL;
                  if (open_block) ++indentation_base;
                }
              break;
            case TOKEN_ID_PARAGRAPH_SECTION_MARKER:
              if (matched_input.size() < 2)
                {
                  XML_string_t match;
                  match.UCS4(matched_input);
                  cerr << "QMath: Get_next_token(): Wrong match for "
                       << "paragraph section marker: '" << match << "'"
                       << endl << flush;
                  longest_match_token_id = TOKEN_ID_NONE;
                  longest_match_size = 0;
                  break;
                }
              matched_input.pop_back();// Get rid of the ending colon (':').
              counter = 0;
              while (   XML_UCS4_char_t(' ')  == matched_input.front()
                     || XML_UCS4_char_t('\t') == matched_input.front())
                {
                  matched_input.erase(matched_input.begin());
                  ++counter;
                  // The result of this gets raised by indentation_base later.
                }
              if (!Translate_token(matched_input,
                                   &translated_input, &token_type))
                {
                  if (debug)
                    {
                      XML_string_t message_UCS4;
                      string message_UTF8;
                      message_UCS4.UCS4(matched_input);
                      message_UCS4.UTF8(&message_UTF8);
                      cerr << "Can't translate '" << message_UTF8 << "'."
                           << endl;
                    }
                  matched_input.erase(matched_input.begin(),
                                      matched_input.end());
                  single_characters_pending = longest_match_size;
                  single_character_identifiers_pending = 0;
                  yylval_p->character = input_buffer.front().character;
                  input_buffer.erase(input_buffer.begin());
                  --single_characters_pending;
                  if (index_of_current_character > 0)
                    --index_of_current_character;
                  return T_UCS4_CHAR;
                  break;
                }
              break;
            case TOKEN_ID_METADATA_LABEL:
              matched_input.erase(matched_input.begin());// First colon.
              matched_input.pop_back();// Ending colon.
              --longest_match_size;// Leave the last colon to be parsed again.
              // Parse the rest of the line like it was a complete line.
              // The current longest_match_size gives us the index of the colon.
              input_buffer[longest_match_size]
                .special_condition = Unicode_regexp::
                SPECIAL_CONDITION_AT_BEGINNING_OF_LINE;
              if (!Translate_token(matched_input,
                                   &translated_input, &token_type))
                {
                  XML_string_t label;
                  label.UCS4(matched_input);
                  cerr << Current_file_name() << ":"
                       << Current_line_number() << ":"
                       << " error:"
                       << " Unrecognized metadata label \""
                       << label
                       << "\""
                       << endl << flush;
                }
              break;
            case TOKEN_ID_TAG_OPEN:
              matched_input.erase(matched_input.begin());// Opening '['.
              matched_input.pop_back();// Ending '>'.
              if (!Translate_token(matched_input,
                                   &translated_input, &token_type))
                {
                  XML_string_t label;
                  label.UCS4(matched_input);
                  cerr << Current_file_name() << ":"
                       << Current_line_number() << ":"
                       << " error:"
                       << " Unrecognized tag name \""
                       << label
                       << "\""
                       << endl << flush;
                }
              previous_tag_open = translated_input;
              break;
              // No default.
            }// switch (longest_match_token_id) ...
          break;
        default:
          // NOP
          break;
        }// switch (lexer_state) ...
      
      // Token actions.
      if (longest_match_token_id == TOKEN_ID_NONE)
        {
          // Well, there was no match. Return the first character in the buffer
          // and let the rest to be processed in further calls.
          yylval_p->character = input_buffer.front().character;
          input_buffer.erase(input_buffer.begin());
          --index_of_current_character;
          return T_UCS4_CHAR;
        }
      else
        {
          input_buffer.erase(input_buffer.begin(),
                             input_buffer.begin() + longest_match_size);
          index_of_current_character -= longest_match_size;
          
          Token_buffer_entry_t token_buffer_entry;
          Token_entry_t token_entry;
          switch (lexer_state)
            {
            case LEXER_STATE_INITIAL:
              switch (longest_match_token_id)
                {
                case TOKEN_ID_QMATH_DOC_BEGIN:
                  embedded_mode = false;
                  State(LEXER_STATE_QMATH);
                  break;
                case TOKEN_ID_XML_DOC_BEGIN:
                  embedded_mode = true;
                  token_entry.id = TOKEN_ID_XML_ENTITY_AMP;
                  token_entry.regexp.Compile("&amp;");
                  tokens[LEXER_STATE_MATH].push_back(token_entry);
                  token_entry.id = TOKEN_ID_XML_ENTITY_LT;
                  token_entry.regexp.Compile("&lt;");
                  tokens[LEXER_STATE_MATH].push_back(token_entry);
                  token_entry.id = TOKEN_ID_XML_ENTITY_GT;
                  token_entry.regexp.Compile("&gt;");
                  tokens[LEXER_STATE_MATH].push_back(token_entry);
                  token_entry.id = TOKEN_ID_XML_ENTITY_APOS;
                  token_entry.regexp.Compile("&apos;");
                  tokens[LEXER_STATE_MATH].push_back(token_entry);
                  token_entry.id = TOKEN_ID_XML_ENTITY_QUOT;
                  token_entry.regexp.Compile("&quot;");
                  tokens[LEXER_STATE_MATH].push_back(token_entry);
                  if (0x0000FEFF == matched_input[0])
                    matched_input.erase(matched_input.begin());
                  yylval_p->text = (new XML_string)->UCS4(matched_input);
                  State(LEXER_STATE_XML);
                  token_buffer_entry.value.text = yylval_p->text;
                  yylval_p->text = 0;
                  token_buffer_entry.type = T_UCS4_CHAR_RUN;
                  token_buffer.push_back(token_buffer_entry);
                  return T_EMBEDDED_MARKER;
                  break;
                case TOKEN_ID_START_DEBUGGING:
                  debug = true;
                  return T_START_DEBUGGING;
                  break;
                case TOKEN_ID_STOP_DEBUGGING:
                  debug = false;
                  return T_STOP_DEBUGGING;
                  break;
                case TOKEN_ID_DEBUGGING_REPORT:
                  yylval_p->text = (new XML_string)->UCS4(matched_input);
                  return T_DEBUGGING_REPORT;
                  break;
                case TOKEN_ID_SINGLE_CHARACTER:
                  ;// Pass over until we reach the QMATH header.
                  break;
                default:
                  yylval_p->text = (new XML_string)->UCS4(matched_input);
                  cerr << "Error in Get_next_token(): unhandled sequence '"
                       << yylval_p->text
                       << "' in state " << lexer_state << "." << endl;
                  return T_DEBUGGING_REPORT;
                  break;
                }
              break;
            case LEXER_STATE_QMATH:
              switch (longest_match_token_id)
                {
                case TOKEN_ID_START_DEBUGGING:
                  debug = true;
                  return T_START_DEBUGGING;
                  break;
                case TOKEN_ID_STOP_DEBUGGING:
                  debug = false;
                  return T_STOP_DEBUGGING;
                  break;
                case TOKEN_ID_DEBUGGING_REPORT:
                  yylval_p->text = (new XML_string)->UCS4(matched_input);
                  return T_DEBUGGING_REPORT;
                  break;
                case TOKEN_ID_COMMENT:
                  break;
                case TOKEN_ID_MATH_BEGIN:
                  if (LEXER_STATE_XML == State())
                    {
                      token_buffer_entry.value.text = yylval_p->text;
                      yylval_p->text = 0;
                      token_buffer_entry.type = T_MATH_BEGIN;
                      token_buffer.push_back(token_buffer_entry);
                      State_push(LEXER_STATE_MATH);
                      return T_EMBEDDED_MARKER;
                    }
                  State_push(LEXER_STATE_MATH);
                  return T_MATH_BEGIN;
                  break;
                case TOKEN_ID_EOL:
                  if (LEXER_STATE_XML == State())
                    {
                      token_buffer_entry.value.text = yylval_p->text;
                      yylval_p->text = 0;
                      token_buffer_entry.type = T_END_OF_LINE;
                      token_buffer.push_back(token_buffer_entry);
                      return T_EMBEDDED_MARKER;
                    }
                  return T_END_OF_LINE;
                  break;
                case TOKEN_ID_XML_QMATH_PROCESSING_INSTRUCTION_EOL:
                  State_pop();// This should bring us back to the XML state.
                  if (LEXER_STATE_XML != State())
                    {
                      cerr << "Unexpected state after PI end: "
                           << State() << endl << flush;
                    }
                  return T_NEW_PARAGRAPH;
                  break;
                case TOKEN_ID_PARAGRAPH_SEPARATOR:
                  State(LEXER_STATE_QMATH);
                  return T_NEW_PARAGRAPH;
                  break;
                case TOKEN_ID_TITLE:
                  token_buffer_entry.value.text = (new XML_string)
                    ->UCS4(matched_input, 2, matched_input.size() - 1);
                  token_buffer_entry.type = T_METADATA_TITLE;
                  token_buffer.push_back(token_buffer_entry);
                  State(LEXER_STATE_PARAGRAPH);
                  return T_METADATA_BEGIN;
                  break;
                case TOKEN_ID_AUTHOR:
                  token_buffer_entry.value.text = (new XML_string)
                    ->UCS4(matched_input, 1, matched_input.size());
                  token_buffer_entry.type = T_METADATA_AUTHOR;
                  token_buffer.push_back(token_buffer_entry);
                  State(LEXER_STATE_PARAGRAPH);
                  return T_METADATA_BEGIN;
                  break;
                case TOKEN_ID_DATE:
                  for (vector<XML_UCS4_char_t>::iterator
                         character_iterator = matched_input.begin();
                       character_iterator < matched_input.end();
                       ++character_iterator)
                    if (XML_UCS4_char_t(' ') == *character_iterator)
                      *character_iterator = XML_UCS4_char_t('T');
                  token_buffer_entry.value.text = (new XML_string)
                    ->UCS4(matched_input, 1, matched_input.size());
                  token_buffer_entry.type = T_METADATA_DATE;
                  token_buffer.push_back(token_buffer_entry);
                  State(LEXER_STATE_PARAGRAPH);
                  return T_METADATA_BEGIN;
                  break;
                case TOKEN_ID_LANG:
                  token_buffer_entry.value.text = (new XML_string)
                    ->UCS4(matched_input, 1, matched_input.size());
                  token_buffer_entry.type = T_METADATA_LANG;
                  token_buffer.push_back(token_buffer_entry);
                  if (Language())
                    {
                      State(LEXER_STATE_PARAGRAPH);
                    }
                  else
                    {
                      bool success;
                      Language(*token_buffer_entry.value.text);
                      // Load basic context for language:
                      success = Input_stream("", Tokenizer::LEXER_STATE_QMATH);
                      if (!success)
                        {
                          string message;
                          message = "Can't load basic context file."
                            " Is the environment variable QMATH_HOME='";
                          message += getenv("QMATH_HOME");
                          message += "' correctly set?";
                          Report_error(message.c_str());
                          return T_DOCUMENT_END;
                        }
                      token_buffer_entry.value.markup = 0;
                      token_buffer_entry.type = T_NEW_PARAGRAPH;
                      token_buffer.push_back(token_buffer_entry);
                      State(LEXER_STATE_QMATH);
                    }
                  return T_METADATA_BEGIN;
                  break;
                case TOKEN_ID_METADATA_REFERENCE:
                  yylval_p->text = (new XML_string)
                    ->UCS4(matched_input, 1, matched_input.size() - 1);
                  return T_METADATA_REFERENCE;
                  break;
                case TOKEN_ID_METADATA_LABEL:
                  token_buffer_entry.value.text = (new XML_string)
                    ->UTF8(translated_input);
                  token_buffer_entry.type = token_type;
                  token_buffer.push_back(token_buffer_entry);
                  State(LEXER_STATE_PARAGRAPH);
                  return T_METADATA_BEGIN;
                  break;
                case TOKEN_ID_PARAGRAPH_TYPE:
                  yylval_p->text = (new XML_string)
                    ->UTF8(translated_input);
                  switch (token_type)
                    {
                    case T_CONTAINER_TYPE:
                    case T_CONTAINER_TYPE_LEVEL_0:
                    case T_CONTAINER_TYPE_LEVEL_1:
                    case T_CONTAINER_TYPE_LEVEL_2:
                    case T_CONTAINER_TYPE_LEVEL_3:
                    case T_CONTAINER_TYPE_LEVEL_4:
                    case T_CONTAINER_TYPE_LEVEL_5:
                      token_buffer_entry.value.text = yylval_p->text;
                      yylval_p->text = 0;
                      token_buffer_entry.type = T_PARAGRAPH_TYPE;
                      token_buffer.push_back(token_buffer_entry);
                      return T_PARAGRAPH_AS_CONTAINER_MARKER;
                      break;
                    case T_PARAGRAPH_TYPE:
                    case T_PARAGRAPH_TYPE_LEVEL_0:
                    case T_PARAGRAPH_TYPE_LEVEL_1:
                    case T_PARAGRAPH_TYPE_LEVEL_2:
                    case T_PARAGRAPH_TYPE_LEVEL_3:
                    case T_PARAGRAPH_TYPE_LEVEL_4:
                    case T_PARAGRAPH_TYPE_LEVEL_5:
                      if (LEXER_STATE_QMATH_DIRECTIVE == State())
                        {
                          cerr << "Unexpected paragraph type: "
                               << yylval_p->text << endl << flush;
                        }
                      State(LEXER_STATE_PARAGRAPH);
                      return T_PARAGRAPH_TYPE;
                      break;
                    case T_QMATH_DIRECTIVE:
                      if ("qmath_symbol_definition" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DEF_HEADER);
                          return T_DIRECTIVE_DEFINITION;
                        }
                      else if ("qmath_context_directive" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_CONTEXT;
                        }
                      else if ("qmath_directive_root_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_ROOT_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_mime_format" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_MIME_FORMAT;
                        }
                      else if ("qmath_directive_metadata_mime_format_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_METADATA_MIME_FORMAT_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_id_attribute_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_ID_ATTRIBUTE_NAME;
                        }
                      else if ("qmath_directive_paragraph_default_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_PARAGRAPH_DEFAULT_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_paragraph_text_container_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_PARAGRAPH_TEXT_CONTAINER_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_paragraph_formula_container_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_PARAGRAPH_FORMULA_CONTAINER_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_metadata_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_METADATA_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_metadata_title_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_METADATA_TITLE_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_metadata_author_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_METADATA_AUTHOR_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_metadata_date_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_METADATA_DATE_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_reference_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_REFERENCE_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_system_identifier" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_SYSTEM_IDENTIFIER;
                        }
                      else if ("qmath_directive_public_identifier" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_PUBLIC_IDENTIFIER;
                        }
                      else
                        {
                          string error_message;
                          error_message = "Undefined QMath directive '";
                          error_message += translated_input;
                          error_message += "'";
                          Report_error(error_message);
                          State(LEXER_STATE_QMATH);
                        }
                      return T_PARAGRAPH_TYPE;
                      break;
                    default:
                      Report_error("Unhandled token type as paragraph.");
                    }
                  break;
                case TOKEN_ID_REFERENCE_ID:
                  yylval_p->text = (new XML_string)
                    ->UCS4(matched_input, 3, matched_input.size() - 1);
                  State(LEXER_STATE_PARAGRAPH);
                  return T_REFERENCE_ID;
                  break;
                case TOKEN_ID_REFERENCE:
                  yylval_p->text = (new XML_string)
                    ->UCS4(matched_input, 1, matched_input.size() - 1);
                  return T_REFERENCE;
                  break;
                case TOKEN_ID_REFERENCE_OPEN:
                  yylval_p->text = (new XML_string)
                    ->UCS4(matched_input, 1, matched_input.size() - 1);
                  return T_REFERENCE_OPEN;
                  break;
                case TOKEN_ID_REFERENCE_CLOSE:
                  return T_REFERENCE_CLOSE;
                  break;
                case TOKEN_ID_TAG_OPEN:
                  yylval_p->text = new XML_string;
                  *(yylval_p->text) = translated_input;
                  return T_TAG_OPEN;
                  break;
                case TOKEN_ID_TAG_CLOSE:
                  return T_TAG_CLOSE;
                  break;
                case TOKEN_ID_PLAIN_CHARACTERS:
                  yylval_p->text = (new XML_string)->UCS4(matched_input);
                  return T_UCS4_CHAR_RUN;
                  break;
                case TOKEN_ID_SINGLE_CHARACTER:
                  yylval_p->character = matched_input.front();
                  State(LEXER_STATE_PARAGRAPH);
                  return T_UCS4_CHAR;
                  break;
                case TOKEN_ID_CHAR_STRING:
                  yylval_p->text = (new XML_string)
                    ->UCS4(matched_input, 1, matched_input.size() - 1);
                  State(LEXER_STATE_PARAGRAPH);
                  return T_STRING;
                  break;
                default:
                  yylval_p->text = (new XML_string)->UCS4(matched_input);
                  cerr << "Error in Get_next_token(): unhandled sequence '"
                       << yylval_p->text
                       << "' in state " << lexer_state << "." << endl;
                  return T_DEBUGGING_REPORT;
                  break;
                }
              break;
            case LEXER_STATE_XML:
              switch (longest_match_token_id)
                {
                case TOKEN_ID_XML_QMATH_PROCESSING_INSTRUCTION:
                  State_push(LEXER_STATE_QMATH);
                  return T_END_OF_LINE;
                  break;
                case TOKEN_ID_MATH_BEGIN:
                  if (LEXER_STATE_XML == State())
                    {
                      token_buffer_entry.value.text = yylval_p->text;
                      yylval_p->text = 0;
                      token_buffer_entry.type = T_MATH_BEGIN;
                      token_buffer.push_back(token_buffer_entry);
                      State_push(LEXER_STATE_MATH);
                      return T_EMBEDDED_MARKER;
                    }
                  State_push(LEXER_STATE_MATH);
                  return T_MATH_BEGIN;
                  break;
                case TOKEN_ID_EOL:
                  if (LEXER_STATE_XML == State())
                    {
                      token_buffer_entry.value.text = yylval_p->text;
                      yylval_p->text = 0;
                      token_buffer_entry.type = T_END_OF_LINE;
                      token_buffer.push_back(token_buffer_entry);
                      return T_EMBEDDED_MARKER;
                    }
                  return T_END_OF_LINE;
                  break;
                case TOKEN_ID_XML_QMATH_PROCESSING_INSTRUCTION_EOL:
                  State_pop();// This should bring us back to the XML state.
                  if (LEXER_STATE_XML != State())
                    {
                      cerr << "Unexpected state after PI end: "
                           << State() << endl << flush;
                    }
                  return T_NEW_PARAGRAPH;
                  break;
                case TOKEN_ID_PLAIN_CHARACTERS:
                  yylval_p->text = (new XML_string)->UCS4(matched_input);
                  token_buffer_entry.value.text = yylval_p->text;
                  yylval_p->text = 0;
                  token_buffer_entry.type = T_UCS4_CHAR_RUN;
                  token_buffer.push_back(token_buffer_entry);
                  return T_EMBEDDED_MARKER;
                  break;
                case TOKEN_ID_SINGLE_CHARACTER:
                  yylval_p->character = matched_input.front();
                  token_buffer_entry.value.character = yylval_p->character;
                  token_buffer_entry.type = T_UCS4_CHAR;
                  token_buffer.push_back(token_buffer_entry);
                  return T_EMBEDDED_MARKER;
                  break;
                default:
                  yylval_p->text = (new XML_string)->UCS4(matched_input);
                  cerr << "Error in Get_next_token(): unhandled sequence '"
                       << yylval_p->text
                       << "' in state " << lexer_state << "." << endl;
                  return T_DEBUGGING_REPORT;
                  break;
                }
              break;
            case LEXER_STATE_QMATH_DEF_HEADER:
              switch (longest_match_token_id)
                {
                case TOKEN_ID_EOL:
                  State(LEXER_STATE_QMATH_DEF_COMPLETE);
                  break;
                case TOKEN_ID_SEPARATOR_SPACE:
                  State(LEXER_STATE_QMATH_DEF_SIMPLE);
                  break;
                case TOKEN_ID_IDENTIFIER:
                  yylval_p->text = (new XML_string)
                    ->UCS4(matched_input);
                  State(LEXER_STATE_QMATH_DEF_SIMPLE);
                  return T_IDENTIFIER;
                  break;
                case TOKEN_ID_REFERENCE_ID:
                  yylval_p->text = (new XML_string)
                    ->UCS4(matched_input, 3, matched_input.size() - 1);
                  return T_REFERENCE_ID;
                  break;
                default:
                  yylval_p->text = (new XML_string)->UCS4(matched_input);
                  cerr << "Error in Get_next_token(): unhandled sequence '"
                       << yylval_p->text
                       << "' in state " << lexer_state << "." << endl;
                  return T_DEBUGGING_REPORT;
                  break;
                }
              break;
            case LEXER_STATE_QMATH_DEF_SIMPLE:
              switch (longest_match_token_id)
                {
                case TOKEN_ID_IDENTIFIER:
                  yylval_p->text = (new XML_string)
                    ->UCS4(matched_input);
                  return T_IDENTIFIER;
                  break;
                case TOKEN_ID_DEF_TYPE_SPECIFICATION:
                  yylval_p->text = (new XML_string)
                    ->UCS4(matched_input);
                  return T_SYMBOL_DEFINITION_TYPE;
                  break;
                case TOKEN_ID_SEPARATOR_COMMA:
                  break;
                case TOKEN_ID_CHAR_STRING:
                  yylval_p->text = (new XML_string)
                    ->UCS4(matched_input, 1, matched_input.size() - 1);
                  token_buffer_entry.type = T_NEW_PARAGRAPH;
                  token_buffer.push_back(token_buffer_entry);
                  State(LEXER_STATE_QMATH);
                  return T_STRING;
                  break;
                case TOKEN_ID_EOL:
                  break;
                case TOKEN_ID_SEPARATOR_SPACE:
                  break;
                case TOKEN_ID_COMMENT:
                  break;
                case TOKEN_ID_PARAGRAPH_TYPE:
                  yylval_p->text = (new XML_string)
                    ->UTF8(translated_input);
                  switch (token_type)
                    {
                    case T_CONTAINER_TYPE:
                    case T_CONTAINER_TYPE_LEVEL_0:
                    case T_CONTAINER_TYPE_LEVEL_1:
                    case T_CONTAINER_TYPE_LEVEL_2:
                    case T_CONTAINER_TYPE_LEVEL_3:
                    case T_CONTAINER_TYPE_LEVEL_4:
                    case T_CONTAINER_TYPE_LEVEL_5:
                      token_buffer_entry.value.text = yylval_p->text;
                      yylval_p->text = 0;
                      token_buffer_entry.type = T_PARAGRAPH_TYPE;
                      token_buffer.push_back(token_buffer_entry);
                      return T_PARAGRAPH_AS_CONTAINER_MARKER;
                      break;
                    case T_PARAGRAPH_TYPE:
                    case T_PARAGRAPH_TYPE_LEVEL_0:
                    case T_PARAGRAPH_TYPE_LEVEL_1:
                    case T_PARAGRAPH_TYPE_LEVEL_2:
                    case T_PARAGRAPH_TYPE_LEVEL_3:
                    case T_PARAGRAPH_TYPE_LEVEL_4:
                    case T_PARAGRAPH_TYPE_LEVEL_5:
                      if (LEXER_STATE_QMATH_DIRECTIVE == State())
                        {
                          cerr << "Unexpected paragraph type: "
                               << yylval_p->text << endl << flush;
                        }
                      State(LEXER_STATE_PARAGRAPH);
                      return T_PARAGRAPH_TYPE;
                      break;
                    case T_QMATH_DIRECTIVE:
                      if ("qmath_symbol_definition" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DEF_HEADER);
                          return T_DIRECTIVE_DEFINITION;
                        }
                      else if ("qmath_context_directive" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_CONTEXT;
                        }
                      else if ("qmath_directive_root_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_ROOT_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_mime_format" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_MIME_FORMAT;
                        }
                      else if ("qmath_directive_metadata_mime_format_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_METADATA_MIME_FORMAT_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_id_attribute_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_ID_ATTRIBUTE_NAME;
                        }
                      else if ("qmath_directive_paragraph_default_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_PARAGRAPH_DEFAULT_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_paragraph_text_container_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_PARAGRAPH_TEXT_CONTAINER_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_paragraph_formula_container_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_PARAGRAPH_FORMULA_CONTAINER_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_metadata_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_METADATA_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_metadata_title_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_METADATA_TITLE_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_metadata_author_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_METADATA_AUTHOR_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_metadata_date_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_METADATA_DATE_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_reference_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_REFERENCE_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_system_identifier" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_SYSTEM_IDENTIFIER;
                        }
                      else if ("qmath_directive_public_identifier" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_PUBLIC_IDENTIFIER;
                        }
                      else
                        {
                          string error_message;
                          error_message = "Undefined QMath directive '";
                          error_message += translated_input;
                          error_message += "'";
                          Report_error(error_message);
                          State(LEXER_STATE_QMATH);
                        }
                      return T_PARAGRAPH_TYPE;
                      break;
                    default:
                      Report_error("Unhandled token type as paragraph.");
                    }
                  break;
                case TOKEN_ID_PARAGRAPH_SEPARATOR:
                  State(LEXER_STATE_QMATH);
                  return T_NEW_PARAGRAPH;
                  break;
                default:
                  yylval_p->text = (new XML_string)->UCS4(matched_input);
                  cerr << "Error in Get_next_token(): unhandled sequence '"
                       << yylval_p->text
                       << "' in state " << lexer_state << "." << endl;
                  return T_DEBUGGING_REPORT;
                  break;
                }
              break;
            case LEXER_STATE_QMATH_DEF_COMPLETE:
              switch (longest_match_token_id)
                {
                case TOKEN_ID_IDENTIFIER:
                  yylval_p->text = (new XML_string)
                    ->UCS4(matched_input);
                  return T_IDENTIFIER;
                  break;
                case TOKEN_ID_DEF_TYPE_SPECIFICATION:
                  yylval_p->text = (new XML_string)
                    ->UCS4(matched_input);
                  return T_SYMBOL_DEFINITION_TYPE;
                  break;
                case TOKEN_ID_SEPARATOR_COMMA:
                  break;
                case TOKEN_ID_CHAR_STRING:
                  yylval_p->text = (new XML_string)
                    ->UCS4(matched_input, 1, matched_input.size() - 1);
                  return T_STRING;
                  break;
                case TOKEN_ID_EOL:
                  break;
                case TOKEN_ID_SEPARATOR_SPACE:
                  break;
                case TOKEN_ID_COMMENT:
                  break;
                case TOKEN_ID_DEF_PARAMETER_LABELLED:
                  yylval_p->text = (new XML_string)
                    ->UCS4(matched_input, 0, matched_input.size() - 1);
                  return T_PARAMETER_LABEL;
                  break;
                case TOKEN_ID_PARAGRAPH_SEPARATOR:
                  State(LEXER_STATE_QMATH);
                  return T_NEW_PARAGRAPH;
                  break;
                default:
                  yylval_p->text = (new XML_string)->UCS4(matched_input);
                  cerr << "Error in Get_next_token(): unhandled sequence '"
                       << yylval_p->text
                       << "' in state " << lexer_state << "." << endl;
                  return T_DEBUGGING_REPORT;
                  break;
                }
              break;
            case LEXER_STATE_QMATH_DIRECTIVE:
              switch (longest_match_token_id)
                {
                case TOKEN_ID_CHAR_STRING:
                  if (matched_input.size() > 2)
                    {
                      yylval_p->text = (new XML_string)
                        ->UCS4(matched_input,
                               1, matched_input.size() - 1);
                    }
                  else
                    {
                      yylval_p->text = 0;
                    }
                  token_buffer_entry.type = T_NEW_PARAGRAPH;
                  token_buffer.push_back(token_buffer_entry);
                  State(LEXER_STATE_QMATH);
                  return T_STRING;
                  break;
                case TOKEN_ID_EOL:
                  break;
                case TOKEN_ID_SEPARATOR_SPACE:
                  break;
                case TOKEN_ID_COMMENT:
                  break;
                case TOKEN_ID_PARAGRAPH_TYPE:
                  yylval_p->text = (new XML_string)
                    ->UTF8(translated_input);
                  switch (token_type)
                    {
                    case T_CONTAINER_TYPE:
                    case T_CONTAINER_TYPE_LEVEL_0:
                    case T_CONTAINER_TYPE_LEVEL_1:
                    case T_CONTAINER_TYPE_LEVEL_2:
                    case T_CONTAINER_TYPE_LEVEL_3:
                    case T_CONTAINER_TYPE_LEVEL_4:
                    case T_CONTAINER_TYPE_LEVEL_5:
                      token_buffer_entry.value.text = yylval_p->text;
                      yylval_p->text = 0;
                      token_buffer_entry.type = T_PARAGRAPH_TYPE;
                      token_buffer.push_back(token_buffer_entry);
                      return T_PARAGRAPH_AS_CONTAINER_MARKER;
                      break;
                    case T_PARAGRAPH_TYPE:
                    case T_PARAGRAPH_TYPE_LEVEL_0:
                    case T_PARAGRAPH_TYPE_LEVEL_1:
                    case T_PARAGRAPH_TYPE_LEVEL_2:
                    case T_PARAGRAPH_TYPE_LEVEL_3:
                    case T_PARAGRAPH_TYPE_LEVEL_4:
                    case T_PARAGRAPH_TYPE_LEVEL_5:
                      if (LEXER_STATE_QMATH_DIRECTIVE == State())
                        {
                          cerr << "Unexpected paragraph type: "
                               << yylval_p->text << endl << flush;
                        }
                      State(LEXER_STATE_PARAGRAPH);
                      return T_PARAGRAPH_TYPE;
                      break;
                    case T_QMATH_DIRECTIVE:
                      if ("qmath_symbol_definition" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DEF_HEADER);
                          return T_DIRECTIVE_DEFINITION;
                        }
                      else if ("qmath_context_directive" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_CONTEXT;
                        }
                      else if ("qmath_directive_root_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_ROOT_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_mime_format" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_MIME_FORMAT;
                        }
                      else if ("qmath_directive_metadata_mime_format_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_METADATA_MIME_FORMAT_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_id_attribute_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_ID_ATTRIBUTE_NAME;
                        }
                      else if ("qmath_directive_paragraph_default_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_PARAGRAPH_DEFAULT_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_paragraph_text_container_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_PARAGRAPH_TEXT_CONTAINER_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_paragraph_formula_container_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_PARAGRAPH_FORMULA_CONTAINER_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_metadata_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_METADATA_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_metadata_title_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_METADATA_TITLE_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_metadata_author_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_METADATA_AUTHOR_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_metadata_date_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_METADATA_DATE_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_reference_element_name" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_REFERENCE_ELEMENT_NAME;
                        }
                      else if ("qmath_directive_system_identifier" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_SYSTEM_IDENTIFIER;
                        }
                      else if ("qmath_directive_public_identifier" == translated_input)
                        {
                          State(LEXER_STATE_QMATH_DIRECTIVE);
                          return T_DIRECTIVE_PUBLIC_IDENTIFIER;
                        }
                      else
                        {
                          string error_message;
                          error_message = "Undefined QMath directive '";
                          error_message += translated_input;
                          error_message += "'";
                          Report_error(error_message);
                          State(LEXER_STATE_QMATH);
                        }
                      return T_PARAGRAPH_TYPE;
                      break;
                    default:
                      Report_error("Unhandled token type as paragraph.");
                    }
                  break;
                case TOKEN_ID_PARAGRAPH_SEPARATOR:
                  State(LEXER_STATE_QMATH);
                  return T_NEW_PARAGRAPH;
                  break;
                default:
                  yylval_p->text = (new XML_string)->UCS4(matched_input);
                  cerr << "Error in Get_next_token(): unhandled sequence '"
                       << yylval_p->text
                       << "' in state " << lexer_state << "." << endl;
                  return T_DEBUGGING_REPORT;
                  break;
                }
              break;
            case LEXER_STATE_PARAGRAPH:
              switch (longest_match_token_id)
                {
                case TOKEN_ID_START_DEBUGGING:
                  debug = true;
                  return T_START_DEBUGGING;
                  break;
                case TOKEN_ID_STOP_DEBUGGING:
                  debug = false;
                  return T_STOP_DEBUGGING;
                  break;
                case TOKEN_ID_DEBUGGING_REPORT:
                  yylval_p->text = (new XML_string)->UCS4(matched_input);
                  return T_DEBUGGING_REPORT;
                  break;
                case TOKEN_ID_COMMENT:
                  break;
                case TOKEN_ID_MATH_BEGIN:
                  if (LEXER_STATE_XML == State())
                    {
                      token_buffer_entry.value.text = yylval_p->text;
                      yylval_p->text = 0;
                      token_buffer_entry.type = T_MATH_BEGIN;
                      token_buffer.push_back(token_buffer_entry);
                      State_push(LEXER_STATE_MATH);
                      return T_EMBEDDED_MARKER;
                    }
                  State_push(LEXER_STATE_MATH);
                  return T_MATH_BEGIN;
                  break;
                case TOKEN_ID_EOL:
                  if (LEXER_STATE_XML == State())
                    {
                      token_buffer_entry.value.text = yylval_p->text;
                      yylval_p->text = 0;
                      token_buffer_entry.type = T_END_OF_LINE;
                      token_buffer.push_back(token_buffer_entry);
                      return T_EMBEDDED_MARKER;
                    }
                  return T_END_OF_LINE;
                  break;
                case TOKEN_ID_XML_QMATH_PROCESSING_INSTRUCTION_EOL:
                  State_pop();// This should bring us back to the XML state.
                  if (LEXER_STATE_XML != State())
                    {
                      cerr << "Unexpected state after PI end: "
                           << State() << endl << flush;
                    }
                  return T_NEW_PARAGRAPH;
                  break;
                case TOKEN_ID_PARAGRAPH_SEPARATOR:
                  State(LEXER_STATE_QMATH);
                  return T_NEW_PARAGRAPH;
                  break;
                case TOKEN_ID_TITLE:
                  token_buffer_entry.value.text = (new XML_string)
                    ->UCS4(matched_input, 2, matched_input.size() - 1);
                  token_buffer_entry.type = T_METADATA_TITLE;
                  token_buffer.push_back(token_buffer_entry);
                  State(LEXER_STATE_PARAGRAPH);
                  return T_METADATA_BEGIN;
                  break;
                case TOKEN_ID_AUTHOR:
                  token_buffer_entry.value.text = (new XML_string)
                    ->UCS4(matched_input, 1, matched_input.size());
                  token_buffer_entry.type = T_METADATA_AUTHOR;
                  token_buffer.push_back(token_buffer_entry);
                  State(LEXER_STATE_PARAGRAPH);
                  return T_METADATA_BEGIN;
                  break;
                case TOKEN_ID_DATE:
                  for (vector<XML_UCS4_char_t>::iterator
                         character_iterator = matched_input.begin();
                       character_iterator < matched_input.end();
                       ++character_iterator)
                    if (XML_UCS4_char_t(' ') == *character_iterator)
                      *character_iterator = XML_UCS4_char_t('T');
                  token_buffer_entry.value.text = (new XML_string)
                    ->UCS4(matched_input, 1, matched_input.size());
                  token_buffer_entry.type = T_METADATA_DATE;
                  token_buffer.push_back(token_buffer_entry);
                  State(LEXER_STATE_PARAGRAPH);
                  return T_METADATA_BEGIN;
                  break;
                case TOKEN_ID_LANG:
                  token_buffer_entry.value.text = (new XML_string)
                    ->UCS4(matched_input, 1, matched_input.size());
                  token_buffer_entry.type = T_METADATA_LANG;
                  token_buffer.push_back(token_buffer_entry);
                  if (Language())
                    {
                      State(LEXER_STATE_PARAGRAPH);
                    }
                  else
                    {
                      bool success;
                      Language(*token_buffer_entry.value.text);
                      // Load basic context for language:
                      success = Input_stream("", Tokenizer::LEXER_STATE_QMATH);
                      if (!success)
                        {
                          string message;
                          message = "Can't load basic context file."
                            " Is the environment variable QMATH_HOME='";
                          message += getenv("QMATH_HOME");
                          message += "' correctly set?";
                          Report_error(message.c_str());
                          return T_DOCUMENT_END;
                        }
                      token_buffer_entry.value.markup = 0;
                      token_buffer_entry.type = T_NEW_PARAGRAPH;
                      token_buffer.push_back(token_buffer_entry);
                      State(LEXER_STATE_QMATH);
                    }
                  return T_METADATA_BEGIN;
                  break;
                case TOKEN_ID_METADATA_REFERENCE:
                  yylval_p->text = (new XML_string)
                    ->UCS4(matched_input, 1, matched_input.size() - 1);
                  return T_METADATA_REFERENCE;
                  break;
                case TOKEN_ID_METADATA_LABEL:
                  token_buffer_entry.value.text = (new XML_string)
                    ->UTF8(translated_input);
                  token_buffer_entry.type = token_type;
                  token_buffer.push_back(token_buffer_entry);
                  State(LEXER_STATE_PARAGRAPH);
                  return T_METADATA_BEGIN;
                  break;
                case TOKEN_ID_REFERENCE_ID:
                  yylval_p->text = (new XML_string)
                    ->UCS4(matched_input, 3, matched_input.size() - 1);
                  return T_REFERENCE_ID;
                  break;
                case TOKEN_ID_PLAIN_CHARACTERS:
                  yylval_p->text = (new XML_string)->UCS4(matched_input);
                  return T_UCS4_CHAR_RUN;
                  break;
                case TOKEN_ID_SINGLE_CHARACTER:
                  yylval_p->character = matched_input.front();
                  State(LEXER_STATE_PARAGRAPH);
                  return T_UCS4_CHAR;
                  break;
                case TOKEN_ID_CHAR_STRING:
                  yylval_p->text = (new XML_string)
                    ->UCS4(matched_input, 1, matched_input.size() - 1);
                  State(LEXER_STATE_PARAGRAPH);
                  return T_STRING;
                  break;
                case TOKEN_ID_REFERENCE:
                  yylval_p->text = (new XML_string)
                    ->UCS4(matched_input, 1, matched_input.size() - 1);
                  return T_REFERENCE;
                  break;
                case TOKEN_ID_REFERENCE_OPEN:
                  yylval_p->text = (new XML_string)
                    ->UCS4(matched_input, 1, matched_input.size() - 1);
                  return T_REFERENCE_OPEN;
                  break;
                case TOKEN_ID_REFERENCE_CLOSE:
                  return T_REFERENCE_CLOSE;
                  break;
                case TOKEN_ID_TAG_OPEN:
                  yylval_p->text = new XML_string;
                  *(yylval_p->text) = translated_input;
                  return T_TAG_OPEN;
                  break;
                case TOKEN_ID_TAG_CLOSE:
                  return T_TAG_CLOSE;
                  break;
                case TOKEN_ID_PARAGRAPH_SECTION_MARKER:
                  yylval_p->text = (new XML_string)->UCS4(matched_input);
                  if ("qmath_paragraph_section_FMP" == translated_input)
                    {
                      return T_PARAGRAPH_SECTION_MARKER_FMP;
                    }
                  return T_UCS4_CHAR_RUN;
                  break;
                default:
                  yylval_p->text = (new XML_string)->UCS4(matched_input);
                  cerr << "Error in Get_next_token(): unhandled sequence '"
                       << yylval_p->text
                       << "' in state " << lexer_state << "." << endl;
                  return T_DEBUGGING_REPORT;
                  break;
                }
              break;
            case LEXER_STATE_MATH:
              switch (longest_match_token_id)
                {
                case TOKEN_ID_MATH_END:
                  State_pop();
                  return T_MATH_END;
                  break;
                case TOKEN_ID_INT_NUMBER:
                  yylval_p->OM_object = (new OpenMath_object)
                    ->Integer(translated_input);
                  return T_INT_NUM;
                  break;
                case TOKEN_ID_FP_NUMBER:
                  yylval_p->OM_object = (new OpenMath_object)
                    ->Fp_number(translated_input);
                  return T_FP_NUM;
                  break;
                case TOKEN_ID_CHAR_STRING:
                  yylval_p->OM_object = (new OpenMath_object)
                    ->Char_string(translated_input);
                  return T_STR;
                  break;
                case TOKEN_ID_SEPARATOR_SPACE:
                  ;// Eat blanks.
                  break;
                case TOKEN_ID_SEPARATOR_COMMA:
                  return T_SEP_COMMA;
                  break;
                case TOKEN_ID_SEPARATOR_COLON:
                  return T_SEP_COLON;
                  break;
                case TOKEN_ID_SEPARATOR_SEMICOLON:
                  return T_SEP_SCOLON;
                  break;
                case TOKEN_ID_DOUBLE_QUOTE:
                  return T_DOUBLE_QUOTE;
                  break;
                case TOKEN_ID_SINGLE_QUOTE:
                  return T_SINGLE_QUOTE;
                  break;
                case TOKEN_ID_BACK_QUOTE:
                  return T_BACK_QUOTE;
                  break;
                case TOKEN_ID_EOL:
                  if (LEXER_STATE_XML == State())
                    {
                      token_buffer_entry.value.text = yylval_p->text;
                      yylval_p->text = 0;
                      token_buffer_entry.type = T_END_OF_LINE;
                      token_buffer.push_back(token_buffer_entry);
                      return T_EMBEDDED_MARKER;
                    }
                  return T_END_OF_LINE;
                  break;
                case TOKEN_ID_XML_QMATH_PROCESSING_INSTRUCTION_EOL:
                  State_pop();// This should bring us back to the XML state.
                  if (LEXER_STATE_XML != State())
                    {
                      cerr << "Unexpected state after PI end: "
                           << State() << endl << flush;
                    }
                  return T_NEW_PARAGRAPH;
                  break;
                case TOKEN_ID_FENCE_OPEN:
                  yylval_p->character = matched_input.front();
                  return T_FENCE_OPEN;
                  break;
                case TOKEN_ID_FENCE_CLOSE:
                  yylval_p->character = matched_input.front();
                  return T_FENCE_CLOSE;
                  break;
                case TOKEN_ID_HASH:
                  return T_HASH;
                  break;
                case TOKEN_ID_SPACE:
                  return T_SPACE;
                  break;
                case TOKEN_ID_TAB:
                  return T_TAB;
                  break;
                case TOKEN_ID_PARAGRAPH_SEPARATOR:
                  State(LEXER_STATE_QMATH);
                  return T_NEW_PARAGRAPH;
                  break;
                case TOKEN_ID_IDENTIFIER:
                  switch (token_type)
                    {
                    case T_SYMBOL:
                    case T_APPLICATION:
                    case T_ATTRIBUTION:
                    case T_BINDING:
                    case T_OP_IMPL:
                    case T_OP_OR:
                    case T_OP_AND:
                    case T_OP_EQ:
                    case T_OP_PLUS:
                    case T_OP_PROD:
                    case T_OP_EXP:
                    case T_OP_FACT:
                    case T_OP_BIND:
                      yylval_p->OM_object = (new OpenMath_object)
                        ->Symbol(translated_input);
                      return token_type;
                      break;
                    case T_VARIABLE:
                      yylval_p->OM_object = (new OpenMath_object)
                        ->Variable(translated_input);
                      return token_type;
                      break;
                    }
                  break;
                default:
                  yylval_p->text = (new XML_string)->UCS4(matched_input);
                  cerr << "Error in Get_next_token(): unhandled sequence '"
                       << yylval_p->text
                       << "' in state " << lexer_state << "." << endl;
                  return T_DEBUGGING_REPORT;
                  break;
                }
              break;
            default:
              yylval_p->text = (new XML_string)->UCS4(matched_input);
              cerr << "Error in Get_next_token(): unhandled state with '"
                   << yylval_p->text
                   << "' in state " << lexer_state << "." << endl;
              return T_DEBUGGING_REPORT;
              break;
            }// switch (lexer_state) ...
        }// if (longest_match_token_id == ~0) ... else ...
      // We get here when there haven't been a return statement. So far, this
      // only occurs in LEXER_STATE_INITIAL, when we can't match
      // TOKEN_ID_QMATH_DOC_BEGIN. In this case, we keep advancing until match.
      // If we wanted to end parsing instead, we could use:
      //return T_DOCUMENT_END;
    }// while (!document_finished) ...
  
  if (debug)
    cerr << "exit from loop while(!document_finished)."
         << endl << flush;
  
  return T_DOCUMENT_END;
}

/////////////
// Error_count - Tell count of reported errors.
// no arguments.
// returns current error count.
//public:
unsigned int Tokenizer::Error_count()
{
  return error_count;
}


/////////////
// Tab_stops - Get tab stops stack.
// no arguments.
// returns a copy of the tab stops stack. Indentation is in characters, so "HORIZONTAL TABULATION" counts as 1.
//public:
vector<Tab_stop_t>* Tokenizer::Tab_stops()
{
  return &tab_stops;
}

/////////////
// Tab_stops - Get tab stop at index.
// no arguments.
// returns a tab stop. Indentation is in characters, so "HORIZONTAL TABULATION" counts as 1.
//public:
Tab_stop_p_t Tokenizer::Tab_stops(int index)
{
  if (index+1 > int(tab_stops.size()))
    return &tab_stops.back();
  if (index < 0)
    return &tab_stops[0];
  
  return &tab_stops[index];
}


using std::cout;
using std::cin;
using std::endl;
using std::flush;
using std::string;
using std::vector;

/////////////
// Report_debug - Output debug message.
// message: debug message.
// returns nothing.
//public:
void Tokenizer::Report_debug(const char* message)
{
  cerr << Current_file_name() << ":"
       << Current_line_number() << ":"
       << " debug: "
       << message
       << ", at column " << Current_column_number() << "."
       << endl << flush;
}

/////////////
// Report_debug - Output debug message.
// message: debug message.
// returns nothing.
//public:
void Tokenizer::Report_debug(string message)
{
  cerr << Current_file_name() << ":"
       << Current_line_number() << ":"
       << " debug: "
       << message
       << ", at column " << Current_column_number() << "."
       << endl << flush;
}

/////////////
// Report_debug - Output debug message.
// message: debug message.
// returns nothing.
//public:
void Tokenizer::Report_debug(XML_string_t message)
{
  cerr << Current_file_name() << ":"
       << Current_line_number() << ":"
       << " debug: "
       << message
       << ", at column " << Current_column_number() << "."
       << endl << flush;
}

/////////////
// Report_warning - Output warning message.
// message: warning message.
// returns nothing.
//public:
void Tokenizer::Report_warning(const char* message)
{
  cerr << Current_file_name() << ":"
       << Current_line_number() << ":"
       << " warning: "
       << message
       << ", at column " << Current_column_number() << "."
       << endl << flush;
}

/////////////
// Report_warning - Output warning message.
// message: warning message.
// returns nothing.
//public:
void Tokenizer::Report_warning(string message)
{
  cerr << Current_file_name() << ":"
       << Current_line_number() << ":"
       << " warning: "
       << message
       << ", at column " << Current_column_number() << "."
       << endl << flush;
}

/////////////
// Report_warning - Output warning message.
// message: warning message.
// returns nothing.
//public:
void Tokenizer::Report_warning(XML_string_t message)
{
  cerr << Current_file_name() << ":"
       << Current_line_number() << ":"
       << " warning: "
       << message
       << ", at column " << Current_column_number() << "."
       << endl << flush;
}

/////////////
// Report_error - Output error message.
// message: error message.
// returns nothing.
//public:
void Tokenizer::Report_error(const char* message)
{
  cerr << Current_file_name() << ":"
       << Current_line_number() << ":"
       << " error: "
       << message
       << ", at column " << Current_column_number() << "."
       << endl << flush;
  ++error_count;
}

/////////////
// Report_error - Output error message.
// message: error message.
// returns nothing.
//public:
void Tokenizer::Report_error(string message)
{
  cerr << Current_file_name() << ":"
       << Current_line_number() << ":"
       << " error: "
       << message
       << ", at column " << Current_column_number() << "."
       << endl << flush;
  ++error_count;
}

/////////////
// Report_error - Output error message.
// message: error message.
// returns nothing.
//public:
void Tokenizer::Report_error(XML_string_t message)
{
  cerr << Current_file_name() << ":"
       << Current_line_number() << ":"
       << " error: "
       << message
       << ", at column " << Current_column_number() << "."
       << endl << flush;
  ++error_count;
}


/////////////
// Increment_current_column_number - Increment current column number.
// no arguments.
// returns column number, counting from 0, or 0 if error.
//public:
int Tokenizer::Increment_current_column_number()
{
  if (input_streams_stack.size() > 0)
    {
      ++input_streams_stack.back().current_input_column_number;
      return input_streams_stack.back().current_input_column_number;
    }
  else
    return 0;
}

/////////////
// Increment_current_column_number - Increment current column number.
// count: number of columns to add.
// returns column number, counting from 0, or 0 if error.
//public:
int Tokenizer::Increment_current_column_number(unsigned int count)
{
  if (input_streams_stack.size() > 0)
    {
      input_streams_stack.back().current_input_column_number += count;
      return input_streams_stack.back().current_input_column_number;
    }
  else
    return 0;
}


/////////////
// Embedded_mode - Tell whether the lexer detected an embedded grammar or not.
// returns true if the QMath expression grammar was used embedded in an XML document, and false otherwise.
//public:
bool Tokenizer::Embedded_mode()
{
  return embedded_mode;
}
using std::cout;
using std::cin;
using std::endl;
using std::flush;

/////////////
// Append_translation_entry - Append an input token translation table entry.
// no arguments.
// returns parsed token type on success, 0 (EOF) on fail.
//public:
int Tokenizer::Append_translation_entry(XML_string_t &context_name, XML_string_t &token, XML_string_t &token_type, XML_string_t &translation)
{
  if (debug)
    {
      cerr << "+++ Appending translation for " << token
           << ", with token type " << token_type
           << ": " << translation << endl;
    }
  Translation_table_entry_t entry;
  entry.context_name = context_name;
  translation.UTF8(&(entry.openmath_symbol));
  if ("OP_BIND" == token_type)
    entry.token_type = T_OP_BIND;
  else if ("OP_FACT" == token_type)
    entry.token_type = T_OP_FACT;
  else if ("OP_EXP" == token_type)
    entry.token_type = T_OP_EXP;
  else if ("OP_PROD" == token_type)
    entry.token_type = T_OP_PROD;
  else if ("OP_PLUS" == token_type)
    entry.token_type = T_OP_PLUS;
  else if ("OP_EQ" == token_type)
    entry.token_type = T_OP_EQ;
  else if ("OP_AND" == token_type)
    entry.token_type = T_OP_AND;
  else if ("OP_OR" == token_type)
    entry.token_type = T_OP_OR;
  else if ("OP_IMPL" == token_type)
    entry.token_type = T_OP_IMPL;
  else if ("VARIABLE" == token_type)
    entry.token_type = T_VARIABLE;
  else if ("SYMBOL" == token_type)
    entry.token_type = T_SYMBOL;
  else if ("BINDING" == token_type)
    entry.token_type = T_BINDING;
  else if ("ATTRIBUTION" == token_type)
    entry.token_type = T_ATTRIBUTION;
  else if ("APPLICATION" == token_type)
    entry.token_type = T_APPLICATION;
  else if ("METADATA" == token_type)
    entry.token_type = T_METADATA;
  else if ("PARAGRAPH_SECTION_MARKER" == token_type)
    entry.token_type = T_PARAGRAPH_SECTION_MARKER;
  else if ("CONTAINER_TYPE" == token_type)
    entry.token_type = T_CONTAINER_TYPE;
  else if ("PARAGRAPH_TYPE" == token_type)
    entry.token_type = T_PARAGRAPH_TYPE;
  else if ("QMATH_DIRECTIVE" == token_type)
    entry.token_type = T_QMATH_DIRECTIVE;
  else if ("PARAGRAPH_TYPE_LEVEL_0" == token_type)
    entry.token_type = T_PARAGRAPH_TYPE_LEVEL_0;
  else if ("CONTAINER_TYPE_LEVEL_0" == token_type)
    entry.token_type = T_CONTAINER_TYPE_LEVEL_0;
  else if ("PARAGRAPH_TYPE_LEVEL_1" == token_type)
    entry.token_type = T_PARAGRAPH_TYPE_LEVEL_1;
  else if ("CONTAINER_TYPE_LEVEL_1" == token_type)
    entry.token_type = T_CONTAINER_TYPE_LEVEL_1;
  else if ("PARAGRAPH_TYPE_LEVEL_2" == token_type)
    entry.token_type = T_PARAGRAPH_TYPE_LEVEL_2;
  else if ("CONTAINER_TYPE_LEVEL_2" == token_type)
    entry.token_type = T_CONTAINER_TYPE_LEVEL_2;
  else if ("PARAGRAPH_TYPE_LEVEL_3" == token_type)
    entry.token_type = T_PARAGRAPH_TYPE_LEVEL_3;
  else if ("CONTAINER_TYPE_LEVEL_3" == token_type)
    entry.token_type = T_CONTAINER_TYPE_LEVEL_3;
  else if ("PARAGRAPH_TYPE_LEVEL_4" == token_type)
    entry.token_type = T_PARAGRAPH_TYPE_LEVEL_4;
  else if ("CONTAINER_TYPE_LEVEL_4" == token_type)
    entry.token_type = T_CONTAINER_TYPE_LEVEL_4;
  else if ("PARAGRAPH_TYPE_LEVEL_5" == token_type)
    entry.token_type = T_PARAGRAPH_TYPE_LEVEL_5;
  else if ("CONTAINER_TYPE_LEVEL_5" == token_type)
    entry.token_type = T_CONTAINER_TYPE_LEVEL_5;
  else
    {
      cerr << "Error: QMath: Invalid token type '" << token_type << "'"
           << endl << flush;
      return 0;
    }
  
  entry.ucs4_string = token;
  
  if (entry.openmath_symbol.size() > 0 && entry.ucs4_string.UCS4_length() > 0)
    translation_table.push_back(entry);
  else
    return 0;
  
  return entry.token_type;
}



/////////////
// Input_UCS4_from_UTF8_stream - Get UCS4 character from an UTF-8 stream.
// input_stream: Stream to read UTF-8 encoded char from.
// p_character: Pointer to a XML_UCS4_char_t to receive the UCS4 code
//        point. On error, it's set to 0x0000FFFD (REPLACEMENT CHAR), as
//        recommended in the Unicode FAQ.
// returns true if there are still characters to read, false otherwise
//        (e.g. EOF or error).
//public:
//static
bool Tokenizer::Input_UCS4_from_UTF8_stream(istream &input_stream, XML_UCS4_char_p_t p_character)
{
  if (!p_character) return false;
  if (!(&input_stream))
    {
      // We don't set "*p_character".
      return false;
    }
  
  int run_length = 0;
  int correct_run_length = 1;// If 0, it would match run_length and return.
  XML_UCS4_char_t ucs4_char = 0;
  char utf8_byte;
  
  while (!input_stream.eof())
    {
      input_stream.get(utf8_byte);
      if (input_stream.eof())
        {
          // If run_length is 0, we haven't yet begun a sequence, so we just
          // report EOF. But and EOF at the middle of a sequence is considered
          // a UTF-8 encoding error.
          if (0 == run_length)
            correct_run_length = 0;
          break;// Out of: while (!input_stream.eof()) ...
        }
      
      // Normalize EOL characters.
      // \n 0x0A LF ^C-j LINE FEED
      // \r 0x0D CR ^C-m CARRIAGE RETURN
      // Unix:   LF
      // Mac:    CR
      // MS-DOS: CRLF
      if (0x0D == utf8_byte)
        {
          if (input_stream.peek() == 0x0000000A)
            continue;// It's safe to just drop this character since no valid UTF-8 sequence can contain this value if it doesn't encode this very character.
          else
            utf8_byte = 0x0A;
        }
      
      Unicode_translator::UTF8(&ucs4_char,
                               utf8_byte,
                               &run_length,
                               &correct_run_length);
      
      if (run_length == correct_run_length)
        {
          *p_character = ucs4_char;
          break;// OK.
        }
      else if (run_length > correct_run_length)
        {
          // Something failed in XML_character_info_item::UTF8().
          *p_character = 0x0000FFFD;// REPLACEMENT CHAR. (Unicode FAQ)
          return false;// Error.
        }
    }
  if (run_length != correct_run_length)
    {
      *p_character = 0x0000FFFD;// REPLACEMENT CHAR. (Unicode FAQ)
      return false;// Error.
    }
  else if (input_stream.eof())
    return false;// Normal EOF.
  else
    return true;// Keep reading.
}
