// Clase:    Tokenizer
// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:40 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002, 2003 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#ifndef _H_Tokenizer
#define _H_Tokenizer

#include <unistd.h>
#include <iostream>
using std::istream;
#include <fstream>
using std::ifstream;
#include <vector> // STL
using std::vector;
#include <map> // STL
using std::map;
#include "Unicode_regexp/Unicode_regexp.h"
using std::ios;
using std::cout;
using std::cin;
using std::endl;
using std::flush;
#include "../OpenMath_object.H"
#include "../Bison_typedefs.h"
#include "../QMath_parser.tab.cc.h"
#include "../../Unicode_translator/Unicode_translator.h"
#include <string>
using std::string;
typedef class Tokenizer Tokenizer_t, *Tokenizer_p_t;

class Tokenizer
{
 public:
  typedef enum
  {
    LEXER_STATE_INITIAL,//                 0
    LEXER_STATE_QMATH,//                   1
    LEXER_STATE_QMATH_DEF_HEADER,//        2
    LEXER_STATE_QMATH_DEF_SIMPLE,//        3
    LEXER_STATE_QMATH_DEF_COMPLETE,//      4
    LEXER_STATE_QMATH_DIRECTIVE,//         5
    LEXER_STATE_PARAGRAPH,//               6
    LEXER_STATE_MATH,//                    7
    LEXER_STATE_XML//                      8
  } Lexer_state_t, *Lexer_state_p_t;
  typedef struct
  {
    Unicode_regexp_t regexp;
    bool match_at_beginning_of_line;
    bool match_at_end_of_line;
    unsigned int id;
    unsigned int match_length;
  } Token_entry_t, *Token_entry_p_t;
  typedef struct
  {
    istream* p_istream;
    ifstream* p_ifstream;
    unsigned int current_input_line_number;
    unsigned int current_input_column_number;
    string file_name;
  } Stream_entry_t, *Stream_entry_p_t;
  typedef struct
  {
    XML_UCS4_char_t character;
    Unicode_regexp::Special_condition_t special_condition;
  } Input_buffer_entry_t, *Input_buffer_entry_p_t;
 private:
  typedef struct
  {
    int type;
    YYSTYPE value;
  } Token_buffer_entry_t, *Token_buffer_entry_p_t;
  bool initialized;
  vector<Input_buffer_entry_t> input_buffer;
  unsigned int index_of_current_character;
  vector<Token_buffer_entry_t> token_buffer;
  vector<Stream_entry_t> input_streams_stack;
  string base_directory;
  vector<Tab_stop_t> tab_stops;
  unsigned int indentation_base;
  bool at_beginning_of_line;
  bool at_end_of_line;
  Lexer_state_t lexer_state;
  vector<Lexer_state_t> lexer_state_stack;
  bool debug;
  unsigned int single_characters_pending;
  unsigned int single_character_identifiers_pending;
  string previous_tag_open;
  map<Lexer_state_t, vector<Token_entry_t> > tokens;
  #include "language_codes.h"
  XML_string_t lang;
  typedef struct
  {
    XML_string_t context_name;
    XML_string_t ucs4_string;
    int token_type;
    //map<Language_id_t, vector<XML_UCS4_char_t> > openmath_symbol;
    string openmath_symbol;
  } Translation_table_entry_t, *Translation_table_entry_p_t;
  typedef vector<Translation_table_entry_t> Translation_table_t;
  Translation_table_t translation_table;
  Unicode_data unicode;
  unsigned int error_count;
  bool embedded_mode;
 public:
  Tokenizer();
  ~Tokenizer();
  int Current_column_number();
  bool Translate_token(const vector<XML_UCS4_char_t> input, string *p_translated_input, int *p_token_type);
  bool Language();
  bool Language(XML_string_t &language);
  bool Language(XML_string_p_t p_language);
  bool Language(string* p_language);
  int Increment_current_line_number();
  int Increment_current_line_number(unsigned int count);
  int Current_line_number();
  bool Load_translation_table();
  Lexer_state_t State();
  bool State(Lexer_state_t state);
  bool State_push(Lexer_state_t state);
  bool State_pop();
  istream* Input_stream();
  bool Input_stream(istream &input_stream);
  bool Input_stream(istream &input_stream, Lexer_state_t new_lexer_state);
  bool Input_stream(const char* file_name);
  bool Input_stream(const char* file_name, Lexer_state_t new_lexer_state);
  bool Input_stream(const string& file_name);
  bool Input_stream(const string& file_name, Lexer_state_t new_lexer_state);
  bool Input_stream(XML_string_t &file_name);
  bool Input_stream(XML_string_t &file_name, Lexer_state_t new_lexer_state);
  unsigned int Indentation_base();
  const char *Current_file_name();
  int Get_next_token(YYSTYPE *yylval_p);
  unsigned int Error_count();
  vector<Tab_stop_t>* Tab_stops();
  Tab_stop_p_t Tab_stops(int index);
  void Report_debug(const char* message);
  void Report_debug(string message);
  void Report_debug(XML_string_t message);
  void Report_warning(const char* message);
  void Report_warning(string message);
  void Report_warning(XML_string_t message);
  void Report_error(const char* message);
  void Report_error(string message);
  void Report_error(XML_string_t message);
  int Increment_current_column_number();
  int Increment_current_column_number(unsigned int count);
  bool Embedded_mode();
  int Append_translation_entry(XML_string_t &context_name, XML_string_t &token, XML_string_t &token_type, XML_string_t &translation);
  static bool Input_UCS4_from_UTF8_stream(istream &input_stream, XML_UCS4_char_p_t p_character);
};



#endif
