// Clase:    Unicode_regexp
// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:40 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#include "Unicode_regexp.h"

/////////////
// Unicode_regexp - Class constructor.
// no arguments.
//public:
Unicode_regexp::Unicode_regexp()
{
  p_simple_node = NULL;
  p_sequence_array = NULL;
  p_alternative_array = NULL;
  match_count = 0;
  match_count_min = 1;
  match_count_max = 1;
  accumulated_match_count = 0;
  match_failed = false;
  //match_is_complete = false;
  special_condition = SPECIAL_CONDITION_NONE;
  special_condition_met = SPECIAL_CONDITION_NONE;
  current_regexp = 0;
  current_alternative = 0;
}

/////////////
// Unicode_regexp - Copy constructor.
// model: Object to be copied.
//public:
Unicode_regexp::Unicode_regexp(const Unicode_regexp_t &model)
{
  if (model.p_simple_node)
    p_simple_node = new Unicode_regexp_node_t(*model.p_simple_node);
  else p_simple_node = NULL;
  if (model.p_sequence_array)
    p_sequence_array = new Unicode_regexp_array_t(*model.p_sequence_array);
  else p_sequence_array = NULL;
  if (model.p_alternative_array)
    p_alternative_array = new Unicode_regexp_array_t(*model.p_alternative_array);
  else p_alternative_array = NULL;
  match_count = model.match_count;
  match_count_min = model.match_count_min;
  match_count_max = model.match_count_max;
  accumulated_match_count = model.accumulated_match_count;
  match_failed = model.match_failed;
  //match_is_complete = model.match_is_complete;
  special_condition = model.special_condition;
  special_condition_met = model.special_condition_met;
  current_regexp = model.current_regexp;
  current_alternative = model.current_alternative;
}

/////////////
// ~Unicode_regexp - Class destructor.
// no arguments.
//public:
Unicode_regexp::~Unicode_regexp()
{
  Erase();
}


/////////////
// Reset - Reset match counters.
// no arguments.
// returns nothing.
//public:
void Unicode_regexp::Reset()
{
  if (0 == match_count
      && 0 == accumulated_match_count
      && !match_failed)
    return;
  
  match_count = 0;
  //match_count_min = 1;// These two, of course,
  //match_count_max = 1;// mustn't be reset.
  accumulated_match_count = 0;
  match_failed = false;
  special_condition_met = SPECIAL_CONDITION_NONE;
  current_regexp = 0;
  current_alternative = 0;
  
  if (p_sequence_array)
    {
      for (vector<Unicode_regexp_t>::iterator
             regexp_iterator = p_sequence_array->begin();
           regexp_iterator != p_sequence_array->end();
           ++regexp_iterator)
        regexp_iterator->Reset();
    }
  else if (p_alternative_array)
    {
      for (vector<Unicode_regexp_t>::iterator
             regexp_iterator = p_alternative_array->begin();
           regexp_iterator != p_alternative_array->end();
           ++regexp_iterator)
        regexp_iterator->Reset();
    }
}


/////////////
// Erase - Erase regexp data.
// no arguments.
// returns nothing.
//public:
void Unicode_regexp::Erase()
{
  if (p_simple_node)
    {
      delete p_simple_node;
      p_simple_node = NULL;
    }
  if (p_sequence_array)
    {
      delete p_sequence_array;
      p_sequence_array = NULL;
    }
  if (p_alternative_array)
    {
      delete p_alternative_array;
      p_alternative_array = NULL;
    }
  
  Reset();
  match_count_min = 1;
  match_count_max = 1;
}


/////////////
// Compile - Compile a regexp given as a C string encoded in UTF8.
// regexp_string: C char array containing the textual representation.
//                The escape codes recognized are:
//                  \a  0x07 BEL      BELL
//                  \b  0x08 BS ^C-h  BACKSPACE
//                  \f  0x0C FF ^C-l  FORM FEED
//                  \n  0x0A LF ^C-j  LINE FEED
//                  \r  0x0D CR ^C-m  CARRIAGE RETURN
//                  \t  0x09 HT ^C-i  HORIZONTAL TABULATION
//                  \v  0x0B VT ^C-k  VERTICAL TABULATION
//                  \Uxxxxxxxx        32-bit hexadecimal escape code.
//                UTF-16 escapes aren't recognized: \u1234 matches "u1234".
// returns true on success, false on fail, such as an invalid regexp string.
//public:
bool Unicode_regexp::Compile(const char* regexp_string)
{
  XML_string xml_string;
  xml_string.UTF8(regexp_string);
  
  return Compile(xml_string.UCS4_string());
}

/////////////
// Compile - Compile a regexp given as an XML_string.
// regexp_string: UCS4 char array containing the textual representation.
//                The escape codes recognized are:
//                  \a  0x07 BEL      BELL
//                  \b  0x08 BS ^C-h  BACKSPACE
//                  \f  0x0C FF ^C-l  FORM FEED
//                  \n  0x0A LF ^C-j  LINE FEED
//                  \r  0x0D CR ^C-m  CARRIAGE RETURN
//                  \t  0x09 HT ^C-i  HORIZONTAL TABULATION
//                  \v  0x0B VT ^C-k  VERTICAL TABULATION
//                  \Uxxxxxxxx        32-bit hexadecimal escape code.
//                UTF-16 escapes aren't recognized: \u1234 matches "u1234".
// returns true on success, false on fail, such as an invalid regexp string.
//public:
bool Unicode_regexp::Compile(const vector<UCS4_char_t> &regexp_string)
{
  unsigned int char_index;
  char_index = 0;
  
  return Compile(regexp_string, &char_index);
}

/////////////
// Compile - Compile a regexp given as an XML_string.
// regexp_string: UCS4 char array containing the textual representation.
//                The escape codes recognized are:
//                  \a  0x07 BEL      BELL
//                  \b  0x08 BS ^C-h  BACKSPACE
//                  \f  0x0C FF ^C-l  FORM FEED
//                  \n  0x0A LF ^C-j  LINE FEED
//                  \r  0x0D CR ^C-m  CARRIAGE RETURN
//                  \t  0x09 HT ^C-i  HORIZONTAL TABULATION
//                  \v  0x0B VT ^C-k  VERTICAL TABULATION
//                  \Uxxxxxxxx        32-bit hexadecimal escape code.
//                UTF-16 escapes aren't recognized: \u1234 matches "u1234".
// p_char_index: Pointer to the character index, that gets modified.
// returns true on success, false on fail, such as an invalid regexp string.
//public:
bool Unicode_regexp::Compile(const vector<UCS4_char_t> &regexp_string, unsigned int *p_char_index)
{
  Erase();
  
  if (!(&regexp_string)) return false;
  if (!p_char_index) return false;
  if (*p_char_index >= regexp_string.size()) return false;
  if (regexp_string.size() == 0) return true;// A void regexp is a valid one.
  
  Unicode_regexp_t regexp;
  
  Unicode_regexp_node::character_range_t character_range;
  Unicode_regexp_node::character_category_t character_category;
  
  Parser_state_t parser_state;
  parser_state = STATE_INITIAL;
  int state_count_range_section;
  state_count_range_section = 0;
  unsigned int accumulator;// For reading numeric values.
  accumulator = 0;
  XML_string_t text_accumulator;// For text values.
  bool state_char_class_at_beginning;
  bool state_char_class_range_open;
  bool state_char_class_range_pending;
  bool state_char_class_negated;
  state_char_class_at_beginning = false;
  state_char_class_range_open = false;
  state_char_class_range_pending = false;
  state_char_class_negated = false;
  Special_condition_t state_special_condition;
  state_special_condition = SPECIAL_CONDITION_NONE;
  
  bool regexp_pending;
  regexp_pending = false;
  bool pending_regexp_is_alternative;
  pending_regexp_is_alternative = false;
  
  
  while (*p_char_index < regexp_string.size())
    {
      switch (parser_state)
        {
        case STATE_INITIAL:
          switch (regexp_string[*p_char_index])
            {
            case UCS4_char_t('('):
              if (regexp_pending)
                {
                  regexp.Special_condition(state_special_condition);
                  state_special_condition = SPECIAL_CONDITION_NONE;
                  if (pending_regexp_is_alternative)
                    Alternative(&regexp);
                  else
                    Concatenated(&regexp);
                  //regexp.Erase(); Already done in Concat./Altern.
                  regexp_pending = false;
                  pending_regexp_is_alternative = false;
                }
              
              ++(*p_char_index);
              if (*p_char_index >= regexp_string.size())
                return false;
              
              regexp.Compile(regexp_string, p_char_index);
              regexp_pending = true;
              break;
            case UCS4_char_t(')'):
              if (regexp_pending)
                {
                  regexp.Special_condition(state_special_condition);
                  state_special_condition = SPECIAL_CONDITION_NONE;
                  if (pending_regexp_is_alternative)
                    Alternative(&regexp);
                  else
                    Concatenated(&regexp);
                  //regexp.Erase(); Already done in Concat./Altern.
                  regexp_pending = false;
                  pending_regexp_is_alternative = false;
                }
              
              return true;
              break;
            case UCS4_char_t('\\'):
              if (regexp_pending)
                {
                  regexp.Special_condition(state_special_condition);
                  state_special_condition = SPECIAL_CONDITION_NONE;
                  if (pending_regexp_is_alternative)
                    Alternative(&regexp);
                  else
                    Concatenated(&regexp);
                  //regexp.Erase(); Already done in Concat./Altern.
                  regexp_pending = false;
                  pending_regexp_is_alternative = false;
                }
              
              ++(*p_char_index);
              if (*p_char_index >= regexp_string.size())
                return false;
              
              if (!regexp.p_simple_node)
                regexp.p_simple_node = new Unicode_regexp_node_t;
              regexp.p_simple_node->Erase();
              switch (regexp_string[*p_char_index])
                {
                case UCS4_char_t('a'):
                  character_range.begin = UCS4_char_t(0x00000007);
                  break;
                case UCS4_char_t('b'):
                  character_range.begin = UCS4_char_t(0x00000008);
                  break;
                case UCS4_char_t('f'):
                  character_range.begin = UCS4_char_t(0x0000000C);
                  break;
                case UCS4_char_t('n'):
                  character_range.begin = UCS4_char_t(0x0000000A);
                  break;
                case UCS4_char_t('r'):
                  character_range.begin = UCS4_char_t(0x0000000D);
                  break;
                case UCS4_char_t('t'):
                  character_range.begin = UCS4_char_t(0x00000009);
                  break;
                case UCS4_char_t('v'):
                  character_range.begin = UCS4_char_t(0x0000000B);
                  break;
                case UCS4_char_t('U'):
                  {
                    UCS4_char_t escaped_character;
                    escaped_character = 0x00000000;
                    const UCS4_char_t char_0 = UCS4_char_t('0');
                    const UCS4_char_t char_a = UCS4_char_t('a');
                    const UCS4_char_t char_A = UCS4_char_t('A');
                    for (int i = 0; i < 8; ++i)
                      {
                        ++(*p_char_index);
                        if (*p_char_index >= regexp_string.size())
                          return false;
                        UCS4_char_t c;
                        c = regexp_string[*p_char_index];
                        if      (c >= char_a) c = c - char_a + 10;
                        else if (c >= char_A) c = c - char_A + 10;
                        else                  c = c - char_0;
                        escaped_character |= ((c & 0x0000000F) << (28 - i*4));
                      }
                    character_range.begin = escaped_character;
                  }
                  break;
                default:
                  character_range.begin = regexp_string[*p_char_index];
                  break;
                }
              character_range.end   = character_range.begin;
              regexp.p_simple_node->Append_range(character_range);
              regexp_pending = true;
              break;
            case UCS4_char_t('?'):
              if (!regexp_pending)
                {
                  cerr << "Compile(): No regexp_pending for closure!\n";
                  string buffer;
                  for (vector<UCS4_char_t>::const_iterator
                       i = regexp_string.begin(); i < regexp_string.end();
                       ++i)
                    Unicode_translator::UTF8(*i, &buffer);
                  cerr << buffer << "\n";
                  for (unsigned int i = *p_char_index; i > 0; --i)
                    cerr << "-";
                  cerr << "^\n";
                  return false;
                }
              // We could have managed this unexpected character as if
              // it was escaped. But I prefer being strict here.
              regexp.Match_count_min(0);
              regexp.Match_count_max(1);
              break;
            case UCS4_char_t('*'):
              if (!regexp_pending)
                {
                  cerr << "Compile(): No regexp_pending for closure!\n";
                  string buffer;
                  for (vector<UCS4_char_t>::const_iterator
                       i = regexp_string.begin(); i < regexp_string.end();
                       ++i)
                    Unicode_translator::UTF8(*i, &buffer);
                  cerr << buffer << "\n";
                  for (unsigned int i = *p_char_index; i > 0; --i)
                    cerr << "-";
                  cerr << "^\n";
                  return false;
                }
              // We could have managed this unexpected character as if
              // it was escaped. But I prefer being strict here.
              regexp.Match_count_min(0);
              regexp.Match_count_max(UNLIMITED);
              break;
            case UCS4_char_t('+'):
              if (!regexp_pending)
                {
                  cerr << "Compile(): No regexp_pending for closure!\n";
                  string buffer;
                  for (vector<UCS4_char_t>::const_iterator
                       i = regexp_string.begin(); i < regexp_string.end();
                       ++i)
                    Unicode_translator::UTF8(*i, &buffer);
                  cerr << buffer << "\n";
                  for (unsigned int i = *p_char_index; i > 0; --i)
                    cerr << "-";
                  cerr << "^\n";
                  return false;
                }
              // We could have managed this unexpected character as if
              // it was escaped. But I prefer being strict here.
              regexp.Match_count_min(1);
              regexp.Match_count_max(UNLIMITED);
              break;
            case UCS4_char_t('{'):
              parser_state = STATE_COUNT_RANGE;
              state_count_range_section = 1;
              accumulator = 0;
              break;
            case UCS4_char_t('['):
              if (regexp_pending)
                {
                  regexp.Special_condition(state_special_condition);
                  state_special_condition = SPECIAL_CONDITION_NONE;
                  if (pending_regexp_is_alternative)
                    Alternative(&regexp);
                  else
                    Concatenated(&regexp);
                  //regexp.Erase(); Already done in Concat./Altern.
                  regexp_pending = false;
                  pending_regexp_is_alternative = false;
                }
              parser_state = STATE_CHAR_CLASS;
              state_char_class_at_beginning = true;
              state_char_class_range_open = false;
              state_char_class_range_pending = false;
              state_char_class_negated = false;
              character_category.general_category = Unicode_data::none;
              break;
            case UCS4_char_t('|'):
              if (regexp_pending)
                {
                  regexp.Special_condition(state_special_condition);
                  state_special_condition = SPECIAL_CONDITION_NONE;
                  if (pending_regexp_is_alternative)
                    Alternative(&regexp);
                  else
                    Concatenated(&regexp);
                  //regexp.Erase(); Already done in Concat./Altern.
                  regexp_pending = false;
                  pending_regexp_is_alternative = false;
                }
              pending_regexp_is_alternative = true;
              break;
            case UCS4_char_t('^'):
              if (regexp_pending)
                {
                  regexp.Special_condition(state_special_condition);
                  state_special_condition = SPECIAL_CONDITION_NONE;
                  if (pending_regexp_is_alternative)
                    Alternative(&regexp);
                  else
                    Concatenated(&regexp);
                  //regexp.Erase(); Already done in Concat./Altern.
                  regexp_pending = false;
                  pending_regexp_is_alternative = false;
                }
              state_special_condition = Special_condition_t
                (state_special_condition
                 | SPECIAL_CONDITION_AT_BEGINNING_OF_LINE);
              break;
            case UCS4_char_t('$'):
              state_special_condition = Special_condition_t
                (state_special_condition
                 | SPECIAL_CONDITION_AT_END_OF_LINE);
              if (regexp_pending)
                {
                  regexp.Special_condition(state_special_condition);
                  state_special_condition = SPECIAL_CONDITION_NONE;
                  if (pending_regexp_is_alternative)
                    Alternative(&regexp);
                  else
                    Concatenated(&regexp);
                  //regexp.Erase(); Already done in Concat./Altern.
                  regexp_pending = false;
                  pending_regexp_is_alternative = false;
                }
              break;
            default:
              if (regexp_pending)
                {
                  regexp.Special_condition(state_special_condition);
                  state_special_condition = SPECIAL_CONDITION_NONE;
                  if (pending_regexp_is_alternative)
                    Alternative(&regexp);
                  else
                    Concatenated(&regexp);
                  //regexp.Erase(); Already done in Concat./Altern.
                  regexp_pending = false;
                  pending_regexp_is_alternative = false;
                }
              character_range.begin = regexp_string[*p_char_index];
              character_range.end   = regexp_string[*p_char_index];
              if (!regexp.p_simple_node)
                regexp.p_simple_node = new Unicode_regexp_node_t;
              regexp.p_simple_node->Append_range(character_range);
              regexp_pending = true;
              break;
            }// switch (regexp_string[*p_char_index]) ...
          break;
        case STATE_COUNT_RANGE:
          switch (regexp_string[*p_char_index])
            {
            case UCS4_char_t('0'):
            case UCS4_char_t('1'):
            case UCS4_char_t('2'):
            case UCS4_char_t('3'):
            case UCS4_char_t('4'):
            case UCS4_char_t('5'):
            case UCS4_char_t('6'):
            case UCS4_char_t('7'):
            case UCS4_char_t('8'):
            case UCS4_char_t('9'):
              {
                accumulator = accumulator * 10
                  + (
                     (unsigned int) regexp_string[*p_char_index]
                     - (unsigned int) UCS4_char_t('0')
                     );
              }
              break;
            case UCS4_char_t(','):
              regexp.Match_count_min(accumulator);
              accumulator = 0;
              state_count_range_section = 2;
              break;
            case UCS4_char_t('}'):
              if (2 == state_count_range_section)
                {
                  if (0 == accumulator)
                    regexp.Match_count_max(UNLIMITED);
                  else
                    regexp.Match_count_max(accumulator);
                }
              else
                {
                  regexp.Match_count_min(accumulator);
                  regexp.Match_count_max(accumulator);
                }
              parser_state = STATE_INITIAL;
              break;
            default:
              break;
            }// switch (regexp_string[*p_char_index]) ...
          break;
        case STATE_CHAR_CLASS:
          switch (regexp_string[*p_char_index])
            {
            case UCS4_char_t('['):
              // Character category.
              if (state_char_class_range_pending)
                {
                  character_range.end = character_range.begin;
                  if (!regexp.p_simple_node)
                    regexp.p_simple_node = new Unicode_regexp_node_t;
                  regexp.p_simple_node->Append_range(character_range);
                  regexp_pending = true;
                  regexp.p_simple_node
                    ->Negated_char_ranges(state_char_class_negated);
                }
              if (state_char_class_range_open)
                {
                  character_range.begin = UCS4_char_t('-');
                  character_range.end = character_range.begin;
if (!regexp.p_simple_node)
  regexp.p_simple_node = new Unicode_regexp_node_t;
regexp.p_simple_node->Append_range(character_range);
regexp_pending = true;
                  regexp.p_simple_node
                    ->Negated_char_ranges(state_char_class_negated);
                }
              state_char_class_range_open = false;
              state_char_class_range_pending = false;
              parser_state = STATE_CHAR_CATEGORY;
              text_accumulator = "";
              break;
            case UCS4_char_t(']'):
              if (state_char_class_range_pending)
                {
                  character_range.end = character_range.begin;
                  if (!regexp.p_simple_node)
                    regexp.p_simple_node = new Unicode_regexp_node_t;
                  regexp.p_simple_node->Append_range(character_range);
                  regexp_pending = true;
                  regexp.p_simple_node
                    ->Negated_char_ranges(state_char_class_negated);
                }
              if (state_char_class_range_open)
                {
                  character_range.begin = UCS4_char_t('-');
                  character_range.end = character_range.begin;
if (!regexp.p_simple_node)
  regexp.p_simple_node = new Unicode_regexp_node_t;
regexp.p_simple_node->Append_range(character_range);
regexp_pending = true;
                  regexp.p_simple_node
                    ->Negated_char_ranges(state_char_class_negated);
                }
              state_char_class_range_open = false;
              state_char_class_range_pending = false;
              parser_state = STATE_INITIAL;
              break;
            case UCS4_char_t('-'):
              state_char_class_range_open = true;
              break;
            case UCS4_char_t('^'):
              if (state_char_class_at_beginning)
                {
                  state_char_class_negated = true;
                  state_char_class_at_beginning = false;
                  break;
                }
              // If not, leak to "default".
            default:
              state_char_class_at_beginning = false;
              if (state_char_class_range_open)
                {
                  character_range.end = regexp_string[*p_char_index];
if (!regexp.p_simple_node)
  regexp.p_simple_node = new Unicode_regexp_node_t;
regexp.p_simple_node->Append_range(character_range);
regexp_pending = true;
                  regexp.p_simple_node
                    ->Negated_char_ranges(state_char_class_negated);
                  state_char_class_range_pending = false;
                  state_char_class_range_open = false;
                }
              else
                {
                  if (state_char_class_range_pending)
                    {
                      character_range.end = character_range.begin;
                      if (!regexp.p_simple_node)
                        regexp.p_simple_node = new Unicode_regexp_node_t;
                      regexp.p_simple_node->Append_range(character_range);
                      regexp_pending = true;
                      regexp.p_simple_node
                        ->Negated_char_ranges(state_char_class_negated);
                    }
                  character_range.begin = regexp_string[*p_char_index];
                  state_char_class_range_pending = true;
                }
              break;
            }// switch (regexp_string[*p_char_index]) ...
          break;
        case STATE_CHAR_CATEGORY:
          switch (regexp_string[*p_char_index])
            {
            case UCS4_char_t(']'):
              parser_state = STATE_CHAR_CLASS;
              break;
            case UCS4_char_t('['):
              // Ignore
              break;
            case UCS4_char_t(':'):
              if (text_accumulator.UCS4_string().size() > 0)
                {
                  if ("Lu" == text_accumulator)
                    character_category.general_category = Unicode_data::Lu;
                  else if ("Ll" == text_accumulator)
                    character_category.general_category = Unicode_data::Ll;
                  else if ("Lt" == text_accumulator)
                    character_category.general_category = Unicode_data::Lt;
                  else if ("Mn" == text_accumulator)
                    character_category.general_category = Unicode_data::Mn;
                  else if ("Mc" == text_accumulator)
                    character_category.general_category = Unicode_data::Mc;
                  else if ("Me" == text_accumulator)
                    character_category.general_category = Unicode_data::Me;
                  else if ("Nd" == text_accumulator)
                    character_category.general_category = Unicode_data::Nd;
                  else if ("Nl" == text_accumulator)
                    character_category.general_category = Unicode_data::Nl;
                  else if ("No" == text_accumulator)
                    character_category.general_category = Unicode_data::No;
                  else if ("Zs" == text_accumulator)
                    character_category.general_category = Unicode_data::Zs;
                  else if ("Zl" == text_accumulator)
                    character_category.general_category = Unicode_data::Zl;
                  else if ("Zp" == text_accumulator)
                    character_category.general_category = Unicode_data::Zp;
                  else if ("Cc" == text_accumulator)
                    character_category.general_category = Unicode_data::Cc;
                  else if ("Cf" == text_accumulator)
                    character_category.general_category = Unicode_data::Cf;
                  else if ("Cs" == text_accumulator)
                    character_category.general_category = Unicode_data::Cs;
                  else if ("Co" == text_accumulator)
                    character_category.general_category = Unicode_data::Co;
                  else if ("Cn" == text_accumulator)
                    character_category.general_category = Unicode_data::Cn;
                  else if ("Lm" == text_accumulator)
                    character_category.general_category = Unicode_data::Lm;
                  else if ("Lo" == text_accumulator)
                    character_category.general_category = Unicode_data::Lo;
                  else if ("Pc" == text_accumulator)
                    character_category.general_category = Unicode_data::Pc;
                  else if ("Pd" == text_accumulator)
                    character_category.general_category = Unicode_data::Pd;
                  else if ("Ps" == text_accumulator)
                    character_category.general_category = Unicode_data::Ps;
                  else if ("Pe" == text_accumulator)
                    character_category.general_category = Unicode_data::Pe;
                  else if ("Pi" == text_accumulator)
                    character_category.general_category = Unicode_data::Pi;
                  else if ("Pf" == text_accumulator)
                    character_category.general_category = Unicode_data::Pf;
                  else if ("Po" == text_accumulator)
                    character_category.general_category = Unicode_data::Po;
                  else if ("Sm" == text_accumulator)
                    character_category.general_category = Unicode_data::Sm;
                  else if ("Sc" == text_accumulator)
                    character_category.general_category = Unicode_data::Sc;
                  else if ("Sk" == text_accumulator)
                    character_category.general_category = Unicode_data::Sk;
                  else if ("So" == text_accumulator)
                    character_category.general_category = Unicode_data::So;
                  else
                    cerr << "Unicode_regexp: unrecognized category: "
                         << text_accumulator << endl;
                  if (!regexp.p_simple_node)
                    regexp.p_simple_node = new Unicode_regexp_node_t;
                  regexp.p_simple_node->Append_category(character_category);
                  regexp_pending = true;
                  regexp.p_simple_node
                    ->Negated_char_ranges(state_char_class_negated);
                  character_category.general_category = Unicode_data::none;
                  text_accumulator = "";
                }
              else
                text_accumulator = "";
              break;
            default:
              text_accumulator += regexp_string[*p_char_index];
              break;
            }// switch (regexp_string[*p_char_index]) ...
          break;
        }// switch (parser_state) ...
      ++(*p_char_index);
    }// while ((*p_char_index) < ...)
  if (regexp_pending)
    {
      regexp.Special_condition(state_special_condition);
      state_special_condition = SPECIAL_CONDITION_NONE;
      if (pending_regexp_is_alternative)
        Alternative(&regexp);
      else
        Concatenated(&regexp);
      //regexp.Erase(); Already done in Concat./Altern.
      regexp_pending = false;
      pending_regexp_is_alternative = false;
    }
  
  return true;
}


/////////////
// Alternative - Add and alternative regexp to this one.
//                The argument regexp gets transferred, not copied, which means
//                that the regexp pointed to by p_regext gets reset.
// p_regexp: Pointer to the regexp to be concatenated.
// returns this for operator composition.
//public:
Unicode_regexp_p_t Unicode_regexp::Alternative(Unicode_regexp_p_t p_regexp)
{
  if (!p_regexp) return this;
  
  Unicode_regexp_t void_regexp;
  Unicode_regexp_t temp_regexp;
  if (p_simple_node || p_sequence_array)
    {
      // Transfer the member pointers instead of copying everything.
      this->Transfer_to(temp_regexp);
      p_alternative_array = new Unicode_regexp_array_t;
      p_alternative_array->push_back(void_regexp);// Pushes a copy.
      temp_regexp.Transfer_to(p_alternative_array->back());
    }
  
  p_alternative_array->push_back(void_regexp);// Pushes a copy.
  p_regexp->Transfer_to(p_alternative_array->back());
  
  return this;
}


/////////////
// operator= - Overloading of = operator.
// model: Unicode_regexp to be copied.
// returns object reference for assignment concatenation.
//public:
const Unicode_regexp &Unicode_regexp::operator= (const Unicode_regexp &model)
{
  if (this == &model)
    return *this;
  
  Erase();
  
  if (model.p_simple_node)
    p_simple_node = new Unicode_regexp_node_t(*model.p_simple_node);
  else p_simple_node = NULL;
  if (model.p_sequence_array)
    p_sequence_array = new Unicode_regexp_array_t(*model.p_sequence_array);
  else p_sequence_array = NULL;
  if (model.p_alternative_array)
    p_alternative_array = new Unicode_regexp_array_t(*model.p_alternative_array);
  else p_alternative_array = NULL;
  match_count = model.match_count;
  match_count_min = model.match_count_min;
  match_count_max = model.match_count_max;
  accumulated_match_count = model.accumulated_match_count;
  match_failed = model.match_failed;
  //match_is_complete = model.match_is_complete;
  special_condition = model.special_condition;
  special_condition_met = model.special_condition_met;
  current_regexp = model.current_regexp;
  current_alternative = model.current_alternative;
  
  return *this;
}

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
// regexp: Unicode_regexp to output.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, Unicode_regexp &regexp)
{
  regexp.Output(output);
  
  return output;
}

/////////////
// Match_count_is_valid - Tell whether the current match count is valid.
// no arguments.
// returns true is count is within bounds, false otherwise.
//public:
bool Unicode_regexp::Match_count_is_valid()
{
  return (match_count >= match_count_min && match_count <= match_count_max);
}

/////////////
// Match_count - Set match_count value.
// value: new value.
// returns true on success, false on fail.
//public:
bool Unicode_regexp::Match_count(unsigned int value)
{
  match_count = value;
  if (value > match_count_max)
    match_count_max = value;
  if (value < match_count_min)
    match_count_min = value;
  
  return true;
}

/////////////
// Match_count - Get match_count value.
// no arguments.
// returns current match_count value.
//public:
unsigned int Unicode_regexp::Match_count()
{
  return match_count;
}

/////////////
// Match_count_min - Set match_count_min value.
// value: new value.
// returns true on success, false on fail.
//public:
bool Unicode_regexp::Match_count_min(unsigned int value)
{
  match_count_min = value;
  if (value > match_count_max)
    match_count_max = value;
  
  return true;
}

/////////////
// Match_count_min - Get match_count value.
// no arguments.
// returns current match_count_min value.
//public:
unsigned int Unicode_regexp::Match_count_min()
{
  return match_count_min;
}

/////////////
// Match_count_max - Set match_count_max value.
// value: new value.
// returns true on success, false on fail.
//public:
bool Unicode_regexp::Match_count_max(unsigned int value)
{
  match_count_max = value;
  if (value < match_count_min)
    match_count_min = value;
  
  return true;
}

/////////////
// Match_count_max - Get match_count_max value.
// no arguments.
// returns current match_count_max value.
//public:
unsigned int Unicode_regexp::Match_count_max()
{
  return match_count_max;
}


void Unicode_regexp::Output_closure_symbol(ostream &output)
{
  if (match_count_min == 0
      && match_count_max == 1)
    output << "?";
  else if (match_count_min == 0
           && match_count_max == UNLIMITED)
    output << "*";
  else if (match_count_min == 1
           && match_count_max == UNLIMITED)
    output << "+";
  else if (match_count_min
           == match_count_max)
    {
      if (match_count_max != 1)
        output << "{"
               << match_count_max
               << "}";
    }
  else if (match_count_max == UNLIMITED)
    output << "{"
           << match_count_min << ","
           << "}";
  else
    output << "{"
           << match_count_min << ","
           << match_count_max
           << "}";
}

/////////////
// Output - Output object to a c++ stream.
// output: output c++ stream.
// returns output stream for operator composition.
//public:
ostream &Unicode_regexp::Output(ostream &output)
{
  if (Special_condition() == SPECIAL_CONDITION_AT_BEGINNING_OF_LINE)
    output << "^";
  
  if (p_simple_node)
    {
      output << *p_simple_node;
      Output_closure_symbol(output);
    }
  else if (p_sequence_array)
    {
      output << "(";
      for (Unicode_regexp_array_t::iterator
             regexp_iterator = p_sequence_array->begin();
           regexp_iterator != p_sequence_array->end();
           ++regexp_iterator)
        {
          output << *regexp_iterator;
        }
      output << ")";
      Output_closure_symbol(output);
    }
  else if (p_alternative_array)
    {
      output << "(";// TODO: show parentheses only if closure symbol is needed.
      for (Unicode_regexp_array_t::iterator
             regexp_iterator = p_alternative_array->begin();
           regexp_iterator != p_alternative_array->end();
           ++regexp_iterator)
        {
          output << *regexp_iterator;
          if (regexp_iterator + 1 != p_alternative_array->end())
            output << "|";
        }
      output << ")";
      Output_closure_symbol(output);
    }
  else
    output << "<<NULL regexp>>";
  
  if (Special_condition() == SPECIAL_CONDITION_AT_END_OF_LINE)
    output << "$";
  
  return output;
}


/////////////
// Match_is_complete - Whether or not the match is complete.
// no arguments.
// returns true if the entire regexp has been matched, false otherwise.
//   Further characters can still match after the match is completed when
//  the last node is a closure. This only indicates when the minimum match has
//  been reached.
//public:
bool Unicode_regexp::Match_is_complete()
{
  if (Special_condition() != SPECIAL_CONDITION_NONE)
    if (!(Special_condition() == special_condition_met))
      return false;
  
  if (p_simple_node)
    {
      return Match_count_is_valid();
    }
  else if (p_sequence_array)
    {
      if (current_regexp == 0
          && p_sequence_array->begin()->match_count == 0)
        {
          // This regexp is not at a partial match.
          // So either it has been rewinded after a full match, or still
          // hasn't been tested at all.
          // Just check match_count.
          return Match_count_is_valid();
        }
      for (Unicode_regexp_array_t::iterator
             regexp_iterator = p_sequence_array->begin() + current_regexp;
           regexp_iterator < p_sequence_array->end();
           ++regexp_iterator)
        {
          if (!regexp_iterator->Match_is_complete())
            {
              return false;
            }
        }
      
      return Match_count_is_valid();
    }
  else if (p_alternative_array)
    {
      for (Unicode_regexp_array_t::iterator
             alternative_iterator = p_alternative_array->begin();
           alternative_iterator != p_alternative_array->end();
           ++alternative_iterator)
        {
          if (alternative_iterator->Match_is_complete())
            return true;
        }
      return false;
    }
  else
    return true;// A void regexp is complete always.
}


/////////////
// Special_condition - Set special condition value.
// condition: condition to set.
// returns true if set, false otherwise.
//public:
bool Unicode_regexp::Special_condition(Special_condition_t condition)
{
  special_condition = condition;
  
  return true;
}

/////////////
// Special_condition - Get special condition value.
// no arguments.
// returns value of special condition.
//public:
Unicode_regexp::Special_condition_t Unicode_regexp::Special_condition()
{
  return special_condition;
}


/////////////
// Increment_match_count - Add 1 to match_count value, and return validity.
// no arguments.
// returns true if the incremented value is inside bounds, false otherwise.
//public:
bool Unicode_regexp::Increment_match_count()
{
  if (UNLIMITED == match_count_max)
    {
      ++match_count;
      return true;
    }
  if (match_count < match_count_max)
    {
      ++match_count;
      return true;
    }
  else
    {
      return false;
    }
}


/////////////
// Transfer_to - Transfer contents to another regexp, to avoid deep copying.
// regexp: The destination regexp.
// returns pointer to new regexp for operator composition.
//public:
Unicode_regexp_p_t Unicode_regexp::Transfer_to(Unicode_regexp_t &regexp)
{
  if (!(&regexp)) return this;
  
  regexp.Erase();
  regexp.p_simple_node = p_simple_node;
  regexp.p_sequence_array = p_sequence_array;
  regexp.p_alternative_array = p_alternative_array;
  regexp.match_count = match_count;
  regexp.match_count_min = match_count_min;
  regexp.match_count_max = match_count_max;
  regexp.accumulated_match_count = accumulated_match_count;
  regexp.match_failed = match_failed;
  //regexp.match_is_complete = match_is_complete;
  regexp.special_condition     = special_condition;
  regexp.special_condition_met = special_condition_met;
  regexp.current_regexp = current_regexp;
  regexp.current_alternative = current_alternative;
  
  p_simple_node = NULL;
  p_sequence_array = NULL;
  p_alternative_array = NULL;
  match_count = 0;
  match_count_min = 1;
  match_count_max = 1;
  accumulated_match_count = 0;
  //match_failed = match_failed;
  //match_is_complete = match_is_complete;
  special_condition = SPECIAL_CONDITION_NONE;
  special_condition_met = SPECIAL_CONDITION_NONE;
  current_regexp = 0;
  current_alternative = 0;
  
  return &regexp;
}

using std::cerr;
using std::endl;
using std::flush;

/////////////
// Concatenated - Concatenate a regexp to this one.
//                The argument regexp gets transferred, not copied, which means
//                that the regexp pointed to by p_regext gets reset.
// p_regexp: Pointer to the regexp to be concatenated.
// returns this for operator composition.
//public:
Unicode_regexp_p_t Unicode_regexp::Concatenated(Unicode_regexp_p_t p_regexp)
{
  if (!p_regexp) return this;
  
  Unicode_regexp_t void_regexp;
  Unicode_regexp_t temp_regexp;
  if (!p_simple_node && !p_sequence_array && !p_alternative_array)
    {
      p_regexp->Transfer_to(*this);
    }
  else if (p_simple_node)
    {
      if (p_sequence_array)
        cerr << "Error: Unicode_regexp::Concatenated(): p_sequence_array is not NULL." << endl;
      // Transfer the member pointers instead of copying everything.
      this->Transfer_to(temp_regexp);
      p_sequence_array = new Unicode_regexp_array_t;
      p_sequence_array->push_back(void_regexp);// Pushes a copy.
      temp_regexp.Transfer_to(p_sequence_array->back());
      
      p_sequence_array->push_back(void_regexp);// Pushes a copy.
      p_regexp->Transfer_to(p_sequence_array->back());
    }
  else if (p_sequence_array)
    {
      // Transfer the member pointers instead of copying everything.
      p_sequence_array->push_back(void_regexp);// Pushes a copy.
      p_regexp->Transfer_to(p_sequence_array->back());
    }
  else if (p_alternative_array)
    {
      // Append to the last alternative in the array.
      if (p_alternative_array->size() > 0)
        p_alternative_array->back().Concatenated(p_regexp);
      else
        {
          cerr << "Error: Unicode_regexp::Concatenated(): p_alternative_array"
               << " has no elements." << endl << flush;
          p_regexp->Erase();
        }
    }
  
  return this;
}


/////////////
// Match - Match character against regexp.
// character: next UCS4 character to match.
// returns true on match, false otherwise.
//public:
bool Unicode_regexp::Match(UCS4_char_t character)
{
  if (match_failed)
    return false;// A previous match failed. Don't skip mismatches.
  
  if (p_simple_node)
    {
      if (p_simple_node->Match(character))
        {
          return Increment_match_count();
        }
      else
        {
          if (!Match_is_complete())
            Reset();
          match_failed = true;
          return false;
        }
    }
  else if (p_sequence_array)
    {
      int matches_found;
      matches_found = 0;
      while (current_regexp < p_sequence_array->size())
        {
          // Try to match as many as possible in parallel, just in case the
          // current regexp turns out to match only partially.
          // E.g.: 'a(bc)+bd' matching 'abcbd': it would try to match
          // the second 'b' as the beginning of another iteration of
          // (bc). When we find that it isn't the correct path, we
          // will need to change track to a(bc)b.
          // If the current one ends up being completed, we will
          // discard all the alternative paths.
          // (sort of "speculative matching")
          matches_found = 0;
          if ((*p_sequence_array)[current_regexp].match_failed)
            {
              // It had already failed. Give it a last chance for redemption.
              if (! (*p_sequence_array)[current_regexp].Match_is_complete())
                return false;
            }
          
          // Keep account of the match count before the matching attempt.
          // If it fails, the regexp will be reset, and if the minimum
          // match count is 0, a partial failed match would be considered
          // a completed one.
          unsigned int previous_match_count;
          previous_match_count = (*p_sequence_array)[current_regexp].Match();
          
          // We only search for further matches if the current one does.
          if ((*p_sequence_array)[current_regexp].Match(character))
            {
              ++matches_found;
              if (!(*p_sequence_array)[current_regexp].Match_is_complete())
                {
                  // We don't know whether current_regexp is THE ONE or not,
                  // so we have to try all the "other potentials" as well.
                  for (Unicode_regexp_array_t::iterator
                         regexp_iterator
                         = p_sequence_array->begin() + current_regexp + 1;
                       regexp_iterator < p_sequence_array->end();
                       ++regexp_iterator)
                    {
                      if (regexp_iterator->Match(character))
                        ++matches_found;
                      else if (!regexp_iterator->Match_is_complete())
                        {
                          // Clear failed match.
                          //regexp_iterator->Reset();
                          //regexp_iterator->match_failed = true;
                          break;
                        }
                    }
                }
            }// if (...[current_regexp].Match(...)) ...
          
          if ((*p_sequence_array)[current_regexp].Match()
              < previous_match_count)
            {
              Reset();
              match_failed = true;
              return false;
            }
          
          if (matches_found > 0 && 0 == match_count)
            {
              if (!Increment_match_count())
                {
                  // This is the first match, but match_count_max == 0.
                  if (!Match_is_complete())
                    Reset();
                  match_failed = true;
                  return false;
                }
            }
          
          // Pass over completed matches until we find (or not) one new.
          if ((*p_sequence_array)[current_regexp].Match_is_complete())
            {
              // current_regexp is complete. Discard alternative paths.
              for (Unicode_regexp_array_t::iterator
                     regexp_iterator
                     = p_sequence_array->begin() + current_regexp + 1;
                   regexp_iterator < p_sequence_array->end();
                   ++regexp_iterator)
                {
                  regexp_iterator->Reset();
                }
              // TODO: Greediness can be controlled from here.
              if (matches_found > 0)
                {
                  // Matched and complete. Oh, we are soooo OK...
                  return true;
                }
              else
                {
                  // current_regexp doesn't match, but it's complete. Try next.
                  ++current_regexp;
                  if (current_regexp == p_sequence_array->size())
                    {
                      // We have reached the end, but the last regexp was
                      // complete. Rewind and try again...
                      if (Increment_match_count())
                        {
                          for (Unicode_regexp_array_t::iterator
                                 regexp_iterator = p_sequence_array->begin();
                               regexp_iterator != p_sequence_array->end();
                               ++regexp_iterator)
                            {
                              accumulated_match_count
                                += regexp_iterator->Match();
                              regexp_iterator->Reset();
                            }
                          current_regexp = 0;
                        }
                    }
                }
            }// if (...[current_regexp].Match_is_complete()) ...
          else
            {
              // The current regexp isn't complete yet, but we hope it will.
              if (matches_found > 0)
                return true;
              else
                {
                  // Match broken!! We are doomed!!
                  // Current regexp isn't complete, so we can't escape to the
                  // next one in hope of finding a match there.
                  (*p_sequence_array)[current_regexp].Reset();
                  (*p_sequence_array)[current_regexp].match_failed = true;
                  if (current_regexp == 0 && accumulated_match_count > 0)
                    --match_count;// Just rewinded. Undo Increment_match_count.
                  if (!Match_is_complete())
                    Reset();// Reject partial matches.
                  match_failed = true;
                  return false;// Oh, shit!! A real mismatch!!
                }
            }
        }// while (current_regexp < ...) ...
    }// else if (p_sequence_array) ...
  else if (p_alternative_array)
    {
      unsigned int maximum_match_length;
      unsigned int alternative_index;
      unsigned int alternative_matches_count;
      unsigned int complete_alternatives_count;
      maximum_match_length = 0;
      alternative_index = 0;
      alternative_matches_count = 0;
      complete_alternatives_count = 0;
      
      while (!match_failed)
        {
          maximum_match_length = 0;
          alternative_index = 0;
          current_alternative = 0;
          alternative_matches_count = 0;
          complete_alternatives_count = 0;
          for (Unicode_regexp_array_t::iterator
                 alternative_iterator = p_alternative_array->begin();
               alternative_iterator != p_alternative_array->end();
               ++alternative_iterator)
            {
              if (alternative_iterator->Match(character))
                {
                  ++alternative_matches_count;
                  if (alternative_iterator->Match()
                      > maximum_match_length)
                    {
                      maximum_match_length = alternative_iterator->Match();
                      current_alternative = alternative_index;
                      //break;
                    }
                }
              if (alternative_iterator->Match_is_complete())
                ++complete_alternatives_count;
              
              ++alternative_index;
            }
          if (alternative_matches_count > 0 && 0 == match_count)
            if (!Increment_match_count())
              {
                if (!Match_is_complete())
                  Reset();
                match_failed = true;
                return false;
              }
          
          if (alternative_matches_count > 0)
            return true;
          else if (complete_alternatives_count > 0)
            {
              if (Increment_match_count())
                {
                  maximum_match_length = 0;
                  
                  for (Unicode_regexp_array_t::iterator
                         alternative_iterator = p_alternative_array->begin();
                       alternative_iterator != p_alternative_array->end();
                       ++alternative_iterator)
                    {
                      if (alternative_iterator->Match()
                          > maximum_match_length)
                        maximum_match_length = alternative_iterator->Match();
                  
                      if (alternative_iterator->Match_is_complete())
                        alternative_iterator->Reset();
                    }
                  
                  if (maximum_match_length > 0)
                    accumulated_match_count += maximum_match_length;
                  else
                    match_failed = true;
                }
              else
                match_failed = true;
            }
          else match_failed = true;
          
        }// while (!match_failed) ...
    }// else if (p_alternative_array) ...
  
  if (!Match_is_complete())
    Reset();
  match_failed = true;
  return false;
}

/////////////
// Match - Match UCS4 string against regexp.
// ucs4_string: ucs4_string to match.
// returns longest match count.
//public:
unsigned int Unicode_regexp::Match(const vector<UCS4_char_t> &ucs4_string)
{
  return Match(ucs4_string, 0);
}

/////////////
// Match - Match UCS4 string against regexp.
// ucs4_string: ucs4_string to match.
// offset: offset at which to begin matching.
// returns longest match count.
//public:
unsigned int Unicode_regexp::Match(const vector<UCS4_char_t> &ucs4_string, unsigned int offset)
{
  Reset();
  
  if (offset < ucs4_string.size())
    for (vector<UCS4_char_t>::const_iterator
           character_iterator = ucs4_string.begin() + offset;
         character_iterator != ucs4_string.end();
         ++character_iterator)
      {
        if (!Match(*character_iterator))
          break;
      }
  
  return Match();
}

/////////////
// Match - Return total match count.
// no arguments.
// returns total match count.
//public:
unsigned int Unicode_regexp::Match()
{
  unsigned int match_count_total;
  match_count_total = 0;
  
  if (p_simple_node)
    {
      match_count_total = match_count;
    }
  else if (p_sequence_array)
    {
      for (Unicode_regexp_array_t::iterator
             regexp_iterator = p_sequence_array->begin();
           regexp_iterator != p_sequence_array->end();
           ++regexp_iterator)
        match_count_total += regexp_iterator->Match();
      match_count_total += accumulated_match_count;
    }
  else if (p_alternative_array)
    {
      unsigned int regexp_match_count;
      regexp_match_count = 0;
      for (Unicode_regexp_array_t::iterator
             regexp_iterator = p_alternative_array->begin();
           regexp_iterator != p_alternative_array->end();
           ++regexp_iterator)
        {
          regexp_match_count = regexp_iterator->Match();
          if (regexp_match_count > match_count_total)
            match_count_total = regexp_match_count;
        }
      match_count_total += accumulated_match_count;
    }
  
  return match_count_total;
}


/////////////
// Signal_condition - Signal a special condition.
//                 Special conditions don't increment match_count, so regexps
//               containing them don't take precedence when comparing
//               alternatives. This function only makes sure that regexps that
//               require a certain condition can't match without it.
// condition: condition to match.
// returns true on acknowledge (condition was required), false otherwise.
//public:
bool Unicode_regexp::Signal_condition(Unicode_regexp::Special_condition_t condition)
{
  if (match_failed)
    return false;// If match failed, there is no point in signaling anything.
  
  bool there_was_some_ack;
  there_was_some_ack = false;
  if (p_sequence_array)
    {
      //(*p_sequence_array)[current_regexp].Signal_condition(condition);
      for (Unicode_regexp_array_t::iterator
             regexp_iterator = p_sequence_array->begin() + current_regexp;
           regexp_iterator < p_sequence_array->end();
           ++regexp_iterator)
        {
          there_was_some_ack
            |= regexp_iterator->Signal_condition(condition);
          if (!regexp_iterator->Match_is_complete())
            {
              break;
            }
        }
      
    }
  else if (p_alternative_array)
    {
      for (Unicode_regexp_array_t::iterator
             alternative_iterator = p_alternative_array->begin();
           alternative_iterator != p_alternative_array->end();
           ++alternative_iterator)
        {
          there_was_some_ack
            |= alternative_iterator->Signal_condition(condition);
        }
    }
  
  if (!(Special_condition() & condition))
    return there_was_some_ack;// NAK if no subexp acknowledged.
  special_condition_met = Special_condition_t(special_condition_met
                                              | condition);
  
  return true;// ACK.
}


/////////////
// Fail_match - Reset match counters and set match_failed flag.
// no arguments.
// returns nothing.
//public:
void Unicode_regexp::Fail_match()
{
  if (!Match_is_complete())
    Reset();
  match_failed = true;
}
