// Clase:    Unicode_regexp
// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:40 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#ifndef _H_Unicode_regexp
#define _H_Unicode_regexp

#include "Unicode_regexp_node/Unicode_regexp_node.h"
#include <vector> // STL
using std::vector;
#include "../../../XML_infoset/XML_string/XML_string.h"
#include <iostream>
using std::cerr;
using std::endl;
using std::flush;
typedef class Unicode_regexp Unicode_regexp_t, *Unicode_regexp_p_t;

class Unicode_regexp
{
 public:
  typedef Unicode_regexp_node::UCS4_char_t UCS4_char_t, *UCS4_char_p_t;
  static const unsigned int UNLIMITED = ~0;// For closure range ends.
  typedef enum
  {
    SPECIAL_CONDITION_NONE                  = 0x00000000,
    SPECIAL_CONDITION_AT_BEGINNING_OF_LINE  = 0x00000001,
    SPECIAL_CONDITION_AT_END_OF_LINE        = 0x00000002
  } Special_condition_t, *Special_condition_p_t;
 private:
  // A regexp can be either a simple node, a sequence of regexps, or
  // a list of alternative regexps.
  // Read chapter 4 of "C++ Annotations" (ISBN 90-367-0470-7) for an
  // explanation on how to manage these pointers. I did it *after* spending
  // two weeks [20001222] tracking down the bug and stupidly blaming either
  // gcc or glibc until I found I hadn't defined a copy constructor, so
  // the seemingly innocent ->push_back() was deallocating them when it
  // called the destructor for the intermediate copy it makes.
  typedef vector<Unicode_regexp_t> Unicode_regexp_array_t, *Unicode_regexp_array_p_t;
  Unicode_regexp_node_p_t p_simple_node;
  Unicode_regexp_array_p_t p_sequence_array;
  Unicode_regexp_array_p_t p_alternative_array;
  unsigned int match_count;
  unsigned int match_count_min;
  unsigned int match_count_max;
  unsigned int accumulated_match_count;
  bool match_failed;
  //bool match_is_complete;
  Special_condition_t special_condition;
  Special_condition_t special_condition_met;
  unsigned int current_regexp;
  unsigned int current_alternative;
  typedef enum
  {
    STATE_INITIAL,
    STATE_COUNT_RANGE,
    STATE_CHAR_CLASS,
    STATE_CHAR_CATEGORY
  } Parser_state_t, *Parser_state_p_t;
 public:
  Unicode_regexp();
  Unicode_regexp(const Unicode_regexp_t &model);
  ~Unicode_regexp();
  void Reset();
  void Erase();
  bool Compile(const char* regexp_string);
  bool Compile(const vector<UCS4_char_t> &regexp_string);
  bool Compile(const vector<UCS4_char_t> &regexp_string, unsigned int *p_char_index);
  Unicode_regexp_p_t Alternative(Unicode_regexp_p_t p_regexp);
  const Unicode_regexp &operator= (const Unicode_regexp &model);
  bool Match_count_is_valid();
  bool Match_count(unsigned int value);
  unsigned int Match_count();
  bool Match_count_min(unsigned int value);
  unsigned int Match_count_min();
  bool Match_count_max(unsigned int value);
  unsigned int Match_count_max();
  void Output_closure_symbol(ostream &output);
  ostream &Output(ostream &output);
  bool Match_is_complete();
  bool Special_condition(Special_condition_t condition);
  Special_condition_t Special_condition();
  bool Increment_match_count();
  Unicode_regexp_p_t Transfer_to(Unicode_regexp_t &regexp);
  Unicode_regexp_p_t Concatenated(Unicode_regexp_p_t p_regexp);
  bool Match(UCS4_char_t character);
  unsigned int Match(const vector<UCS4_char_t> &ucs4_string);
  unsigned int Match(const vector<UCS4_char_t> &ucs4_string, unsigned int offset);
  unsigned int Match();
  bool Signal_condition(Special_condition_t condition);
  void Fail_match();
};

extern ostream &operator<< (ostream &, Unicode_regexp &);

#endif
