// Clase:    Unicode_regexp_node
// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:40 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#include "Unicode_regexp_node.h"

/////////////
// Unicode_regexp_node - Class constructor.
// no arguments.
//public:
Unicode_regexp_node::Unicode_regexp_node()
{
  this->negated = false;
}

/////////////
// ~Unicode_regexp_node - Class destructor.
// no arguments.
//public:
Unicode_regexp_node::~Unicode_regexp_node()
{

}

/////////////
// Erase - Erase node data.
// no arguments.
// returns nothing.
//public:
void Unicode_regexp_node::Erase()
{
  character_categories.erase(character_categories.begin(),
                             character_categories.end());
  character_ranges.erase(character_ranges.begin(),
                         character_ranges.end());
}

/////////////
// Append_category - Append a caracter category.
// category: category to append.
// returns true if the new category is valid, false otherwise.
//public:
bool Unicode_regexp_node::Append_category(character_category_t &category)
{
  if (Unicode_data::none == category.general_category)
    return false;
  
  character_categories.push_back(category);
  
  return true;
}


/////////////
// Character_ranges - Access to character_ranges list.
// no arguments.
// returns const character_ranges.
//public:
const vector<Unicode_regexp_node::character_range_t> &Unicode_regexp_node::Character_ranges()
{
  return character_ranges;
}

using std::ostream;
using std::cout;
using std::endl;
using std::dec;
using std::hex;

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
//         Characters are output as UTF-8. Hence, no \u nor \v escapes are
//        used.
// regexp_node: Unicode_regexp_node to output.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, Unicode_regexp_node &regexp_node)
{
  vector<char> buffer;
  bool inside_bracket;
  inside_bracket = false;
  
  cout << dec;// Just in case. All integers are to be expressed in decimal.
  if (0 == regexp_node.Character_ranges().size()
      && 0 == regexp_node.Character_categories().size()
      )
    output << "<<NULL regexp_node>>";
  
  for (vector<Unicode_regexp_node::character_range_t>::const_iterator
         range_iterator = regexp_node.Character_ranges().begin();
       range_iterator != regexp_node.Character_ranges().end();
       ++range_iterator)
    {
      buffer.erase(buffer.begin(), buffer.end());
      if (range_iterator->begin == range_iterator->end
          && regexp_node.Character_ranges().size()     == 1
          && regexp_node.Character_categories().size() == 0
          && !regexp_node.Negated_char_ranges())
        {
          // One single character, instead of a proper character class.
          switch (range_iterator->begin)
            {// This crude char casting works only with ASCII (7 bit) chars.
            case Unicode_regexp_node::UCS4_char_t('.'):
            case Unicode_regexp_node::UCS4_char_t('*'):
            case Unicode_regexp_node::UCS4_char_t('+'):
            case Unicode_regexp_node::UCS4_char_t('?'):
            case Unicode_regexp_node::UCS4_char_t('['):
            case Unicode_regexp_node::UCS4_char_t(']'):
            case Unicode_regexp_node::UCS4_char_t('('):
            case Unicode_regexp_node::UCS4_char_t(')'):
            case Unicode_regexp_node::UCS4_char_t('|'):
            case Unicode_regexp_node::UCS4_char_t('\\'):
            case Unicode_regexp_node::UCS4_char_t('^'):
            case Unicode_regexp_node::UCS4_char_t('$'):
              buffer.push_back('\\');
            }
          Unicode_translator::UTF8(range_iterator->begin, &buffer);
          for (vector<char>::const_iterator
                 char_iterator = buffer.begin();
               char_iterator != buffer.end();
               ++char_iterator)
            output << *char_iterator;
        }
      else
        {
          if (!inside_bracket)
            {
              output << "[";
              inside_bracket = true;
              if (regexp_node.Negated_char_ranges())
                output << "^";
            }
          switch (range_iterator->begin)
            {
            case Unicode_regexp_node::UCS4_char_t('\a'):
              buffer.push_back('\\');
              buffer.push_back('a');
              break;
            case Unicode_regexp_node::UCS4_char_t('\b'):
              buffer.push_back('\\');
              buffer.push_back('b');
              break;
            case Unicode_regexp_node::UCS4_char_t('\f'):
              buffer.push_back('\\');
              buffer.push_back('f');
              break;
            case Unicode_regexp_node::UCS4_char_t('\n'):
              buffer.push_back('\\');
              buffer.push_back('n');
              break;
            case Unicode_regexp_node::UCS4_char_t('\r'):
              buffer.push_back('\\');
              buffer.push_back('r');
              break;
            case Unicode_regexp_node::UCS4_char_t('\t'):
              buffer.push_back('\\');
              buffer.push_back('t');
              break;
            case Unicode_regexp_node::UCS4_char_t('\v'):
              buffer.push_back('\\');
              buffer.push_back('v');
              break;
            case Unicode_regexp_node::UCS4_char_t('['):
            case Unicode_regexp_node::UCS4_char_t(']'):
            case Unicode_regexp_node::UCS4_char_t('\\'):
            case Unicode_regexp_node::UCS4_char_t('^'):
              buffer.push_back('\\');
              Unicode_translator::UTF8(range_iterator->begin, &buffer);
              break;
            default:
              Unicode_translator::UTF8(range_iterator->begin, &buffer);
              break;
            }
          if (range_iterator->end != range_iterator->begin)
            {
              buffer.push_back('-');
              switch (range_iterator->end)
                {
                case Unicode_regexp_node::UCS4_char_t('\a'):
                  buffer.push_back('\\');
                  buffer.push_back('a');
                  break;
                case Unicode_regexp_node::UCS4_char_t('\b'):
                  buffer.push_back('\\');
                  buffer.push_back('b');
                  break;
                case Unicode_regexp_node::UCS4_char_t('\f'):
                  buffer.push_back('\\');
                  buffer.push_back('f');
                  break;
                case Unicode_regexp_node::UCS4_char_t('\n'):
                  buffer.push_back('\\');
                  buffer.push_back('n');
                  break;
                case Unicode_regexp_node::UCS4_char_t('\r'):
                  buffer.push_back('\\');
                  buffer.push_back('r');
                  break;
                case Unicode_regexp_node::UCS4_char_t('\t'):
                  buffer.push_back('\\');
                  buffer.push_back('t');
                  break;
                case Unicode_regexp_node::UCS4_char_t('\v'):
                  buffer.push_back('\\');
                  buffer.push_back('v');
                  break;
                case Unicode_regexp_node::UCS4_char_t('['):
                case Unicode_regexp_node::UCS4_char_t(']'):
                case Unicode_regexp_node::UCS4_char_t('\\'):
                case Unicode_regexp_node::UCS4_char_t('^'):
                  buffer.push_back('\\');
                  Unicode_translator::UTF8(range_iterator->end, &buffer);
                  break;
                default:
                  Unicode_translator::UTF8(range_iterator->end, &buffer);
                  break;
                }
            }
          for (vector<char>::const_iterator
                 char_iterator = buffer.begin();
               char_iterator != buffer.end();
               ++char_iterator)
            output << *char_iterator;
        }
    }
  for (vector<Unicode_regexp_node::character_category_t>::const_iterator
         category_iterator = regexp_node.Character_categories().begin();
       category_iterator != regexp_node.Character_categories().end();
       ++category_iterator)
    {
      if (!inside_bracket)
        {
          output << "[";
          inside_bracket = true;
        }
      switch (category_iterator->general_category)
        {
        case Unicode_data::Lu: output << "[:Lu:]"; break;
        case Unicode_data::Ll: output << "[:Ll:]"; break;
        case Unicode_data::Lt: output << "[:Lt:]"; break;
        case Unicode_data::Mn: output << "[:Mn:]"; break;
        case Unicode_data::Mc: output << "[:Mc:]"; break;
        case Unicode_data::Me: output << "[:Me:]"; break;
        case Unicode_data::Nd: output << "[:Nd:]"; break;
        case Unicode_data::Nl: output << "[:Nl:]"; break;
        case Unicode_data::No: output << "[:No:]"; break;
        case Unicode_data::Zs: output << "[:Zs:]"; break;
        case Unicode_data::Zl: output << "[:Zl:]"; break;
        case Unicode_data::Zp: output << "[:Zp:]"; break;
        case Unicode_data::Cc: output << "[:Cc:]"; break;
        case Unicode_data::Cf: output << "[:Cf:]"; break;
        case Unicode_data::Cs: output << "[:Cs:]"; break;
        case Unicode_data::Co: output << "[:Co:]"; break;
        case Unicode_data::Cn: output << "[:Cn:]"; break;
        case Unicode_data::Lm: output << "[:Lm:]"; break;
        case Unicode_data::Lo: output << "[:Lo:]"; break;
        case Unicode_data::Pc: output << "[:Pc:]"; break;
        case Unicode_data::Pd: output << "[:Pd:]"; break;
        case Unicode_data::Ps: output << "[:Ps:]"; break;
        case Unicode_data::Pe: output << "[:Pe:]"; break;
        case Unicode_data::Pi: output << "[:Pi:]"; break;
        case Unicode_data::Pf: output << "[:Pf:]"; break;
        case Unicode_data::Po: output << "[:Po:]"; break;
        case Unicode_data::Sm: output << "[:Sm:]"; break;
        case Unicode_data::Sc: output << "[:Sc:]"; break;
        case Unicode_data::Sk: output << "[:Sk:]"; break;
        case Unicode_data::So: output << "[:So:]"; break;
        default: output << "[:unknown_class_id:]"; break;
        }
    }
  if (inside_bracket)
    {
      output << "]";
      inside_bracket = false;
    }
  
  return output;
}


/////////////
// Character_categories - Access to character_categories list.
// no arguments.
// returns const character_categories.
//public:
const vector<Unicode_regexp_node::character_category_t> &Unicode_regexp_node::Character_categories()
{
  return character_categories;
}


/////////////
// Negated_char_ranges - Get 'negated' property.
// no arguments.
// returns whether or not the character ranges and categories are to be considered negated when matching.
//public:
bool Unicode_regexp_node::Negated_char_ranges()
{
  return negated;
}

/////////////
// Negated_char_ranges - Set 'negated' property.
// value: true or false for the property.
// returns nothing.
//public:
void Unicode_regexp_node::Negated_char_ranges(bool value)
{
  negated = value;
}


/////////////
// Append_range - Append a character range.
// range: range to append.
//        You can use equal ".begin" and ".end" to specify a single character.
//        ".end" must be greater or equal to ".begin".
// returns true if the new range is valid (UCS), false otherwise.
//public:
bool Unicode_regexp_node::Append_range(character_range_t &range)
{
  if (range.end < range.begin)
    return false;
  if (range.begin < 0)
    return false;
  
  character_ranges.push_back(range);
  
  return true;
}


/////////////
// Match - Try to match an UCS4 character.
// character: UCS4_char_t to match.
// returns true if character matches, false otherwise.
//public:
bool Unicode_regexp_node::Match(UCS4_char_t character)
{
  for (vector<character_range_t>::const_iterator
         range_iterator = character_ranges.begin();
       range_iterator != character_ranges.end();
       ++range_iterator)
    {
      if (range_iterator->begin <= character
          && character <= range_iterator->end)
        {
          return !(this->negated);
        }
    }
  
  if (character_categories.size() > 0)
    {
      Unicode_data unicode;
      Unicode_data::character_entry_t character_entry;
      if (! unicode.Character_entry(character, &character_entry))
        return false;
      
      // Check general categories.
      for (vector<character_category_t>::const_iterator
             category_iterator = character_categories.begin();
           category_iterator != character_categories.end();
           ++category_iterator)
        {
          if (category_iterator->general_category
              == character_entry.general_category)
            {
              return !(this->negated);
            }
        }
    }
  
  return  (this->negated);
}
