// Clase:    Unicode_data
// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:41 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#ifndef _H_Unicode_data
#define _H_Unicode_data

#include <vector>
using std::vector;


class Unicode_data
{
 public:
  typedef long UCS4_char_t;// 32 bit.
  typedef enum
  {
    none,
    // Normative categories
    Lu,// Letter, Uppercase
    Ll,// Letter, Lowercase
    Lt,// Letter, Titlecase
    Mn,// Mark, Non-Spacing
    Mc,// Mark, Spacing Combining
    Me,// Mark, Enclosing
    Nd,// Number, Decimal Digit
    Nl,// Number, Letter
    No,// Number, Other
    Zs,// Separator, Space
    Zl,// Separator, Line
    Zp,// Separator, Paragraph
    Cc,// Other, Control
    Cf,// Other, Format
    Cs,// Other, Surrogate
    Co,// Other, Private Use
    Cn,// Other, Not Assigned (no characters in the file have this property)
    // Informative categories
    Lm,// Letter, Modifier
    Lo,// Letter, Other
    Pc,// Punctuation, Connector
    Pd,// Punctuation, Dash
    Ps,// Punctuation, Open
    Pe,// Punctuation, Close
    Pi,// Punctuation, Initial quote (may behave like Ps or Pe depending on
    //                                    usage)
    Pf,// Punctuation, Final quote (may behave like Ps or Pe depending on
    //                                  usage)
    Po,// Punctuation, Other
    Sm,// Symbol, Math
    Sc,// Symbol, Currency
    Sk,// Symbol, Modifier
    So // Symbol, Other
  } general_category_t, *general_category_p_t;
  typedef struct
  {
    UCS4_char_t code_point;
    general_category_t general_category;
    int decimal_digit_value;
  } character_entry_t, *character_entry_p_t;
 private:
  typedef struct
  {
    UCS4_char_t code_range_begin;
    UCS4_char_t code_range_end;
    general_category_t general_category;
    int decimal_digit_value;
  } character_entry_internal_t, *character_entry_internal_p_t;
  typedef vector<character_entry_internal_t> character_table_t, *character_table_p_t;
  static character_table_t character_table;
  static int character_table_lock;
 public:
  Unicode_data();
  ~Unicode_data();
 private:
  static bool Load_character_table();
 public:
  bool Character_entry(UCS4_char_t character, character_entry_p_t p_result);
};



#endif
