// Clase:    Unicode_translator
// Proyecto: QMath
// Inicio: 2002-07-18 19:32:13 UTC
// Fecha:  2009-12-17 21:56:41 UTC
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#include "Unicode_translator.h"

/////////////
// Unicode_translator - Class constructor.
// no arguments.
//public:
Unicode_translator::Unicode_translator()
{

}

/////////////
// Unicode_translator - Copy constructor.
// model: Object to be copied.
//public:
Unicode_translator::Unicode_translator(const Unicode_translator &model)
{

}

/////////////
// ~Unicode_translator - Class destructor.
// no arguments.
//public:
Unicode_translator::~Unicode_translator()
{

}


using std::vector;
using std::string;

/////////////
// UTF8 - Append the UTF-8 encoding to a C++ vector<unsigned char>, and return char count.
// character: UCS4 character.
// p_result: STL vector<unsigned char> to receive UTF-8 characters.
// returns length of UTF-8 encoding. -1 means error.
//public:
//static
int Unicode_translator::UTF8(const UCS4_char_t character, vector<unsigned char>* p_result)
{
  return UTF8(character, (vector<char>*)p_result);
}

/////////////
// UTF8 - Append the UTF-8 encoding to a C++ vector<char>, and return char count.
// character: UCS4 character.
// p_result: STL vector<char> to receive UTF-8 characters.
// returns length of UTF-8 encoding. -1 means error.
//public:
//static
int Unicode_translator::UTF8(const UCS4_char_t character, vector<char>* p_result)
{
  if (!p_result) return -1;
  
  UCS4_char_t ucs4_char;
  ucs4_char = (UCS4_char_t)character;
  if (ucs4_char < 0x00000080)
    {
      p_result->insert(p_result->end(), char(ucs4_char));
      return 1;
    }
  if ((unsigned long)ucs4_char < 0x00000800)
    {
      p_result->insert(p_result->end(), char(ucs4_char>>6 & 0x0000001F) | 0xC0);
      p_result->insert(p_result->end(), char(ucs4_char & 0x0000003F) | 0x80);
      return 2;
    }
  if ((unsigned long)ucs4_char < 0x00010000)
    {
      p_result->insert(p_result->end(), char(ucs4_char>>12 & 0x0000001F) | 0xE0);
      p_result->insert(p_result->end(), char(ucs4_char>>6 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char & 0x0000003F) | 0x80);
      return 3;
    }
  if ((unsigned long)ucs4_char < 0x00200000)
    {
      p_result->insert(p_result->end(), char(ucs4_char>>18 & 0x0000001F) | 0xF0);
      p_result->insert(p_result->end(), char(ucs4_char>>12 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char>>6 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char & 0x0000003F) | 0x80);
      return 4;
    }
  if ((unsigned long)ucs4_char < 0x04000000)
    {
      p_result->insert(p_result->end(), char(ucs4_char>>24 & 0x0000001F) | 0xF8);
      p_result->insert(p_result->end(), char(ucs4_char>>18 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char>>12 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char>>6 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char & 0x0000003F) | 0x80);
      return 5;
    }
  if ((unsigned long)ucs4_char < 0x80000000)
    {
      p_result->insert(p_result->end(), char(ucs4_char>>30 & 0x0000001F) | 0xFC);
      p_result->insert(p_result->end(), char(ucs4_char>>24 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char>>18 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char>>12 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char>>6 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char & 0x0000003F) | 0x80);
      return 6;
    }
  return -1;
}


/////////////
// UTF8 - Append the UTF-8 encoding to a C++ string, and return char count.
// character: UCS4 character.
// p_result: STL string to receive UTF-8 characters.
// returns length of UTF-8 encoding. -1 means error.
//public:
//static
int Unicode_translator::UTF8(const UCS4_char_t character, string* p_result)
{
  if (!p_result) return -1;
  
  UCS4_char_t ucs4_char;
  ucs4_char = (UCS4_char_t)character;
  if (ucs4_char < 0x00000080)
    {
      p_result->insert(p_result->end(), char(ucs4_char));
      return 1;
    }
  if ((unsigned long)ucs4_char < 0x00000800)
    {
      p_result->insert(p_result->end(), char(ucs4_char>>6 & 0x0000001F) | 0xC0);
      p_result->insert(p_result->end(), char(ucs4_char & 0x0000003F) | 0x80);
      return 2;
    }
  if ((unsigned long)ucs4_char < 0x00010000)
    {
      p_result->insert(p_result->end(), char(ucs4_char>>12 & 0x0000001F) | 0xE0);
      p_result->insert(p_result->end(), char(ucs4_char>>6 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char & 0x0000003F) | 0x80);
      return 3;
    }
  if ((unsigned long)ucs4_char < 0x00200000)
    {
      p_result->insert(p_result->end(), char(ucs4_char>>18 & 0x0000001F) | 0xF0);
      p_result->insert(p_result->end(), char(ucs4_char>>12 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char>>6 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char & 0x0000003F) | 0x80);
      return 4;
    }
  if ((unsigned long)ucs4_char < 0x04000000)
    {
      p_result->insert(p_result->end(), char(ucs4_char>>24 & 0x0000001F) | 0xF8);
      p_result->insert(p_result->end(), char(ucs4_char>>18 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char>>12 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char>>6 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char & 0x0000003F) | 0x80);
      return 5;
    }
  if ((unsigned long)ucs4_char < 0x80000000)
    {
      p_result->insert(p_result->end(), char(ucs4_char>>30 & 0x0000001F) | 0xFC);
      p_result->insert(p_result->end(), char(ucs4_char>>24 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char>>18 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char>>12 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char>>6 & 0x0000003F) | 0x80);
      p_result->insert(p_result->end(), char(ucs4_char & 0x0000003F) | 0x80);
      return 6;
    }
  return -1;
}


//public:
//static
bool Unicode_translator::UTF8(UCS4_char_p_t p_ucs4_char, const unsigned char utf8_char, int *p_run_length, int *p_correct_run_length)
{
  if ((utf8_char & 0x80) == 0x00 && *p_run_length == 0)
    {
      *p_run_length = 1;
      *p_correct_run_length = 1;
      *p_ucs4_char = utf8_char;
    }
  else if ((utf8_char & 0xFE) == 0xFE) // error in UTF-8 string.
    {
      *p_run_length = 1;
      *p_correct_run_length = -1;// Flag error.
      //*p_ucs4_char = utf8_char;
    }
  else if (((utf8_char & 0xC0) == 0xC0 && *p_run_length == 0) ||
           ((utf8_char & 0xC0) == 0x80 && *p_run_length > 0)
           )
    {
      ++*p_run_length;
      if (*p_run_length == 1)
        {
          int zero_pos;
          for (zero_pos = 7; zero_pos >= 0; --zero_pos)
            if (!(utf8_char & (1<<zero_pos))) break;
          *p_correct_run_length = 7 - zero_pos;
          *p_ucs4_char = utf8_char & ~(0xFF << zero_pos);
        }
      else if (*p_run_length <= *p_correct_run_length)
        {
          *p_ucs4_char = (*p_ucs4_char << 6) | (utf8_char & 0x3F);
        }
    }
  else
    *p_correct_run_length = -1;// Flag error.
  
  return false;
}


/////////////
// UTF8 - Set UCS4 char from UTF-8 encoded char vector.
// p_character: Pointer to UCS4_char_t.
// utf8_iterator: STL char vector iterator pointing to UTF-8 characters.
// utf8_iterator_end: End of STL char vector.
// returns count of UTF-8 chars (bytes) read. -1 means error.
//public:
//static
int Unicode_translator::UTF8(UCS4_char_p_t p_character, vector<char>::const_iterator* p_utf8_iterator, vector<char>::const_iterator utf8_iterator_end)
{
  if (!p_character || !p_utf8_iterator) return -1;
  
  int run_length = 0;
  int correct_run_length = 1;// If 0, it would match run_length and return.
  UCS4_char_t ucs4_char = 0;
  unsigned char utf8_char;
  while (*p_utf8_iterator < utf8_iterator_end)
    {
      utf8_char = (unsigned char)*((*p_utf8_iterator)++);
      
      UTF8(&ucs4_char, utf8_char, &run_length, &correct_run_length);
      
      if (run_length == correct_run_length)
        {
          *p_character = ucs4_char;
          break;// OK.
        }
      else if (run_length > correct_run_length)
        {
          *p_character = 0x0000FFFD;// REPLACEMENT CHAR. (Unicode FAQ)
          break;// Error.
        }
    }
  if (run_length < correct_run_length || correct_run_length == -1)
    return -1;// Error.
  else
    return run_length;
}

/////////////
// UTF8 - Set UCS4 char from UTF-8 encoded unsigned char vector.
// p_character: Pointer to UCS4_char_t.
// utf8_iterator: STL char vector iterator pointing to UTF-8 characters.
// utf8_iterator_end: End of STL char vector.
// returns count of UTF-8 chars (bytes) read. -1 means error.
//public:
//static
int Unicode_translator::UTF8(UCS4_char_p_t p_character, vector<unsigned char>::const_iterator* p_utf8_iterator, vector<unsigned char>::const_iterator utf8_iterator_end)
{
  if (!p_character || !p_utf8_iterator) return -1;
  
  int run_length = 0;
  int correct_run_length = 1;// If 0, it would match run_length and return.
  UCS4_char_t ucs4_char = 0;
  unsigned char utf8_char;
  while (*p_utf8_iterator < utf8_iterator_end)
    {
      utf8_char = (unsigned char)*((*p_utf8_iterator)++);
      
      UTF8(&ucs4_char, utf8_char, &run_length, &correct_run_length);
      
      if (run_length == correct_run_length)
        {
          *p_character = ucs4_char;
          break;// OK.
        }
      else if (run_length > correct_run_length)
        {
          *p_character = 0x0000FFFD;// REPLACEMENT CHAR. (Unicode FAQ)
          break;// Error.
        }
    }
  if (run_length < correct_run_length || correct_run_length == -1)
    return -1;// Error.
  else
    return run_length;
}

#ifndef USING_GCC_2_95
/////////////
// UTF8 - Set UCS4 char from UTF-8 encoded C++ string.
// p_character: Pointer to UCS4_char_t.
// utf8_iterator: C++ string iterator pointing to UTF-8 characters.
// utf8_iterator_end: End of C++ string.
// returns count of UTF-8 chars (bytes) read. -1 means error.
//public:
//static
int Unicode_translator::UTF8(UCS4_char_p_t p_character, string::const_iterator* p_utf8_iterator, string::const_iterator utf8_iterator_end)
{
  int run_length = 0;
  int correct_run_length = 1;// If 0, it would match run_length and return.
  UCS4_char_t ucs4_char = 0;
  unsigned char   utf8_char;
  while (*p_utf8_iterator < utf8_iterator_end)
    {
      utf8_char = *((*p_utf8_iterator)++);
      
      UTF8(&ucs4_char, utf8_char, &run_length, &correct_run_length);
      
      if (run_length == correct_run_length)
        {
          *p_character = ucs4_char;
          break;// OK.
        }
      else if (run_length > correct_run_length)
        {
          *p_character = 0x0000FFFD;// REPLACEMENT CHAR. (Unicode FAQ)
          break;// Error.
        }
    }
  if (run_length < correct_run_length || correct_run_length == -1)
    return -1;// Error.
  else
    return run_length;
}


/////////////
// UTF8 - Set UCS4 char from UTF-8 encoded char array.
// p_character: Pointer to UCS4_char_t.
// p_utf8_pointer: Pointer to char array pointer to UTF-8 characters.
// utf8_pointer_end: End of C char array.
// returns count of UTF8 chars (bytes) read. -1 means error.
//public:
//static
int Unicode_translator::UTF8(UCS4_char_p_t p_character, const char** p_utf8_pointer, const char* utf8_pointer_end)
{
  return UTF8(p_character,
              (const unsigned char**) p_utf8_pointer,
              (const unsigned char*)  utf8_pointer_end
              );
}

/////////////
// UTF8 - Set UCS4 char from UTF-8 encoded char array.
// p_character: Pointer to UCS4_char_t.
// p_utf8_pointer: Pointer to char array pointer to UTF-8 characters.
// utf8_pointer_end: End of C char array.
// returns count of UTF8 chars (bytes) read. -1 means error.
//public:
//static
int Unicode_translator::UTF8(UCS4_char_p_t p_character, const unsigned char** p_utf8_pointer, const unsigned char* utf8_pointer_end)
{
  if (!p_character || !p_utf8_pointer || !utf8_pointer_end) return -1;
  if (!*p_utf8_pointer || utf8_pointer_end <= *p_utf8_pointer) return -1;
  
  int run_length = 0;
  int correct_run_length = 1;// If 0, it would match run_length and return.
  UCS4_char_t ucs4_char = 0;
  unsigned char   utf8_char;
  while (*p_utf8_pointer < utf8_pointer_end)
    {
      utf8_char = *((*p_utf8_pointer)++);
      
      UTF8(&ucs4_char, utf8_char, &run_length, &correct_run_length);
      
      if (run_length == correct_run_length)
        {
          *p_character = ucs4_char;
          break;// OK.
        }
      else if (run_length > correct_run_length)
        {
          *p_character = 0x0000FFFD;// REPLACEMENT CHAR. (Unicode FAQ)
          break;// Error.
        }
    }
  if (run_length < correct_run_length || correct_run_length == -1)
    return -1;// Error.
  else
    return run_length;
}
#endif // USING_GCC_2_95


/////////////
// UTF8_length - Return the length of the string in UTF-8 encoding.
// no arguments.
// returns length of string in UTF8 format.
//public:
//static
int Unicode_translator::UTF8_length(const long character)
{
  UCS4_char_t ucs4_char;
  ucs4_char = (UCS4_char_t)character;
  if ((unsigned long)ucs4_char < 0x00000080) return 1;
  if ((unsigned long)ucs4_char < 0x00000800) return 2;
  if ((unsigned long)ucs4_char < 0x00010000) return 3;
  if ((unsigned long)ucs4_char < 0x00200000) return 4;
  if ((unsigned long)ucs4_char < 0x04000000) return 5;
  if ((unsigned long)ucs4_char < 0x80000000) return 6;
  return -1;// Error.
}
