// Clase:    XML_character_info_item
// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:43 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#include "XML_character_info_item.h"

/////////////
// XML_character_info_item - Class constructor.
// no arguments.
//public:
XML_character_info_item::XML_character_info_item()
{
  this->parent = NULL;
  this->code_point = -1;// This is an invalid value.
  Type(character_info_item);
}

/////////////
// ~XML_character_info_item - Class destructor.
// no arguments.
//public:
XML_character_info_item::~XML_character_info_item()
{

}

using std::ostream;

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
// XML_character_info_item: XML_character_info_item to output.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, XML_character_info_item_t &xml_character_info_item)
{
  XML_character_info_item::Output(&xml_character_info_item, output);
  return output;
}

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
// XML_character_info_item: XML_character_info_item to output, passed as pointer.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, XML_character_info_item_p_t xml_character_info_item)
{
  XML_character_info_item::Output(xml_character_info_item, output);
  return output;
}

using std::ostream;

/////////////
// Output - Write to a stream in UTF8 format.
// output: stream which receives data.
// returns true on success, false on fail.
//public:
//static
bool XML_character_info_item::Output(XML_character_info_item_p_t character, ostream &output)
{
  if (!character) return false;
  
  vector<char> buffer;
  switch (character->Character_code())
    {
    case XML_UCS4_char_t('<'):  output << "&lt;";   break;
    case XML_UCS4_char_t('>'):  output << "&gt;";   break;
    case XML_UCS4_char_t('&'):  output << "&amp;";  break;
    default:
      character->UTF8(&buffer);
      if (buffer.size() > 0)// Else, it's an invalid UCS4 char.
        {
          for (vector<char>::iterator buffer_iterator = buffer.begin();
               buffer_iterator != buffer.end();
               ++buffer_iterator)
            {
              output << *buffer_iterator;
            }
        }
      break;
    }
  
  return true;
}

using std::vector;
using std::string;

/////////////
// UTF8 - Append the UTF8 encoding to a char vector.
// p_result: STL char vector to receive UTF-8 characters.
// returns object pointer for operator concatenation.
//public:
XML_character_info_item_p_t XML_character_info_item::UTF8(vector<char>* p_result)
{
  Unicode_translator::UTF8(this->Character_code(), p_result);
  
  return this;
}


/////////////
// UTF8 - Set UCS4 char from UTF-8 encoded char vector.
// utf8_iterator: STL char vector iterator pointing to UTF-8 characters.
// utf8_iterator_end: End of STL char vector.
// returns object pointer for operator concatenation.
//public:
XML_character_info_item_p_t XML_character_info_item::UTF8(vector<char>::const_iterator* p_utf8_iterator, vector<char>::const_iterator utf8_iterator_end)
{
  XML_UCS4_char_t character_code;
  Unicode_translator::UTF8(&(character_code),
                           p_utf8_iterator, utf8_iterator_end);
  this->Character_code(character_code);
  return this;
}


/////////////
// UTF8 - Set UCS4 char from UTF-8 encoded C char array.
// p_utf8_pointer: Pointer to C char pointer to UTF-8 characters.
// utf8_pointer_end: End of C char array.
// returns object pointer for operator concatenation.
//public:
XML_character_info_item_p_t XML_character_info_item::UTF8(const unsigned char** p_utf8_pointer, const unsigned char* utf8_pointer_end)
{
  XML_UCS4_char_t character_code;
  Unicode_translator::UTF8(&(character_code),
                           p_utf8_pointer, utf8_pointer_end);
  this->Character_code(character_code);
  return this;
}


/////////////
// UTF8 - Set UCS4 char from UTF-8 encoded C char array.
// p_utf8_pointer: Pointer to C char pointer to UTF-8 characters.
// utf8_pointer_end: End of C char array.
// returns object pointer for operator concatenation.
//public:
XML_character_info_item_p_t XML_character_info_item::UTF8(const unsigned char* utf8_pointer, const unsigned char* utf8_pointer_end)
{
  XML_UCS4_char_t character_code;
  Unicode_translator::UTF8(&(character_code),
                           &utf8_pointer, utf8_pointer_end);
  this->Character_code(character_code);
  return this;
}


/////////////
// UTF8 - Set UCS4 char from zero-terminated UTF-8 encoded C char array.
// p_utf8_pointer: Pointer to C char pointer to utf-8 characters.
// returns object pointer for operator concatenation.
//public:
XML_character_info_item_p_t XML_character_info_item::UTF8(const unsigned char** p_utf8_pointer)
{
  XML_UCS4_char_t character_code;
  Unicode_translator::UTF8(&(character_code),
                           p_utf8_pointer,
                           (*p_utf8_pointer) + strlen((char*)*p_utf8_pointer));
  this->Character_code(character_code);
  return this;
}


/////////////
// UTF8_length - Return the length of the string in UTF-8 encoding.
// no arguments.
// returns length of UTF-8 encoding. -1 means error.
//public:
int XML_character_info_item::UTF8_length()
{
  return Unicode_translator::UTF8_length(this->Character_code());
}



/////////////
// Character_code - Set [character code] property.
// character_code: new value for property.
// returns object pointer for operator concatenation.
//public:
XML_character_info_item_p_t XML_character_info_item::Character_code(XML_UCS4_char_t character_code)
{
  this->code_point = character_code;
  
  return this;
}

/////////////
// Character_code - Report [character code] property.
// no arguments.
// returns property value.
//public:
XML_UCS4_char_t XML_character_info_item::Character_code()
{
  return this->code_point;
}




/////////////
// XML_character_info_item - Class constructor.
// parent: *_info_item to be the [parent] of the new object.
//         The new object is automatically inserted in [parent].
//public:
XML_character_info_item::XML_character_info_item(XML_elem_info_item_p_t parent)
{
  this->parent = NULL;
  this->code_point = -1;// This is an invalid value.
  Type(character_info_item);
  parent->Children(this);
  Parent(parent);
}


/////////////
// Parent - Set [parent] property.
// parent: new value for property. Can be 0.
// returns object pointer for operator concatenation.
//public:
XML_character_info_item_p_t XML_character_info_item::Parent(XML_elem_info_item_p_t parent)
{
  this->parent = parent;
  
  return this;
}

/////////////
// Parent - Access to [parent] property.
// no arguments.
// returns property value.
//public:
const XML_info_item_p_t XML_character_info_item::Parent()
{
  return (XML_info_item_p_t)(this->parent);
}
