// Clase:    XML_comment_info_item
// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:43 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#include "XML_comment_info_item.h"

/////////////
// XML_comment_info_item - Class constructor.
// no arguments.
//public:
XML_comment_info_item::XML_comment_info_item()
{
  this->content = NULL;
  this->parent = NULL;
  Type(comment_info_item);
}

/////////////
// ~XML_comment_info_item - Class destructor.
// no arguments.
//public:
XML_comment_info_item::~XML_comment_info_item()
{
  if (content)
    {
      delete content;
      content = NULL;
    }
}


using std::vector;


/////////////
// Content - Set [content] property.
// content: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_comment_info_item_p_t XML_comment_info_item::Content(XML_string_t &content)
{
  if (!(&content)) return this;
  if (!this->content)
    this->content = new XML_string();
  *(this->content) = content;
  return this;
}

/////////////
// Content - Set [content] property.
// content: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_comment_info_item_p_t XML_comment_info_item::Content(XML_string_p_t content)
{
  if (!content)
    {
      if (this->content)
        {
          delete this->content;
          this->content = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->content)
    this->content = new XML_string();
  *(this->content) = *content;
  return this;
}

/////////////
// Content - Set [content] property.
// content: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_comment_info_item_p_t XML_comment_info_item::Content(const vector<XML_UCS4_char_t> &content)
{
  if (!(&content)) return this;
  if (!this->content)
    this->content = new XML_string();
  this->content->UCS4(content);
  return this;
}

/////////////
// Content - Set [content] property.
// content: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_comment_info_item_p_t XML_comment_info_item::Content(const vector<char> &content)
{
  if (!(&content)) return this;
  if (!this->content)
    this->content = new XML_string();
  this->content->UTF8(content);
  return this;
}

/////////////
// Content - Set [content] property.
// content: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_comment_info_item_p_t XML_comment_info_item::Content(const char* content)
{
  if (!content)
    {
      if (this->content)
        {
          delete this->content;
          this->content = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->content)
    this->content = new XML_string();
  vector<char> content_vector;
  for (unsigned int index = 0; index < strlen(content); ++index)
    content_vector.push_back(content[index]);
  return Content(content_vector);
}

/////////////
// Content - Access to [content] property.
// no arguments.
// returns property value.
//public:
XML_string_p_t XML_comment_info_item::Content()
{
  return this->content;
}


using std::ostream;

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
// XML_comment_info_item: XML_comment_info_item to output.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, XML_comment_info_item_t &xml_comment_info_item)
{
  XML_comment_info_item::Output(&xml_comment_info_item, output);
  return output;
}

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
// XML_comment_info_item: XML_comment_info_item to output, passed as pointer.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, XML_comment_info_item_p_t xml_comment_info_item)
{
  XML_comment_info_item::Output(xml_comment_info_item, output);
  return output;
}

using std::ostream;
using std::endl;

/////////////
// Output - Write to a stream in XML format.
// item: XML_comment_info_item to be output, passed as pointer.
// output: stream which receives data.
// returns true on success, false on fail.
//public:
//static
bool XML_comment_info_item::Output(XML_comment_info_item_p_t item, ostream &output)
{
  return Output(item, output, 0);
}

/////////////
// Output - Write to a stream in XML format.
// item: XML_comment_info_item to be output, passed as pointer.
// output: stream which receives data.
// returns true on success, false on fail.
//public:
//static
bool XML_comment_info_item::Output(XML_comment_info_item_p_t item, ostream &output, unsigned int indentation)
{
  if (!item || !(&output)) return false;
  
  if (item->Content())
    output << endl << "<!-- " << item->Content() << " -->";
  
  return true;
}




/////////////
// XML_comment_info_item - Class constructor.
// parent: *_info_item to be the [parent] of the new object.
//         The new object is automatically inserted in [parent].
//public:
XML_comment_info_item::XML_comment_info_item(XML_doc_info_item_p_t parent)
{
  this->content = NULL;
  this->parent = NULL;
  Type(comment_info_item);
  parent->Children(this);
  Parent(parent);
}

/////////////
// XML_comment_info_item - Class constructor.
// parent: *_info_item to be the [parent] of the new object.
//         The new object is automatically inserted in [parent].
//public:
XML_comment_info_item::XML_comment_info_item(XML_elem_info_item_p_t parent)
{
  this->content = NULL;
  this->parent = NULL;
  Type(comment_info_item);
  parent->Children(this);
  Parent(parent);
}

/////////////
// XML_comment_info_item - Class constructor.
// parent: *_info_item to be the [parent] of the new object.
//         The new object is automatically inserted in [parent].
//public:
XML_comment_info_item::XML_comment_info_item(XML_doc_type_decl_info_item_p_t parent)
{
  this->content = NULL;
  this->parent = NULL;
  Type(comment_info_item);
  parent->Children(this);
  Parent(parent);
}


/////////////
// Parent - Set [parent] property.
// parent: new value for property. Can be 0.
// returns object pointer for operator concatenation.
//public:
XML_comment_info_item_p_t XML_comment_info_item::Parent(XML_doc_info_item_p_t parent)
{
  this->parent = parent;
  
  return this;
}
/////////////
// Parent - Set [parent] property.
// parent: new value for property. Can be 0.
// returns object pointer for operator concatenation.
//public:
XML_comment_info_item_p_t XML_comment_info_item::Parent(XML_elem_info_item_p_t parent)
{
  this->parent = parent;
  
  return this;
}
/////////////
// Parent - Set [parent] property.
// parent: new value for property. Can be 0.
// returns object pointer for operator concatenation.
//public:
XML_comment_info_item_p_t XML_comment_info_item::Parent(XML_doc_type_decl_info_item_p_t parent)
{
  this->parent = parent;
  
  return this;
}

/////////////
// Parent - Access to [parent] property.
// no arguments.
// returns property value.
//public:
const XML_info_item_p_t XML_comment_info_item::Parent()
{
  return (XML_info_item_p_t)(this->parent);
}
