// Clase:    XML_doc_info_item
// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:44 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#include "XML_doc_info_item.h"

/////////////
// XML_doc_info_item - Class constructor.
// no arguments.
//public:
XML_doc_info_item::XML_doc_info_item()
{
  this->document_entity = NULL;
  this->document_element = NULL;
  this->base_URI = NULL;
  this->standalone = not_present;
  this->version = NULL;
  this->all_declarations_processed = false;
  Type(doc_info_item);
  Version("1.0");
}

/////////////
// ~XML_doc_info_item - Class destructor.
// no arguments.
//public:
XML_doc_info_item::~XML_doc_info_item()
{
  if (document_entity)
    {
      delete document_entity;
      document_entity = NULL;
    }
  if (document_element)
    {
      delete document_element;
      document_element = NULL;
    }
  if (base_URI)
    {
      delete base_URI;
      base_URI = NULL;
    }
  if (version)
    {
      delete version;
      version = NULL;
    }
  Children_clear();
}




/////////////
// Children - Append to [children] list from STL vector.
// children: vector of pointers to objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Children(vector<XML_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    {
      switch ((*i)->Type())
        {
        case elem_info_item:
          this->Children(static_cast<XML_elem_info_item_p_t>(*i));
          break;
        case proc_inst_info_item:
          this->Children(static_cast<XML_proc_inst_info_item_p_t>(*i));
          break;
        case comment_info_item:
          this->Children(static_cast<XML_comment_info_item_p_t>(*i));
          break;
        case doc_type_decl_info_item:
          this->Children(static_cast<XML_doc_type_decl_info_item_p_t>(*i));
          break;
        default:
          ;// Ignore any others.
          break;
        }
    }
  
  return this;
}


/////////////
// Children - Append to [children] list from C array.
// array: pointer to array of pointers to objects to add.
// count: number of objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Children(XML_elem_info_item_p_t* array, int count)
{
  if (!array || count <= 0) return this;
  
  for (int index = 0; index < count; ++index)
    this->Children(array[index]);
  
  return this;
}

/////////////
// Children - Append to [children] list from STL vector.
// children: vector of pointers to objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Children(vector<XML_elem_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_elem_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    this->Children(*i);
  
  return this;
}

/////////////
// Children - Append a [children] element to list.
// item: pointer to item to add.
//       Object is appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Children(XML_elem_info_item_p_t item)
{
  if (!item) return this;
  
  this->children.push_back(item);
  item->Parent(this);
  
  return this;
}

/////////////
// Children - Append to [children] list from C array.
// array: pointer to array of pointers to objects to add.
// count: number of objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Children(XML_proc_inst_info_item_p_t* array, int count)
{
  if (!array || count <= 0) return this;
  
  for (int index = 0; index < count; ++index)
    this->Children(array[index]);
  
  return this;
}

/////////////
// Children - Append to [children] list from STL vector.
// children: vector of pointers to objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Children(vector<XML_proc_inst_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_proc_inst_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    this->Children(*i);
  
  return this;
}

/////////////
// Children - Append a [children] element to list.
// item: pointer to item to add.
//       Object is appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Children(XML_proc_inst_info_item_p_t item)
{
  if (!item) return this;
  
  this->children.push_back(item);
  item->Parent(this);
  
  return this;
}

/////////////
// Children - Append to [children] list from C array.
// array: pointer to array of pointers to objects to add.
// count: number of objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Children(XML_comment_info_item_p_t* array, int count)
{
  if (!array || count <= 0) return this;
  
  for (int index = 0; index < count; ++index)
    this->Children(array[index]);
  
  return this;
}

/////////////
// Children - Append to [children] list from STL vector.
// children: vector of pointers to objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Children(vector<XML_comment_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_comment_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    this->Children(*i);
  
  return this;
}

/////////////
// Children - Append a [children] element to list.
// item: pointer to item to add.
//       Object is appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Children(XML_comment_info_item_p_t item)
{
  if (!item) return this;
  
  this->children.push_back(item);
  item->Parent(this);
  
  return this;
}

/////////////
// Children - Append to [children] list from C array.
// array: pointer to array of pointers to objects to add.
// count: number of objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Children(XML_doc_type_decl_info_item_p_t* array, int count)
{
  if (!array || count <= 0) return this;
  
  for (int index = 0; index < count; ++index)
    this->Children(array[index]);
  
  return this;
}

/////////////
// Children - Append to [children] list from STL vector.
// children: vector of pointers to objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Children(vector<XML_doc_type_decl_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_doc_type_decl_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    this->Children(*i);
  
  return this;
}

/////////////
// Children - Append a [children] element to list.
// item: pointer to item to add.
//       Object is appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Children(XML_doc_type_decl_info_item_p_t item)
{
  if (!item) return this;
  
  this->children.push_back(item);
  item->Parent(this);
  
  return this;
}


/////////////
// Children - Access to [children] list for inspection.
// no arguments.
// returns [children] list as const. This means that the list
//       itself can't be modified by adding, removing, or reordering elements,
//       but the elements themselves can be modified.
//public:
const list<XML_info_item_p_t>& XML_doc_info_item::Children()
{
  return this->children;
}

/////////////
// Children_clear - Erase all items in [children].
// no arguments.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Children_clear()
{
  XML_info_item_p_t item;
  for (list<XML_info_item_p_t>::iterator
         item_iterator = children.begin();
       item_iterator != children.end();
       ++item_iterator)
    {
      item = *item_iterator;
      switch (item->Type())
        {
        case elem_info_item:
          if (this == XML_elem_info_item_p_t(item)->Parent())
            delete XML_elem_info_item_p_t(item);
          break;
        case proc_inst_info_item:
          if (this == XML_proc_inst_info_item_p_t(item)->Parent())
            delete XML_proc_inst_info_item_p_t(item);
          break;
        case unexp_ent_ref_info_item:
          if (this == XML_unexp_ent_ref_info_item_p_t(item)->Parent())
            delete XML_unexp_ent_ref_info_item_p_t(item);
          break;
        case character_info_item:
          if (this == XML_character_info_item_p_t(item)->Parent())
            delete XML_character_info_item_p_t(item);
          break;
        case comment_info_item:
          if (this == XML_comment_info_item_p_t(item)->Parent())
            delete XML_comment_info_item_p_t(item);
          break;
        case doc_type_decl_info_item:
          if (this == XML_doc_type_decl_info_item_p_t(item)->Parent())
            delete XML_doc_type_decl_info_item_p_t(item);
          break;
        case ent_start_marker_info_item:
          if (this == XML_ent_start_marker_info_item_p_t(item)->Parent())
            delete XML_ent_start_marker_info_item_p_t(item);
          break;
        case ent_end_marker_info_item:
          if (this == XML_ent_end_marker_info_item_p_t(item)->Parent())
            delete XML_ent_end_marker_info_item_p_t(item);
          break;
        case cdata_start_marker_info_item:
          if (this == XML_cdata_start_marker_info_item_p_t(item)->Parent())
            delete XML_cdata_start_marker_info_item_p_t(item);
          break;
        case cdata_end_marker_info_item:
          if (this == XML_cdata_end_marker_info_item_p_t(item)->Parent())
            delete XML_cdata_end_marker_info_item_p_t(item);
          break;
        case attr_info_item:
          if (this == XML_attr_info_item_p_t(item)->Owner_element())
            delete XML_attr_info_item_p_t(item);
          break;
        default:
          std::cerr << "Non-critical internal error: XML item type not handled." << std::endl;
        }
    }
  children.clear();
  
  return this;
}



using std::vector;


/////////////
// Version - Set [version] property.
// version: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Version(XML_string_t &version)
{
  if (!(&version)) return this;
  if (!this->version)
    this->version = new XML_string();
  *(this->version) = version;
  return this;
}

/////////////
// Version - Set [version] property.
// version: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Version(XML_string_p_t version)
{
  if (!version)
    {
      if (this->version)
        {
          delete this->version;
          this->version = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->version)
    this->version = new XML_string();
  *(this->version) = *version;
  return this;
}

/////////////
// Version - Set [version] property.
// version: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Version(const vector<XML_UCS4_char_t> &version)
{
  if (!(&version)) return this;
  if (!this->version)
    this->version = new XML_string();
  this->version->UCS4(version);
  return this;
}

/////////////
// Version - Set [version] property.
// version: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Version(const vector<char> &version)
{
  if (!(&version)) return this;
  if (!this->version)
    this->version = new XML_string();
  this->version->UTF8(version);
  return this;
}

/////////////
// Version - Set [version] property.
// version: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Version(const char* version)
{
  if (!version)
    {
      if (this->version)
        {
          delete this->version;
          this->version = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->version)
    this->version = new XML_string();
  vector<char> version_vector;
  for (unsigned int index = 0; index < strlen(version); ++index)
    version_vector.push_back(version[index]);
  return Version(version_vector);
}

/////////////
// Version - Access to [version] property.
// no arguments.
// returns property value.
//public:
XML_string_p_t XML_doc_info_item::Version()
{
  return this->version;
}


using std::ostream;

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
// XML_doc_info_item: XML_doc_info_item to output.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, XML_doc_info_item_t &xml_doc_info_item)
{
  XML_doc_info_item::Output(&xml_doc_info_item, output);
  return output;
}

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
// XML_doc_info_item: XML_doc_info_item to output, passed as pointer.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, XML_doc_info_item_p_t xml_doc_info_item)
{
  XML_doc_info_item::Output(xml_doc_info_item, output);
  return output;
}




/////////////
// Standalone - Set [standalone] property.
// standalone: new value for property.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Standalone(XML_standalone_t standalone)
{
  this->standalone = standalone;
  return this;
}

/////////////
// Standalone - Report [standalone] property.
// no arguments.
// returns property value.
//public:
XML_doc_info_item::XML_standalone_t XML_doc_info_item::Standalone()
{
  return this->standalone;
}



using std::vector;


/////////////
// Base_URI - Set [base URI] property.
// base_URI: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Base_URI(XML_string_t &base_URI)
{
  if (!(&base_URI)) return this;
  if (!this->base_URI)
    this->base_URI = new XML_string();
  *(this->base_URI) = base_URI;
  return this;
}

/////////////
// Base_URI - Set [base URI] property.
// base_URI: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Base_URI(XML_string_p_t base_URI)
{
  if (!base_URI)
    {
      if (this->base_URI)
        {
          delete this->base_URI;
          this->base_URI = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->base_URI)
    this->base_URI = new XML_string();
  *(this->base_URI) = *base_URI;
  return this;
}

/////////////
// Base_URI - Set [base URI] property.
// base_URI: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Base_URI(const vector<XML_UCS4_char_t> &base_URI)
{
  if (!(&base_URI)) return this;
  if (!this->base_URI)
    this->base_URI = new XML_string();
  this->base_URI->UCS4(base_URI);
  return this;
}

/////////////
// Base_URI - Set [base URI] property.
// base_URI: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Base_URI(const vector<char> &base_URI)
{
  if (!(&base_URI)) return this;
  if (!this->base_URI)
    this->base_URI = new XML_string();
  this->base_URI->UTF8(base_URI);
  return this;
}

/////////////
// Base_URI - Set [base URI] property.
// base_URI: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Base_URI(const char* base_URI)
{
  if (!base_URI)
    {
      if (this->base_URI)
        {
          delete this->base_URI;
          this->base_URI = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->base_URI)
    this->base_URI = new XML_string();
  vector<char> base_URI_vector;
  for (unsigned int index = 0; index < strlen(base_URI); ++index)
    base_URI_vector.push_back(base_URI[index]);
  return Base_URI(base_URI_vector);
}

/////////////
// Base_URI - Access to [base URI] property.
// no arguments.
// returns property value.
//public:
XML_string_p_t XML_doc_info_item::Base_URI()
{
  return this->base_URI;
}





/////////////
// Entities - Add to [entities] set from STL vector.
// children: vector of pointers to objects to add.
//        Objects are added to whatever was previously in the set.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Entities(vector<XML_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    {
      switch ((*i)->Type())
        {
        case internal_ent_info_item:
          this->Entities(static_cast<XML_internal_ent_info_item_p_t>(*i));
          break;
        case external_ent_info_item:
          this->Entities(static_cast<XML_external_ent_info_item_p_t>(*i));
          break;
        default:
          ;// Ignore any others.
          break;
        }
    }
  
  return this;
}


/////////////
// Entities - Add a [entities] element to set.
// item: pointer to item to add.
//       Object is added to whatever was previously in the set.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Entities(XML_internal_ent_info_item_p_t item)
{
  if (!item) return this;
  
  this->entities.insert(item);
  
  return this;
}

/////////////
// Entities - Add a [entities] element to set.
// item: pointer to item to add.
//       Object is added to whatever was previously in the set.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Entities(XML_external_ent_info_item_p_t item)
{
  if (!item) return this;
  
  this->entities.insert(item);
  
  return this;
}


/////////////
// Entities - Access to [entities] set.
// no arguments.
// returns [entities] list.
//public:
set<XML_info_item_p_t>& XML_doc_info_item::Entities()
{
  return this->entities;
}

using std::ostream;
using std::endl;

/////////////
// Output - Write to a stream in XML format.
// item: XML_doc_info_item to be output, passed as pointer.
// output: stream which receives data.
// returns true on success, false on fail.
//public:
//static
bool XML_doc_info_item::Output(XML_doc_info_item_p_t item, ostream &output)
{
  return Output(item, output, 0);
}

/////////////
// Output - Write to a stream in XML format.
// item: XML_doc_info_item to be output, passed as pointer.
// output: stream which receives data.
// returns true on success, false on fail.
//public:
//static
bool XML_doc_info_item::Output(XML_doc_info_item_p_t item, ostream &output, unsigned int indentation)
{
  if (!item || !(&output)) return false;
  
  if (indentation)
    output << endl;
  for (unsigned int count = 0; count < indentation; ++count)
    output << "  ";
  output << "<?xml";
  if (item->Version())
    output << " version='" << item->Version() << "'";
  output << " encoding='UTF-8'";
  switch (item->Standalone())
    {
    case yes:    output << " standalone='yes'"; break;
    case no:     output << " standalone='no'";  break;
    default:     break;// not_present
    }
  output << "?>";
  
  for (list<XML_info_item_p_t>::const_iterator
         item_p_iterator = item->Children().begin();
       item_p_iterator != item->Children().end();
       ++item_p_iterator)
    {
      if (0 == indentation)
        output << "\n";
      XML_info_item::Output(*item_p_iterator, output, indentation);
    }
  
  if (item->Document_element())
    {
      if (0 == indentation)
        output << "\n";
      XML_info_item::Output(item->Document_element(), output, indentation);
    }
  
  if (item->Document_entity())
    {
      if (0 == indentation)
        output << "\n";
      XML_info_item::Output(item->Document_entity(), output, indentation);
    }
  
  return true;
}





/////////////
// Document_element - Set [document element] property.
// document_element: new value for property. Can be 0.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Document_element(XML_elem_info_item_p_t document_element)
{
  this->document_element = document_element;
  
  if (!document_element) return this;
  
  document_element->Parent(this);
  
  return this;
}

/////////////
// Document_element - Access to [document element] property.
// no arguments.
// returns property value.
//public:
const XML_info_item_p_t XML_doc_info_item::Document_element()
{
  return (XML_info_item_p_t)(this->document_element);
}






/////////////
// Document_entity - Set [document entity] property.
// document_entity: new value for property. Can be 0.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Document_entity(XML_internal_ent_info_item_p_t document_entity)
{
  this->document_entity = document_entity;
  
  return this;
}
/////////////
// Document_entity - Set [document entity] property.
// document_entity: new value for property. Can be 0.
// returns object pointer for operator concatenation.
//public:
XML_doc_info_item_p_t XML_doc_info_item::Document_entity(XML_external_ent_info_item_p_t document_entity)
{
  this->document_entity = document_entity;
  
  return this;
}

/////////////
// Document_entity - Access to [document entity] property.
// no arguments.
// returns property value.
//public:
const XML_info_item_p_t XML_doc_info_item::Document_entity()
{
  return (XML_info_item_p_t)(this->document_entity);
}
