// Clase:    XML_doc_type_decl_info_item
// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:44 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#include "XML_doc_type_decl_info_item.h"

/////////////
// XML_doc_type_decl_info_item - Class constructor.
// no arguments.
//public:
XML_doc_type_decl_info_item::XML_doc_type_decl_info_item()
{
  this->system_identifier = NULL;
  this->public_identifier = NULL;
  this->parent = NULL;
  Type(doc_type_decl_info_item);
}

/////////////
// ~XML_doc_type_decl_info_item - Class destructor.
// no arguments.
//public:
XML_doc_type_decl_info_item::~XML_doc_type_decl_info_item()
{
  if (system_identifier)
    {
      delete system_identifier;
      system_identifier = NULL;
    }
  if (public_identifier)
    {
      delete public_identifier;
      public_identifier = NULL;
    }
  Children_clear();
}




/////////////
// Children - Append to [children] list from STL vector.
// children: vector of pointers to objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_type_decl_info_item_p_t XML_doc_type_decl_info_item::Children(vector<XML_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    {
      switch ((*i)->Type())
        {
        case proc_inst_info_item:
          this->Children(static_cast<XML_proc_inst_info_item_p_t>(*i));
          break;
        case comment_info_item:
          this->Children(static_cast<XML_comment_info_item_p_t>(*i));
          break;
        default:
          ;// Ignore any others.
          break;
        }
    }
  
  return this;
}


/////////////
// Children - Append to [children] list from C array.
// array: pointer to array of pointers to objects to add.
// count: number of objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_type_decl_info_item_p_t XML_doc_type_decl_info_item::Children(XML_proc_inst_info_item_p_t* array, int count)
{
  if (!array || count <= 0) return this;
  
  for (int index = 0; index < count; ++index)
    this->Children(array[index]);
  
  return this;
}

/////////////
// Children - Append to [children] list from STL vector.
// children: vector of pointers to objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_type_decl_info_item_p_t XML_doc_type_decl_info_item::Children(vector<XML_proc_inst_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_proc_inst_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    this->Children(*i);
  
  return this;
}

/////////////
// Children - Append a [children] element to list.
// item: pointer to item to add.
//       Object is appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_type_decl_info_item_p_t XML_doc_type_decl_info_item::Children(XML_proc_inst_info_item_p_t item)
{
  if (!item) return this;
  
  this->children.push_back(item);
  item->Parent(this);
  
  return this;
}

/////////////
// Children - Append to [children] list from C array.
// array: pointer to array of pointers to objects to add.
// count: number of objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_type_decl_info_item_p_t XML_doc_type_decl_info_item::Children(XML_comment_info_item_p_t* array, int count)
{
  if (!array || count <= 0) return this;
  
  for (int index = 0; index < count; ++index)
    this->Children(array[index]);
  
  return this;
}

/////////////
// Children - Append to [children] list from STL vector.
// children: vector of pointers to objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_type_decl_info_item_p_t XML_doc_type_decl_info_item::Children(vector<XML_comment_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_comment_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    this->Children(*i);
  
  return this;
}

/////////////
// Children - Append a [children] element to list.
// item: pointer to item to add.
//       Object is appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_doc_type_decl_info_item_p_t XML_doc_type_decl_info_item::Children(XML_comment_info_item_p_t item)
{
  if (!item) return this;
  
  this->children.push_back(item);
  item->Parent(this);
  
  return this;
}


/////////////
// Children - Access to [children] list for inspection.
// no arguments.
// returns [children] list as const. This means that the list
//       itself can't be modified by adding, removing, or reordering elements,
//       but the elements themselves can be modified.
//public:
const list<XML_info_item_p_t>& XML_doc_type_decl_info_item::Children()
{
  return this->children;
}

/////////////
// Children_clear - Erase all items in [children].
// no arguments.
// returns object pointer for operator concatenation.
//public:
XML_doc_type_decl_info_item_p_t XML_doc_type_decl_info_item::Children_clear()
{
  XML_info_item_p_t item;
  for (list<XML_info_item_p_t>::iterator
         item_iterator = children.begin();
       item_iterator != children.end();
       ++item_iterator)
    {
      item = *item_iterator;
      switch (item->Type())
        {
        case elem_info_item:
          if (this == XML_elem_info_item_p_t(item)->Parent())
            delete XML_elem_info_item_p_t(item);
          break;
        case proc_inst_info_item:
          if (this == XML_proc_inst_info_item_p_t(item)->Parent())
            delete XML_proc_inst_info_item_p_t(item);
          break;
        case unexp_ent_ref_info_item:
          if (this == XML_unexp_ent_ref_info_item_p_t(item)->Parent())
            delete XML_unexp_ent_ref_info_item_p_t(item);
          break;
        case character_info_item:
          if (this == XML_character_info_item_p_t(item)->Parent())
            delete XML_character_info_item_p_t(item);
          break;
        case comment_info_item:
          if (this == XML_comment_info_item_p_t(item)->Parent())
            delete XML_comment_info_item_p_t(item);
          break;
        case doc_type_decl_info_item:
          if (this == XML_doc_type_decl_info_item_p_t(item)->Parent())
            delete XML_doc_type_decl_info_item_p_t(item);
          break;
        case ent_start_marker_info_item:
          if (this == XML_ent_start_marker_info_item_p_t(item)->Parent())
            delete XML_ent_start_marker_info_item_p_t(item);
          break;
        case ent_end_marker_info_item:
          if (this == XML_ent_end_marker_info_item_p_t(item)->Parent())
            delete XML_ent_end_marker_info_item_p_t(item);
          break;
        case cdata_start_marker_info_item:
          if (this == XML_cdata_start_marker_info_item_p_t(item)->Parent())
            delete XML_cdata_start_marker_info_item_p_t(item);
          break;
        case cdata_end_marker_info_item:
          if (this == XML_cdata_end_marker_info_item_p_t(item)->Parent())
            delete XML_cdata_end_marker_info_item_p_t(item);
          break;
        case attr_info_item:
          if (this == XML_attr_info_item_p_t(item)->Owner_element())
            delete XML_attr_info_item_p_t(item);
          break;
        default:
          std::cerr << "Non-critical internal error: XML item type not handled." << std::endl;
        }
    }
  children.clear();
  
  return this;
}


using std::ostream;

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
// XML_doc_type_decl_info_item: XML_doc_type_decl_info_item to output.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, XML_doc_type_decl_info_item_t &xml_doc_type_decl_info_item)
{
  XML_doc_type_decl_info_item::Output(&xml_doc_type_decl_info_item, output);
  return output;
}

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
// XML_doc_type_decl_info_item: XML_doc_type_decl_info_item to output, passed as pointer.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, XML_doc_type_decl_info_item_p_t xml_doc_type_decl_info_item)
{
  XML_doc_type_decl_info_item::Output(xml_doc_type_decl_info_item, output);
  return output;
}

using std::ostream;
using std::endl;

/////////////
// Output - Write to a stream in XML format.
// item: XML_doc_type_decl_info_item to be output, passed as pointer.
// output: stream which receives data.
// returns true on success, false on fail.
//public:
//static
bool XML_doc_type_decl_info_item::Output(XML_doc_type_decl_info_item_p_t item, ostream &output)
{
  return Output(item, output, 0);
}

/////////////
// Output - Write to a stream in XML format.
// item: XML_doc_type_decl_info_item to be output, passed as pointer.
// output: stream which receives data.
// returns true on success, false on fail.
//public:
//static
bool XML_doc_type_decl_info_item::Output(XML_doc_type_decl_info_item_p_t item, ostream &output, unsigned int indentation)
{
  if (!item || !(&output)) return false;
  
  if (indentation)
    output << endl;
  for (unsigned int count = 0; count < indentation; ++count)
    output << "  ";
  output << "<";
  output << "!DOCTYPE";
  
  XML_doc_info_item_p_t document = NULL;
  XML_string_p_t document_element_name = NULL;
  XML_string_p_t document_base_uri = NULL;
  document = ((XML_doc_info_item_p_t)item->Parent());
  if (document)
    if (document->Document_element())
      {
        XML_elem_info_item_p_t document_element;
        document_element = (XML_elem_info_item_p_t)
          document->Document_element();
        if (document_element->Local_name())
          document_element_name = document_element->Local_name();
        if (document->Base_URI())
          document_base_uri = document->Base_URI();
      }
  
  if (document_element_name)
    output << " " << document_element_name;
  if (item->Public_identifier())
    {
      output << "\n PUBLIC '"
             << item->Public_identifier()
             << "'";
      if (item->System_identifier())
        {
          output << "\n '";
          if (document_base_uri)
            output << document_base_uri;
          output << item->System_identifier()
                 << "'";
        }
    }
  else if (item->System_identifier())
    {
      output << "\n SYSTEM '";
      if (document_base_uri)
        output << document_base_uri;
      output << item->System_identifier()
             << "'";
    }
  
  if (item->Parent())
    {
      XML_doc_info_item_p_t document;
      document = ((XML_doc_info_item_p_t)item->Parent());
      if (document->Entities().size() > 0)
        {
          output << " [";
          for (set<XML_info_item_p_t>::const_iterator
                 info_item_iterator
                 = ((XML_doc_info_item_p_t)item->Parent())->Entities().begin();
               info_item_iterator
                 != ((XML_doc_info_item_p_t)item->Parent())->Entities().end();
               ++info_item_iterator)
            {
              output << endl << *info_item_iterator;
            }
          output << "\n]";
        }
    }
  
  output << ">";
  
  for (list<XML_info_item_p_t>::const_iterator item_p_iterator
         = item->Children().begin();
       item_p_iterator != item->Children().end();
       ++item_p_iterator)
    {
      XML_info_item::Output(*item_p_iterator, output, indentation + 1);
    }
  
  return true;
}


using std::vector;


/////////////
// System_identifier - Set [system identifier] property.
// system_identifier: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_doc_type_decl_info_item_p_t XML_doc_type_decl_info_item::System_identifier(XML_string_t &system_identifier)
{
  if (!(&system_identifier)) return this;
  if (!this->system_identifier)
    this->system_identifier = new XML_string();
  *(this->system_identifier) = system_identifier;
  return this;
}

/////////////
// System_identifier - Set [system identifier] property.
// system_identifier: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_doc_type_decl_info_item_p_t XML_doc_type_decl_info_item::System_identifier(XML_string_p_t system_identifier)
{
  if (!system_identifier)
    {
      if (this->system_identifier)
        {
          delete this->system_identifier;
          this->system_identifier = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->system_identifier)
    this->system_identifier = new XML_string();
  *(this->system_identifier) = *system_identifier;
  return this;
}

/////////////
// System_identifier - Set [system identifier] property.
// system_identifier: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_doc_type_decl_info_item_p_t XML_doc_type_decl_info_item::System_identifier(const vector<XML_UCS4_char_t> &system_identifier)
{
  if (!(&system_identifier)) return this;
  if (!this->system_identifier)
    this->system_identifier = new XML_string();
  this->system_identifier->UCS4(system_identifier);
  return this;
}

/////////////
// System_identifier - Set [system identifier] property.
// system_identifier: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_doc_type_decl_info_item_p_t XML_doc_type_decl_info_item::System_identifier(const vector<char> &system_identifier)
{
  if (!(&system_identifier)) return this;
  if (!this->system_identifier)
    this->system_identifier = new XML_string();
  this->system_identifier->UTF8(system_identifier);
  return this;
}

/////////////
// System_identifier - Set [system identifier] property.
// system_identifier: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_doc_type_decl_info_item_p_t XML_doc_type_decl_info_item::System_identifier(const char* system_identifier)
{
  if (!system_identifier)
    {
      if (this->system_identifier)
        {
          delete this->system_identifier;
          this->system_identifier = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->system_identifier)
    this->system_identifier = new XML_string();
  vector<char> system_identifier_vector;
  for (unsigned int index = 0; index < strlen(system_identifier); ++index)
    system_identifier_vector.push_back(system_identifier[index]);
  return System_identifier(system_identifier_vector);
}

/////////////
// System_identifier - Access to [system identifier] property.
// no arguments.
// returns property value.
//public:
XML_string_p_t XML_doc_type_decl_info_item::System_identifier()
{
  return this->system_identifier;
}



using std::vector;


/////////////
// Public_identifier - Set [public identifier] property.
// public_identifier: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_doc_type_decl_info_item_p_t XML_doc_type_decl_info_item::Public_identifier(XML_string_t &public_identifier)
{
  if (!(&public_identifier)) return this;
  if (!this->public_identifier)
    this->public_identifier = new XML_string();
  *(this->public_identifier) = public_identifier;
  return this;
}

/////////////
// Public_identifier - Set [public identifier] property.
// public_identifier: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_doc_type_decl_info_item_p_t XML_doc_type_decl_info_item::Public_identifier(XML_string_p_t public_identifier)
{
  if (!public_identifier)
    {
      if (this->public_identifier)
        {
          delete this->public_identifier;
          this->public_identifier = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->public_identifier)
    this->public_identifier = new XML_string();
  *(this->public_identifier) = *public_identifier;
  return this;
}

/////////////
// Public_identifier - Set [public identifier] property.
// public_identifier: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_doc_type_decl_info_item_p_t XML_doc_type_decl_info_item::Public_identifier(const vector<XML_UCS4_char_t> &public_identifier)
{
  if (!(&public_identifier)) return this;
  if (!this->public_identifier)
    this->public_identifier = new XML_string();
  this->public_identifier->UCS4(public_identifier);
  return this;
}

/////////////
// Public_identifier - Set [public identifier] property.
// public_identifier: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_doc_type_decl_info_item_p_t XML_doc_type_decl_info_item::Public_identifier(const vector<char> &public_identifier)
{
  if (!(&public_identifier)) return this;
  if (!this->public_identifier)
    this->public_identifier = new XML_string();
  this->public_identifier->UTF8(public_identifier);
  return this;
}

/////////////
// Public_identifier - Set [public identifier] property.
// public_identifier: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_doc_type_decl_info_item_p_t XML_doc_type_decl_info_item::Public_identifier(const char* public_identifier)
{
  if (!public_identifier)
    {
      if (this->public_identifier)
        {
          delete this->public_identifier;
          this->public_identifier = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->public_identifier)
    this->public_identifier = new XML_string();
  vector<char> public_identifier_vector;
  for (unsigned int index = 0; index < strlen(public_identifier); ++index)
    public_identifier_vector.push_back(public_identifier[index]);
  return Public_identifier(public_identifier_vector);
}

/////////////
// Public_identifier - Access to [public identifier] property.
// no arguments.
// returns property value.
//public:
XML_string_p_t XML_doc_type_decl_info_item::Public_identifier()
{
  return this->public_identifier;
}





/////////////
// XML_doc_type_decl_info_item - Class constructor.
// parent: *_info_item to be the [parent] of the new object.
//         The new object is automatically inserted in [parent].
//public:
XML_doc_type_decl_info_item::XML_doc_type_decl_info_item(XML_doc_info_item_p_t parent)
{
  this->system_identifier = NULL;
  this->public_identifier = NULL;
  this->parent = NULL;
  Type(doc_type_decl_info_item);
  parent->Children(this);
  Parent(parent);
}


/////////////
// Parent - Set [parent] property.
// parent: new value for property. Can be 0.
// returns object pointer for operator concatenation.
//public:
XML_doc_type_decl_info_item_p_t XML_doc_type_decl_info_item::Parent(XML_doc_info_item_p_t parent)
{
  this->parent = parent;
  
  return this;
}

/////////////
// Parent - Access to [parent] property.
// no arguments.
// returns property value.
//public:
const XML_info_item_p_t XML_doc_type_decl_info_item::Parent()
{
  return (XML_info_item_p_t)(this->parent);
}
