// Clase:    XML_elem_info_item
// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:45 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#include "XML_elem_info_item.h"

/////////////
// XML_elem_info_item - Class constructor.
// no arguments.
//public:
XML_elem_info_item::XML_elem_info_item()
{
  this->namespace_name = NULL;
  this->local_name = NULL;
  this->prefix = NULL;
  this->base_URI = NULL;
  this->parent = NULL;
  
  Type(elem_info_item);
}

/////////////
// ~XML_elem_info_item - Class destructor.
// no arguments.
//public:
XML_elem_info_item::~XML_elem_info_item()
{
  if (namespace_name)
    {
      delete namespace_name;
      namespace_name = NULL;
    }
  if (local_name)
    {
      delete local_name;
      local_name = NULL;
    }
  if (prefix)
    {
      delete prefix;
      prefix = NULL;
    }
  if (base_URI)
    {
      delete base_URI;
      base_URI = NULL;
    }
  Children_clear();
}


using std::vector;


/////////////
// Namespace_name - Set [namespace name] property.
// namespace_name: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Namespace_name(XML_string_t &namespace_name)
{
  if (!(&namespace_name)) return this;
  if (!this->namespace_name)
    this->namespace_name = new XML_string();
  *(this->namespace_name) = namespace_name;
  return this;
}

/////////////
// Namespace_name - Set [namespace name] property.
// namespace_name: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Namespace_name(XML_string_p_t namespace_name)
{
  if (!namespace_name)
    {
      if (this->namespace_name)
        {
          delete this->namespace_name;
          this->namespace_name = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->namespace_name)
    this->namespace_name = new XML_string();
  *(this->namespace_name) = *namespace_name;
  return this;
}

/////////////
// Namespace_name - Set [namespace name] property.
// namespace_name: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Namespace_name(const vector<XML_UCS4_char_t> &namespace_name)
{
  if (!(&namespace_name)) return this;
  if (!this->namespace_name)
    this->namespace_name = new XML_string();
  this->namespace_name->UCS4(namespace_name);
  return this;
}

/////////////
// Namespace_name - Set [namespace name] property.
// namespace_name: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Namespace_name(const vector<char> &namespace_name)
{
  if (!(&namespace_name)) return this;
  if (!this->namespace_name)
    this->namespace_name = new XML_string();
  this->namespace_name->UTF8(namespace_name);
  return this;
}

/////////////
// Namespace_name - Set [namespace name] property.
// namespace_name: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Namespace_name(const char* namespace_name)
{
  if (!namespace_name)
    {
      if (this->namespace_name)
        {
          delete this->namespace_name;
          this->namespace_name = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->namespace_name)
    this->namespace_name = new XML_string();
  vector<char> namespace_name_vector;
  for (unsigned int index = 0; index < strlen(namespace_name); ++index)
    namespace_name_vector.push_back(namespace_name[index]);
  return Namespace_name(namespace_name_vector);
}

/////////////
// Namespace_name - Access to [namespace name] property.
// no arguments.
// returns property value.
//public:
XML_string_p_t XML_elem_info_item::Namespace_name()
{
  return this->namespace_name;
}





/////////////
// Children - Append to [children] list from STL vector.
// children: vector of pointers to objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(vector<XML_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    {
      switch ((*i)->Type())
        {
        case elem_info_item:
          this->Children(static_cast<XML_elem_info_item_p_t>(*i));
          break;
        case proc_inst_info_item:
          this->Children(static_cast<XML_proc_inst_info_item_p_t>(*i));
          break;
        case unexp_ent_ref_info_item:
          this->Children(static_cast<XML_unexp_ent_ref_info_item_p_t>(*i));
          break;
        case character_info_item:
          this->Children(static_cast<XML_character_info_item_p_t>(*i));
          break;
        case comment_info_item:
          this->Children(static_cast<XML_comment_info_item_p_t>(*i));
          break;
        case ent_start_marker_info_item:
          this->Children(static_cast<XML_ent_start_marker_info_item_p_t>(*i));
          break;
        case ent_end_marker_info_item:
          this->Children(static_cast<XML_ent_end_marker_info_item_p_t>(*i));
          break;
        case cdata_start_marker_info_item:
          this->Children(static_cast<XML_cdata_start_marker_info_item_p_t>(*i));
          break;
        case cdata_end_marker_info_item:
          this->Children(static_cast<XML_cdata_end_marker_info_item_p_t>(*i));
          break;
        default:
          ;// Ignore any others.
          break;
        }
    }
  
  return this;
}


/////////////
// Children - Append to [children] list from C array.
// array: pointer to array of pointers to objects to add.
// count: number of objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_elem_info_item_p_t* array, int count)
{
  if (!array || count <= 0) return this;
  
  for (int index = 0; index < count; ++index)
    this->Children(array[index]);
  
  return this;
}

/////////////
// Children - Append to [children] list from STL vector.
// children: vector of pointers to objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(vector<XML_elem_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_elem_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    this->Children(*i);
  
  return this;
}

/////////////
// Children - Append a [children] element to list.
// item: pointer to item to add.
//       Object is appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_elem_info_item_p_t item)
{
  if (!item) return this;
  
  this->children.push_back(item);
  item->Parent(this);
  
  return this;
}

/////////////
// Children - Append to [children] list from C array.
// array: pointer to array of pointers to objects to add.
// count: number of objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_proc_inst_info_item_p_t* array, int count)
{
  if (!array || count <= 0) return this;
  
  for (int index = 0; index < count; ++index)
    this->Children(array[index]);
  
  return this;
}

/////////////
// Children - Append to [children] list from STL vector.
// children: vector of pointers to objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(vector<XML_proc_inst_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_proc_inst_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    this->Children(*i);
  
  return this;
}

/////////////
// Children - Append a [children] element to list.
// item: pointer to item to add.
//       Object is appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_proc_inst_info_item_p_t item)
{
  if (!item) return this;
  
  this->children.push_back(item);
  item->Parent(this);
  
  return this;
}

/////////////
// Children - Append to [children] list from C array.
// array: pointer to array of pointers to objects to add.
// count: number of objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_unexp_ent_ref_info_item_p_t* array, int count)
{
  if (!array || count <= 0) return this;
  
  for (int index = 0; index < count; ++index)
    this->Children(array[index]);
  
  return this;
}

/////////////
// Children - Append to [children] list from STL vector.
// children: vector of pointers to objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(vector<XML_unexp_ent_ref_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_unexp_ent_ref_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    this->Children(*i);
  
  return this;
}

/////////////
// Children - Append a [children] element to list.
// item: pointer to item to add.
//       Object is appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_unexp_ent_ref_info_item_p_t item)
{
  if (!item) return this;
  
  this->children.push_back(item);
  item->Parent(this);
  
  return this;
}

/////////////
// Children - Append to [children] list from C array.
// array: pointer to array of pointers to objects to add.
// count: number of objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_character_info_item_p_t* array, int count)
{
  if (!array || count <= 0) return this;
  
  for (int index = 0; index < count; ++index)
    this->Children(array[index]);
  
  return this;
}

/////////////
// Children - Append to [children] list from STL vector.
// children: vector of pointers to objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(vector<XML_character_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_character_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    this->Children(*i);
  
  return this;
}

/////////////
// Children - Append a [children] element to list.
// item: pointer to item to add.
//       Object is appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_character_info_item_p_t item)
{
  if (!item) return this;
  
  this->children.push_back(item);
  item->Parent(this);
  
  return this;
}

/////////////
// Children - Fill [children] list from XML string.
// source: XML_string.
//        Objects are appended to whatever was previously in the vector.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_string_t &source)
{
  if (!(&source)) return this;
  
  return Children(source.UCS4_string());
}

/////////////
// Children - Fill [children] list from XML string pointer.
// source: pointer to XML_string.
//        Objects are appended to whatever was previously in the vector.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_string_p_t source_p)
{
  if (!source_p) return this;
  
  return Children(source_p->UCS4_string());
}

/////////////
// Children - Fill [children] list from STL vector.
// source: XML_UCS4_char vector.
//        Objects are appended to whatever was previously in the vector.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(const vector<XML_UCS4_char_t> &source)
{
  if (!(&source)) return this;
  
  vector<XML_UCS4_char_t>::const_iterator source_iterator;
  for (source_iterator = source.begin();
       source_iterator < source.end();
       ++source_iterator)
    this->Children((new XML_character_info_item)
                   ->Parent(this)
                   ->Character_code(*source_iterator)
                   );
  
  return this;
}

/////////////
// Children - Fill [children] list from STL vector pointer.
// source: pointer to XML_UCS4_char vector.
//        Objects are appended to whatever was previously in the vector.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(const vector<XML_UCS4_char_t>* source_p)
{
  if (!source_p) return this;
  
  vector<XML_UCS4_char_t>::const_iterator source_iterator;
  for (source_iterator = source_p->begin();
       source_iterator < source_p->end();
       ++source_iterator)
    this->Children((new XML_character_info_item)
                   ->Parent(this)
                   ->Character_code(*source_iterator)
                   );
  
  return this;
}

/////////////
// Children - Fill [children] list from C++ char vector.
// array: pointer to char array.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(vector<char> &array)
{
  vector<char>::const_iterator buffer_begin;
  vector<char>::const_iterator buffer_end;
  buffer_begin = array.begin();
  buffer_end   = array.end();
  while (buffer_begin < buffer_end)
    this->Children((new XML_character_info_item)
                   ->Parent(this)
                   ->UTF8(&buffer_begin, buffer_end)
                   );
  
  return this;
}

/////////////
// Children - Fill [children] list from C array.
// array: pointer to char array.
// array_end: first non valid address for array elements.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(const char* array, const char* array_end)
{
  if (!array || array >= array_end) return this;
  
  const unsigned char* buffer;
  const unsigned char* buffer_end;
  buffer     = (const unsigned char*)array;
  buffer_end = (const unsigned char*)array_end;
  while (buffer < buffer_end)
    this->Children((new XML_character_info_item)
                   ->Parent(this)
                   ->UTF8(&buffer, buffer_end)
                   );
  
  return this;
}

/////////////
// Children - Fill [children] list from C array.
// array: pointer to zero-terminated char array.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(const char* array)
{
  if (!array) return this;

  Children(array, array + strlen(array));
  
  return this;
}

/////////////
// Children - Append to [children] list from C array.
// array: pointer to array of pointers to objects to add.
// count: number of objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_comment_info_item_p_t* array, int count)
{
  if (!array || count <= 0) return this;
  
  for (int index = 0; index < count; ++index)
    this->Children(array[index]);
  
  return this;
}

/////////////
// Children - Append to [children] list from STL vector.
// children: vector of pointers to objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(vector<XML_comment_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_comment_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    this->Children(*i);
  
  return this;
}

/////////////
// Children - Append a [children] element to list.
// item: pointer to item to add.
//       Object is appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_comment_info_item_p_t item)
{
  if (!item) return this;
  
  this->children.push_back(item);
  item->Parent(this);
  
  return this;
}

/////////////
// Children - Append to [children] list from C array.
// array: pointer to array of pointers to objects to add.
// count: number of objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_ent_start_marker_info_item_p_t* array, int count)
{
  if (!array || count <= 0) return this;
  
  for (int index = 0; index < count; ++index)
    this->Children(array[index]);
  
  return this;
}

/////////////
// Children - Append to [children] list from STL vector.
// children: vector of pointers to objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(vector<XML_ent_start_marker_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_ent_start_marker_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    this->Children(*i);
  
  return this;
}

/////////////
// Children - Append a [children] element to list.
// item: pointer to item to add.
//       Object is appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_ent_start_marker_info_item_p_t item)
{
  if (!item) return this;
  
  this->children.push_back(item);
  item->Parent(this);
  
  return this;
}

/////////////
// Children - Append to [children] list from C array.
// array: pointer to array of pointers to objects to add.
// count: number of objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_ent_end_marker_info_item_p_t* array, int count)
{
  if (!array || count <= 0) return this;
  
  for (int index = 0; index < count; ++index)
    this->Children(array[index]);
  
  return this;
}

/////////////
// Children - Append to [children] list from STL vector.
// children: vector of pointers to objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(vector<XML_ent_end_marker_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_ent_end_marker_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    this->Children(*i);
  
  return this;
}

/////////////
// Children - Append a [children] element to list.
// item: pointer to item to add.
//       Object is appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_ent_end_marker_info_item_p_t item)
{
  if (!item) return this;
  
  this->children.push_back(item);
  item->Parent(this);
  
  return this;
}

/////////////
// Children - Append to [children] list from C array.
// array: pointer to array of pointers to objects to add.
// count: number of objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_cdata_start_marker_info_item_p_t* array, int count)
{
  if (!array || count <= 0) return this;
  
  for (int index = 0; index < count; ++index)
    this->Children(array[index]);
  
  return this;
}

/////////////
// Children - Append to [children] list from STL vector.
// children: vector of pointers to objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(vector<XML_cdata_start_marker_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_cdata_start_marker_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    this->Children(*i);
  
  return this;
}

/////////////
// Children - Append a [children] element to list.
// item: pointer to item to add.
//       Object is appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_cdata_start_marker_info_item_p_t item)
{
  if (!item) return this;
  
  this->children.push_back(item);
  item->Parent(this);
  
  return this;
}

/////////////
// Children - Append to [children] list from C array.
// array: pointer to array of pointers to objects to add.
// count: number of objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_cdata_end_marker_info_item_p_t* array, int count)
{
  if (!array || count <= 0) return this;
  
  for (int index = 0; index < count; ++index)
    this->Children(array[index]);
  
  return this;
}

/////////////
// Children - Append to [children] list from STL vector.
// children: vector of pointers to objects to add.
//        Objects are appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(vector<XML_cdata_end_marker_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_cdata_end_marker_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    this->Children(*i);
  
  return this;
}

/////////////
// Children - Append a [children] element to list.
// item: pointer to item to add.
//       Object is appended to whatever was previously in the list.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children(XML_cdata_end_marker_info_item_p_t item)
{
  if (!item) return this;
  
  this->children.push_back(item);
  item->Parent(this);
  
  return this;
}


/////////////
// Children - Access to [children] list for inspection.
// no arguments.
// returns [children] list as const. This means that the list
//       itself can't be modified by adding, removing, or reordering elements,
//       but the elements themselves can be modified.
//public:
const list<XML_info_item_p_t>& XML_elem_info_item::Children()
{
  return this->children;
}

/////////////
// Children_clear - Erase all items in [children].
// no arguments.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Children_clear()
{
  XML_info_item_p_t item;
  for (list<XML_info_item_p_t>::iterator
         item_iterator = children.begin();
       item_iterator != children.end();
       ++item_iterator)
    {
      item = *item_iterator;
      switch (item->Type())
        {
        case elem_info_item:
          if (this == XML_elem_info_item_p_t(item)->Parent())
            delete XML_elem_info_item_p_t(item);
          break;
        case proc_inst_info_item:
          if (this == XML_proc_inst_info_item_p_t(item)->Parent())
            delete XML_proc_inst_info_item_p_t(item);
          break;
        case unexp_ent_ref_info_item:
          if (this == XML_unexp_ent_ref_info_item_p_t(item)->Parent())
            delete XML_unexp_ent_ref_info_item_p_t(item);
          break;
        case character_info_item:
          if (this == XML_character_info_item_p_t(item)->Parent())
            delete XML_character_info_item_p_t(item);
          break;
        case comment_info_item:
          if (this == XML_comment_info_item_p_t(item)->Parent())
            delete XML_comment_info_item_p_t(item);
          break;
        case doc_type_decl_info_item:
          if (this == XML_doc_type_decl_info_item_p_t(item)->Parent())
            delete XML_doc_type_decl_info_item_p_t(item);
          break;
        case ent_start_marker_info_item:
          if (this == XML_ent_start_marker_info_item_p_t(item)->Parent())
            delete XML_ent_start_marker_info_item_p_t(item);
          break;
        case ent_end_marker_info_item:
          if (this == XML_ent_end_marker_info_item_p_t(item)->Parent())
            delete XML_ent_end_marker_info_item_p_t(item);
          break;
        case cdata_start_marker_info_item:
          if (this == XML_cdata_start_marker_info_item_p_t(item)->Parent())
            delete XML_cdata_start_marker_info_item_p_t(item);
          break;
        case cdata_end_marker_info_item:
          if (this == XML_cdata_end_marker_info_item_p_t(item)->Parent())
            delete XML_cdata_end_marker_info_item_p_t(item);
          break;
        case attr_info_item:
          if (this == XML_attr_info_item_p_t(item)->Owner_element())
            delete XML_attr_info_item_p_t(item);
          break;
        default:
          std::cerr << "Non-critical internal error: XML item type not handled." << std::endl;
        }
    }
  children.clear();
  
  return this;
}


using std::ostream;

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
// XML_elem_info_item: XML_elem_info_item to output.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, XML_elem_info_item_t &xml_elem_info_item)
{
  XML_elem_info_item::Output(&xml_elem_info_item, output);
  return output;
}

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
// XML_elem_info_item: XML_elem_info_item to output, passed as pointer.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, XML_elem_info_item_p_t xml_elem_info_item)
{
  XML_elem_info_item::Output(xml_elem_info_item, output);
  return output;
}



using std::vector;


/////////////
// Prefix - Set [prefix] property.
// prefix: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Prefix(XML_string_t &prefix)
{
  if (!(&prefix)) return this;
  if (!this->prefix)
    this->prefix = new XML_string();
  *(this->prefix) = prefix;
  return this;
}

/////////////
// Prefix - Set [prefix] property.
// prefix: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Prefix(XML_string_p_t prefix)
{
  if (!prefix)
    {
      if (this->prefix)
        {
          delete this->prefix;
          this->prefix = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->prefix)
    this->prefix = new XML_string();
  *(this->prefix) = *prefix;
  return this;
}

/////////////
// Prefix - Set [prefix] property.
// prefix: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Prefix(const vector<XML_UCS4_char_t> &prefix)
{
  if (!(&prefix)) return this;
  if (!this->prefix)
    this->prefix = new XML_string();
  this->prefix->UCS4(prefix);
  return this;
}

/////////////
// Prefix - Set [prefix] property.
// prefix: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Prefix(const vector<char> &prefix)
{
  if (!(&prefix)) return this;
  if (!this->prefix)
    this->prefix = new XML_string();
  this->prefix->UTF8(prefix);
  return this;
}

/////////////
// Prefix - Set [prefix] property.
// prefix: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Prefix(const char* prefix)
{
  if (!prefix)
    {
      if (this->prefix)
        {
          delete this->prefix;
          this->prefix = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->prefix)
    this->prefix = new XML_string();
  vector<char> prefix_vector;
  for (unsigned int index = 0; index < strlen(prefix); ++index)
    prefix_vector.push_back(prefix[index]);
  return Prefix(prefix_vector);
}

/////////////
// Prefix - Access to [prefix] property.
// no arguments.
// returns property value.
//public:
XML_string_p_t XML_elem_info_item::Prefix()
{
  return this->prefix;
}





/////////////
// Attributes - Add to [attributes] set from STL vector.
// children: vector of pointers to objects to add.
//        Objects are added to whatever was previously in the set.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Attributes(vector<XML_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    {
      switch ((*i)->Type())
        {
        case attr_info_item:
          this->Attributes(static_cast<XML_attr_info_item_p_t>(*i));
          break;
        default:
          ;// Ignore any others.
          break;
        }
    }
  
  return this;
}


/////////////
// Attributes - Add a [attributes] element to set.
// item: pointer to item to add.
//       Object is added to whatever was previously in the set.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Attributes(XML_attr_info_item_p_t item)
{
  if (!item) return this;
  
  if (item->Local_name())
    for (set<XML_info_item_p_t>::iterator
           item_iterator = attributes.begin();
         item_iterator != attributes.end();
         ++item_iterator)
      {
        XML_attr_info_item_p_t current_item;
        current_item = (XML_attr_info_item_p_t)(*item_iterator);
        if ( *item->Local_name() == *current_item->Local_name() )
          if (!item->Prefix()
              || *item->Prefix() == *current_item->Prefix())
            attributes.erase(item_iterator);
      }
  
  this->attributes.insert(item);
  item->Owner_element(this);
  
  return this;
}


/////////////
// Attributes - Access to [attributes] set.
// no arguments.
// returns [attributes] list.
//public:
set<XML_info_item_p_t>& XML_elem_info_item::Attributes()
{
  return this->attributes;
}



using std::vector;


/////////////
// Base_URI - Set [base URI] property.
// base_URI: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Base_URI(XML_string_t &base_URI)
{
  if (!(&base_URI)) return this;
  if (!this->base_URI)
    this->base_URI = new XML_string();
  *(this->base_URI) = base_URI;
  return this;
}

/////////////
// Base_URI - Set [base URI] property.
// base_URI: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Base_URI(XML_string_p_t base_URI)
{
  if (!base_URI)
    {
      if (this->base_URI)
        {
          delete this->base_URI;
          this->base_URI = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->base_URI)
    this->base_URI = new XML_string();
  *(this->base_URI) = *base_URI;
  return this;
}

/////////////
// Base_URI - Set [base URI] property.
// base_URI: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Base_URI(const vector<XML_UCS4_char_t> &base_URI)
{
  if (!(&base_URI)) return this;
  if (!this->base_URI)
    this->base_URI = new XML_string();
  this->base_URI->UCS4(base_URI);
  return this;
}

/////////////
// Base_URI - Set [base URI] property.
// base_URI: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Base_URI(const vector<char> &base_URI)
{
  if (!(&base_URI)) return this;
  if (!this->base_URI)
    this->base_URI = new XML_string();
  this->base_URI->UTF8(base_URI);
  return this;
}

/////////////
// Base_URI - Set [base URI] property.
// base_URI: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Base_URI(const char* base_URI)
{
  if (!base_URI)
    {
      if (this->base_URI)
        {
          delete this->base_URI;
          this->base_URI = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->base_URI)
    this->base_URI = new XML_string();
  vector<char> base_URI_vector;
  for (unsigned int index = 0; index < strlen(base_URI); ++index)
    base_URI_vector.push_back(base_URI[index]);
  return Base_URI(base_URI_vector);
}

/////////////
// Base_URI - Access to [base URI] property.
// no arguments.
// returns property value.
//public:
XML_string_p_t XML_elem_info_item::Base_URI()
{
  return this->base_URI;
}





/////////////
// In_scope_namespaces - Add to [in-scope namespaces] set from STL vector.
// children: vector of pointers to objects to add.
//        Objects are added to whatever was previously in the set.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::In_scope_namespaces(vector<XML_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    {
      switch ((*i)->Type())
        {
        case namespace_info_item:
          this->In_scope_namespaces(static_cast<XML_namespace_info_item_p_t>(*i));
          break;
        default:
          ;// Ignore any others.
          break;
        }
    }
  
  return this;
}


/////////////
// In_scope_namespaces - Add a [in-scope namespaces] element to set.
// item: pointer to item to add.
//       Object is added to whatever was previously in the set.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::In_scope_namespaces(XML_namespace_info_item_p_t item)
{
  if (!item) return this;
  
  this->in_scope_namespaces.insert(item);
  
  return this;
}


/////////////
// In_scope_namespaces - Access to [in-scope namespaces] set.
// no arguments.
// returns [in-scope namespaces] list.
//public:
set<XML_info_item_p_t>& XML_elem_info_item::In_scope_namespaces()
{
  return this->in_scope_namespaces;
}






/////////////
// Child - Access to [child] property.
// name: name of the element to retrieve.
// returns property value.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Child(XML_string_t& name)
{
  // type[-4:] != '_p_t': name = xml_string_name, type = XML_string_t&
  XML_string_t xml_string_name;
  xml_string_name = name;
  XML_string_p_t prefix_p;
  XML_string_p_t local_name_p;
  vector<XML_string_t> parts;
  xml_string_name.Split(XML_UCS4_char_t(':'), &parts);
  if (parts.size() == 1)
    {
      prefix_p     = (XML_string_p_t)NULL;
      local_name_p = &(parts[0]);
    }
  else
    {
      prefix_p     = &(parts[0]);
      local_name_p = &(parts[1]);
    }
  for (list<XML_info_item_p_t>::iterator
         item_iterator = children.begin();
       item_iterator != children.end();
       ++item_iterator)
    {
      if ((*item_iterator)->Type() == elem_info_item)
        {
          XML_elem_info_item_p_t item;
          item = (XML_elem_info_item_p_t)(*item_iterator);
          if (*item->Local_name() == *local_name_p)
            if (!prefix_p
                || *item->Prefix() == *prefix_p)
              return item;
        }
    }
  
  return (XML_elem_info_item_p_t)NULL;
}

/////////////
// Child - Access to [child] property.
// name: name of the element to retrieve.
// returns property value.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Child(XML_string_p_t name)
{
  if (0 == name) return 0;

  XML_string_p_t prefix_p;
  XML_string_p_t local_name_p;
  vector<XML_string_t> parts;
  name->Split(XML_UCS4_char_t(':'), &parts);
  if (parts.size() == 1)
    {
      prefix_p     = (XML_string_p_t)NULL;
      local_name_p = &(parts[0]);
    }
  else
    {
      prefix_p     = &(parts[0]);
      local_name_p = &(parts[1]);
    }
  for (list<XML_info_item_p_t>::iterator
         item_iterator = children.begin();
       item_iterator != children.end();
       ++item_iterator)
    {
      if ((*item_iterator)->Type() == elem_info_item)
        {
          XML_elem_info_item_p_t item;
          item = (XML_elem_info_item_p_t)(*item_iterator);
          if (*item->Local_name() == *local_name_p)
            if (!prefix_p
                || *item->Prefix() == *prefix_p)
              return item;
        }
    }
  
  return (XML_elem_info_item_p_t)NULL;
}


/////////////
// Child - Erase [child] property.
// name: name of the element to erase.
// returns true if erased, false otherwise, such as inexistant key.
//public:
bool XML_elem_info_item::Child_erase(XML_string_t& name)
{
  // type[-4:] != '_p_t': name = xml_string_name, type = XML_string_t&
  XML_string_t xml_string_name;
  xml_string_name = name;
  XML_string_p_t prefix_p;
  XML_string_p_t local_name_p;
  vector<XML_string_t> parts;
  xml_string_name.Split(XML_UCS4_char_t(':'), &parts);
  if (parts.size() == 1)
    {
      prefix_p     = (XML_string_p_t)NULL;
      local_name_p = &(parts[0]);
    }
  else
    {
      prefix_p     = &(parts[0]);
      local_name_p = &(parts[1]);
    }
  for (list<XML_info_item_p_t>::iterator
         item_iterator = children.begin();
       item_iterator != children.end();
       ++item_iterator)
    {
      switch ((*item_iterator)->Type())
        {
        case elem_info_item:
          XML_elem_info_item_p_t elem_item;
          elem_item = (XML_elem_info_item_p_t)(*item_iterator);
          if (*elem_item->Local_name() == *local_name_p)
            if (!prefix_p
                || *elem_item->Prefix() == *prefix_p)
              {
                children.erase(item_iterator);
                if (this == elem_item->Parent())
                  delete elem_item;
                return true;
              }
          break;
        default:
          ;// NOP
          break;
        }
    }
  
  return false;
}

/////////////
// Child - Erase [child] property.
// name: name of the element to erase.
// returns true if erased, false otherwise, such as inexistant key.
//public:
bool XML_elem_info_item::Child_erase(XML_string_p_t name)
{
  if (0 == name) return 0;

  XML_string_p_t prefix_p;
  XML_string_p_t local_name_p;
  vector<XML_string_t> parts;
  name->Split(XML_UCS4_char_t(':'), &parts);
  if (parts.size() == 1)
    {
      prefix_p     = (XML_string_p_t)NULL;
      local_name_p = &(parts[0]);
    }
  else
    {
      prefix_p     = &(parts[0]);
      local_name_p = &(parts[1]);
    }
  for (list<XML_info_item_p_t>::iterator
         item_iterator = children.begin();
       item_iterator != children.end();
       ++item_iterator)
    {
      switch ((*item_iterator)->Type())
        {
        case elem_info_item:
          XML_elem_info_item_p_t elem_item;
          elem_item = (XML_elem_info_item_p_t)(*item_iterator);
          if (*elem_item->Local_name() == *local_name_p)
            if (!prefix_p
                || *elem_item->Prefix() == *prefix_p)
              {
                children.erase(item_iterator);
                if (this == elem_item->Parent())
                  delete elem_item;
                return true;
              }
          break;
        default:
          ;// NOP
          break;
        }
    }
  
  return false;
}


/////////////
// Child_extract - Extraction of [child] property.
// name: name of the element to extract.
// returns property value, with its [parent] property set to NULL.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Child_extract(XML_string_t& name)
{
  // type[-4:] != '_p_t': name = xml_string_name, type = XML_string_t&
  XML_string_t xml_string_name;
  xml_string_name = name;
  XML_string_p_t prefix_p;
  XML_string_p_t local_name_p;
  vector<XML_string_t> parts;
  xml_string_name.Split(XML_UCS4_char_t(':'), &parts);
  if (parts.size() == 1)
    {
      prefix_p     = (XML_string_p_t)NULL;
      local_name_p = &(parts[0]);
    }
  else
    {
      prefix_p     = &(parts[0]);
      local_name_p = &(parts[1]);
    }
  for (list<XML_info_item_p_t>::iterator
         item_iterator = children.begin();
       item_iterator != children.end();
       ++item_iterator)
    {
      if ((*item_iterator)->Type() == elem_info_item)
        {
          XML_elem_info_item_p_t item;
          item = (XML_elem_info_item_p_t)(*item_iterator);
          if (*item->Local_name() == *local_name_p)
            if (!prefix_p
                || *item->Prefix() == *prefix_p)
              {
                children.erase(item_iterator);
                switch (item->Type())
                  {
                  case doc_type_decl_info_item:
                    if (this == XML_doc_type_decl_info_item_p_t(item)->Parent())
                      XML_doc_type_decl_info_item_p_t(item)->Parent(XML_doc_info_item_p_t(NULL));
                    break;
                  case elem_info_item:
                    if (this == XML_elem_info_item_p_t(item)->Parent())
                      XML_elem_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case proc_inst_info_item:
                    if (this == XML_proc_inst_info_item_p_t(item)->Parent())
                      XML_proc_inst_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case unexp_ent_ref_info_item:
                    if (this == XML_unexp_ent_ref_info_item_p_t(item)->Parent())
                      XML_unexp_ent_ref_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case character_info_item:
                    if (this == XML_character_info_item_p_t(item)->Parent())
                      XML_character_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case comment_info_item:
                    if (this == XML_comment_info_item_p_t(item)->Parent())
                      XML_comment_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case ent_start_marker_info_item:
                    if (this == XML_ent_start_marker_info_item_p_t(item)->Parent())
                      XML_ent_start_marker_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case ent_end_marker_info_item:
                    if (this == XML_ent_end_marker_info_item_p_t(item)->Parent())
                      XML_ent_end_marker_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case cdata_start_marker_info_item:
                    if (this == XML_cdata_start_marker_info_item_p_t(item)->Parent())
                      XML_cdata_start_marker_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case cdata_end_marker_info_item:
                    if (this == XML_cdata_end_marker_info_item_p_t(item)->Parent())
                      XML_cdata_end_marker_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case attr_info_item:
                    if (this == XML_attr_info_item_p_t(item)->Owner_element())
                      XML_attr_info_item_p_t(item)->Owner_element(NULL);
                    break;
                  default:
                    std::cerr << "Non-critical internal error: XML item type not handled." << std::endl;
                  }
              
                return item;
              }
        }
    }
  
  return (XML_elem_info_item_p_t)NULL;
}

/////////////
// Child_extract - Extraction of [child] property.
// name: name of the element to extract.
// returns property value, with its [parent] property set to NULL.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Child_extract(XML_string_p_t name)
{
  if (0 == name) return 0;

  XML_string_p_t prefix_p;
  XML_string_p_t local_name_p;
  vector<XML_string_t> parts;
  name->Split(XML_UCS4_char_t(':'), &parts);
  if (parts.size() == 1)
    {
      prefix_p     = (XML_string_p_t)NULL;
      local_name_p = &(parts[0]);
    }
  else
    {
      prefix_p     = &(parts[0]);
      local_name_p = &(parts[1]);
    }
  for (list<XML_info_item_p_t>::iterator
         item_iterator = children.begin();
       item_iterator != children.end();
       ++item_iterator)
    {
      if ((*item_iterator)->Type() == elem_info_item)
        {
          XML_elem_info_item_p_t item;
          item = (XML_elem_info_item_p_t)(*item_iterator);
          if (*item->Local_name() == *local_name_p)
            if (!prefix_p
                || *item->Prefix() == *prefix_p)
              {
                children.erase(item_iterator);
                switch (item->Type())
                  {
                  case doc_type_decl_info_item:
                    if (this == XML_doc_type_decl_info_item_p_t(item)->Parent())
                      XML_doc_type_decl_info_item_p_t(item)->Parent(XML_doc_info_item_p_t(NULL));
                    break;
                  case elem_info_item:
                    if (this == XML_elem_info_item_p_t(item)->Parent())
                      XML_elem_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case proc_inst_info_item:
                    if (this == XML_proc_inst_info_item_p_t(item)->Parent())
                      XML_proc_inst_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case unexp_ent_ref_info_item:
                    if (this == XML_unexp_ent_ref_info_item_p_t(item)->Parent())
                      XML_unexp_ent_ref_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case character_info_item:
                    if (this == XML_character_info_item_p_t(item)->Parent())
                      XML_character_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case comment_info_item:
                    if (this == XML_comment_info_item_p_t(item)->Parent())
                      XML_comment_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case ent_start_marker_info_item:
                    if (this == XML_ent_start_marker_info_item_p_t(item)->Parent())
                      XML_ent_start_marker_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case ent_end_marker_info_item:
                    if (this == XML_ent_end_marker_info_item_p_t(item)->Parent())
                      XML_ent_end_marker_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case cdata_start_marker_info_item:
                    if (this == XML_cdata_start_marker_info_item_p_t(item)->Parent())
                      XML_cdata_start_marker_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case cdata_end_marker_info_item:
                    if (this == XML_cdata_end_marker_info_item_p_t(item)->Parent())
                      XML_cdata_end_marker_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case attr_info_item:
                    if (this == XML_attr_info_item_p_t(item)->Owner_element())
                      XML_attr_info_item_p_t(item)->Owner_element(NULL);
                    break;
                  default:
                    std::cerr << "Non-critical internal error: XML item type not handled." << std::endl;
                  }
              
                return item;
              }
        }
    }
  
  return (XML_elem_info_item_p_t)NULL;
}


/////////////
// Child - Access to [child] property.
// name: name of the element to retrieve.
// returns property value.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Child(const char* name)
{
  // type[-4:] != '_p_t': name = xml_string_name, type = const char*
  XML_string_t xml_string_name;
  xml_string_name = name;
  XML_string_p_t prefix_p;
  XML_string_p_t local_name_p;
  vector<XML_string_t> parts;
  xml_string_name.Split(XML_UCS4_char_t(':'), &parts);
  if (parts.size() == 1)
    {
      prefix_p     = (XML_string_p_t)NULL;
      local_name_p = &(parts[0]);
    }
  else
    {
      prefix_p     = &(parts[0]);
      local_name_p = &(parts[1]);
    }
  for (list<XML_info_item_p_t>::iterator
         item_iterator = children.begin();
       item_iterator != children.end();
       ++item_iterator)
    {
      if ((*item_iterator)->Type() == elem_info_item)
        {
          XML_elem_info_item_p_t item;
          item = (XML_elem_info_item_p_t)(*item_iterator);
          if (*item->Local_name() == *local_name_p)
            if (!prefix_p
                || *item->Prefix() == *prefix_p)
              return item;
        }
    }
  
  return (XML_elem_info_item_p_t)NULL;
}

/////////////
// Child_extract - Extraction of [child] property.
// name: name of the element to extract.
// returns property value, with its [parent] property set to NULL.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Child_extract(const char* name)
{
  // type[-4:] != '_p_t': name = xml_string_name, type = const char*
  XML_string_t xml_string_name;
  xml_string_name = name;
  XML_string_p_t prefix_p;
  XML_string_p_t local_name_p;
  vector<XML_string_t> parts;
  xml_string_name.Split(XML_UCS4_char_t(':'), &parts);
  if (parts.size() == 1)
    {
      prefix_p     = (XML_string_p_t)NULL;
      local_name_p = &(parts[0]);
    }
  else
    {
      prefix_p     = &(parts[0]);
      local_name_p = &(parts[1]);
    }
  for (list<XML_info_item_p_t>::iterator
         item_iterator = children.begin();
       item_iterator != children.end();
       ++item_iterator)
    {
      if ((*item_iterator)->Type() == elem_info_item)
        {
          XML_elem_info_item_p_t item;
          item = (XML_elem_info_item_p_t)(*item_iterator);
          if (*item->Local_name() == *local_name_p)
            if (!prefix_p
                || *item->Prefix() == *prefix_p)
              {
                children.erase(item_iterator);
                switch (item->Type())
                  {
                  case doc_type_decl_info_item:
                    if (this == XML_doc_type_decl_info_item_p_t(item)->Parent())
                      XML_doc_type_decl_info_item_p_t(item)->Parent(XML_doc_info_item_p_t(NULL));
                    break;
                  case elem_info_item:
                    if (this == XML_elem_info_item_p_t(item)->Parent())
                      XML_elem_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case proc_inst_info_item:
                    if (this == XML_proc_inst_info_item_p_t(item)->Parent())
                      XML_proc_inst_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case unexp_ent_ref_info_item:
                    if (this == XML_unexp_ent_ref_info_item_p_t(item)->Parent())
                      XML_unexp_ent_ref_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case character_info_item:
                    if (this == XML_character_info_item_p_t(item)->Parent())
                      XML_character_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case comment_info_item:
                    if (this == XML_comment_info_item_p_t(item)->Parent())
                      XML_comment_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case ent_start_marker_info_item:
                    if (this == XML_ent_start_marker_info_item_p_t(item)->Parent())
                      XML_ent_start_marker_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case ent_end_marker_info_item:
                    if (this == XML_ent_end_marker_info_item_p_t(item)->Parent())
                      XML_ent_end_marker_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case cdata_start_marker_info_item:
                    if (this == XML_cdata_start_marker_info_item_p_t(item)->Parent())
                      XML_cdata_start_marker_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case cdata_end_marker_info_item:
                    if (this == XML_cdata_end_marker_info_item_p_t(item)->Parent())
                      XML_cdata_end_marker_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                    break;
                  case attr_info_item:
                    if (this == XML_attr_info_item_p_t(item)->Owner_element())
                      XML_attr_info_item_p_t(item)->Owner_element(NULL);
                    break;
                  default:
                    std::cerr << "Non-critical internal error: XML item type not handled." << std::endl;
                  }
              
                return item;
              }
        }
    }
  
  return (XML_elem_info_item_p_t)NULL;
}

/////////////
// Child - Erase [child] property.
// name: name of the element to erase.
// returns true if erased, false otherwise, such as inexistant key.
//public:
bool XML_elem_info_item::Child_erase(const char* name)
{
  // type[-4:] != '_p_t': name = xml_string_name, type = const char*
  XML_string_t xml_string_name;
  xml_string_name = name;
  XML_string_p_t prefix_p;
  XML_string_p_t local_name_p;
  vector<XML_string_t> parts;
  xml_string_name.Split(XML_UCS4_char_t(':'), &parts);
  if (parts.size() == 1)
    {
      prefix_p     = (XML_string_p_t)NULL;
      local_name_p = &(parts[0]);
    }
  else
    {
      prefix_p     = &(parts[0]);
      local_name_p = &(parts[1]);
    }
  for (list<XML_info_item_p_t>::iterator
         item_iterator = children.begin();
       item_iterator != children.end();
       ++item_iterator)
    {
      switch ((*item_iterator)->Type())
        {
        case elem_info_item:
          XML_elem_info_item_p_t elem_item;
          elem_item = (XML_elem_info_item_p_t)(*item_iterator);
          if (*elem_item->Local_name() == *local_name_p)
            if (!prefix_p
                || *elem_item->Prefix() == *prefix_p)
              {
                children.erase(item_iterator);
                if (this == elem_item->Parent())
                  delete elem_item;
                return true;
              }
          break;
        default:
          ;// NOP
          break;
        }
    }
  
  return false;
}


/////////////
// Child - Erase [child] property.
// item: pointer to element to erase.
// returns true if erased, false otherwise, such as inexistant key.
//public:
bool XML_elem_info_item::Child_erase(XML_info_item_p_t item)
{
  for (list<XML_info_item_p_t>::iterator
         item_iterator = children.begin();
       item_iterator != children.end();
       ++item_iterator)
    {
      if (*item_iterator == item)
        {
          children.erase(item_iterator);
          switch (item->Type())
            {
            case doc_type_decl_info_item:
              if (this == XML_doc_type_decl_info_item_p_t(item)->Parent())
                delete XML_doc_type_decl_info_item_p_t(item);
              break;
            case elem_info_item:
              if (this == XML_elem_info_item_p_t(item)->Parent())
                delete XML_elem_info_item_p_t(item);
              break;
            case proc_inst_info_item:
              if (this == XML_proc_inst_info_item_p_t(item)->Parent())
                delete XML_proc_inst_info_item_p_t(item);
              break;
            case unexp_ent_ref_info_item:
              if (this == XML_unexp_ent_ref_info_item_p_t(item)->Parent())
                delete XML_unexp_ent_ref_info_item_p_t(item);
              break;
            case character_info_item:
              if (this == XML_character_info_item_p_t(item)->Parent())
                delete XML_character_info_item_p_t(item);
              break;
            case comment_info_item:
              if (this == XML_comment_info_item_p_t(item)->Parent())
                delete XML_comment_info_item_p_t(item);
              break;
            case ent_start_marker_info_item:
              if (this == XML_ent_start_marker_info_item_p_t(item)->Parent())
                delete XML_ent_start_marker_info_item_p_t(item);
              break;
            case ent_end_marker_info_item:
              if (this == XML_ent_end_marker_info_item_p_t(item)->Parent())
                delete XML_ent_end_marker_info_item_p_t(item);
              break;
            case cdata_start_marker_info_item:
              if (this == XML_cdata_start_marker_info_item_p_t(item)->Parent())
                delete XML_cdata_start_marker_info_item_p_t(item);
              break;
            case cdata_end_marker_info_item:
              if (this == XML_cdata_end_marker_info_item_p_t(item)->Parent())
                delete XML_cdata_end_marker_info_item_p_t(item);
              break;
            case attr_info_item:
              if (this == XML_attr_info_item_p_t(item)->Owner_element())
                delete XML_attr_info_item_p_t(item);
              break;
            default:
              std::cerr << "Non-critical internal error: XML item type not handled." << std::endl;
            }
          return true;
        }
    }
  
  return false;
}

/////////////
// Child_extract - Extraction of [child] property.
// item: pointer to element to extract.
// returns property value, with its [parent] property set to NULL.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Child_extract(XML_elem_info_item_p_t item)
{
  for (list<XML_info_item_p_t>::iterator
         item_iterator = children.begin();
       item_iterator != children.end();
       ++item_iterator)
    {
      if ((*item_iterator)->Type() == elem_info_item)
        {
          if (*item_iterator == item)
            {
              children.erase(item_iterator);
              switch (item->Type())
                {
                case doc_type_decl_info_item:
                  if (this == XML_doc_type_decl_info_item_p_t(item)->Parent())
                    XML_doc_type_decl_info_item_p_t(item)->Parent(XML_doc_info_item_p_t(NULL));
                  break;
                case elem_info_item:
                  if (this == XML_elem_info_item_p_t(item)->Parent())
                    XML_elem_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                  break;
                case proc_inst_info_item:
                  if (this == XML_proc_inst_info_item_p_t(item)->Parent())
                    XML_proc_inst_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                  break;
                case unexp_ent_ref_info_item:
                  if (this == XML_unexp_ent_ref_info_item_p_t(item)->Parent())
                    XML_unexp_ent_ref_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                  break;
                case character_info_item:
                  if (this == XML_character_info_item_p_t(item)->Parent())
                    XML_character_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                  break;
                case comment_info_item:
                  if (this == XML_comment_info_item_p_t(item)->Parent())
                    XML_comment_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                  break;
                case ent_start_marker_info_item:
                  if (this == XML_ent_start_marker_info_item_p_t(item)->Parent())
                    XML_ent_start_marker_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                  break;
                case ent_end_marker_info_item:
                  if (this == XML_ent_end_marker_info_item_p_t(item)->Parent())
                    XML_ent_end_marker_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                  break;
                case cdata_start_marker_info_item:
                  if (this == XML_cdata_start_marker_info_item_p_t(item)->Parent())
                    XML_cdata_start_marker_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                  break;
                case cdata_end_marker_info_item:
                  if (this == XML_cdata_end_marker_info_item_p_t(item)->Parent())
                    XML_cdata_end_marker_info_item_p_t(item)->Parent(XML_elem_info_item_p_t(NULL));
                  break;
                case attr_info_item:
                  if (this == XML_attr_info_item_p_t(item)->Owner_element())
                    XML_attr_info_item_p_t(item)->Owner_element(NULL);
                  break;
                default:
                  std::cerr << "Non-critical internal error: XML item type not handled." << std::endl;
                }
              
              return item;
            }
        }
    }
  
  return (XML_elem_info_item_p_t)NULL;
}

using std::ostream;
using std::endl;

/////////////
// Output - Write to a stream in XML format.
// item: XML_elem_info_item to be output, passed as pointer.
// output: stream which receives data.
// returns true on success, false on fail.
//public:
//static
bool XML_elem_info_item::Output(XML_elem_info_item_p_t item, ostream &output)
{
  return Output(item, output, 0);
}

/////////////
// Output - Write to a stream in XML format.
// item: XML_elem_info_item to be output, passed as pointer.
// output: stream which receives data.
// returns true on success, false on fail.
//public:
//static
bool XML_elem_info_item::Output(XML_elem_info_item_p_t item, ostream &output, unsigned int indentation)
{
  if (!item || !(&output)) return false;
  
  unsigned int character_count_begin;
  character_count_begin = output.tellp();
  if (indentation)
    {
      output << endl;
      character_count_begin = output.tellp();
      for (unsigned int count = 0; count < indentation; ++count)
        output << "  ";
    }
  output << "<";
  if (item->Prefix())
    output << item->Prefix() << ":";
  if (item->Local_name())
    output << item->Local_name();
  
  for (set<XML_info_item_p_t>::const_iterator item_p_iterator
         = item->Attributes().begin();
       item_p_iterator != item->Attributes().end();
       ++item_p_iterator)
    {
      if ((*item_p_iterator)->Type() == attr_info_item)
        output << " " << *item_p_iterator;
    }
  for (set<XML_info_item_p_t>::const_iterator item_p_iterator
         = item->Namespace_attributes().begin();
       item_p_iterator != item->Namespace_attributes().end();
       ++item_p_iterator)
    {
      if ((*item_p_iterator)->Type() == attr_info_item)
        output << " " << *item_p_iterator;
    }
  for (set<XML_info_item_p_t>::const_iterator item_p_iterator
         = item->In_scope_namespaces().begin();
       item_p_iterator != item->In_scope_namespaces().end();
       ++item_p_iterator)
    {
      output << " " << "in-scope-namespace" << "='" << *item_p_iterator << "'";
    }
  bool valid_break_point;
  valid_break_point = false;
  if (item->Children().size() > 0)
    {
      output << ">";
      
      int character_count;
      character_count = int(output.tellp()) - character_count_begin;
      int output_width_hint;
      output_width_hint = item->Output_width_hint();
      for (list<XML_info_item_p_t>::const_iterator item_p_iterator
             = item->Children().begin();
           item_p_iterator != item->Children().end();
           ++item_p_iterator)
        {
          if ((*item_p_iterator)->Type() == character_info_item)
            {
              if (character_count > output_width_hint)
                {
                  switch (((XML_character_info_item_p_t)(*item_p_iterator))
                          ->Character_code())
                    {
                    case XML_UCS4_char_t(' '):
                    case XML_UCS4_char_t('\t'):
                    case XML_UCS4_char_t('\n'):
                      valid_break_point = true;
                      character_count = 0;
                      break;
                    default:
                      valid_break_point = false;
                      XML_info_item::Output(*item_p_iterator, output, 0);
                      ++character_count;
                      break;
                    }
                }
              else
                {
                  switch (((XML_character_info_item_p_t)(*item_p_iterator))
                          ->Character_code())
                    {
                    case XML_UCS4_char_t(' '):
                    case XML_UCS4_char_t('\t'):
                    case XML_UCS4_char_t('\n'):
                      if (character_count > 0)
                        {
                          XML_info_item::Output(*item_p_iterator, output, 0);
                          ++character_count;
                        }
                      break;
                    default:
                      if (valid_break_point)
                        {
                          output << endl;
                          character_count_begin = output.tellp();
                          for (unsigned int count = 0; count <= indentation;
                               ++count)
                            {
                              output << "  ";
                            }
                          character_count = (indentation + 1) * 2;
                          valid_break_point = false;
                        }
                      XML_info_item::Output(*item_p_iterator, output, 0);
                      ++character_count;
                    }
                }
            }
          else
            {
              XML_info_item::Output(*item_p_iterator, output, indentation + 1);
              character_count = int(output.tellp()) - character_count_begin;
              valid_break_point = true;
            }
        }
      
      if (valid_break_point)
        {
          output << endl;
          for (unsigned int count = 0; count < indentation; ++count)
            output << "  ";
        }
      output << "</";
      if (item->Prefix())
        output << item->Prefix() << ":";
      if (item->Local_name())
        output << item->Local_name();
      output << ">";
    }
  else
    {
      output << "/>";
    }
  
  return true;
}




/////////////
// Namespace_attributes - Add to [namespace attributes] set from STL vector.
// children: vector of pointers to objects to add.
//        Objects are added to whatever was previously in the set.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Namespace_attributes(vector<XML_info_item_p_t> &new_children)
{
  if (!&new_children) return this;
  
  for (vector<XML_info_item_p_t>::iterator
         i = new_children.begin();
       i  != new_children.end();
       ++i)
    {
      switch ((*i)->Type())
        {
        case attr_info_item:
          this->Namespace_attributes(static_cast<XML_attr_info_item_p_t>(*i));
          break;
        default:
          ;// Ignore any others.
          break;
        }
    }
  
  return this;
}


/////////////
// Namespace_attributes - Add a [namespace attributes] element to set.
// item: pointer to item to add.
//       Object is added to whatever was previously in the set.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Namespace_attributes(XML_attr_info_item_p_t item)
{
  if (!item) return this;
  
  if (item->Local_name())
    for (set<XML_info_item_p_t>::iterator
           item_iterator = namespace_attributes.begin();
         item_iterator != namespace_attributes.end();
         ++item_iterator)
      {
        XML_attr_info_item_p_t current_item;
        current_item = (XML_attr_info_item_p_t)(*item_iterator);
        if ( *item->Local_name() == *current_item->Local_name() )
          if (!item->Prefix()
              || *item->Prefix() == *current_item->Prefix())
            namespace_attributes.erase(item_iterator);
      }
  
  this->namespace_attributes.insert(item);
  item->Owner_element(this);
  
  return this;
}


/////////////
// Namespace_attributes - Access to [namespace attributes] set.
// no arguments.
// returns [namespace attributes] list.
//public:
set<XML_info_item_p_t>& XML_elem_info_item::Namespace_attributes()
{
  return this->namespace_attributes;
}



using std::vector;


/////////////
// Local_name - Set [local name] property.
// local_name: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Local_name(XML_string_t &local_name)
{
  if (!(&local_name)) return this;
  if (!this->local_name)
    this->local_name = new XML_string();
  *(this->local_name) = local_name;
  return this;
}

/////////////
// Local_name - Set [local name] property.
// local_name: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Local_name(XML_string_p_t local_name)
{
  if (!local_name)
    {
      if (this->local_name)
        {
          delete this->local_name;
          this->local_name = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->local_name)
    this->local_name = new XML_string();
  *(this->local_name) = *local_name;
  return this;
}

/////////////
// Local_name - Set [local name] property.
// local_name: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Local_name(const vector<XML_UCS4_char_t> &local_name)
{
  if (!(&local_name)) return this;
  if (!this->local_name)
    this->local_name = new XML_string();
  this->local_name->UCS4(local_name);
  return this;
}

/////////////
// Local_name - Set [local name] property.
// local_name: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Local_name(const vector<char> &local_name)
{
  if (!(&local_name)) return this;
  if (!this->local_name)
    this->local_name = new XML_string();
  this->local_name->UTF8(local_name);
  return this;
}

/////////////
// Local_name - Set [local name] property.
// local_name: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Local_name(const char* local_name)
{
  if (!local_name)
    {
      if (this->local_name)
        {
          delete this->local_name;
          this->local_name = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->local_name)
    this->local_name = new XML_string();
  vector<char> local_name_vector;
  for (unsigned int index = 0; index < strlen(local_name); ++index)
    local_name_vector.push_back(local_name[index]);
  return Local_name(local_name_vector);
}

/////////////
// Local_name - Access to [local name] property.
// no arguments.
// returns property value.
//public:
XML_string_p_t XML_elem_info_item::Local_name()
{
  return this->local_name;
}






/////////////
// Attribute - Access to [attribute] property.
// name: name of the element to retrieve.
// returns property value.
//public:
const XML_attr_info_item_p_t XML_elem_info_item::Attribute(XML_string_t &name)
{
  // type[-4:] != '_p_t': name = xml_string_name, type = XML_string_t&
  XML_string_t xml_string_name;
  xml_string_name = name;
  XML_string_p_t prefix_p;
  XML_string_p_t local_name_p;
  vector<XML_string_t> parts;
  xml_string_name.Split(XML_UCS4_char_t(':'), &parts);
  if (parts.size() == 1)
    {
      prefix_p     = (XML_string_p_t)NULL;
      local_name_p = &(parts[0]);
    }
  else
    {
      prefix_p     = &(parts[0]);
      local_name_p = &(parts[1]);
    }
  for (set<XML_info_item_p_t>::iterator
         item_iterator = attributes.begin();
       item_iterator != attributes.end();
       ++item_iterator)
    {
      if ((*item_iterator)->Type() == attr_info_item)
        {
          XML_attr_info_item_p_t current_item;
          current_item = (XML_attr_info_item_p_t)(*item_iterator);
          if (*current_item->Local_name() == *local_name_p)
            if (!prefix_p
                || *current_item->Prefix() == *prefix_p)
              return current_item;
        }
    }
  
  return (XML_attr_info_item_p_t)NULL;
}

/////////////
// Attribute - Erase [attribute] property.
// name: name of the element to erase.
// returns true if erased, false otherwise, such as inexistant key.
//public:
bool XML_elem_info_item::Attribute_erase(XML_string_t &name)
{
  // type[-4:] != '_p_t': name = xml_string_name, type = XML_string_t&
  XML_string_t xml_string_name;
  xml_string_name = name;
  XML_string_p_t prefix_p;
  XML_string_p_t local_name_p;
  vector<XML_string_t> parts;
  xml_string_name.Split(XML_UCS4_char_t(':'), &parts);
  if (parts.size() == 1)
    {
      prefix_p     = (XML_string_p_t)NULL;
      local_name_p = &(parts[0]);
    }
  else
    {
      prefix_p     = &(parts[0]);
      local_name_p = &(parts[1]);
    }
  for (set<XML_info_item_p_t>::iterator
         item_iterator = attributes.begin();
       item_iterator != attributes.end();
       ++item_iterator)
    {
      switch ((*item_iterator)->Type())
        {
        case attr_info_item:
          XML_attr_info_item_p_t attr_item;
          attr_item = (XML_attr_info_item_p_t)(*item_iterator);
          if (*attr_item->Local_name() == *local_name_p)
            if (!prefix_p
                || *attr_item->Prefix() == *prefix_p)
              {
                attributes.erase(item_iterator);
                return true;
              }
          break;
        default:
          ;// NOP
          break;
        }
    }
  
  return false;
}


/////////////
// Attribute - Access to [attribute] property.
// name: name of the element to retrieve.
// returns property value.
//public:
const XML_attr_info_item_p_t XML_elem_info_item::Attribute(const char* name)
{
  XML_string_t name_string;
  name_string = name;
  
  return Attribute(name_string);
}


/////////////
// Attribute - Erase [attribute] property.
// name: name of the element to erase.
// returns true if erased, false otherwise, such as inexistant key.
//public:
bool XML_elem_info_item::Attribute_erase(const char* name)
{
  XML_string_t name_string;
  name_string = name;
  
  return Attribute_erase(name_string);
}





/////////////
// XML_elem_info_item - Class constructor.
// parent: *_info_item to be the [parent] of the new object.
//         The new object is automatically inserted in [parent].
//public:
XML_elem_info_item::XML_elem_info_item(XML_doc_info_item_p_t parent)
{
  this->namespace_name = NULL;
  this->local_name = NULL;
  this->prefix = NULL;
  this->base_URI = NULL;
  this->parent = NULL;
  
  Type(elem_info_item);
  parent->Children(this);
  Parent(parent);
}

/////////////
// XML_elem_info_item - Class constructor.
// parent: *_info_item to be the [parent] of the new object.
//         The new object is automatically inserted in [parent].
//public:
XML_elem_info_item::XML_elem_info_item(XML_elem_info_item_p_t parent)
{
  this->namespace_name = NULL;
  this->local_name = NULL;
  this->prefix = NULL;
  this->base_URI = NULL;
  this->parent = NULL;
  
  Type(elem_info_item);
  parent->Children(this);
  Parent(parent);
}


/////////////
// Parent - Set [parent] property.
// parent: new value for property. Can be 0.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Parent(XML_doc_info_item_p_t parent)
{
  this->parent = parent;
  
  return this;
}
/////////////
// Parent - Set [parent] property.
// parent: new value for property. Can be 0.
// returns object pointer for operator concatenation.
//public:
XML_elem_info_item_p_t XML_elem_info_item::Parent(XML_elem_info_item_p_t parent)
{
  this->parent = parent;
  
  return this;
}

/////////////
// Parent - Access to [parent] property.
// no arguments.
// returns property value.
//public:
const XML_info_item_p_t XML_elem_info_item::Parent()
{
  return (XML_info_item_p_t)(this->parent);
}
