// Clase:    XML_external_ent_info_item
// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:47 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#include "XML_external_ent_info_item.h"

/////////////
// XML_external_ent_info_item - Class constructor.
// no arguments.
//public:
XML_external_ent_info_item::XML_external_ent_info_item()
{
  this->name = NULL;
  this->system_identifier = NULL;
  this->public_identifier = NULL;
  this->base_URI = NULL;
  this->charset = NULL;
  Type(external_ent_info_item);
}

/////////////
// ~XML_external_ent_info_item - Class destructor.
// no arguments.
//public:
XML_external_ent_info_item::~XML_external_ent_info_item()
{
  if (name)
    {
      delete name;
      name = NULL;
    }
  if (system_identifier)
    {
      delete system_identifier;
      system_identifier = NULL;
    }
  if (public_identifier)
    {
      delete public_identifier;
      public_identifier = NULL;
    }
  if (base_URI)
    {
      delete base_URI;
      base_URI = NULL;
    }
  if (charset)
    {
      delete charset;
      charset = NULL;
    }
}

using std::ostream;

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
// XML_external_ent_info_item: XML_external_ent_info_item to output.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, XML_external_ent_info_item_t &xml_external_ent_info_item)
{
  XML_external_ent_info_item::Output(&xml_external_ent_info_item, output);
  return output;
}

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
// XML_external_ent_info_item: XML_external_ent_info_item to output, passed as pointer.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, XML_external_ent_info_item_p_t xml_external_ent_info_item)
{
  XML_external_ent_info_item::Output(xml_external_ent_info_item, output);
  return output;
}



using std::vector;


/////////////
// Base_URI - Set [base URI] property.
// base_URI: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Base_URI(XML_string_t &base_URI)
{
  if (!(&base_URI)) return this;
  if (!this->base_URI)
    this->base_URI = new XML_string();
  *(this->base_URI) = base_URI;
  return this;
}

/////////////
// Base_URI - Set [base URI] property.
// base_URI: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Base_URI(XML_string_p_t base_URI)
{
  if (!base_URI)
    {
      if (this->base_URI)
        {
          delete this->base_URI;
          this->base_URI = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->base_URI)
    this->base_URI = new XML_string();
  *(this->base_URI) = *base_URI;
  return this;
}

/////////////
// Base_URI - Set [base URI] property.
// base_URI: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Base_URI(const vector<XML_UCS4_char_t> &base_URI)
{
  if (!(&base_URI)) return this;
  if (!this->base_URI)
    this->base_URI = new XML_string();
  this->base_URI->UCS4(base_URI);
  return this;
}

/////////////
// Base_URI - Set [base URI] property.
// base_URI: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Base_URI(const vector<char> &base_URI)
{
  if (!(&base_URI)) return this;
  if (!this->base_URI)
    this->base_URI = new XML_string();
  this->base_URI->UTF8(base_URI);
  return this;
}

/////////////
// Base_URI - Set [base URI] property.
// base_URI: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Base_URI(const char* base_URI)
{
  if (!base_URI)
    {
      if (this->base_URI)
        {
          delete this->base_URI;
          this->base_URI = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->base_URI)
    this->base_URI = new XML_string();
  vector<char> base_URI_vector;
  for (unsigned int index = 0; index < strlen(base_URI); ++index)
    base_URI_vector.push_back(base_URI[index]);
  return Base_URI(base_URI_vector);
}

/////////////
// Base_URI - Access to [base URI] property.
// no arguments.
// returns property value.
//public:
XML_string_p_t XML_external_ent_info_item::Base_URI()
{
  return this->base_URI;
}

using std::ostream;
using std::endl;

/////////////
// Output - Write to a stream in XML format.
// item: XML_external_ent_info_item to be output, passed as pointer.
// output: stream which receives data.
// returns true on success, false on fail.
//public:
//static
bool XML_external_ent_info_item::Output(XML_external_ent_info_item_p_t item, ostream &output)
{
  return Output(item, output, 0);
}

/////////////
// Output - Write to a stream in XML format.
// item: XML_external_ent_info_item to be output, passed as pointer.
// output: stream which receives data.
// returns true on success, false on fail.
//public:
//static
bool XML_external_ent_info_item::Output(XML_external_ent_info_item_p_t item, ostream &output, unsigned int indentation)
{
  if (!item || !(&output)) return false;
  
  if (indentation)
    output << endl;
  for (unsigned int count = 0; count < indentation; ++count)
    output << "  ";
  output << "<";
  output << "!SYSTEM";
  output << " " << item->Name();
  if (item->Public_identifier())
    {
      output << "\n PUBLIC '"
             << item->Public_identifier()
             << "'";
      if (item->System_identifier())
        {
          output << "\n '";
          if (item->Base_URI())
            output << item->Base_URI();
          output << item->System_identifier()
                 << "'";
        }
    }
  else if (item->System_identifier())
    {
      output << "\n SYSTEM '";
      if (item->Base_URI())
        output << item->Base_URI();
      output << item->System_identifier()
             << "'";
    }
  
  if (item->Charset())
    output << "\n encoding='" << item->Charset() << "'";
  output << ">";
  
  return true;
}


using std::vector;


/////////////
// Name - Set [name] property.
// name: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Name(XML_string_t &name)
{
  if (!(&name)) return this;
  if (!this->name)
    this->name = new XML_string();
  *(this->name) = name;
  return this;
}

/////////////
// Name - Set [name] property.
// name: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Name(XML_string_p_t name)
{
  if (!name)
    {
      if (this->name)
        {
          delete this->name;
          this->name = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->name)
    this->name = new XML_string();
  *(this->name) = *name;
  return this;
}

/////////////
// Name - Set [name] property.
// name: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Name(const vector<XML_UCS4_char_t> &name)
{
  if (!(&name)) return this;
  if (!this->name)
    this->name = new XML_string();
  this->name->UCS4(name);
  return this;
}

/////////////
// Name - Set [name] property.
// name: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Name(const vector<char> &name)
{
  if (!(&name)) return this;
  if (!this->name)
    this->name = new XML_string();
  this->name->UTF8(name);
  return this;
}

/////////////
// Name - Set [name] property.
// name: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Name(const char* name)
{
  if (!name)
    {
      if (this->name)
        {
          delete this->name;
          this->name = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->name)
    this->name = new XML_string();
  vector<char> name_vector;
  for (unsigned int index = 0; index < strlen(name); ++index)
    name_vector.push_back(name[index]);
  return Name(name_vector);
}

/////////////
// Name - Access to [name] property.
// no arguments.
// returns property value.
//public:
XML_string_p_t XML_external_ent_info_item::Name()
{
  return this->name;
}



using std::vector;


/////////////
// System_identifier - Set [system identifier] property.
// system_identifier: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::System_identifier(XML_string_t &system_identifier)
{
  if (!(&system_identifier)) return this;
  if (!this->system_identifier)
    this->system_identifier = new XML_string();
  *(this->system_identifier) = system_identifier;
  return this;
}

/////////////
// System_identifier - Set [system identifier] property.
// system_identifier: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::System_identifier(XML_string_p_t system_identifier)
{
  if (!system_identifier)
    {
      if (this->system_identifier)
        {
          delete this->system_identifier;
          this->system_identifier = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->system_identifier)
    this->system_identifier = new XML_string();
  *(this->system_identifier) = *system_identifier;
  return this;
}

/////////////
// System_identifier - Set [system identifier] property.
// system_identifier: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::System_identifier(const vector<XML_UCS4_char_t> &system_identifier)
{
  if (!(&system_identifier)) return this;
  if (!this->system_identifier)
    this->system_identifier = new XML_string();
  this->system_identifier->UCS4(system_identifier);
  return this;
}

/////////////
// System_identifier - Set [system identifier] property.
// system_identifier: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::System_identifier(const vector<char> &system_identifier)
{
  if (!(&system_identifier)) return this;
  if (!this->system_identifier)
    this->system_identifier = new XML_string();
  this->system_identifier->UTF8(system_identifier);
  return this;
}

/////////////
// System_identifier - Set [system identifier] property.
// system_identifier: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::System_identifier(const char* system_identifier)
{
  if (!system_identifier)
    {
      if (this->system_identifier)
        {
          delete this->system_identifier;
          this->system_identifier = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->system_identifier)
    this->system_identifier = new XML_string();
  vector<char> system_identifier_vector;
  for (unsigned int index = 0; index < strlen(system_identifier); ++index)
    system_identifier_vector.push_back(system_identifier[index]);
  return System_identifier(system_identifier_vector);
}

/////////////
// System_identifier - Access to [system identifier] property.
// no arguments.
// returns property value.
//public:
XML_string_p_t XML_external_ent_info_item::System_identifier()
{
  return this->system_identifier;
}



using std::vector;


/////////////
// Charset - Set [charset] property.
// charset: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Charset(XML_string_t &charset)
{
  if (!(&charset)) return this;
  if (!this->charset)
    this->charset = new XML_string();
  *(this->charset) = charset;
  return this;
}

/////////////
// Charset - Set [charset] property.
// charset: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Charset(XML_string_p_t charset)
{
  if (!charset)
    {
      if (this->charset)
        {
          delete this->charset;
          this->charset = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->charset)
    this->charset = new XML_string();
  *(this->charset) = *charset;
  return this;
}

/////////////
// Charset - Set [charset] property.
// charset: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Charset(const vector<XML_UCS4_char_t> &charset)
{
  if (!(&charset)) return this;
  if (!this->charset)
    this->charset = new XML_string();
  this->charset->UCS4(charset);
  return this;
}

/////////////
// Charset - Set [charset] property.
// charset: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Charset(const vector<char> &charset)
{
  if (!(&charset)) return this;
  if (!this->charset)
    this->charset = new XML_string();
  this->charset->UTF8(charset);
  return this;
}

/////////////
// Charset - Set [charset] property.
// charset: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Charset(const char* charset)
{
  if (!charset)
    {
      if (this->charset)
        {
          delete this->charset;
          this->charset = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->charset)
    this->charset = new XML_string();
  vector<char> charset_vector;
  for (unsigned int index = 0; index < strlen(charset); ++index)
    charset_vector.push_back(charset[index]);
  return Charset(charset_vector);
}

/////////////
// Charset - Access to [charset] property.
// no arguments.
// returns property value.
//public:
XML_string_p_t XML_external_ent_info_item::Charset()
{
  return this->charset;
}



using std::vector;


/////////////
// Public_identifier - Set [public identifier] property.
// public_identifier: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Public_identifier(XML_string_t &public_identifier)
{
  if (!(&public_identifier)) return this;
  if (!this->public_identifier)
    this->public_identifier = new XML_string();
  *(this->public_identifier) = public_identifier;
  return this;
}

/////////////
// Public_identifier - Set [public identifier] property.
// public_identifier: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Public_identifier(XML_string_p_t public_identifier)
{
  if (!public_identifier)
    {
      if (this->public_identifier)
        {
          delete this->public_identifier;
          this->public_identifier = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->public_identifier)
    this->public_identifier = new XML_string();
  *(this->public_identifier) = *public_identifier;
  return this;
}

/////////////
// Public_identifier - Set [public identifier] property.
// public_identifier: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Public_identifier(const vector<XML_UCS4_char_t> &public_identifier)
{
  if (!(&public_identifier)) return this;
  if (!this->public_identifier)
    this->public_identifier = new XML_string();
  this->public_identifier->UCS4(public_identifier);
  return this;
}

/////////////
// Public_identifier - Set [public identifier] property.
// public_identifier: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Public_identifier(const vector<char> &public_identifier)
{
  if (!(&public_identifier)) return this;
  if (!this->public_identifier)
    this->public_identifier = new XML_string();
  this->public_identifier->UTF8(public_identifier);
  return this;
}

/////////////
// Public_identifier - Set [public identifier] property.
// public_identifier: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_external_ent_info_item_p_t XML_external_ent_info_item::Public_identifier(const char* public_identifier)
{
  if (!public_identifier)
    {
      if (this->public_identifier)
        {
          delete this->public_identifier;
          this->public_identifier = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->public_identifier)
    this->public_identifier = new XML_string();
  vector<char> public_identifier_vector;
  for (unsigned int index = 0; index < strlen(public_identifier); ++index)
    public_identifier_vector.push_back(public_identifier[index]);
  return Public_identifier(public_identifier_vector);
}

/////////////
// Public_identifier - Access to [public identifier] property.
// no arguments.
// returns property value.
//public:
XML_string_p_t XML_external_ent_info_item::Public_identifier()
{
  return this->public_identifier;
}
