// Clase:    XML_info_item
// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:47 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#include "XML_info_item.h"

/////////////
// XML_info_item - Class constructor.
// no arguments.
//public:
XML_info_item::XML_info_item()
{
  Type(info_item);
  output_width_hint = 70;
}

/////////////
// ~XML_info_item - Class destructor.
// no arguments.
//public:
XML_info_item::~XML_info_item()
{

}

using std::ostream;

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
// XML_info_item: XML_info_item to output.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, XML_info_item_t &xml_info_item)
{
  XML_info_item::Output(&xml_info_item, output);
  return output;
}

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
// XML_info_item: XML_info_item to output, passed as pointer.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, XML_info_item_p_t xml_info_item)
{
  XML_info_item::Output(xml_info_item, output);
  return output;
}

using std::ostream;

 #include "../XML_doc_info_item/XML_doc_info_item.h"
 #include "../XML_elem_info_item/XML_elem_info_item.h"
 #include "../XML_attr_info_item/XML_attr_info_item.h"
 #include "../XML_character_info_item/XML_character_info_item.h"
 #include "../XML_comment_info_item/XML_comment_info_item.h"
 #include "../XML_doc_type_decl_info_item/XML_doc_type_decl_info_item.h"
 #include "../XML_internal_ent_info_item/XML_internal_ent_info_item.h"
 #include "../XML_external_ent_info_item/XML_external_ent_info_item.h"
 #include "../XML_unparsed_ent_info_item/XML_unparsed_ent_info_item.h"
/////////////
// Output - Write to a stream in XML format.
// item: XML_info_item to be output, passed as pointer.
// output: stream which receives data.
// returns true on success, false on fail.
//public:
//static
bool XML_info_item::Output(XML_info_item_p_t item, ostream &output)
{
  return Output(item, output, 0);
}

/////////////
// Output - Write to a stream in XML format.
// item: XML_info_item to be output, passed as pointer.
// output: stream which receives data.
// returns true on success, false on fail.
//public:
//static
bool XML_info_item::Output(XML_info_item_p_t item, ostream &output, unsigned int indentation)
{
  if (!item || !(&output)) return false;
  
  switch (item->Type())
    {
    case info_item:
      output << "<info_item address='" << (void*)item << "'/>";
      break;
    case doc_info_item:
      XML_doc_info_item::Output((XML_doc_info_item_p_t)item, output, indentation);
      break;
    case elem_info_item:
      XML_elem_info_item::Output((XML_elem_info_item_p_t)item, output, indentation);
      break;
    case attr_info_item:
      XML_attr_info_item::Output((XML_attr_info_item_p_t)item, output, indentation);
      break;
    case character_info_item:
      XML_character_info_item::Output((XML_character_info_item_p_t)item, output);
      break;
    case comment_info_item:
      XML_comment_info_item::Output((XML_comment_info_item_p_t)item, output, indentation);
      break;
    case doc_type_decl_info_item:
      XML_doc_type_decl_info_item::Output((XML_doc_type_decl_info_item_p_t)item, output, indentation);
      break;
    case internal_ent_info_item:
      XML_internal_ent_info_item::Output((XML_internal_ent_info_item_p_t)item, output, indentation);
      break;
    case external_ent_info_item:
      XML_external_ent_info_item::Output((XML_external_ent_info_item_p_t)item, output, indentation);
      break;
    case unparsed_ent_info_item:
      XML_unparsed_ent_info_item::Output((XML_unparsed_ent_info_item_p_t)item, output, indentation);
      break;
      // If still unimplemented, output its type and address.
    case proc_inst_info_item:
      output << "<proc_inst_info_item address='" << (void*)item << "'/>";
      break;
    case unexp_ent_ref_info_item:
      output << "<unexp_ent_ref_info_item address='" << (void*)item << "'/>";
      break;
    case notation_info_item:
      output << "<notation_info_item address='" << (void*)item << "'/>";
      break;
    case ent_start_marker_info_item:
      output << "<ent_start_marker_info_item address='" << (void*)item << "'/>";
      break;
    case ent_end_marker_info_item:
      output << "<ent_end_marker_info_item address='" << (void*)item << "'/>";
      break;
    case namespace_info_item:
      output << "<namespace_info_item address='" << (void*)item << "'/>";
      break;
    case cdata_start_marker_info_item:
      output << "<cdata_start_marker_info_item address='" << (void*)item << "'/>";
      break;
    case cdata_end_marker_info_item:
      output << "<cdata_end_marker_info_item address='" << (void*)item << "'/>";
      break;
    }
  
  return true;
}

/////////////
// Type - Set Information Item type.
// type: new type for item.
// returns old type.
//public:
XML_info_item::XML_info_item_type_t XML_info_item::Type(XML_info_item::XML_info_item_type_t type)
{
  XML_info_item_type_t old_type;
  old_type = this->type;
  this->type = type;
  return old_type;
}

/////////////
// Type - Report Information Item type.
// no arguments.
// returns current type.
//public:
XML_info_item::XML_info_item_type_t XML_info_item::Type()
{
  return this->type;
}

/////////////
// Output_width_hint - Set output width hint.
// output_width_hint: new output width hint for item.
// returns old output width hint.
//public:
unsigned int XML_info_item::Output_width_hint(unsigned int output_width_hint)
{
  unsigned int old_output_width_hint;
  old_output_width_hint = this->output_width_hint;
  this->output_width_hint = output_width_hint;
  return old_output_width_hint;
}

/////////////
// Output_width_hint - Report output width hint.
// no arguments.
// returns current output width hint.
//public:
unsigned int XML_info_item::Output_width_hint()
{
  return this->output_width_hint;
}
