// Clase:    XML_internal_ent_info_item
// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:47 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#include "XML_internal_ent_info_item.h"

/////////////
// XML_internal_ent_info_item - Class constructor.
// no arguments.
//public:
XML_internal_ent_info_item::XML_internal_ent_info_item()
{
  this->name = NULL;
  this->content = NULL;
  Type(internal_ent_info_item);
}

/////////////
// ~XML_internal_ent_info_item - Class destructor.
// no arguments.
//public:
XML_internal_ent_info_item::~XML_internal_ent_info_item()
{
  if (name)
    {
      delete name;
      name = NULL;
    }
  if (content)
    {
      delete content;
      content = NULL;
    }
}


using std::vector;


/////////////
// Content - Set [content] property.
// content: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_internal_ent_info_item_p_t XML_internal_ent_info_item::Content(XML_string_t &content)
{
  if (!(&content)) return this;
  if (!this->content)
    this->content = new XML_string();
  *(this->content) = content;
  return this;
}

/////////////
// Content - Set [content] property.
// content: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_internal_ent_info_item_p_t XML_internal_ent_info_item::Content(XML_string_p_t content)
{
  if (!content)
    {
      if (this->content)
        {
          delete this->content;
          this->content = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->content)
    this->content = new XML_string();
  *(this->content) = *content;
  return this;
}

/////////////
// Content - Set [content] property.
// content: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_internal_ent_info_item_p_t XML_internal_ent_info_item::Content(const vector<XML_UCS4_char_t> &content)
{
  if (!(&content)) return this;
  if (!this->content)
    this->content = new XML_string();
  this->content->UCS4(content);
  return this;
}

/////////////
// Content - Set [content] property.
// content: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_internal_ent_info_item_p_t XML_internal_ent_info_item::Content(const vector<char> &content)
{
  if (!(&content)) return this;
  if (!this->content)
    this->content = new XML_string();
  this->content->UTF8(content);
  return this;
}

/////////////
// Content - Set [content] property.
// content: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_internal_ent_info_item_p_t XML_internal_ent_info_item::Content(const char* content)
{
  if (!content)
    {
      if (this->content)
        {
          delete this->content;
          this->content = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->content)
    this->content = new XML_string();
  vector<char> content_vector;
  for (unsigned int index = 0; index < strlen(content); ++index)
    content_vector.push_back(content[index]);
  return Content(content_vector);
}

/////////////
// Content - Access to [content] property.
// no arguments.
// returns property value.
//public:
XML_string_p_t XML_internal_ent_info_item::Content()
{
  return this->content;
}


using std::ostream;

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
// XML_internal_ent_info_item: XML_internal_ent_info_item to output.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, XML_internal_ent_info_item_t &xml_internal_ent_info_item)
{
  XML_internal_ent_info_item::Output(&xml_internal_ent_info_item, output);
  return output;
}

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
// XML_internal_ent_info_item: XML_internal_ent_info_item to output, passed as pointer.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, XML_internal_ent_info_item_p_t xml_internal_ent_info_item)
{
  XML_internal_ent_info_item::Output(xml_internal_ent_info_item, output);
  return output;
}

using std::ostream;
using std::endl;

/////////////
// Output - Write to a stream in XML format.
// item: XML_internal_ent_info_item to be output, passed as pointer.
// output: stream which receives data.
// returns true on success, false on fail.
//public:
//static
bool XML_internal_ent_info_item::Output(XML_internal_ent_info_item_p_t item, ostream &output)
{
  return Output(item, output, 0);
}

/////////////
// Output - Write to a stream in XML format.
// item: XML_internal_ent_info_item to be output, passed as pointer.
// output: stream which receives data.
// returns true on success, false on fail.
//public:
//static
bool XML_internal_ent_info_item::Output(XML_internal_ent_info_item_p_t item, ostream &output, unsigned int indentation)
{
  if (!item || !(&output)) return false;
  
  if (indentation)
    output << endl;
  for (unsigned int count = 0; count < indentation; ++count)
    output << "  ";
  output << "<";
  output << "!ENTITY";
  output << " " << item->Name();
  if (item->Content())
    {
      string unescaped_content;
      item->Content()->UTF8(&unescaped_content);
      unsigned int position;
      position = 0;
      while (position < unescaped_content.size())
        {
          position = unescaped_content.find('\'', position);
          if (position < unescaped_content.size())
            {
              unescaped_content.replace(position, 1, "&apos;");
              ++position;
            }
        }
      output << " '" << unescaped_content << "'";
    }
  output << ">";
  
  return true;
}


using std::vector;


/////////////
// Name - Set [name] property.
// name: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_internal_ent_info_item_p_t XML_internal_ent_info_item::Name(XML_string_t &name)
{
  if (!(&name)) return this;
  if (!this->name)
    this->name = new XML_string();
  *(this->name) = name;
  return this;
}

/////////////
// Name - Set [name] property.
// name: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_internal_ent_info_item_p_t XML_internal_ent_info_item::Name(XML_string_p_t name)
{
  if (!name)
    {
      if (this->name)
        {
          delete this->name;
          this->name = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->name)
    this->name = new XML_string();
  *(this->name) = *name;
  return this;
}

/////////////
// Name - Set [name] property.
// name: new value for property, encoded as UCS4.
// returns object pointer for operator concatenation.
//public:
XML_internal_ent_info_item_p_t XML_internal_ent_info_item::Name(const vector<XML_UCS4_char_t> &name)
{
  if (!(&name)) return this;
  if (!this->name)
    this->name = new XML_string();
  this->name->UCS4(name);
  return this;
}

/////////////
// Name - Set [name] property.
// name: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_internal_ent_info_item_p_t XML_internal_ent_info_item::Name(const vector<char> &name)
{
  if (!(&name)) return this;
  if (!this->name)
    this->name = new XML_string();
  this->name->UTF8(name);
  return this;
}

/////////////
// Name - Set [name] property.
// name: new value for property, encoded as UTF8.
// returns object pointer for operator concatenation.
//public:
XML_internal_ent_info_item_p_t XML_internal_ent_info_item::Name(const char* name)
{
  if (!name)
    {
      if (this->name)
        {
          delete this->name;
          this->name = (XML_string_p_t)NULL;
        }
      return this;
    }
  if (!this->name)
    this->name = new XML_string();
  vector<char> name_vector;
  for (unsigned int index = 0; index < strlen(name); ++index)
    name_vector.push_back(name[index]);
  return Name(name_vector);
}

/////////////
// Name - Access to [name] property.
// no arguments.
// returns property value.
//public:
XML_string_p_t XML_internal_ent_info_item::Name()
{
  return this->name;
}
