// Clase:    XML_string
// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:49 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#include "XML_string.h"

/////////////
// XML_string - Class constructor.
// no arguments.
//public:
XML_string::XML_string()
{

}

/////////////
// XML_string - Copy constructor.
// model Object to be copied.
//public:
XML_string::XML_string(const XML_string &model)
{
  *this = model;
}

/////////////
// ~XML_string - Class destructor.
// no arguments.
//public:
XML_string::~XML_string()
{

}


using std::vector;

/////////////
// ISO_8859_1 - Fill in a char vector with the ISO_8859_1 encoding.
// buffer: STL char vector to receive ISO_8859_1 characters.
//         Characters not directly mappable into this charset get
//         expanded to XML entity names.
// returns object pointer for operator concatenation.
//public:
XML_string_p_t XML_string::ISO_8859_1(vector<char>* p_result)
{
  if (!p_result) return this;
  
  int length;
  length = ISO_8859_1_length();
  if (length < 0) return this;
  p_result->erase(p_result->begin(), p_result->end());
  if (length == 0) return this;
  
  vector<XML_UCS4_char_t>::iterator UCS4_iterator;
  for (UCS4_iterator = ucs4_string.begin();
       UCS4_iterator < ucs4_string.end();
       ++UCS4_iterator)
    {
      XML_UCS4_char_t UCS4_char;
      UCS4_char = *UCS4_iterator;
      if (!(UCS4_char & 0xFFFFFF00))
        {
          p_result->push_back(char(UCS4_char));
        }
      else
        {
          Entity_for_UCS4(UCS4_char, p_result);
        }
    }
  return this;
}


/////////////
// ISO_8859_1 - Set UCS4 chars from ISO_8859_1 encoded char vector.
// source: STL char vector that contains ISO_8859_1 characters.
// returns object pointer for operator concatenation.
//public:
XML_string_p_t XML_string::ISO_8859_1(const vector<char> &source)
{
  if (!(&source)) return this;
  
  ucs4_string.erase(ucs4_string.begin(), ucs4_string.end());
  if (source.size() == 0) return this;
  
  vector<char>::const_iterator ISO_8859_1_iterator;
  ISO_8859_1_iterator = source.begin();
  while (ISO_8859_1_iterator < source.end())
    {
      XML_UCS4_char_t UCS4_char;
      char ISO_8859_1_char;
      ISO_8859_1_char = *ISO_8859_1_iterator;
      if (ISO_8859_1_char != '&')
        {
          UCS4_char = XML_UCS4_char_t((unsigned char)ISO_8859_1_char);
          ++ISO_8859_1_iterator;
        }
      else
        {
          UCS4_char = UCS4_for_entity(source, ISO_8859_1_iterator);
          // ISO_8859_1_iterator gets increased by UCS4_for_entity().
        }
      
      ucs4_string.push_back(UCS4_char);
    }
  return this;
}


/////////////
// ISO_8859_1 - Set UCS4 chars from ISO_8859_1 encoded C string.
// source: zero-terminated C string that contains ISO_8859_1 characters.
// returns object pointer for operator concatenation.
//public:
XML_string_p_t XML_string::ISO_8859_1(const char* source)
{
  if (!source) return this;
  
  ucs4_string.erase(ucs4_string.begin(), ucs4_string.end());
  if (strlen(source) == 0) return this;
  
  int index;
  index = 0;
  while (index < (int)strlen(source))
    {
      ISO_8859_1(source[index]);
      ++index;
    }
  return this;
}


/////////////
// ISO_8859_1 - Append UCS4 char from ISO_8859_1 encoded char.
// ISO_8859_1_char: ISO_8859_1 character.
// returns length of string in UCS4 format. -1 means error.
//public:
XML_string_p_t XML_string::ISO_8859_1(const char ISO_8859_1_char)
{
  XML_UCS4_char_t UCS4_char;
  
  UCS4_char = XML_UCS4_char_t((unsigned char)ISO_8859_1_char);
  ucs4_string.push_back(UCS4_char);
  
  return this;
}


/////////////
// ISO_8859_1_length - Return the length of the string in ISO_8859_1 encoding.
// no arguments.
// returns length of string in ISO_8859_1 format.
//public:
int XML_string::ISO_8859_1_length()
{
  return ucs4_string.size();
}

/////////////
// UCS4_string - Read only access to UCS4_string.
// no arguments.
// returns const UCS4_string.
//public:
const vector<XML_UCS4_char_t>& XML_string::UCS4_string()
{
  return this->ucs4_string;
}
using std::vector;

using std::ostream;

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
// XML_string: XML_string to output.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, XML_string_t &xml_string)
{
  XML_string::Output(&xml_string, output);
  return output;
}

/////////////
// operator<< - Overloading of << operator.
// output: output c++ stream.
// XML_string: XML_string to output, passed as pointer.
// returns output stream for operator composition.
//public:
ostream &operator<< (ostream &output, XML_string_p_t xml_string)
{
  XML_string::Output(xml_string, output);
  return output;
}


using std::istream;

/////////////
// operator>> - Overloading of >> operator.
// input: input c++ stream.
// XML_string: XML_string to get input.
// returns input stream for operator composition.
//public:
istream &operator>> (istream &input, XML_string_t &xml_string)
{
  XML_string::Input(&xml_string, input);
  return input;
}

/////////////
// operator>> - Overloading of >> operator.
// input: input c++ stream.
// XML_string: XML_string to get input, passed as
// pointer.
// returns input stream for operator composition.
//public:
istream &operator>> (istream &input, XML_string_p_t xml_string)
{
  XML_string::Input(xml_string, input);
  return input;
}


/////////////
// operator= - Overloading of = operator.
// source: C++ string encoded in UTF-8.
// returns object reference.
//public:
XML_string_t &XML_string::operator= (const string &source)
{
  return *this = source.c_str();
}

/////////////
// operator= - Overloading of = operator.
// source: C string encoded in UTF-8.
// returns object reference.
//public:
XML_string_t &XML_string::operator= (const char* source)
{
  if (!source) return *this;
  
  vector<unsigned char> buffer;
  for (unsigned int index = 0; index < strlen((const char*)source); ++index)
    buffer.push_back(source[index]);
  UTF8(buffer);
  
  return *this;
}

/////////////
// operator= - Overloading of = operator.
// source: XML string.
// returns object reference.
//public:
XML_string_t &XML_string::operator= (const XML_string &source)
{
  ucs4_string.clear();
  ucs4_string.insert(ucs4_string.end(),
                     source.ucs4_string.begin(),
                     source.ucs4_string.end()
                     );
  
  return *this;
}

/////////////
// operator== - Overloading of == operator.
// source: UCS4 character array.
// returns equality value. If the address of any term is NULL, returns false.
//public:
bool XML_string::operator== (const XML_string &source)
{
  if (!(&source) || !this) return false;
  if (source.ucs4_string.size() != ucs4_string.size())
    return false;
  
  vector<XML_UCS4_char_t>::const_iterator source_character_iterator;
  source_character_iterator = source.ucs4_string.begin();
  for (vector<XML_UCS4_char_t>::iterator
         character_iterator = ucs4_string.begin();
       character_iterator != ucs4_string.end();
       ++character_iterator)
    {
      if (*character_iterator != *source_character_iterator)
        return false;
      ++source_character_iterator;
    }
  
  return true;
}

/////////////
// operator== - Overloading of == operator.
// source: UCS4 character array.
// returns equality value. If the address of any term is NULL, returns false.
//public:
bool operator== (const vector<XML_UCS4_char_t> &source, XML_string &xml_string)
{
  return xml_string == source;
}

/////////////
// operator== - Overloading of == operator.
// source: UCS4 character array.
// returns equality value. If the address of any term is NULL, returns false.
//public:
bool XML_string::operator== (const vector<XML_UCS4_char_t> &source)
{
  if (!(&source) || !this) return false;
  if (source.size() != ucs4_string.size())
    return false;
  
  vector<XML_UCS4_char_t>::const_iterator source_character_iterator;
  source_character_iterator = source.begin();
  for (vector<XML_UCS4_char_t>::iterator
         character_iterator = ucs4_string.begin();
       character_iterator != ucs4_string.end();
       ++character_iterator)
    {
      if (*character_iterator != *source_character_iterator)
        return false;
      ++source_character_iterator;
    }
  
  return true;
}

/////////////
// operator== - Overloading of == operator.
// source: C++ string encoded in UTF-8.
// returns equality value. If the address of any term is NULL, returns false.
//public:
bool operator== (const string &source, XML_string &xml_string)
{
  return xml_string == source;
}

/////////////
// operator== - Overloading of == operator.
// source: C++ string encoded in UTF-8.
// returns equality value. If the address of any term is NULL, returns false.
//public:
bool operator== (const char* source, XML_string &xml_string)
{
  return xml_string == source;
}

/////////////
// operator== - Overloading of == operator.
// source: C++ string encoded in UTF-8.
// returns object reference. If the address of any term is NULL, returns false.
//public:
bool XML_string::operator== (const string &source)
{
  return *this == source.c_str();
}

/////////////
// operator== - Overloading of == operator.
// source: C string encoded in UTF-8.
// returns equality value. If the address of any term is NULL, returns false.
//public:
bool XML_string::operator== (const char* source)
{
  return operator== ((const unsigned char*)source);
}

/////////////
// operator== - Overloading of == operator.
// source: C string encoded in UTF-8.
// returns equality value. If the address of any term is NULL, returns false.
//public:
bool XML_string::operator== (const unsigned char* source)
{
  if (!source || !this) return false;
  
  unsigned int character_index;
  character_index = 0;
  
  const unsigned char* p_utf8_char;
  const unsigned char* source_end;
  p_utf8_char = source;
  source_end = source + strlen((const char*)source);
  while (character_index < ucs4_string.size())
    {
      int utf8_chars_decoded;
      XML_UCS4_char_t UCS4_char;
      utf8_chars_decoded =
        Unicode_translator::UTF8(&UCS4_char, &p_utf8_char, source_end);
      if (utf8_chars_decoded == -1)
        break;// Error or end of buffer.
      else
        if (UCS4_char != ucs4_string[character_index])
          return false;
      ++character_index;
    }
  
  return (character_index == ucs4_string.size()
          && p_utf8_char == source_end);
}

/////////////
// operator+= - Overloading of += operator.
// character: UCS4 character.
// returns object reference.
//public:
XML_string_t &XML_string::operator+= (const XML_UCS4_char_t character)
{
  ucs4_string.push_back(character);
  
  return *this;
}

/////////////
// operator+= - Overloading of += operator.
// source: UCS4 character vector.
// returns object reference.
//public:
XML_string_t &XML_string::operator+= (const vector<XML_UCS4_char_t> &source)
{
  ucs4_string.insert(ucs4_string.end(), source.begin(), source.end());
  
  return *this;
}

/////////////
// operator+= - Overloading of += operator.
// source: XML_string.
// returns object reference.
//public:
XML_string_t &XML_string::operator+= (const XML_string &source)
{
  ucs4_string.insert(ucs4_string.end(),
                     source.ucs4_string.begin(),
                     source.ucs4_string.end()
                     );
  
  return *this;
}

/////////////
// operator+= - Overloading of += operator.
// source: C++ string encoded in UTF-8.
// returns object reference.
//public:
XML_string_t &XML_string::operator+= (const string &source)
{
  return *this += source.c_str();
}

/////////////
// operator+= - Overloading of += operator.
// source: C string encoded in UTF-8.
// returns object reference.
//public:
XML_string_t &XML_string::operator+= (const char* source)
{
  return operator+= ((const unsigned char*)source);
}

/////////////
// operator+= - Overloading of += operator.
// source: C string encoded in UTF-8.
// returns object reference.
//public:
XML_string_t &XML_string::operator+= (const unsigned char* source)
{
  if (!source) return *this;
  
  const unsigned char* source_end;
  source_end = source + strlen((const char*)source);
  if (source_end == source) return *this;
  
  const unsigned char* UTF8_iterator;
  UTF8_iterator = source;
  while (UTF8_iterator < source_end)
    {
      int utf8_chars_decoded;
      XML_UCS4_char_t UCS4_char;
      utf8_chars_decoded =
        Unicode_translator::UTF8(&UCS4_char, &UTF8_iterator, source_end);
      if (utf8_chars_decoded == -1)
        break;// Error.
      else
        ucs4_string.push_back(UCS4_char);
    }
  return *this;
}

using std::istream;
using std::string;

/////////////
// Input - Get from stream in UTF-8 format.
// input: stream which receives data.
// returns true on success, false on fail.
//public:
//static
bool XML_string::Input(XML_string_p_t xml_string, istream &input)
{
  if (!xml_string) return false;
  
  string utf8_buffer;
  input >> utf8_buffer;
  xml_string->UTF8(utf8_buffer);
  
  return true;
}



using std::vector;

/////////////
// UCS4 - Fill in a XML_UCS4_char_t vector with the UCS4 encoding.
// p_result: STL XML_UCS4_char_t vector to receive UCS4 characters.
//           It's erased before putting the new characters.
// returns object pointer for operator concatenation.
//public:
XML_string_p_t XML_string::UCS4(vector<XML_UCS4_char_t>* p_result)
{
  if (!p_result) return this;
  
  int length;
  length = UCS4_length();
  if (length < 0) return this;
  p_result->erase(p_result->begin(), p_result->end());
  if (length == 0) return this;
  
  p_result->insert(p_result->end(), ucs4_string.begin(), ucs4_string.end());
  
  return this;
}


/////////////
// UCS4 - Set UCS4 chars from XML_UCS4_char_t vector.
// source: STL XML_UCS4_char_t vector that contains UCS4 characters.
// returns object pointer for operator concatenation.
//public:
XML_string_p_t XML_string::UCS4(const vector<XML_UCS4_char_t> &source)
{
  if (!(&source)) return this;
  
  ucs4_string.erase(ucs4_string.begin(), ucs4_string.end());
  if (source.size() == 0) return this;
  
  ucs4_string.insert(ucs4_string.end(), source.begin(), source.end());
  
  return this;
}


/////////////
// UCS4 - Set UCS4 chars from XML_UCS4_char_t vector.
// source: STL XML_UCS4_char_t vector that contains UCS4 characters.
// begin: index of first character to be copied.
// end: index of last character plus one.
// returns object pointer for operator concatenation.
//public:
XML_string_p_t XML_string::UCS4(const vector<XML_UCS4_char_t> &source, unsigned int begin, unsigned int end)
{
  if (!(&source)) return this;

  if (end > source.size())
    end = source.size();
  if (begin > end)
    begin = end;
  
  ucs4_string.erase(ucs4_string.begin(), ucs4_string.end());
  if (source.size() == 0) return this;
  
  ucs4_string.insert(ucs4_string.end(),
                     source.begin() + begin, source.begin() + end);
  
  return this;
}


/////////////
// UCS4 - Append UCS4 char.
// UCS4_char: UCS4 character.
// returns object pointer for operator concatenation.
//public:
XML_string_p_t XML_string::UCS4(XML_UCS4_char_t UCS4_char)
{
  ucs4_string.push_back(UCS4_char);
  
  return this;
}


/////////////
// UCS4_length - Return the length of the string in UCS4 encoding.
// no arguments.
// returns length of string in UCS4 format.
//public:
int XML_string::UCS4_length()
{
  return ucs4_string.size();
}

using std::ostream;

/////////////
// Output - Write to a stream in UTF-8 format.
// output: stream which receives data.
// returns true on success, false on fail.
//public:
//static
bool XML_string::Output(XML_string_p_t xml_string, ostream &output)
{
  if (!xml_string) return false;
  
  vector<char> buffer;
  for (vector<XML_UCS4_char_t>::const_iterator
         ucs4_iterator = xml_string->UCS4_string().begin();
       ucs4_iterator < xml_string->UCS4_string().end();
       ++ucs4_iterator)
    {
      buffer.erase(buffer.begin(), buffer.end());
      if (Unicode_translator::UTF8(*ucs4_iterator, &buffer) == -1)
        break;// Error.
      
      switch (*ucs4_iterator)
        {
        case XML_UCS4_char_t('<'):  output << "&lt;";   break;
        case XML_UCS4_char_t('>'):  output << "&gt;";   break;
        case XML_UCS4_char_t('&'):  output << "&amp;";  break;
        case XML_UCS4_char_t('\''): output << "&apos;"; break;
        case XML_UCS4_char_t('"'):  output << "&quot;"; break;
        default:
          for (vector<char>::const_iterator
                 buffer_iterator = buffer.begin();
               buffer_iterator != buffer.end();
               ++buffer_iterator)
            {
              output << *buffer_iterator;
            }
        }
    }
  
  return true;
}
using std::vector;

/////////////
// Split - Fill in a XML_string vector with the parts delimited by a character.
// p_result: XML_string STL vector to receive XML strings.
//           It's erased before putting the new strings.
//           Puts at least an unary list with a void string.
// returns object pointer for operator concatenation.
//public:
XML_string_p_t XML_string::Split(const XML_UCS4_char_t delimiter, vector<XML_string_t>* p_result)
{
  if (!p_result) return this;
  
  p_result->erase(p_result->begin(), p_result->end());
  
  XML_string_t piece;
  for (vector<XML_UCS4_char_t>::const_iterator
         character_iterator = ucs4_string.begin();
       character_iterator != ucs4_string.end();
       ++character_iterator)
    {
      if (delimiter == *character_iterator)
        {
          p_result->push_back(piece);
          piece = "";
        }
      else
        {
          piece += *character_iterator;
        }
    }
  p_result->push_back(piece);
  
  return this;
}


using std::vector;

 #include "SGML_entities.h"
 #include "MathML_entities.h"

/////////////
// Search_entity - Search for an entity in a SGML entity list.
// entity_name: name.
// returns UCS4 code point, or ENTITY_LIST_TERMINATOR if not found.
XML_UCS4_char_t Search_entity(const char* entity_name, SGML_entity_entry* entity_list)
{
  if (!entity_name || !entity_list) return 0xFFFFFFFF;// Invalid code point.
  int index;
  index = 0;
  while (entity_list[index].UCS4_code_point != ENTITY_LIST_TERMINATOR)
    {
      if (strcmp(entity_name, entity_list[index].name) == 0) break;
      ++index;
    }
  return entity_list[index].UCS4_code_point;
}

/////////////
// UCS4_for_entity - Return UCS4 code point for SGML entity name.
// buffer: char vector where the entity is.
// entity_begin: iterator pointing to first entity character, that must be '&'.
//               Gets incremented past the entity end char ';'. So, the caller
//               shouldn't increase it.
//               Maximum entity name length is 127. Just a try to stop wild
//               runaways.
// returns UCS4 code point.
//public:
//static
XML_UCS4_char_t XML_string::UCS4_for_entity(const vector<char> &buffer, vector<char>::const_iterator &entity_begin)
{
  if (!(&buffer)) return 0x0000FFFD;
  if (*entity_begin != '&') return 0x0000FFFD;
  int length;
  for (length = 0; length < 128; ++length)
    if (*(entity_begin + length) == ';') break;
  ++length;// Length includes & and ;.
  if (length > 127) return 0x0000FFFD;
  char entity_name[128];
  for (int index = 0; index < length; ++index)
    entity_name[index] = *(entity_begin + index);
  entity_name[length] = 0;
  
  XML_UCS4_char_t UCS4_char;
  if (entity_name[1] == '#')
    {
      if (entity_name[2] == 'x')
        sscanf(entity_name + 3, "%lX", &UCS4_char);
      else
        sscanf(entity_name + 2, "%ld", &UCS4_char);
    }
  else
    {
      UCS4_char = Search_entity(entity_name, mathml_entity_list);
      if (UCS4_char == ENTITY_LIST_TERMINATOR)
        UCS4_char = Search_entity(entity_name, sgml_entity_list);
      if (UCS4_char == ENTITY_LIST_TERMINATOR)
        UCS4_char = 0x0000FFFD;// Unicode REPLACEMENT CHARACTER.
    }
  entity_begin += length;
  return UCS4_char;
}

using std::vector;

/////////////
// UTF8 - Fill in a vector<unsigned char> with the UTF-8 encoding, and return its length.
// p_result: STL vector<unsigned char> to receive UTF-8 characters.
//           It's erased before putting the new characters.
// returns object pointer for operator concatenation.
//public:
XML_string_p_t XML_string::UTF8(vector<unsigned char>* p_result)
{
  if (!p_result) return this;
  
  int length;
  length = UTF8_length();
  if (length < 0) return this;
  p_result->erase(p_result->begin(), p_result->end());
  if (length == 0) return this;
  
  for (vector<XML_UCS4_char_t>::iterator
         ucs4_iterator = ucs4_string.begin();
       ucs4_iterator != ucs4_string.end();
       ++ucs4_iterator)
    {
      if (Unicode_translator::UTF8(*ucs4_iterator, p_result) == -1)
        break;// Error.
    }
  return this;
}


/////////////
// UTF8 - Fill in a string with the UTF-8 encoding, and return its length.
// p_result: STL string to receive UTF-8 characters.
//           It's erased before putting the new characters.
// returns object pointer for operator concatenation.
//public:
XML_string_p_t XML_string::UTF8(string* p_result)
{
  if (!p_result) return this;
  
  int length;
  length = UTF8_length();
  if (length < 0) return this;
  p_result->erase(p_result->begin(), p_result->end());
  if (length == 0) return this;
  
  for (vector<XML_UCS4_char_t>::iterator
         ucs4_iterator = ucs4_string.begin();
       ucs4_iterator != ucs4_string.end();
       ++ucs4_iterator)
    {
      if (Unicode_translator::UTF8(*ucs4_iterator, p_result) == -1)
        break;// Error.
    }
  return this;
}


/////////////
// UTF8 - Set UCS4 chars from UTF-8 encoded char vector.
// source: STL char vector that contains UTF-8 characters.
// returns object pointer for operator concatenation.
//public:
XML_string_p_t XML_string::UTF8(const vector<char> &source)
{
  if (!(&source)) return this;
  
  ucs4_string.erase(ucs4_string.begin(), ucs4_string.end());
  if (source.size() == 0) return this;
  
  vector<char>::const_iterator UTF8_iterator;
  UTF8_iterator = source.begin();
  while (UTF8_iterator < source.end())
    {
      int utf8_chars_decoded;
      XML_UCS4_char_t UCS4_char;
      utf8_chars_decoded =
        Unicode_translator::UTF8(&UCS4_char, &UTF8_iterator, source.end());
      if (utf8_chars_decoded == -1)
        break;// Error.
      else
        ucs4_string.push_back(UCS4_char);
    }
  return this;
}

/////////////
// UTF8 - Set UCS4 chars from UTF-8 encoded char vector.
// source: STL char vector that contains UTF-8 characters.
// returns object pointer for operator concatenation.
//public:
XML_string_p_t XML_string::UTF8(const vector<unsigned char> &source)
{
  if (!(&source)) return this;
  
  ucs4_string.erase(ucs4_string.begin(), ucs4_string.end());
  if (source.size() == 0) return this;
  
  vector<unsigned char>::const_iterator UTF8_iterator;
  UTF8_iterator = source.begin();
  while (UTF8_iterator < source.end())
    {
      int utf8_chars_decoded;
      XML_UCS4_char_t UCS4_char;
      utf8_chars_decoded =
        Unicode_translator::UTF8(&UCS4_char, &UTF8_iterator, source.end());
      if (utf8_chars_decoded == -1)
        break;// Error.
      else
        ucs4_string.push_back(UCS4_char);
    }
  return this;
}


/////////////
// UTF8 - Set UCS4 chars from UTF-8 encoded C++ string.
// source: C++ string that contains UTF-8 characters.
// returns object pointer for operator concatenation.
//public:
XML_string_p_t XML_string::UTF8(const string &source)
{
  if (!(&source)) return this;
  
  ucs4_string.erase(ucs4_string.begin(), ucs4_string.end());
  if (source.size() == 0) return this;
  
  string::const_iterator UTF8_iterator;
  UTF8_iterator = source.begin();
  while (UTF8_iterator < source.end())
    {
      int utf8_chars_decoded;
      XML_UCS4_char_t UCS4_char;
      utf8_chars_decoded =
        Unicode_translator::UTF8(&UCS4_char,
                                 &UTF8_iterator,
                                 source.end());
      if (utf8_chars_decoded == -1)
        break;// Error.
      else
        ucs4_string.push_back(UCS4_char);
    }
  return this;
}


/////////////
// UTF8 - Set UCS4 chars from UTF-8 encoded C string.
// source: zero-terminated C string that contains UTF-8 characters.
// returns object pointer for operator concatenation.
//public:
XML_string_p_t XML_string::UTF8(const unsigned char* source)
{
  if (!source) return this;
  
  ucs4_string.erase(ucs4_string.begin(), ucs4_string.end());
  const unsigned char* source_end;
  source_end = source + strlen((const char*)source);
  if (source_end == source) return this;
  
  const unsigned char* UTF8_iterator;
  UTF8_iterator = source;
  while (UTF8_iterator < source_end)
    {
      int utf8_chars_decoded;
      XML_UCS4_char_t UCS4_char;
      utf8_chars_decoded =
        Unicode_translator::UTF8(&UCS4_char, &UTF8_iterator, source_end);
      if (utf8_chars_decoded == -1)
        break;// Error.
      else
        ucs4_string.push_back(UCS4_char);
    }
  return this;
}


/////////////
// UTF8_length - Return the length of the string in UTF-8 encoding.
// no arguments.
// returns length of string in UTF-8 format.
//public:
int XML_string::UTF8_length()
{
  int length;
  length = 0;
  vector<XML_UCS4_char_t>::iterator UCS4_iterator;
  for (UCS4_iterator = ucs4_string.begin();
       UCS4_iterator < ucs4_string.end();
       ++UCS4_iterator)
    {
      unsigned long UCS4_char;
      UCS4_char = (unsigned long)*UCS4_iterator;
      if (UCS4_char < 0x00000080) { length += 1; continue; };
      if (UCS4_char < 0x00000800) { length += 2; continue; };
      if (UCS4_char < 0x00010000) { length += 3; continue; };
      if (UCS4_char < 0x00200000) { length += 4; continue; };
      if (UCS4_char < 0x04000000) { length += 5; continue; };
      if (UCS4_char < 0x80000000) { length += 6; continue; };
      break;
    }
  return length;
}

using std::vector;

 #include "SGML_entities.h"
 #include "MathML_entities.h"

/////////////
// Search_code_point - Search for an entity in an SGML entity list.
// UCS4_char: code point.
// returns pointer to entity name, or NULL if not found.
const char* Search_code_point(XML_UCS4_char_t UCS4_char, SGML_entity_entry* entity_list)
{
  if (!entity_list) return NULL;
  int index;
  index = 0;
  while (entity_list[index].UCS4_code_point != ENTITY_LIST_TERMINATOR)
    {
      if (UCS4_char == entity_list[index].UCS4_code_point) break;
      ++index;
    }
  return entity_list[index].name;
}

/////////////
// Entity_for_UCS4 - Append SGML entity name to char vector.
// UCS4_char: UCS4 code point.
// buffer: char vector where the entity is to be appended.
// returns entity name length, including & and ;.
//public:
//static
int XML_string::Entity_for_UCS4(long UCS4_char, vector<char>* p_buffer)
{
  if (!p_buffer) return 0;
  int length;
  const char* entity_name = NULL;
  
  entity_name = Search_code_point(UCS4_char, mathml_entity_list);
  if (entity_name == NULL)
    entity_name = Search_code_point(UCS4_char, sgml_entity_list);
  
  if (entity_name)
    {
      length = strlen(entity_name);
      int index;
      index = 0;
      while (entity_name[index])
        p_buffer->push_back(entity_name[index++]);
    }
  else
    {
      char* numerical_entity = new char[16];
      sprintf(numerical_entity, "&#x%lX;", UCS4_char);
      length = strlen(numerical_entity);
      int index;
      index = 0;
      while (numerical_entity[index])
        p_buffer->push_back(numerical_entity[index++]);
      delete [] numerical_entity;
      numerical_entity = NULL;
    }
  
  return length;
}
