// Clase:    XML_string
// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:49 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#ifndef _H_XML_string
#define _H_XML_string

#include <iostream>
#include <vector> // STL
using std::vector;
#include "../XML_typedefs.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../../Unicode_translator/Unicode_translator.h"
using std::ostream;
using std::istream;
#include <string> // STL
using std::string;
#include "../XML_character_info_item/XML_character_info_item.h"


class XML_string
{
 private:
  vector<XML_UCS4_char_t> ucs4_string;// ISO 10646
 public:
  XML_string();
  XML_string(const XML_string &model);
  ~XML_string();
  XML_string_p_t ISO_8859_1(vector<char>* p_result);
  XML_string_p_t ISO_8859_1(const vector<char> &source);
  XML_string_p_t ISO_8859_1(const char* source);
  XML_string_p_t ISO_8859_1(const char ISO_8859_1_char);
  int ISO_8859_1_length();
  const vector<XML_UCS4_char_t>& UCS4_string();
  XML_string_t &operator= (const string &source);
  XML_string_t &operator= (const char* source);
  XML_string_t &operator= (const XML_string &source);
  bool operator== (const XML_string &source);
  bool operator== (const vector<XML_UCS4_char_t> &source);
  bool operator== (const string &source);
  bool operator== (const char* source);
  bool operator== (const unsigned char* source);
  XML_string_t &operator+= (const XML_UCS4_char_t character);
  XML_string_t &operator+= (const vector<XML_UCS4_char_t> &source);
  XML_string_t &operator+= (const XML_string &source);
  XML_string_t &operator+= (const string &source);
  XML_string_t &operator+= (const char* source);
  XML_string_t &operator+= (const unsigned char* source);
  static bool Input(XML_string_p_t xml_string, istream &input);
  XML_string_p_t UCS4(vector<XML_UCS4_char_t>* p_result);
  XML_string_p_t UCS4(const vector<XML_UCS4_char_t> &source);
  XML_string_p_t UCS4(const vector<XML_UCS4_char_t> &source, unsigned int begin, unsigned int end);
  XML_string_p_t UCS4(XML_UCS4_char_t UCS4_char);
  int UCS4_length();
  static bool Output(XML_string_p_t xml_string, ostream &output);
  XML_string_p_t Split(const XML_UCS4_char_t delimiter, vector<XML_string_t>* p_result);
  static XML_UCS4_char_t UCS4_for_entity(const vector<char> &buffer, vector<char>::const_iterator &entity_begin);
  XML_string_p_t UTF8(vector<unsigned char>* p_result);
  XML_string_p_t UTF8(string* p_result);
  XML_string_p_t UTF8(const vector<char> &source);
  XML_string_p_t UTF8(const vector<unsigned char> &source);
  XML_string_p_t UTF8(const string &source);
  XML_string_p_t UTF8(const unsigned char* source);
  int UTF8_length();
  static int Entity_for_UCS4(long UCS4_char, vector<char>* p_buffer);
};

extern ostream &operator<< (ostream &, XML_string_t &);
extern ostream &operator<< (ostream &, XML_string_p_t);
extern istream &operator>> (istream &, XML_string_t &);
extern istream &operator>> (istream &, XML_string_p_t);
extern bool operator== (const vector<XML_UCS4_char_t> &, XML_string &);
extern bool operator== (const string &, XML_string &);
extern bool operator== (const char*, XML_string &);

#endif
