// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:51 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#include <stdio.h>
#include <stdlib.h>

#include "../src/Interface/Interface.h"
#include "../src/OpenMath/OpenMath_object/OpenMath_object.h"
#include "../src/QMath_parser/QMath_parser.h"
#include "../src/QMath_parser/Tokenizer/Tokenizer.h"
#include "../src/QMath_parser/Tokenizer/Unicode_regexp/Unicode_regexp.h"
#include "../src/QMath_parser/Tokenizer/Unicode_regexp/Unicode_regexp_node/Unicode_regexp_node.h"
#include "../src/Unicode_data/Unicode_data.h"
#include "../src/Unicode_translator/Unicode_translator.h"
#include "../src/XML_infoset/XML_ent_start_marker_info_item/XML_ent_start_marker_info_item.h"
#include "../src/XML_infoset/XML_unexp_ent_ref_info_item/XML_unexp_ent_ref_info_item.h"
#include "../src/XML_infoset/XML_info_item/XML_info_item.h"
#include "../src/XML_infoset/XML_doc_info_item/XML_doc_info_item.h"
#include "../src/XML_infoset/XML_string/XML_string.h"
#include "../src/XML_infoset/XML_doc_type_decl_info_item/XML_doc_type_decl_info_item.h"
#include "../src/XML_infoset/XML_comment_info_item/XML_comment_info_item.h"
#include "../src/XML_infoset/XML_character_info_item/XML_character_info_item.h"
#include "../src/XML_infoset/XML_namespace_info_item/XML_namespace_info_item.h"
#include "../src/XML_infoset/XML_external_ent_info_item/XML_external_ent_info_item.h"
#include "../src/XML_infoset/XML_elem_info_item/XML_elem_info_item.h"
#include "../src/XML_infoset/XML_internal_ent_info_item/XML_internal_ent_info_item.h"
#include "../src/XML_infoset/XML_cdata_end_marker_info_item/XML_cdata_end_marker_info_item.h"
#include "../src/XML_infoset/XML_unparsed_ent_info_item/XML_unparsed_ent_info_item.h"
#include "../src/XML_infoset/XML_attr_info_item/XML_attr_info_item.h"
#include "../src/XML_infoset/XML_notation_info_item/XML_notation_info_item.h"
#include "../src/XML_infoset/XML_cdata_start_marker_info_item/XML_cdata_start_marker_info_item.h"
#include "../src/XML_infoset/XML_ent_end_marker_info_item/XML_ent_end_marker_info_item.h"
#include "../src/XML_infoset/XML_proc_inst_info_item/XML_proc_inst_info_item.h"
#include "../src/XML_parser/XML_parser.h"

#include <fstream>
using std::cout;
using std::cin;
using std::endl;

int main(void)
{
  
  //XML_String mi_XML_String;
  //cin >> mi_XML_String;
  //ofstream output;
  //output.open("prueba.utf8");
  //cout << mi_XML_String;
  //output.close();
  
  //<?xml version='1.0'?>
  //<!DOCTYPE book PUBLIC "-//Norman Walsh//DTD DocBk XML V1.4//EN"
  //                  "http://docbook.org/docbook/xml/1.4/db3xml.dtd">
  //<chapter id="chap1"><title>Chapter Title</title>
  //<para>
  //This <emphasis>paragraph</emphasis> is important.
  //</para>
  //</chapter>
  
  XML_doc_info_item_p_t document;
  XML_elem_info_item_p_t hook;
  ((document = new XML_doc_info_item)
   ->Version("1.0")
   ->Standalone(XML_doc_info_item::yes)
   ->Children((new XML_doc_type_decl_info_item)
              ->Public_identifier("-//OASIS//DTD"
                                  " DocBook XML V4.1//EN")
              ->System_identifier("http://www.oasis-open.org"
                                  "/docbook/xml/4.0"
                                  "/docbookx.dtd")
              )
   ->Document_element((new XML_elem_info_item)
                      ->Local_name("book")
                      ->Children((hook = new XML_elem_info_item)
                                 ->Local_name("chapter")
                                 ->Children((new XML_comment_info_item)
                                            ->Content("This is the hand")
                                            )
                                 ->Attributes((new XML_attr_info_item)
                                              ->Local_name("id")
                                              ->Normalized_value("chap1")
                                              )
                                 ->Children((new XML_elem_info_item)
                                            ->Local_name("title")
                                            ->Children("On the Future of "
                                                       "Aviation")
                                            )
                                 ->Children((new XML_elem_info_item)
                                            ->Local_name("emph")
                                            ->Children("Very Important Text")
                                            )
                                 ->Children("And so they lived happily "
                                            "ever after...")
                                 ->Children("\nTill they died.")
                                 )
                      )
   );
  
  cout << document << "\n";
  
  cout << "\ndocument has " << document->Children().size() << " children.\n";
  
  cout << "\n----- Now we test OpenMath\n";
  
  OpenMath_object_p_t expression;
  expression = new OpenMath_object;
  expression->Integer("12888321923912333");
  cout << expression;
  expression->Fp_number("3.1415926535");
  cout << expression;
  expression->Fp_number(3.1415926535);
  cout << expression;
  expression->Integer(0x0000FFFF);
  cout << expression;
  cout << "\nNow a C++ double float cast: " << double(expression->Fp_number(3.1415926535)->Fp_number()) * 2.0;
  cout << "\nConstruct:\n";
  expression->Application(2,
                          (new OpenMath_object)
                          ->Symbol("transc1:sin"),
                          (new OpenMath_object)
                          ->Symbol("x")
                          );
  cout << expression << endl;
  expression->Application(3,
                          (new OpenMath_object)
                          ->Symbol("cartessian-products:pair"),
                          (new OpenMath_object)
                          ->Variable("S"),
                          (new OpenMath_object)
                          ->Variable("*")
                          );
  cout << expression << endl;
  expression->Binding(3,
                      (new OpenMath_object)
                      ->Symbol("fns1:lambda"),
                      (new OpenMath_object)
                      ->Symbol("x"),
                      (new OpenMath_object)
                      ->Application(3,
                                    (new OpenMath_object)
                                    ->Symbol("arith1:plus"),
                                    (new OpenMath_object)
                                    ->Symbol("x"),
                                    (new OpenMath_object)
                                    ->Integer("2")
                                    )
                      );
  cout << expression << endl;
  cout << "\nNow we embed the last OpenMath expression in the XML infoset:\n";
  cout << "Dumping to XML_elem_info_item...";
  hook->Children(expression->To_XML_elem_info_item());
  cout << "Done.\nSerializing document...\n";
  cout << document << endl;
  cout << "\nFinished.\n";
  
  return 0;
}
