// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:51 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#include <stdio.h>
#include <stdlib.h>

#include "../src/Interface/Interface.h"
#include "../src/OpenMath/OpenMath_object/OpenMath_object.h"
#include "../src/QMath_parser/QMath_parser.h"
#include "../src/QMath_parser/Tokenizer/Tokenizer.h"
#include "../src/QMath_parser/Tokenizer/Unicode_regexp/Unicode_regexp.h"
#include "../src/QMath_parser/Tokenizer/Unicode_regexp/Unicode_regexp_node/Unicode_regexp_node.h"
#include "../src/Unicode_data/Unicode_data.h"
#include "../src/Unicode_translator/Unicode_translator.h"
#include "../src/XML_infoset/XML_ent_start_marker_info_item/XML_ent_start_marker_info_item.h"
#include "../src/XML_infoset/XML_unexp_ent_ref_info_item/XML_unexp_ent_ref_info_item.h"
#include "../src/XML_infoset/XML_info_item/XML_info_item.h"
#include "../src/XML_infoset/XML_doc_info_item/XML_doc_info_item.h"
#include "../src/XML_infoset/XML_string/XML_string.h"
#include "../src/XML_infoset/XML_doc_type_decl_info_item/XML_doc_type_decl_info_item.h"
#include "../src/XML_infoset/XML_comment_info_item/XML_comment_info_item.h"
#include "../src/XML_infoset/XML_character_info_item/XML_character_info_item.h"
#include "../src/XML_infoset/XML_namespace_info_item/XML_namespace_info_item.h"
#include "../src/XML_infoset/XML_external_ent_info_item/XML_external_ent_info_item.h"
#include "../src/XML_infoset/XML_elem_info_item/XML_elem_info_item.h"
#include "../src/XML_infoset/XML_internal_ent_info_item/XML_internal_ent_info_item.h"
#include "../src/XML_infoset/XML_cdata_end_marker_info_item/XML_cdata_end_marker_info_item.h"
#include "../src/XML_infoset/XML_unparsed_ent_info_item/XML_unparsed_ent_info_item.h"
#include "../src/XML_infoset/XML_attr_info_item/XML_attr_info_item.h"
#include "../src/XML_infoset/XML_notation_info_item/XML_notation_info_item.h"
#include "../src/XML_infoset/XML_cdata_start_marker_info_item/XML_cdata_start_marker_info_item.h"
#include "../src/XML_infoset/XML_ent_end_marker_info_item/XML_ent_end_marker_info_item.h"
#include "../src/XML_infoset/XML_proc_inst_info_item/XML_proc_inst_info_item.h"
#include "../src/XML_parser/XML_parser.h"

#include <fstream>
using std::cout;
using std::cin;
using std::endl;

void Test_Unicode_regexp()
{
  cout << "Vamos a comprobar Unicode_regexp.\n";
  Unicode_regexp regexp;
  XML_string regexp_string;
  
  string iso_8859_1_string;
  cout << "Expresión regular:";
  cin >> iso_8859_1_string;
  
  regexp_string.ISO_8859_1(iso_8859_1_string.c_str());
  regexp.Compile(regexp_string.UCS4_string());
  
  cout << "Expresión regular (" << iso_8859_1_string
       << ") compilada como: " << regexp << "\n";
  
  while (!cin.eof())
    {
      XML_string input_string;
      
      cin >> iso_8859_1_string;
      input_string.ISO_8859_1(iso_8859_1_string.c_str());
      
      regexp.Reset();
      if (!cin.eof())
        {
          regexp.Signal_condition(Unicode_regexp::
                                  SPECIAL_CONDITION_AT_BEGINNING_OF_LINE);
          for (string::const_iterator
                 char_iterator = iso_8859_1_string.begin();
               char_iterator != iso_8859_1_string.end();
               ++char_iterator)
            {
              cerr<< "Carácter '" << *char_iterator << "' ("
                  << hex << int((unsigned char)*char_iterator) << dec << ") ";
              if (regexp.Match(XML_UCS4_char_t((unsigned char)*char_iterator)))
                cerr << "Encaja en " << regexp << ".";
              else
                cerr << "No encaja en " << regexp << ".";
              if (char_iterator+1 == iso_8859_1_string.end())
                regexp.Signal_condition(Unicode_regexp::
                                        SPECIAL_CONDITION_AT_END_OF_LINE);
              if (regexp.Match_is_complete())
                cerr << "(Regexp completa)";
              cerr << "\n";
            }
          
          cerr << "Mayor longitud encajada: "
               << regexp.Match()//input_string.UCS4_string())
               << "\n" << flush;
        }
    }
}

int main(int argc, void* argv[])
{
  if (argc > 1)
    {
      cout << "Usage: regexp_test\n"
           << "Test Unicode_regexp class.\n"
           << "\nReport bugs to matmota@linuxfreak.com\n";
    }
  else
    {
      Test_Unicode_regexp();
    }
  
  return 0;
}
