// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:51 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#include <stdio.h>
#include <stdlib.h>

#include "../src/Interface/Interface.h"
#include "../src/OpenMath/OpenMath_object/OpenMath_object.h"
#include "../src/QMath_parser/QMath_parser.h"
#include "../src/QMath_parser/Tokenizer/Tokenizer.h"
#include "../src/QMath_parser/Tokenizer/Unicode_regexp/Unicode_regexp.h"
#include "../src/QMath_parser/Tokenizer/Unicode_regexp/Unicode_regexp_node/Unicode_regexp_node.h"
#include "../src/Unicode_data/Unicode_data.h"
#include "../src/Unicode_translator/Unicode_translator.h"
#include "../src/XML_infoset/XML_ent_start_marker_info_item/XML_ent_start_marker_info_item.h"
#include "../src/XML_infoset/XML_unexp_ent_ref_info_item/XML_unexp_ent_ref_info_item.h"
#include "../src/XML_infoset/XML_info_item/XML_info_item.h"
#include "../src/XML_infoset/XML_doc_info_item/XML_doc_info_item.h"
#include "../src/XML_infoset/XML_string/XML_string.h"
#include "../src/XML_infoset/XML_doc_type_decl_info_item/XML_doc_type_decl_info_item.h"
#include "../src/XML_infoset/XML_comment_info_item/XML_comment_info_item.h"
#include "../src/XML_infoset/XML_character_info_item/XML_character_info_item.h"
#include "../src/XML_infoset/XML_namespace_info_item/XML_namespace_info_item.h"
#include "../src/XML_infoset/XML_external_ent_info_item/XML_external_ent_info_item.h"
#include "../src/XML_infoset/XML_elem_info_item/XML_elem_info_item.h"
#include "../src/XML_infoset/XML_internal_ent_info_item/XML_internal_ent_info_item.h"
#include "../src/XML_infoset/XML_cdata_end_marker_info_item/XML_cdata_end_marker_info_item.h"
#include "../src/XML_infoset/XML_unparsed_ent_info_item/XML_unparsed_ent_info_item.h"
#include "../src/XML_infoset/XML_attr_info_item/XML_attr_info_item.h"
#include "../src/XML_infoset/XML_notation_info_item/XML_notation_info_item.h"
#include "../src/XML_infoset/XML_cdata_start_marker_info_item/XML_cdata_start_marker_info_item.h"
#include "../src/XML_infoset/XML_ent_end_marker_info_item/XML_ent_end_marker_info_item.h"
#include "../src/XML_infoset/XML_proc_inst_info_item/XML_proc_inst_info_item.h"
#include "../src/XML_parser/XML_parser.h"

#include <fstream>
using std::cout;
using std::cin;
using std::endl;

int main(void)
{
  Tokenizer_t tokenizer;
  int token_type;
  token_type = T_UCS4_CHAR;
  YYSTYPE token_value;
  
  cout << "Inicio:\n";
  int previous_line_number;
  previous_line_number = 0;
  while (token_type != T_DOCUMENT_END)
    {
      if (tokenizer.Current_line_number() != previous_line_number)
        {
          cout << "\n------- Current line number: "
               << tokenizer.Current_line_number()
               << endl;
          previous_line_number = tokenizer.Current_line_number();
        }
      token_type = tokenizer.Get_next_token(&token_value);
      switch (token_type)
        {
        case T_INT_NUM:
          cout << "T_INT_NUM";
          cout << ": " << token_value.OM_object;
          cout << endl;
          break;
        case T_FP_NUM:
          cout << "T_FP_NUM";
          cout << ": " << token_value.OM_object;
          cout << endl;
          break;
        case T_STR:
          cout << "T_STR";
          cout << ": " << token_value.OM_object;
          cout << endl;
          break;
        case T_SYMBOL:
          cout << "T_SYMBOL";
          cout << ": " << token_value.OM_object;
          cout << endl;
          break;
        case T_VARIABLE:
          cout << "T_VARIABLE";
          cout << ": " << token_value.OM_object;
          cout << endl;
          break;
        case T_APPLICATION:
          cout << "T_APPLICATION";
          cout << ": " << token_value.OM_object;
          cout << endl;
          break;
        case T_ATTRIBUTION:
          cout << "T_ATTRIBUTION";
          cout << ": " << token_value.OM_object;
          cout << endl;
          break;
        case T_BINDING:
          cout << "T_BINDING";
          cout << ": " << token_value.OM_object;
          cout << endl;
          break;
        case T_OP_IMPL:
          cout << "T_OP_IMPL";
          cout << ": " << token_value.OM_object;
          cout << endl;
          break;
        case T_OP_OR:
          cout << "T_OP_OR";
          cout << ": " << token_value.OM_object;
          cout << endl;
          break;
        case T_OP_AND:
          cout << "T_OP_AND";
          cout << ": " << token_value.OM_object;
          cout << endl;
          break;
        case T_OP_EQ:
          cout << "T_OP_EQ";
          cout << ": " << token_value.OM_object;
          cout << endl;
          break;
        case T_OP_PLUS:
          cout << "T_OP_PLUS";
          cout << ": " << token_value.OM_object;
          cout << endl;
          break;
        case T_OP_PROD:
          cout << "T_OP_PROD";
          cout << ": " << token_value.OM_object;
          cout << endl;
          break;
        case T_OP_EXP:
          cout << "T_OP_EXP";
          cout << ": " << token_value.OM_object;
          cout << endl;
          break;
        case T_OP_FACT:
          cout << "T_OP_FACT";
          cout << ": " << token_value.OM_object;
          cout << endl;
          break;
        case T_OP_BIND:
          cout << "T_OP_BIND";
          cout << ": " << token_value.OM_object;
          cout << endl;
          break;
        case T_SPACE:
          cout << "T_SPACE";
          cout << ": (markup)";
          cout << endl;
          break;
        case T_TAB:
          cout << "T_TAB";
          cout << ": (markup)";
          cout << endl;
          break;
        case T_END_OF_LINE:
          cout << "T_END_OF_LINE";
          cout << ": (markup)";
          cout << endl;
          break;
        case T_SEP_COMMA:
          cout << "T_SEP_COMMA";
          cout << ": (markup)";
          cout << endl;
          break;
        case T_SEP_SCOLON:
          cout << "T_SEP_SCOLON";
          cout << ": (markup)";
          cout << endl;
          break;
        case T_SEP_COLON:
          cout << "T_SEP_COLON";
          cout << ": (markup)";
          cout << endl;
          break;
        case T_FENCE_OPEN:
          cout << "T_FENCE_OPEN";
          cout << ": '" << char(token_value.character) << "'";
          cout << endl;
          break;
        case T_FENCE_CLOSE:
          cout << "T_FENCE_CLOSE";
          cout << ": '" << char(token_value.character) << "'";
          cout << endl;
          break;
        case T_UCS4_CHAR:
          cout << "T_UCS4_CHAR";
          cout << ": '" << char(token_value.character) << "'";
          cout << endl;
          break;
        case T_MATH_BEGIN:
          cout << "T_MATH_BEGIN";
          cout << ": (markup)";
          cout << endl;
          break;
        case T_MATH_END:
          cout << "T_MATH_END";
          cout << ": (markup)";
          cout << endl;
          break;
        case T_DOUBLE_QUOTE:
          cout << "T_DOUBLE_QUOTE";
          cout << ": (markup)";
          cout << endl;
          break;
        case T_SINGLE_QUOTE:
          cout << "T_SINGLE_QUOTE";
          cout << ": (markup)";
          cout << endl;
          break;
        case T_BACK_QUOTE:
          cout << "T_BACK_QUOTE";
          cout << ": (markup)";
          cout << endl;
          break;
        case T_HASH:
          cout << "T_HASH";
          cout << ": (markup)";
          cout << endl;
          break;
        case T_NEW_PARAGRAPH:
          cout << "T_NEW_PARAGRAPH";
          cout << ": (markup)";
          cout << endl;
          break;
        case T_DOCUMENT_END:
          cout << "T_DOCUMENT_END";
          cout << ": (markup)";
          cout << endl;
          break;
        case T_IDENTIFIER:
          cout << "T_IDENTIFIER";
          cout << ": \"" << token_value.text << "\"";
          cout << endl;
          break;
        case T_METADATA_BEGIN:
          cout << "T_METADATA_BEGIN";
          cout << ": (markup)";
          cout << endl;
          break;
        case T_METADATA_TITLE:
          cout << "T_METADATA_TITLE";
          cout << ": \"" << token_value.text << "\"";
          cout << endl;
          break;
        case T_METADATA_AUTHOR:
          cout << "T_METADATA_AUTHOR";
          cout << ": \"" << token_value.text << "\"";
          cout << endl;
          break;
        case T_METADATA_DATE:
          cout << "T_METADATA_DATE";
          cout << ": \"" << token_value.text << "\"";
          cout << endl;
          break;
        case T_METADATA_LANG:
          cout << "T_METADATA_LANG";
          cout << ": \"" << token_value.text << "\"";
          cout << endl;
          break;
        case T_METADATA:
          cout << "T_METADATA";
          cout << ": \"" << token_value.text << "\"";
          cout << endl;
          break;
        case T_PARAGRAPH_TYPE:
          cout << "T_PARAGRAPH_TYPE";
          cout << ": \"" << token_value.text << "\"";
          cout << endl;
          break;
        case T_PARAGRAPH_SECTION_MARKER:
          cout << "T_PARAGRAPH_SECTION_MARKER";
          cout << ": \"" << token_value.text << "\"";
          cout << endl;
          break;
        case T_REFERENCE_ID:
          cout << "T_REFERENCE_ID";
          cout << ": \"" << token_value.text << "\"";
          cout << endl;
          break;
        case T_REFERENCE:
          cout << "T_REFERENCE";
          cout << ": \"" << token_value.text << "\"";
          cout << endl;
          break;
        case T_START_DEBUGGING:
          cout << "T_START_DEBUGGING";
          cout << ": (markup)";
          cout << endl;
          break;
        case T_DEBUGGING_REPORT:
          cout << "T_DEBUGGING_REPORT";
          cout << ": \"" << token_value.text << "\"";
          cout << endl;
          break;
        case T_STOP_DEBUGGING:
          cout << "T_STOP_DEBUGGING";
          cout << ": (markup)";
          cout << endl;
          break;
        case T_STRING:
          cout << "T_STRING";
          cout << ": \"" << token_value.text << "\"";
          cout << endl;
          break;
        case T_SYMBOL_DEFINITION_TYPE:
          cout << "T_SYMBOL_DEFINITION_TYPE";
          cout << ": \"" << token_value.text << "\"";
          cout << endl;
          break;
        case T_PARAMETER_LABEL:
          cout << "T_PARAMETER_LABEL";
          cout << ": \"" << token_value.text << "\"";
          cout << endl;
          break;
        default:
          cout << "tokenizer_test.cc: error\n";
          break;
        }
    }
  cout << "\nTerminado.\n" << flush;
  
  return 0;
}
