// Clase:    Unicode_data
// Proyecto: QMath
// Fecha:    jueves, 17 de diciembre de 2009, 22:56:41 CET
///////////////////////////////////////////////////////////////////////////
//
//   Copyright  2001, 2002 Alberto Gonzlez Palomo
//   Author: Alberto Gonzlez Palomo
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////


#include "Unicode_data.h"

// Static data
int Unicode_data::character_table_lock = 0;
Unicode_data::character_table_t Unicode_data::character_table;

/////////////
// Unicode_data - Class constructor.
// no arguments.
//public:
Unicode_data::Unicode_data()
{
  for (int try_count = 0; try_count < 2; ++try_count)
    if (Load_character_table()) break;
}

/////////////
// ~Unicode_data - Class destructor.
// no arguments.
//public:
Unicode_data::~Unicode_data()
{

}

/////////////
// Load_character_table - Load some Unicode character attributes.
// returns true on success, false on fail.
//private:
//static
bool Unicode_data::Load_character_table()
{
  if (character_table.size() > 0)
    return true;// Success if already loaded
  
  // Try to lock the character table:
  if (character_table_lock++ > 0)
    {
      --character_table_lock;
      return false;
    }
  
  character_entry_internal_t character_entry;
  // Made from http://www.unicode.org/Public/3.0-Update/UnicodeData-3.1.0.txt
  // with the following Python code:
  //   import sys
  //   import re
  //   f = open("UnicodeData-3.0.0.txt")
  //   buffer = f.read()
  //   f.close()
  //   fld_Code_value                      =  0;
  //   fld_Character_name                  =  1;
  //   fld_General_category                =  2;
  //   fld_Canonical_combining_classes     =  3;
  //   fld_Bidirectional_category          =  4;
  //   fld_Character_decomposition_mapping =  5;
  //   fld_Decimal_digit_value             =  6;
  //   fld_Digit_value                     =  7;
  //   fld_Numeric_value                   =  8;
  //   fld_Mirrored                        =  9;
  //   fld_Unicode_1_0_name                = 10;
  //   fld_10646_comment_field             = 11;
  //   fld_Uppercase_mapping               = 12;
  //   fld_Lowercase_mapping               = 13;
  //   fld_Titlecase_mapping               = 14;
  //   re_entity = re.compile("^" + 14*"([^;]*);",
  //                          re.MULTILINE)
  //   previous_match = ['','','','','','','','','','','','','','','']
  //   previous_code_point = -1
  //   range_open = 0
  //   for match in re_entity.findall(buffer):
  //     code_point = eval('0x'+match[fld_Code_value])
  //     is_first_of_range = (", First>" == match[fld_Character_name][-8:])
  //     is_last_of_range  = (", Last>"  == match[fld_Character_name][-7:])
  //     interesting_fields_are_different = (
  //          match[fld_General_category]    != previous_match[fld_General_category]
  //       or match[fld_Decimal_digit_value] != previous_match[fld_Decimal_digit_value])
  //     if is_last_of_range:
  //       sys.stdout.write('  character_entry.code_range_end = ' +
  //                        hex(code_point) + ';\n')
  //       sys.stdout.write('  character_table.push_back(character_entry);\n')
  //       range_open = 0
  //     elif (interesting_fields_are_different
  //         or is_first_of_range):
  //       if range_open:
  //         sys.stdout.write('  character_entry.code_range_end = ' +
  //                          hex(previous_code_point) + ';\n')
  //         sys.stdout.write('  character_table.push_back(character_entry);\n')
  //         range_open = 0
  //       sys.stdout.write('  character_entry.code_range_begin = ' +
  //                        hex(code_point) + ';\n')
  //       sys.stdout.write('  character_entry.general_category = ' +
  //                        str(match[fld_General_category]) + ';\n')
  //       sys.stdout.write('  character_entry.decimal_digit_value = ')
  //       if len(str(match[fld_Decimal_digit_value])) > 0:
  //         sys.stdout.write(str(match[fld_Decimal_digit_value]))
  //       else:
  //         sys.stdout.write('-1')
  //       sys.stdout.write(';\n')
  //       range_open = 1
  //     previous_match = match
  //     previous_code_point = code_point
  //   if range_open:
  //     sys.stdout.write('  character_entry.code_range_end = ' +
  //                      hex(code_point - 1) + ';\n')
  //     sys.stdout.write('  character_table.push_back(character_entry);\n')
  //     range_open = 0
  //
  // 2001-02-28 Alberto González Palomo <matmota@linuxfreak.com>
  character_entry.code_range_begin = 0x0;
  character_entry.general_category = Cc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x20;
  character_entry.general_category = Zs;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x20;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x23;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x24;
  character_entry.general_category = Sc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x24;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x25;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x27;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x28;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x28;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x29;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x29;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2a;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2b;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2c;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2d;
  character_entry.general_category = Pd;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2e;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x30;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0x30;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x31;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0x31;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x32;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0x32;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x33;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0x33;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x34;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0x34;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x35;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0x35;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x36;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0x36;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x37;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0x37;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x38;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0x38;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x39;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0x39;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3a;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3c;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3f;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x40;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x41;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x5a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x5b;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x5b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x5c;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x5c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x5d;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x5d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x5e;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x5e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x5f;
  character_entry.general_category = Pc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x5f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x60;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x60;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x61;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x7a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x7b;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x7b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x7c;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x7c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x7d;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x7d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x7e;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x7e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x7f;
  character_entry.general_category = Cc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x9f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa0;
  character_entry.general_category = Zs;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xa0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa1;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xa1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa2;
  character_entry.general_category = Sc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xa5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa6;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xa7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa8;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xa8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa9;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xa9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xaa;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xaa;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xab;
  character_entry.general_category = Pi;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xab;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xac;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xac;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xad;
  character_entry.general_category = Pd;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xad;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xae;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xae;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xaf;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xaf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb0;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb1;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb2;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0xb2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb3;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0xb3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb4;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb5;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb6;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb7;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb8;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb9;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0xb9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xba;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xba;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xbb;
  character_entry.general_category = Pf;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xbb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xbc;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xbe;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xbf;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xbf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xc0;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xd6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd7;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xd7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xde;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xdf;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf7;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf8;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x100;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x100;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x101;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x101;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x102;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x102;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x103;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x103;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x104;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x104;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x105;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x105;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x106;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x106;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x107;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x107;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x108;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x108;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x109;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x109;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x10a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x10a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x10b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x10b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x10c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x10c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x10d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x10d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x10e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x10e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x10f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x10f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x110;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x110;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x111;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x111;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x112;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x112;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x113;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x113;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x114;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x114;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x115;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x115;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x116;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x116;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x117;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x117;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x118;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x118;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x119;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x119;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x11a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x11a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x11b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x11b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x11c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x11c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x11d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x11d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x11e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x11e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x11f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x11f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x120;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x120;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x121;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x121;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x122;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x122;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x123;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x123;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x124;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x124;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x125;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x125;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x126;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x126;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x127;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x127;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x128;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x128;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x129;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x129;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x12a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x12a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x12b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x12b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x12c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x12c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x12d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x12d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x12e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x12e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x12f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x12f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x130;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x130;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x131;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x131;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x132;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x132;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x133;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x133;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x134;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x134;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x135;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x135;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x136;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x136;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x137;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x138;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x139;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x139;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x13a;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x13a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x13b;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x13b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x13c;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x13c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x13d;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x13d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x13e;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x13e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x13f;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x13f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x140;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x140;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x141;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x141;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x142;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x142;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x143;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x143;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x144;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x144;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x145;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x145;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x146;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x146;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x147;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x147;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x148;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x149;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x14a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x14a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x14b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x14b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x14c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x14c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x14d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x14d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x14e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x14e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x14f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x14f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x150;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x150;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x151;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x151;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x152;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x152;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x153;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x153;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x154;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x154;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x155;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x155;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x156;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x156;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x157;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x157;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x158;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x158;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x159;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x159;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x15a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x15a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x15b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x15b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x15c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x15c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x15d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x15d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x15e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x15e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x15f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x15f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x160;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x160;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x161;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x161;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x162;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x162;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x163;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x163;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x164;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x164;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x165;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x165;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x166;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x166;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x167;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x167;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x168;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x168;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x169;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x169;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x16a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x16a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x16b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x16b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x16c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x16c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x16d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x16d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x16e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x16e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x16f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x16f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x170;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x170;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x171;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x171;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x172;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x172;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x173;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x173;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x174;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x174;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x175;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x175;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x176;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x176;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x177;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x177;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x178;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x179;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17a;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x17a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17b;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x17b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17c;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x17c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17d;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x17d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17e;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x180;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x181;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x182;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x183;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x183;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x184;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x184;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x185;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x185;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x186;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x187;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x188;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x188;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x189;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x18b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x18c;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x18d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x18e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x191;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x192;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x192;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x193;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x194;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x195;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x195;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x196;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x198;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x199;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x19b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x19c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x19d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x19e;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x19e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x19f;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1a0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1a1;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1a1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1a2;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1a2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1a3;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1a3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1a4;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1a4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1a5;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1a5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1a6;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1a7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1a8;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1a8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1a9;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1a9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1aa;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ab;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ac;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ac;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ad;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ad;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ae;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1af;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1b0;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1b0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1b1;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1b3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1b4;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1b4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1b5;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1b5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1b6;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1b6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1b7;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1b8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1b9;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ba;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1bb;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1bb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1bc;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1bc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1bd;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1bf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1c0;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1c3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1c4;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1c4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1c5;
  character_entry.general_category = Lt;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1c5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1c6;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1c6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1c7;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1c7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1c8;
  character_entry.general_category = Lt;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1c8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1c9;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1c9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ca;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ca;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1cb;
  character_entry.general_category = Lt;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1cb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1cc;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1cc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1cd;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1cd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ce;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ce;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1cf;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1cf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d0;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d1;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d2;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d3;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d4;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d5;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d6;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d8;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d9;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1da;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1da;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1db;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1db;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1dc;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1dd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1de;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1de;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1df;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1df;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e0;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e1;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e2;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e3;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e4;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e5;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e6;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e7;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e9;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ea;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ea;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eb;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ec;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ec;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ed;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ed;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ee;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ee;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ef;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f1;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f2;
  character_entry.general_category = Lt;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f3;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f4;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f5;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f6;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f9;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fa;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fa;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fb;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fc;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fd;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fe;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fe;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ff;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ff;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x200;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x200;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x201;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x201;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x202;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x202;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x203;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x203;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x204;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x204;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x205;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x205;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x206;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x206;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x207;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x207;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x208;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x208;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x209;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x209;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x20a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x20a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x20b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x20b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x20c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x20c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x20d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x20d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x20e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x20e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x20f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x20f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x210;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x210;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x211;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x211;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x212;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x212;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x213;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x213;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x214;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x214;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x215;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x215;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x216;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x216;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x217;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x217;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x218;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x218;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x219;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x219;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x222;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x222;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x223;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x223;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x224;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x224;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x225;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x225;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x226;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x226;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x227;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x227;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x228;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x228;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x229;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x229;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x22a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x22a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x22b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x22b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x22c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x22c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x22d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x22d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x22e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x22e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x22f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x22f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x230;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x230;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x231;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x231;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x232;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x232;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x233;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2ad;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2b0;
  character_entry.general_category = Lm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2b8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2b9;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2ba;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2bb;
  character_entry.general_category = Lm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2c1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2c2;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2cf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2d0;
  character_entry.general_category = Lm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2d1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2d2;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2df;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2e0;
  character_entry.general_category = Lm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2e4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2e5;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2ed;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2ee;
  character_entry.general_category = Lm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2ee;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x300;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x362;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x374;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x375;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x37a;
  character_entry.general_category = Lm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x37a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x37e;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x37e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x384;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x385;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x386;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x386;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x387;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x387;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x388;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x38f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x390;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x390;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x391;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3ab;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3ac;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3d1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3d2;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3d4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3d5;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3d7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3da;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3da;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3db;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3db;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3dc;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3dc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3dd;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3dd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3de;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3de;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3df;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3df;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3e0;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3e0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3e1;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3e1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3e2;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3e2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3e3;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3e3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3e4;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3e4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3e5;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3e5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3e6;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3e6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3e7;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3e7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3e8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3e8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3e9;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3e9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3ea;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3ea;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3eb;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3eb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3ec;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3ec;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3ed;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3ed;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3ee;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3ee;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3ef;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3f3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3f4;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3f4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3f5;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3f5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x400;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x42f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x430;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x45f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x460;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x460;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x461;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x461;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x462;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x462;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x463;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x463;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x464;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x464;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x465;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x465;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x466;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x466;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x467;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x467;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x468;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x468;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x469;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x469;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x46a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x46a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x46b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x46b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x46c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x46c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x46d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x46d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x46e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x46e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x46f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x46f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x470;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x470;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x471;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x471;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x472;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x472;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x473;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x473;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x474;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x474;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x475;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x475;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x476;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x476;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x477;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x477;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x478;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x478;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x479;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x479;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x47a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x47a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x47b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x47b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x47c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x47c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x47d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x47d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x47e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x47e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x47f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x47f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x480;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x480;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x481;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x481;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x482;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x482;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x483;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x486;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x488;
  character_entry.general_category = Me;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x489;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x48c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x48c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x48d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x48d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x48e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x48e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x48f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x48f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x490;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x490;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x491;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x491;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x492;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x492;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x493;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x493;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x494;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x494;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x495;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x495;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x496;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x496;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x497;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x497;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x498;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x498;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x499;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x499;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x49a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x49a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x49b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x49b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x49c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x49c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x49d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x49d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x49e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x49e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x49f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x49f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4a0;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4a0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4a1;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4a1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4a2;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4a2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4a3;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4a3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4a4;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4a4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4a5;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4a5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4a6;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4a6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4a7;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4a7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4a8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4a8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4a9;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4a9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4aa;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4aa;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4ab;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4ab;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4ac;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4ac;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4ad;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4ad;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4ae;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4ae;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4af;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4af;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4b0;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4b0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4b1;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4b1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4b2;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4b2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4b3;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4b3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4b4;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4b4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4b5;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4b5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4b6;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4b6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4b7;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4b7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4b8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4b8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4b9;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4b9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4ba;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4ba;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4bb;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4bb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4bc;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4bc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4bd;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4bd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4be;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4be;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4bf;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4bf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4c0;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4c1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4c2;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4c2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4c3;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4c3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4c4;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4c4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4c7;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4c7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4c8;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4c8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4cb;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4cb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4cc;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4cc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4d0;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4d0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4d1;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4d1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4d2;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4d2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4d3;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4d3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4d4;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4d4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4d5;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4d5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4d6;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4d6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4d7;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4d7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4d8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4d8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4d9;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4d9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4da;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4da;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4db;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4db;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4dc;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4dc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4dd;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4dd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4de;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4de;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4df;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4df;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4e0;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4e0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4e1;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4e1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4e2;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4e2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4e3;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4e3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4e4;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4e4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4e5;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4e5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4e6;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4e6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4e7;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4e7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4e8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4e8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4e9;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4e9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4ea;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4ea;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4eb;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4eb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4ec;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4ec;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4ed;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4ed;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4ee;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4ee;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4ef;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4ef;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4f0;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4f0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4f1;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4f1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4f2;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4f2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4f3;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4f3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4f4;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4f4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4f5;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4f5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4f8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4f8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4f9;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4f9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x531;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x556;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x559;
  character_entry.general_category = Lm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x559;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x55a;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x55f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x561;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x587;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x589;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x589;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x58a;
  character_entry.general_category = Pd;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x58a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x591;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x5bd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x5be;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x5be;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x5bf;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x5bf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x5c0;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x5c0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x5c1;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x5c2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x5c3;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x5c3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x5c4;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x5c4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x5d0;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x5f2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x5f3;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x61f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x621;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x63a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x640;
  character_entry.general_category = Lm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x640;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x641;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x64a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x64b;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x655;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x660;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0x660;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x661;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0x661;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x662;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0x662;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x663;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0x663;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x664;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0x664;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x665;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0x665;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x666;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0x666;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x667;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0x667;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x668;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0x668;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x669;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0x669;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x66a;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x66d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x670;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x670;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x671;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x6d3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6d4;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x6d4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6d5;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x6d5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6d6;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x6dc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6dd;
  character_entry.general_category = Me;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x6de;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6df;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x6e4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6e5;
  character_entry.general_category = Lm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x6e6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6e7;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x6e8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6e9;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x6e9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6ea;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x6ed;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6f0;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0x6f0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6f1;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0x6f1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6f2;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0x6f2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6f3;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0x6f3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6f4;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0x6f4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6f5;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0x6f5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6f6;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0x6f6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6f7;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0x6f7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6f8;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0x6f8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6f9;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0x6f9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6fa;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x6fc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x6fd;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x6fe;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x700;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x70d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x70f;
  character_entry.general_category = Cf;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x70f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x710;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x710;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x711;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x711;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x712;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x72c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x730;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x74a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x780;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x7a5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x7a6;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x902;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x903;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x903;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x905;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x939;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x93c;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x93c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x93d;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x93d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x93e;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x940;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x941;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x948;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x949;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x94c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x94d;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x94d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x950;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x950;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x951;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x954;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x958;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x961;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x962;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x963;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x964;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x965;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x966;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0x966;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x967;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0x967;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x968;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0x968;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x969;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0x969;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x96a;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0x96a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x96b;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0x96b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x96c;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0x96c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x96d;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0x96d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x96e;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0x96e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x96f;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0x96f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x970;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x970;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x981;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x981;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x982;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x983;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x985;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x9b9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9bc;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x9bc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9be;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x9c0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9c1;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x9c4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9c7;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x9cc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9cd;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x9cd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9d7;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x9d7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9dc;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x9e1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9e2;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x9e3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9e6;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0x9e6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9e7;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0x9e7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9e8;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0x9e8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9e9;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0x9e9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9ea;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0x9ea;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9eb;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0x9eb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9ec;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0x9ec;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9ed;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0x9ed;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9ee;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0x9ee;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9ef;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0x9ef;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9f0;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x9f1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9f2;
  character_entry.general_category = Sc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x9f3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9f4;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x9f9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x9fa;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x9fa;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa02;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xa02;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa05;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xa39;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa3c;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xa3c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa3e;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xa40;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa41;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xa4d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa59;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xa5e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa66;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0xa66;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa67;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0xa67;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa68;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0xa68;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa69;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0xa69;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa6a;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0xa6a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa6b;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0xa6b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa6c;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0xa6c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa6d;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0xa6d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa6e;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0xa6e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa6f;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0xa6f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa70;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xa71;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa72;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xa74;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa81;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xa82;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa83;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xa83;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa85;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xab9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xabc;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xabc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xabd;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xabd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xabe;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xac0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xac1;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xac8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xac9;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xacc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xacd;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xacd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xad0;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xae0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xae6;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0xae6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xae7;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0xae7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xae8;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0xae8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xae9;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0xae9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xaea;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0xaea;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xaeb;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0xaeb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xaec;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0xaec;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xaed;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0xaed;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xaee;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0xaee;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xaef;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0xaef;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb01;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb01;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb02;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb03;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb05;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb39;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb3c;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb3c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb3d;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb3d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb3e;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb3e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb3f;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb3f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb40;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb40;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb41;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb43;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb47;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb4c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb4d;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb56;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb57;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb57;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb5c;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb61;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb66;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0xb66;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb67;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0xb67;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb68;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0xb68;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb69;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0xb69;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb6a;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0xb6a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb6b;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0xb6b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb6c;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0xb6c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb6d;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0xb6d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb6e;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0xb6e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb6f;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0xb6f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb70;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb70;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb82;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb82;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb83;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xb83;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xb85;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xbb9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xbbe;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xbbf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xbc0;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xbc0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xbc1;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xbcc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xbcd;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xbcd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xbd7;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xbd7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xbe7;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0xbe7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xbe8;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0xbe8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xbe9;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0xbe9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xbea;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0xbea;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xbeb;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0xbeb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xbec;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0xbec;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xbed;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0xbed;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xbee;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0xbee;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xbef;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0xbef;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xbf0;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xbf2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xc01;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xc03;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xc05;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xc39;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xc3e;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xc40;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xc41;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xc44;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xc46;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xc56;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xc60;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xc61;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xc66;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0xc66;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xc67;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0xc67;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xc68;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0xc68;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xc69;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0xc69;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xc6a;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0xc6a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xc6b;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0xc6b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xc6c;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0xc6c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xc6d;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0xc6d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xc6e;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0xc6e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xc6f;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0xc6f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xc82;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xc83;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xc85;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xcb9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xcbe;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xcbe;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xcbf;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xcbf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xcc0;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xcc4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xcc6;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xcc6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xcc7;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xccb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xccc;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xccd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xcd5;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xcd6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xcde;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xce1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xce6;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0xce6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xce7;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0xce7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xce8;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0xce8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xce9;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0xce9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xcea;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0xcea;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xceb;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0xceb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xcec;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0xcec;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xced;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0xced;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xcee;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0xcee;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xcef;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0xcef;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd02;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xd03;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd05;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xd39;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd3e;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xd40;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd41;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xd43;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd46;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xd4c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd4d;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xd4d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd57;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xd57;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd60;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xd61;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd66;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0xd66;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd67;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0xd67;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd68;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0xd68;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd69;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0xd69;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd6a;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0xd6a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd6b;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0xd6b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd6c;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0xd6c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd6d;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0xd6d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd6e;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0xd6e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd6f;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0xd6f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd82;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xd83;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd85;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xdc6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xdca;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xdca;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xdcf;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xdd1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xdd2;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xdd6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xdd8;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xdf3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xdf4;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xdf4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe01;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xe30;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe31;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xe31;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe32;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xe33;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe34;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xe3a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe3f;
  character_entry.general_category = Sc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xe3f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe40;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xe45;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe46;
  character_entry.general_category = Lm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xe46;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe47;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xe4e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe4f;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xe4f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe50;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0xe50;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe51;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0xe51;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe52;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0xe52;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe53;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0xe53;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe54;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0xe54;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe55;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0xe55;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe56;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0xe56;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe57;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0xe57;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe58;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0xe58;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe59;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0xe59;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe5a;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xe5b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe81;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xeb0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xeb1;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xeb1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xeb2;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xeb3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xeb4;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xebc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xebd;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xec4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xec6;
  character_entry.general_category = Lm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xec6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xec8;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xecd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xed0;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0xed0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xed1;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0xed1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xed2;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0xed2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xed3;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0xed3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xed4;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0xed4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xed5;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0xed5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xed6;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0xed6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xed7;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0xed7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xed8;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0xed8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xed9;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0xed9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xedc;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf00;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf01;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf03;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf04;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf12;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf13;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf17;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf18;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf19;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf1a;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf1f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf20;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0xf20;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf21;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0xf21;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf22;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0xf22;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf23;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0xf23;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf24;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0xf24;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf25;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0xf25;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf26;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0xf26;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf27;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0xf27;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf28;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0xf28;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf29;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0xf29;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf2a;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf33;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf34;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf34;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf35;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf35;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf36;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf36;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf37;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf37;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf38;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf38;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf39;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf39;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf3a;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf3a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf3b;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf3b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf3c;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf3c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf3d;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf3d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf3e;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf3f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf40;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf6a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf71;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf7e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf7f;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf7f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf80;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf84;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf85;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf85;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf86;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf87;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf88;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf8b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf90;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfbc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfbe;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfc5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfc6;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfc6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfc7;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfcf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1000;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x102a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x102c;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x102c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x102d;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1030;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1031;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1031;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1032;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1037;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1038;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1038;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1039;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1039;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1040;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0x1040;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1041;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0x1041;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1042;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0x1042;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1043;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0x1043;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1044;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0x1044;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1045;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0x1045;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1046;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0x1046;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1047;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0x1047;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1048;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0x1048;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1049;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0x1049;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x104a;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x104f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1050;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1055;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1056;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1057;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1058;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1059;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x10a0;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x10c5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x10d0;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x10f6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x10fb;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x10fb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1100;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x135a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1361;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1368;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1369;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0x1369;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x136a;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0x136a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x136b;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0x136b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x136c;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0x136c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x136d;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0x136d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x136e;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0x136e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x136f;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0x136f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1370;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0x1370;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1371;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0x1371;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1372;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x137c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x13a0;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x166c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x166d;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x166e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x166f;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1676;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1680;
  character_entry.general_category = Zs;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1680;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1681;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x169a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x169b;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x169b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x169c;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x169c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x16a0;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x16ea;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x16eb;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x16ed;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x16ee;
  character_entry.general_category = Nl;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x16f0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1780;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x17b3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17b4;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x17b6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17b7;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x17bd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17be;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x17c5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17c6;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x17c6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17c7;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x17c8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17c9;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x17d3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17d4;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x17da;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17db;
  character_entry.general_category = Sc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x17db;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17dc;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x17dc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17e0;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0x17e0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17e1;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0x17e1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17e2;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0x17e2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17e3;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0x17e3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17e4;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0x17e4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17e5;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0x17e5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17e6;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0x17e6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17e7;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0x17e7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17e8;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0x17e8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x17e9;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0x17e9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1800;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1805;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1806;
  character_entry.general_category = Pd;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1806;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1807;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x180a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x180b;
  character_entry.general_category = Cf;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x180e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1810;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0x1810;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1811;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0x1811;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1812;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0x1812;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1813;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0x1813;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1814;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0x1814;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1815;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0x1815;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1816;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0x1816;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1817;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0x1817;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1818;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0x1818;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1819;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0x1819;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1820;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1842;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1843;
  character_entry.general_category = Lm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1843;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1844;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x18a8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x18a9;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x18a9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e00;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e00;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e01;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e01;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e02;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e02;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e03;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e03;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e04;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e04;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e05;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e05;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e06;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e06;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e07;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e07;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e08;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e08;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e09;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e09;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e0a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e0a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e0b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e0b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e0c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e0c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e0d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e0d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e0e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e0e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e0f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e0f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e10;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e10;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e11;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e11;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e12;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e12;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e13;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e13;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e14;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e14;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e15;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e15;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e16;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e16;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e17;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e17;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e18;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e18;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e19;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e19;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e1a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e1a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e1b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e1b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e1c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e1c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e1d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e1d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e1e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e1e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e1f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e1f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e20;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e20;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e21;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e21;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e22;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e22;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e23;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e23;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e24;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e24;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e25;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e25;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e26;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e26;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e27;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e27;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e28;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e28;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e29;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e29;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e2a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e2a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e2b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e2b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e2c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e2c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e2d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e2d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e2e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e2e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e2f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e2f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e30;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e30;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e31;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e31;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e32;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e32;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e33;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e33;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e34;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e34;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e35;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e35;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e36;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e36;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e37;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e37;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e38;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e38;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e39;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e39;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e3a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e3a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e3b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e3b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e3c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e3c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e3d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e3d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e3e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e3e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e3f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e3f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e40;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e40;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e41;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e41;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e42;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e42;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e43;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e43;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e44;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e44;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e45;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e45;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e46;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e46;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e47;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e47;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e48;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e48;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e49;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e49;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e4a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e4a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e4b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e4b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e4c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e4c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e4d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e4d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e4e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e4e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e4f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e4f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e50;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e50;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e51;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e51;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e52;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e52;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e53;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e53;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e54;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e54;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e55;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e55;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e56;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e56;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e57;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e57;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e58;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e58;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e59;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e59;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e5a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e5a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e5b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e5b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e5c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e5c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e5d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e5d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e5e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e5e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e5f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e5f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e60;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e60;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e61;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e61;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e62;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e62;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e63;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e63;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e64;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e64;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e65;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e65;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e66;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e66;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e67;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e67;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e68;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e68;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e69;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e69;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e6a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e6a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e6b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e6b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e6c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e6c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e6d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e6d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e6e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e6e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e6f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e6f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e70;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e70;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e71;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e71;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e72;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e72;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e73;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e73;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e74;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e74;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e75;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e75;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e76;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e76;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e77;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e77;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e78;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e78;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e79;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e79;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e7a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e7a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e7b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e7b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e7c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e7c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e7d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e7d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e7e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e7e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e7f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e7f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e80;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e80;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e81;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e81;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e82;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e82;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e83;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e83;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e84;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e84;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e85;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e85;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e86;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e86;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e87;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e87;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e88;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e88;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e89;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e89;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e8a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e8a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e8b;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e8b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e8c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e8c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e8d;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e8d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e8e;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e8e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e8f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e8f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e90;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e90;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e91;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e91;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e92;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e92;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e93;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e93;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e94;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e94;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1e95;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1e9b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ea0;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ea0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ea1;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ea1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ea2;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ea2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ea3;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ea3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ea4;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ea4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ea5;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ea5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ea6;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ea6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ea7;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ea7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ea8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ea8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ea9;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ea9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eaa;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eaa;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eab;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eab;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eac;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eac;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ead;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ead;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eae;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eae;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eaf;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eaf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eb0;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eb0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eb1;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eb1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eb2;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eb2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eb3;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eb3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eb4;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eb4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eb5;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eb5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eb6;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eb6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eb7;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eb7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eb8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eb8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eb9;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eb9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eba;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eba;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ebb;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ebb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ebc;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ebc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ebd;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ebd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ebe;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ebe;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ebf;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ebf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ec0;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ec0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ec1;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ec1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ec2;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ec2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ec3;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ec3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ec4;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ec4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ec5;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ec5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ec6;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ec6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ec7;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ec7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ec8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ec8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ec9;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ec9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eca;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eca;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ecb;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ecb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ecc;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ecc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ecd;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ecd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ece;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ece;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ecf;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ecf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ed0;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ed0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ed1;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ed1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ed2;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ed2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ed3;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ed3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ed4;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ed4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ed5;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ed5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ed6;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ed6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ed7;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ed7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ed8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ed8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ed9;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ed9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eda;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eda;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1edb;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1edb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1edc;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1edc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1edd;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1edd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ede;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ede;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1edf;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1edf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ee0;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ee0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ee1;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ee1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ee2;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ee2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ee3;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ee3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ee4;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ee4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ee5;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ee5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ee6;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ee6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ee7;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ee7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ee8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ee8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ee9;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ee9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eea;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eea;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eeb;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eeb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eec;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eec;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eed;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eed;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eee;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eee;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1eef;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1eef;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ef0;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ef0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ef1;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ef1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ef2;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ef2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ef3;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ef3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ef4;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ef4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ef5;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ef5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ef6;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ef6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ef7;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ef7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ef8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ef8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ef9;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f07;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f08;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f0f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f10;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f15;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f18;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f1d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f20;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f27;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f28;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f2f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f30;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f37;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f38;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f3f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f40;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f45;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f48;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f4d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f50;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f57;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f59;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f5f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f60;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f67;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f68;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f6f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f70;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f87;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f88;
  character_entry.general_category = Lt;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f8f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f90;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f97;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1f98;
  character_entry.general_category = Lt;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1f9f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fa0;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fa7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fa8;
  character_entry.general_category = Lt;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1faf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fb0;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fb7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fb8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fbb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fbc;
  character_entry.general_category = Lt;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fbc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fbd;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fbd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fbe;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fbe;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fbf;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fc1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fc2;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fc7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fc8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fcb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fcc;
  character_entry.general_category = Lt;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fcc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fcd;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fcf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fd0;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fd7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fd8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fdb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fdd;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fdf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fe0;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fe7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fe8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fec;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1fed;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1fef;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ff2;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ff7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ff8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ffb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ffc;
  character_entry.general_category = Lt;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ffc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1ffd;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1ffe;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2000;
  character_entry.general_category = Zs;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x200b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x200c;
  character_entry.general_category = Cf;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x200f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2010;
  character_entry.general_category = Pd;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2015;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2016;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2017;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2018;
  character_entry.general_category = Pi;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2018;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2019;
  character_entry.general_category = Pf;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2019;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x201a;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x201a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x201b;
  character_entry.general_category = Pi;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x201c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x201d;
  character_entry.general_category = Pf;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x201d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x201e;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x201e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x201f;
  character_entry.general_category = Pi;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x201f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2020;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2027;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2028;
  character_entry.general_category = Zl;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2028;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2029;
  character_entry.general_category = Zp;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2029;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x202a;
  character_entry.general_category = Cf;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x202e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x202f;
  character_entry.general_category = Zs;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x202f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2030;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2038;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2039;
  character_entry.general_category = Pi;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2039;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x203a;
  character_entry.general_category = Pf;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x203a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x203b;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x203e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x203f;
  character_entry.general_category = Pc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2040;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2041;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2043;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2044;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2044;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2045;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2045;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2046;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2046;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2048;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x204d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x206a;
  character_entry.general_category = Cf;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x206f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2070;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0x2070;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2074;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0x2074;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2075;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0x2075;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2076;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0x2076;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2077;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0x2077;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2078;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0x2078;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2079;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0x2079;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x207a;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x207c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x207d;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x207d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x207e;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x207e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x207f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x207f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2080;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0x2080;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2081;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0x2081;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2082;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0x2082;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2083;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0x2083;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2084;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0x2084;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2085;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0x2085;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2086;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0x2086;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2087;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0x2087;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2088;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0x2088;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2089;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0x2089;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x208a;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x208c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x208d;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x208d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x208e;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x208e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x20a0;
  character_entry.general_category = Sc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x20af;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x20d0;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x20dc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x20dd;
  character_entry.general_category = Me;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x20e0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x20e1;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x20e1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x20e2;
  character_entry.general_category = Me;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x20e3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2100;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2101;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2102;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2102;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2103;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2106;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2107;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2107;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2108;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2109;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x210a;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x210a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x210b;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x210d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x210e;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x210f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2110;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2112;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2113;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2113;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2114;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2114;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2115;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2115;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2116;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2118;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2119;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x211d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x211e;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2123;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2124;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2124;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2125;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2125;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2126;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2126;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2127;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2127;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2128;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2128;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2129;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2129;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x212a;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x212d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x212e;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x212e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x212f;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x212f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2130;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2131;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2132;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2132;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2133;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2133;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2134;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2134;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2135;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2138;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2139;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2139;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x213a;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x213a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2153;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x215f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2160;
  character_entry.general_category = Nl;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2183;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2190;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2194;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2195;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2199;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x219a;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x219b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x219c;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x219f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21a0;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21a0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21a1;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21a2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21a3;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21a3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21a4;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21a5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21a6;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21a6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21a7;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21ad;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21ae;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21ae;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21af;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21cd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21ce;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21cf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21d0;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21d1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21d2;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21d2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21d3;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21d3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21d4;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21d4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x21d5;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x21f3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2200;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x22f1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2300;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2307;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2308;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x230b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x230c;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x231f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2320;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2321;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2322;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2328;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2329;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2329;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x232a;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x232a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x232b;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x244a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2460;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x249b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x249c;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x24e9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x24ea;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x24ea;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2500;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x25b6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x25b7;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x25b7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x25b8;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x25c0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x25c1;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x25c1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x25c2;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x266e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x266f;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x266f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2670;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2767;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2776;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2793;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x2794;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2ffb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3000;
  character_entry.general_category = Zs;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3000;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3001;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3003;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3004;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3004;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3005;
  character_entry.general_category = Lm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3005;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3006;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3006;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3007;
  character_entry.general_category = Nl;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3007;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3008;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3008;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3009;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3009;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x300a;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x300a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x300b;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x300b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x300c;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x300c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x300d;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x300d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x300e;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x300e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x300f;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x300f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3010;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3010;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3011;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3011;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3012;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3013;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3014;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3014;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3015;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3015;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3016;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3016;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3017;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3017;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3018;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3018;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3019;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3019;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x301a;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x301a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x301b;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x301b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x301c;
  character_entry.general_category = Pd;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x301c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x301d;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x301d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x301e;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x301f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3020;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3020;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3021;
  character_entry.general_category = Nl;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3029;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x302a;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x302f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3030;
  character_entry.general_category = Pd;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3030;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3031;
  character_entry.general_category = Lm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3035;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3036;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3037;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3038;
  character_entry.general_category = Nl;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x303a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x303e;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x303f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3041;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3094;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3099;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x309a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x309b;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x309c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x309d;
  character_entry.general_category = Lm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x309e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x30a1;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x30fa;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x30fb;
  character_entry.general_category = Pc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x30fb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x30fc;
  character_entry.general_category = Lm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x30fe;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3105;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x318e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3190;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3191;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3192;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3195;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3196;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x319f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x31a0;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x31b7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3200;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x321c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3220;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3229;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x322a;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x327f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3280;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x3289;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x328a;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x33fe;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x3400;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x4db5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x4e00;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x9fa5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xa490;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xa4c6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xac00;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xd7a3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xd800;
  character_entry.general_category = Cs;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xdb7f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xdb80;
  character_entry.general_category = Cs;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xdbff;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xdc00;
  character_entry.general_category = Cs;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xdfff;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe000;
  character_entry.general_category = Co;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xf8ff;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf900;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfa2d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfb00;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfb17;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfb1d;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfb1d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfb1e;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfb1e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfb1f;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfb28;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfb29;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfb29;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfb2a;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfd3d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfd3e;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfd3e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfd3f;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfd3f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfd50;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfdfb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe20;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe23;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe30;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe30;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe31;
  character_entry.general_category = Pd;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe32;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe33;
  character_entry.general_category = Pc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe34;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe35;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe35;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe36;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe36;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe37;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe37;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe38;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe38;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe39;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe39;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe3a;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe3a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe3b;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe3b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe3c;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe3c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe3d;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe3d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe3e;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe3e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe3f;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe3f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe40;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe40;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe41;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe41;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe42;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe42;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe43;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe43;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe44;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe44;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe49;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe4c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe4d;
  character_entry.general_category = Pc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe4f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe50;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe57;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe58;
  character_entry.general_category = Pd;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe58;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe59;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe59;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe5a;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe5a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe5b;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe5b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe5c;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe5c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe5d;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe5d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe5e;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe5e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe5f;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe61;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe62;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe62;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe63;
  character_entry.general_category = Pd;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe63;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe64;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe66;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe68;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe68;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe69;
  character_entry.general_category = Sc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe69;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe6a;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfe6b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfe70;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfefc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfeff;
  character_entry.general_category = Cf;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfeff;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff01;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff03;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff04;
  character_entry.general_category = Sc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff04;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff05;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff07;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff08;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff08;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff09;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff09;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff0a;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff0a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff0b;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff0b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff0c;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff0c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff0d;
  character_entry.general_category = Pd;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff0d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff0e;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff0f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff10;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0xff10;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff11;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0xff11;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff12;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0xff12;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff13;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0xff13;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff14;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0xff14;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff15;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0xff15;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff16;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0xff16;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff17;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0xff17;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff18;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0xff18;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff19;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0xff19;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff1a;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff1b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff1c;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff1e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff1f;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff20;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff21;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff3a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff3b;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff3b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff3c;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff3c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff3d;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff3d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff3e;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff3e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff3f;
  character_entry.general_category = Pc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff3f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff40;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff40;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff41;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff5a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff5b;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff5b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff5c;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff5c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff5d;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff5d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff5e;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff5e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff61;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff61;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff62;
  character_entry.general_category = Ps;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff62;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff63;
  character_entry.general_category = Pe;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff63;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff64;
  character_entry.general_category = Po;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff64;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff65;
  character_entry.general_category = Pc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff65;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff66;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff6f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff70;
  character_entry.general_category = Lm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff70;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff71;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff9d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xff9e;
  character_entry.general_category = Lm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xff9f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xffa0;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xffdc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xffe0;
  character_entry.general_category = Sc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xffe1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xffe2;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xffe2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xffe3;
  character_entry.general_category = Sk;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xffe3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xffe4;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xffe4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xffe5;
  character_entry.general_category = Sc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xffe6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xffe8;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xffe8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xffe9;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xffec;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xffed;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xffee;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfff9;
  character_entry.general_category = Cf;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfffb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xfffc;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xfffd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x10300;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1031e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x10320;
  character_entry.general_category = No;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x10323;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x10330;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x10349;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1034a;
  character_entry.general_category = Nl;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1034a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x10400;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x10425;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x10428;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1044d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d000;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d164;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d165;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d166;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d167;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d169;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d16a;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d16c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d16d;
  character_entry.general_category = Mc;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d172;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d173;
  character_entry.general_category = Cf;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d17a;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d17b;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d182;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d183;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d184;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d185;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d18b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d18c;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d1a9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d1aa;
  character_entry.general_category = Mn;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d1ad;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d1ae;
  character_entry.general_category = So;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d1dd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d400;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d419;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d41a;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d433;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d434;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d44d;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d44e;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d467;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d468;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d481;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d482;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d49b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d49c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d4b5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d4b6;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d4cf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d4d0;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d4e9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d4ea;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d503;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d504;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d51c;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d51e;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d537;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d538;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d550;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d552;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d56b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d56c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d585;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d586;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d59f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d5a0;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d5b9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d5ba;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d5d3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d5d4;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d5ed;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d5ee;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d607;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d608;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d621;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d622;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d63b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d63c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d655;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d656;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d66f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d670;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d689;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d68a;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d6a3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d6a8;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d6c0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d6c1;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d6c1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d6c2;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d6da;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d6db;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d6db;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d6dc;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d6e1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d6e2;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d6fa;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d6fb;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d6fb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d6fc;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d714;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d715;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d715;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d716;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d71b;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d71c;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d734;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d735;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d735;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d736;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d74e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d74f;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d74f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d750;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d755;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d756;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d76e;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d76f;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d76f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d770;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d788;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d789;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d789;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d78a;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d78f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d790;
  character_entry.general_category = Lu;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d7a8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7a9;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d7a9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7aa;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d7c2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7c3;
  character_entry.general_category = Sm;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d7c3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7c4;
  character_entry.general_category = Ll;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x1d7c9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7ce;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0x1d7ce;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7cf;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0x1d7cf;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7d0;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0x1d7d0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7d1;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0x1d7d1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7d2;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0x1d7d2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7d3;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0x1d7d3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7d4;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0x1d7d4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7d5;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0x1d7d5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7d6;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0x1d7d6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7d7;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0x1d7d7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7d8;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0x1d7d8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7d9;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0x1d7d9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7da;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0x1d7da;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7db;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0x1d7db;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7dc;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0x1d7dc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7dd;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0x1d7dd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7de;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0x1d7de;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7df;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0x1d7df;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7e0;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0x1d7e0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7e1;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0x1d7e1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7e2;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0x1d7e2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7e3;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0x1d7e3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7e4;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0x1d7e4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7e5;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0x1d7e5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7e6;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0x1d7e6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7e7;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0x1d7e7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7e8;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0x1d7e8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7e9;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0x1d7e9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7ea;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0x1d7ea;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7eb;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0x1d7eb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7ec;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0x1d7ec;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7ed;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0x1d7ed;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7ee;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0x1d7ee;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7ef;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0x1d7ef;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7f0;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0x1d7f0;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7f1;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0x1d7f1;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7f2;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0x1d7f2;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7f3;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0x1d7f3;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7f4;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0x1d7f4;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7f5;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0x1d7f5;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7f6;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 0;
  character_entry.code_range_end = 0x1d7f6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7f7;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 1;
  character_entry.code_range_end = 0x1d7f7;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7f8;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 2;
  character_entry.code_range_end = 0x1d7f8;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7f9;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 3;
  character_entry.code_range_end = 0x1d7f9;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7fa;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 4;
  character_entry.code_range_end = 0x1d7fa;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7fb;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 5;
  character_entry.code_range_end = 0x1d7fb;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7fc;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 6;
  character_entry.code_range_end = 0x1d7fc;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7fd;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 7;
  character_entry.code_range_end = 0x1d7fd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7fe;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 8;
  character_entry.code_range_end = 0x1d7fe;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x1d7ff;
  character_entry.general_category = Nd;
  character_entry.decimal_digit_value = 9;
  character_entry.code_range_end = 0x1d7ff;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x20000;
  character_entry.general_category = Lo;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x2a6d6;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xe0001;
  character_entry.general_category = Cf;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xe007f;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0xf0000;
  character_entry.general_category = Co;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0xffffd;
  character_table.push_back(character_entry);
  character_entry.code_range_begin = 0x100000;
  character_entry.general_category = Co;
  character_entry.decimal_digit_value = -1;
  character_entry.code_range_end = 0x10fffd;
  character_table.push_back(character_entry);
  
  --character_table_lock;
  
  return true;
}


/////////////
// Character_entry - Return character entry for UCS4 character.
// character: UCS4 character.
// p_result: pointer to entry to receive the result.
// returns true if the character is defined, false otherwise.
//public:
bool Unicode_data::Character_entry(UCS4_char_t character, character_entry_p_t p_result)
{
  for (character_table_t::iterator
         character_iterator = character_table.begin();
       character_iterator != character_table.end();
       ++character_iterator)
    {
      if (character_iterator->code_range_begin <= character
          && character_iterator->code_range_end >= character)
        {
          if (p_result)
            {
              p_result->code_point = character;
              p_result->general_category
                = character_iterator->general_category;
              p_result->decimal_digit_value
                = character_iterator->decimal_digit_value;
              return true;
            }
          else
            return true;
        }
    }
  
  return false;
}
